/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Configuration page for UTPlus player skin settings.
******************************************************************************/


class UTPGUI_AdminLoginPage extends UTPGUI_ConfigPageBase;


//=============================================================================
// Variables
//=============================================================================

var() automated moEditbox e_AdminName, e_AdminPassword;
var() automated moCheckbox c_SilentLogin;


//== LoadSettings =============================================================
/**
Load the hit sound settings.
*/
//=============================================================================

function LoadSettings()
{
  local UTP_ReplicationInfo UTPlusRI;
  
  foreach PlayerOwner().DynamicActors(class'UTP_ReplicationInfo', UTPlusRI) {
    if ( !UTPlusRI.bAdvancedAdmin )
      DisableComponent(e_AdminName);
    if ( !UTPlusRI.bSilentAdminLogin )
      DisableComponent(c_SilentLogin);
    break;
  }
}


//== SaveSettings =============================================================
/**
Save the hit sound settings.
*/
//=============================================================================

function SaveSettings()
{
  local string Cmd;
  
  Cmd = "ADMINLOGIN";
  if ( c_SilentLogin.IsChecked() )
    Cmd @= "-s";
  if ( e_AdminName.MenuState != MSAT_Disabled )
    Cmd @= e_AdminName.GetComponentValue();
  if ( e_AdminPassword.GetComponentValue() != "" ) {
    Cmd @= e_AdminPassword.GetComponentValue();
    PlayerOwner().ConsoleCommand(Cmd);
  }
}


//== KeyEventAdminName ========================================================
/**
When pressing Enter in the admin name box, move focus to password box.
*/
//=============================================================================

function bool KeyEventAdminName(out byte Key, out byte State, float Delta)
{
  if ( Key == 0x0D && State == 3 ) {
    // Enter released
    e_AdminPassword.SetFocus(e_AdminPassword);
    return true;
  }
  return false;
}


//== KeyEventAdminPassword ====================================================
/**
When pressing Enter in the admin password box, act as if OK was clicked.
*/
//=============================================================================

function bool KeyEventAdminPassword(out byte Key, out byte State, float Delta)
{
  if ( Key == 0x0D && State == 3 ) {
    // Enter released
    Controller.CloseMenu(false);
    return true;
  }
  return false;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  WindowName = "Admin Login"
  
  WinWidth  = 0.5
  WinHeight = 0.3
  WinLeft   = 0.25
  WinTop    = 0.35
  
  DefaultWidth  = 0.5
  DefaultHeight = 0.3
  DefaultLeft   = 0.25
  DefaultTop    = 0.35
  
  Begin Object Class=moEditbox Name=eAdminName
    WinWidth        = 0.8
    WinLeft         = 0.1
    WinTop          = 0.2
    TabOrder        = 1
    bBoundToParent  = True
    bScaleToParent  = True
    Caption         = "User name"
    Hint            = "Enter your admin user name."
    OnKeyEvent      = KeyEventAdminName
  End Object
  e_AdminName = moEditbox'eAdminName'
  
  Begin Object Class=moEditbox Name=eAdminPassword
    WinWidth        = 0.8
    WinLeft         = 0.1
    WinTop          = 0.4
    TabOrder        = 2
    bBoundToParent  = True
    bScaleToParent  = True
    Caption         = "Password"
    Hint            = "Enter your admin password."
    bMaskText       = True
    OnKeyEvent      = KeyEventAdminPassword
  End Object
  e_AdminPassword = moEditbox'eAdminPassword'
  
  Begin Object Class=moCheckbox Name=cSilentLogin
    WinWidth        = 0.8
    WinLeft         = 0.1
    WinTop          = 0.6
    TabOrder        = 3
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Silent login"
    Hint            = "Log in as admin without notifying other players."
  End Object
  c_SilentLogin = moCheckbox'cSilentLogin'
}
