/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Configuration page for UTPlus player skin settings.
******************************************************************************/


class UTPGUI_ConfigPageHitSounds extends UTPGUI_ConfigPageBase;


//=============================================================================
// Variables
//=============================================================================

var() automated moSlider s_HitSoundEnemy, s_HitSoundTeam, s_HitSoundProtected, s_HitSoundVehicle, s_HitSoundEmptyVehicle, s_HitSoundVolume;
var() automated moCheckbox c_VariableHitSounds;
var() automated GUIButton b_EditHitSoundNames, b_ResetHitSoundNames;
var() localized string lblHitSoundNames; 


//== InitComponent ============================================================
/**
Assign caption delegates for sliders.
*/
//=============================================================================

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
  Super.InitComponent(MyController, MyOwner);
  
  s_HitSoundEnemy.MySlider.OnDrawCaption      = GetCaptionHitSoundEnemy;
  s_HitSoundEnemy.MySlider.bShowCaption       = True;
  s_HitSoundEnemy.MySlider.bShowValueTooltip  = False;
  
  s_HitSoundTeam.MySlider.OnDrawCaption     = GetCaptionHitSoundTeam;
  s_HitSoundTeam.MySlider.bShowCaption      = True;
  s_HitSoundTeam.MySlider.bShowValueTooltip = False;
  
  s_HitSoundProtected.MySlider.OnDrawCaption      = GetCaptionHitSoundProtected;
  s_HitSoundProtected.MySlider.bShowCaption       = True;
  s_HitSoundProtected.MySlider.bShowValueTooltip  = False;
  
  s_HitSoundVehicle.MySlider.OnDrawCaption      = GetCaptionHitSoundVehicle;
  s_HitSoundVehicle.MySlider.bShowCaption       = True;
  s_HitSoundVehicle.MySlider.bShowValueTooltip  = False;
  
  s_HitSoundEmptyVehicle.MySlider.OnDrawCaption     = GetCaptionHitSoundEmptyVehicle;
  s_HitSoundEmptyVehicle.MySlider.bShowCaption      = True;
  s_HitSoundEmptyVehicle.MySlider.bShowValueTooltip = False;
}


//== LoadSettings =============================================================
/**
Load the hit sound settings.
*/
//=============================================================================

function LoadSettings()
{
  s_HitSoundEnemy       .Setup(0, class'UTP_PlayerReplicationInfo'.default.HitSoundNames.Length - 1, True);
  s_HitSoundTeam        .Setup(0, class'UTP_PlayerReplicationInfo'.default.HitSoundNames.Length - 1, True);
  s_HitSoundProtected   .Setup(0, class'UTP_PlayerReplicationInfo'.default.HitSoundNames.Length - 1, True);
  s_HitSoundVehicle     .Setup(0, class'UTP_PlayerReplicationInfo'.default.HitSoundNames.Length - 1, True);
  s_HitSoundEmptyVehicle.Setup(0, class'UTP_PlayerReplicationInfo'.default.HitSoundNames.Length - 1, True);
  
  s_HitSoundEnemy       .SetComponentValue(class'UTP_ClientSettings'.default.HitSoundEnemy,         True);
  s_HitSoundTeam        .SetComponentValue(class'UTP_ClientSettings'.default.HitSoundTeam,          True);
  s_HitSoundProtected   .SetComponentValue(class'UTP_ClientSettings'.default.HitSoundProtected,     True);
  s_HitSoundVehicle     .SetComponentValue(class'UTP_ClientSettings'.default.HitSoundVehicle,       True);
  s_HitSoundEmptyVehicle.SetComponentValue(class'UTP_ClientSettings'.default.HitSoundEmptyVehicle,  True);
  s_HitSoundVolume      .SetComponentValue(class'UTP_ClientSettings'.default.HitSoundVolume,        True);
  c_VariableHitSounds   .SetComponentValue(class'UTP_ClientSettings'.default.bVariableHitSounds,    True);
}


//== SaveSettings =============================================================
/**
Save the hit sound settings.
*/
//=============================================================================

function SaveSettings()
{
  class'UTP_ClientSettings'.default.HitSoundEnemy         = s_HitSoundEnemy.GetValue();
  class'UTP_ClientSettings'.default.HitSoundTeam          = s_HitSoundTeam.GetValue();
  class'UTP_ClientSettings'.default.HitSoundProtected     = s_HitSoundProtected.GetValue();
  class'UTP_ClientSettings'.default.HitSoundVehicle       = s_HitSoundVehicle.GetValue();
  class'UTP_ClientSettings'.default.HitSoundEmptyVehicle  = s_HitSoundEmptyVehicle.GetValue();
  class'UTP_ClientSettings'.default.HitSoundVolume        = s_HitSoundVolume.GetValue();
  class'UTP_ClientSettings'.default.bVariableHitSounds    = c_VariableHitSounds.IsChecked();
  class'UTP_ClientSettings'.static.StaticSaveConfig();
}


//== TestHitSound =============================================================
/**
Plays the selected hit sound for testing. This function is simulated so the sound only plays locally even on the server.  
*/
//=============================================================================

simulated function TestHitSound(GUIComponent Sender)
{
  local Sound HitSound;
  
  if ( Sender == s_HitSoundVolume )
    Sender = s_HitSoundEnemy;
  
  if ( moSlider(Sender) != None && s_HitSoundVolume.GetValue() > 0 ) {
    HitSound = Sound(DynamicLoadObject(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[moSlider(Sender).GetValue()], class'Sound'));
    PlayerOwner().ViewTarget.PlaySound(HitSound,, s_HitSoundVolume.GetValue(),,,, False);
  }
}


//== GetCaptionHitSound* ======================================================
/**
Returns the caption for the hit sound sliders. (Crappy delegate declarations require me to have a separate function for each slider. :x)
*/
//=============================================================================

function string GetCaptionHitSoundEnemy()
{
  return GetItemName(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[s_HitSoundEnemy.GetValue()]);
}

function string GetCaptionHitSoundTeam()
{
  return GetItemName(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[s_HitSoundTeam.GetValue()]);
}

function string GetCaptionHitSoundProtected()
{
  return GetItemName(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[s_HitSoundProtected.GetValue()]);
}

function string GetCaptionHitSoundVehicle()
{
  return GetItemName(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[s_HitSoundVehicle.GetValue()]);
}

function string GetCaptionHitSoundEmptyVehicle()
{
  return GetItemName(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[s_HitSoundEmptyVehicle.GetValue()]);
}


//== EditHitSoundList =========================================================
/**
Plays the selected hit sound for testing.
*/
//=============================================================================

function bool EditHitSoundList(GUIComponent Sender)
{
  local GUIArrayPropPage ArrayPage;
  local PlayInfo HitSoundPI;
  local int i;
  
  if ( Sender == b_EditHitSoundNames ) {
    HitSoundPI = new class'PlayInfo';
    HitSoundPI.Clear();
    HitSoundPI.AddClass(class'UTP_PlayerReplicationInfo');
    HitSoundPI.AddSetting("HitSounds", "HitSoundNames", lblHitSoundNames, 0,0, "Text");
    
    if ( Controller.OpenMenu(Controller.DynArrayPropertyMenu) ) {
      ArrayPage = GUIArrayPropPage(Controller.ActivePage);
      ArrayPage.Item = HitSoundPI.Settings[0];
      ArrayPage.OnClose = HitSoundListClosed;
      ArrayPage.SetOwner(Self);
    }
  }
  else if ( Sender == b_ResetHitSoundNames ) {
    class'UTP_PlayerReplicationInfo'.static.StaticClearConfig("HitSoundNames");
    
    // for some weird reason StaticClearConfig adds quotes to the actual values
    for (i = 0; i < class'UTP_PlayerReplicationInfo'.default.HitSoundNames.Length; ++i) {
      if ( Left(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[i], 1) == "\"" && Right(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[i], 1) == "\"" )
        class'UTP_PlayerReplicationInfo'.default.HitSoundNames[i] = Mid(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[i], 1, Len(class'UTP_PlayerReplicationInfo'.default.HitSoundNames[i]) - 2);
    }
    class'UTP_PlayerReplicationInfo'.static.StaticSaveConfig();
    
    class'UTP_ClientSettings'.static.StaticClearConfig("HitSoundEnemy");
    class'UTP_ClientSettings'.static.StaticClearConfig("HitSoundTeam");
    class'UTP_ClientSettings'.static.StaticClearConfig("HitSoundProtected");
    class'UTP_ClientSettings'.static.StaticClearConfig("HitSoundVehicle");
    class'UTP_ClientSettings'.static.StaticClearConfig("HitSoundEmptyVehicle");
    class'UTP_ClientSettings'.static.StaticClearConfig("HitSoundVolume");
    class'UTP_ClientSettings'.static.StaticSaveConfig();
    
    LoadSettings();
  }
  return true;
}


//== HitSoundListClosed =======================================================
/**
Plays the selected hit sound for testing.
*/
//=============================================================================

function HitSoundListClosed(optional bool bCancelled)
{
  local GUIArrayPropPage ArrayPage;
  local UTP_PlayerReplicationInfo UTP_PRI;
  
  if ( !bCancelled ) {
    ArrayPage = GUIArrayPropPage(Controller.ActivePage);
    if ( ArrayPage != None ) {
      class'UTP_PlayerReplicationInfo'.default.HitSoundNames = ArrayPage.PropValue;
      class'UTP_PlayerReplicationInfo'.static.StaticSaveConfig();
      foreach PlayerOwner().DynamicActors(class'UTP_PlayerReplicationInfo', UTP_PRI) {
        UTP_PRI.HitSoundNames = class'UTP_PlayerReplicationInfo'.default.HitSoundNames;
        UTP_PRI.HitSounds.Length = 0;
      }
    }
    
    LoadSettings();
  }
}


//== AlignButtons =============================================================
/**
Aligns the buttons in this menu page.
*/
//=============================================================================

function AlignButtons()
{
  local float X, Y, Xs, Ys;
  local float WIP, HIP;
  
  Super.AlignButtons();
  
  WIP = ActualWidth();
  HIP = ActualHeight();
  
  Xs = b_Ok.ActualWidth()   * 0.1;
  Ys = b_Ok.ActualHeight()  * 0.1;
  
  X = 1 - (b_Ok.ActualWidth()  + Xs) / WIP - EdgeBorder[2] / WIP;
  Y = 1 - (b_Ok.ActualHeight() + Ys) / HIP - EdgeBorder[3] / HIP;
  
  b_Ok.WinLeft = X;
  b_Ok.WinTop = Y;
  
  X = 1 - (b_Ok.ActualWidth() + b_Cancel.ActualWidth() + Xs) / WIP - EdgeBorder[2] / WIP;
  
  b_Cancel.WinLeft = X;
  b_Cancel.WinTop = Y;
  
  X = Xs / WIP + EdgeBorder[0] / WIP;
  Y = 1 - (b_ResetHitSoundNames.ActualHeight() + Ys) / HIP - EdgeBorder[3] / HIP;
  
  b_ResetHitSoundNames.WinLeft = X;
  b_ResetHitSoundNames.WinTop  = Y;
  
  Y = 1 - (b_EditHitSoundNames.ActualHeight() + b_ResetHitSoundNames.ActualHeight() + Ys) / HIP - EdgeBorder[3] / HIP;
  
  b_EditHitSoundNames.WinLeft = X;
  b_EditHitSoundNames.WinTop  = Y;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  WindowName = "UT Plus Hit Sound Options"
  
  WinWidth  = 0.8
  WinHeight = 0.5
  WinLeft   = 0.1
  WinTop    = 0.25
  
  DefaultWidth  = 0.8
  DefaultHeight = 0.5
  DefaultLeft   = 0.1
  DefaultTop    = 0.25
  
  lblHitSoundNames = "Hit Sounds List"
  
  Begin Object Class=moSlider Name=sHitSoundEnemy
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.1
    CaptionWidth    = 0.35
    TabOrder        = 1
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enemy"
    Hint            = "Select the basic hit sound for enemy players."
    OnChange        = TestHitSound
    OnClickSound    = CS_None
  End Object
  s_HitSoundEnemy = moSlider'sHitSoundEnemy'
  
  Begin Object Class=moSlider Name=sHitSoundTeam
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.25
    CaptionWidth    = 0.35
    TabOrder        = 2
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Team"
    Hint            = "Select the hit sound for players on your team."
    OnChange        = TestHitSound
    OnClickSound    = CS_None
  End Object
  s_HitSoundTeam = moSlider'sHitSoundTeam'
  
  Begin Object Class=moSlider Name=sHitSoundProtected
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.4
    CaptionWidth    = 0.35
    TabOrder        = 3
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Protected"
    Hint            = "Select the hit sound for players with spawn protection."
    OnChange        = TestHitSound
    OnClickSound    = CS_None
  End Object
  s_HitSoundProtected = moSlider'sHitSoundProtected'
  
  Begin Object Class=moSlider Name=sHitSoundVehicle
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.1
    CaptionWidth    = 0.35
    TabOrder        = 4
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Vehicle"
    Hint            = "Select the hit sound for enemy vehicles."
    OnChange        = TestHitSound
    OnClickSound    = CS_None
  End Object
  s_HitSoundVehicle = moSlider'sHitSoundVehicle'
  
  Begin Object Class=moSlider Name=sHitSoundEmptyVehicle
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.25
    CaptionWidth    = 0.35
    TabOrder        = 5
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Empty Vehicle"
    Hint            = "Select the hit sound for empty vehicles."
    OnChange        = TestHitSound
    OnClickSound    = CS_None
  End Object
  s_HitSoundEmptyVehicle = moSlider'sHitSoundEmptyVehicle'
  
  Begin Object Class=moSlider Name=sHitSoundVolume
    WinWidth        = 0.94
    WinLeft         = 0.03
    WinTop          = 0.55
    CaptionWidth    = 0.3
    TabOrder        = 6
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Hit Sound Volume"
    Hint            = "Select the volume for all hit sounds."
    OnChange        = TestHitSound
    OnClickSound    = CS_None
    MinValue        = 0.0
    MaxValue        = 4.0
    bIntSlider      = False
  End Object
  s_HitSoundVolume = moSlider'sHitSoundVolume'
  
  Begin Object Class=moCheckbox Name=cVariableHitSounds
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.4
    TabOrder        = 7
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Variable Hit Sounds"
    Hint            = "Select, whether hit sounds should have a damage-based pitch."
  End Object
  c_VariableHitSounds = moCheckbox'cVariableHitSounds'
  
  Begin Object Class=GUIButton Name=btnEditHitSoundNames
    WinWidth    = 0.319298
    bAutoShrink = False
    Caption     = "Edit Hit Sound List"
    Hint        = "Allows you to edit the list of hit sound names."
    TabOrder    = 97
    OnClick     = EditHitSoundList
  End Object
  b_EditHitSoundNames = GUIButton'btnEditHitSoundNames'
  
  Begin Object Class=GUIButton Name=btnResetHitSoundNames
    WinWidth    = 0.319298
    bAutoShrink = False
    Caption     = "Reset Hit Sound List"
    Hint        = "Resets the list of hit sound names to the default values."
    TabOrder    = 98
    OnClick     = EditHitSoundList
  End Object
  b_ResetHitSoundNames = GUIButton'btnResetHitSoundNames'
}
