/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Configuration page for UTPlus player skin settings.
******************************************************************************/


class UTPGUI_ConfigPageMisc extends UTPGUI_ConfigPageBase
    dependson(UTP_ClientSettings);


//=============================================================================
// Localization
//=============================================================================

var localized string PingCompensationTypes[5];
var localized string PingCompensationHint;
var localized string PingCompensationDescription[5];


//=============================================================================
// Variables
//=============================================================================

// ping and ping compensation
var() automated moComboBox cb_PingCompensationPlayers;
var() automated moCheckbox c_PingCompensationProjectiles, c_DisplayConnectionStatus;
var() automated moSlider s_PingSmoothing;

// DeRes
var() automated moCheckbox c_InstantDeRes, c_RisingDeRes, c_TeamDeRes, c_SkipLeaveBodyEffect;

// team beacon options
var() automated moCheckbox c_HideSpectatorBeacons;

// other options
var() automated moCheckbox c_PlayOwnFootsteps, c_PlayOwnLandingSound, c_ConstantCrosshairSize, c_NoDamageIndicators;
var() automated moCheckbox c_EnableClientPlayerIDDB, c_LogUTPlusConsoleActivity;
var() automated moFloatEdit e_RolloffFactor;

// auto-documentation
var() automated moCheckbox c_AutoScreenshot, c_AutoDemoRecording;
var() automated moEditBox  e_AutoScreenshotName, e_AutoDemoName;


//== LoadSettings =============================================================
/**
Load the hit sound settings.
*/
//=============================================================================

function LoadSettings()
{
  local int i;
  
  for (i = 0; i < ArrayCount(PingCompensationTypes); ++i)
    cb_PingCompensationPlayers.AddItem(PingCompensationTypes[i]);
  
  cb_PingCompensationPlayers.SetIndex(class'UTP_ClientSettings'.default.PingCompensationPlayers);
  c_PingCompensationProjectiles.SetComponentValue(class'UTP_ClientSettings'.default.bPingCompensationProjectiles, True);
  c_DisplayConnectionStatus.SetComponentValue(class'UTP_ClientSettings'.default.bDisplayConnectionStatus, True);
  s_PingSmoothing.SetComponentValue(class'UTP_ClientSettings'.default.PingSmoothing, True);
  
  c_InstantDeRes.SetComponentValue(class'UTP_ClientSettings'.default.bInstantDeRes, True);
  c_RisingDeRes.SetComponentValue(class'UTP_ClientSettings'.default.bRisingDeRes, True);
  c_TeamDeRes.SetComponentValue(class'UTP_ClientSettings'.default.bTeamDeRes, True);
  c_SkipLeaveBodyEffect.SetComponentValue(class'UTP_ClientSettings'.default.bSkipLeaveBodyEffect, True);
  
  c_HideSpectatorBeacons.SetComponentValue(class'UTP_ClientSettings'.default.bHideSpectatorBeacons, True);
  
  c_ConstantCrosshairSize.SetComponentValue(class'UTP_ClientSettings'.default.bConstantCrosshairSize, True);
  c_NoDamageIndicators.SetComponentValue(class'UTP_ClientSettings'.default.bNoDamageIndicators, True);
  
  c_PlayOwnFootsteps.SetComponentValue(class'UTP_Pawn'.default.bPlayOwnFootsteps, True);
  c_PlayOwnLandingSound.SetComponentValue(class'UTP_ClientSettings'.default.bPlayOwnLandingSound, True);
  e_RolloffFactor.SetComponentValue(class'UTP_ClientSettings'.default.RolloffFactor, True);
  
  if ( class'UTP_Pawn'.default.bPlayOwnFootsteps )
    DisableComponent(c_PlayOwnLandingSound);
  
  c_EnableClientPlayerIDDB.SetComponentValue(class'UTP_ClientSettings'.default.bEnableClientPlayerIDDB, True);
  c_LogUTPlusConsoleActivity.SetComponentValue(class'UTP_ClientSettings'.default.bLogUTPlusConsoleActivity, True);
  
  c_AutoScreenshot.SetComponentValue(class'UTP_ClientSettings'.default.bAutoScreenshot, True);
  e_AutoScreenshotName.MyEditBox.AllowedCharSet = class'UTP_Util'.default.AllowedFileNameChars;
  e_AutoScreenshotName.MyEditBox.MaxWidth = 50;
  e_AutoScreenshotName.SetComponentValue(class'UTP_ClientSettings'.default.AutoScreenshotName, True);
  c_AutoDemoRecording.SetComponentValue(class'UTP_ClientSettings'.default.bAutoDemoRecording, True);
  e_AutoDemoName.MyEditBox.AllowedCharSet = class'UTP_Util'.default.AllowedFileNameChars;
  e_AutoDemoName.MyEditBox.MaxWidth = 50;
  e_AutoDemoName.SetComponentValue(class'UTP_ClientSettings'.default.AutoDemoName, True);
}


//== SaveSettings =============================================================
/**
Save the hit sound settings.
*/
//=============================================================================

function SaveSettings()
{
  local UnrealPawn P;
  local UTP_PlayerReplicationInfo UTPPRI;
  
  switch (cb_PingCompensationPlayers.GetIndex()) {
    case 1:
      class'UTP_ClientSettings'.default.PingCompensationPlayers = PredictionOnly;
      break;
    case 2:
      class'UTP_ClientSettings'.default.PingCompensationPlayers = PredictionPreferred;
      break;
    case 3:
      class'UTP_ClientSettings'.default.PingCompensationPlayers = ServerPreferred;
      break;
    case 4:
      class'UTP_ClientSettings'.default.PingCompensationPlayers = ServerOnly;
      break;
    default:
      class'UTP_ClientSettings'.default.PingCompensationPlayers = Off;
  }
  UTPPRI = class'UTP_PlayerReplicationInfo'.static.GetFor(PlayerOwner());
  if (UTPPRI != None)
    UTPPRI.ServerSetPingCompensationType(class'UTP_ClientSettings'.default.PingCompensationPlayers);
  
  class'UTP_ClientSettings'.default.bPingCompensationProjectiles= c_PingCompensationProjectiles.IsChecked();
  class'UTP_ClientSettings'.default.bDisplayConnectionStatus    = c_DisplayConnectionStatus.IsChecked();
  class'UTP_ClientSettings'.default.PingSmoothing               = s_PingSmoothing.GetValue();
  
  class'UTP_ClientSettings'.default.bInstantDeRes = c_InstantDeRes.IsChecked();
  class'UTP_ClientSettings'.default.bRisingDeRes  = c_RisingDeRes.IsChecked();
  class'UTP_ClientSettings'.default.bTeamDeRes    = c_TeamDeRes.IsChecked();
  
  class'UTP_ClientSettings'.default.bHideSpectatorBeacons = c_HideSpectatorBeacons.IsChecked();
  
  class'UTP_ClientSettings'.default.bConstantCrosshairSize  = c_ConstantCrosshairSize.IsChecked();
  class'UTP_ClientSettings'.default.bNoDamageIndicators     = c_NoDamageIndicators.IsChecked();
  
  class'UTP_ClientSettings'.default.bPlayOwnLandingSound  = c_PlayOwnLandingSound.IsChecked();
  class'UTP_ClientSettings'.default.RolloffFactor         = e_RolloffFactor.GetValue();
  
  class'UTP_ClientSettings'.default.bEnableClientPlayerIDDB   = c_EnableClientPlayerIDDB.IsChecked();
  class'UTP_ClientSettings'.default.bLogUTPlusConsoleActivity = c_LogUTPlusConsoleActivity.IsChecked();
  
  class'UTP_ClientSettings'.default.bAutoScreenshot     = c_AutoScreenshot.IsChecked();
  class'UTP_ClientSettings'.default.AutoScreenshotName  = e_AutoScreenshotName.GetText();
  class'UTP_ClientSettings'.default.bAutoDemoRecording  = c_AutoDemoRecording.IsChecked();
  class'UTP_ClientSettings'.default.AutoDemoName        = e_AutoDemoName.GetText();
  
  class'UTP_ClientSettings'.static.StaticSaveConfig();
  
  class'UTP_Pawn'.default.bPlayOwnFootsteps = c_PlayOwnFootsteps.IsChecked();
  class'UTP_Pawn'.static.StaticSaveConfig();
  foreach PlayerOwner().AllActors(class'UnrealPawn', P)
    P.bPlayOwnFootsteps = class'UTP_Pawn'.default.bPlayOwnFootsteps;
}


//== UpdateOwnLandingSound ====================================================
/**
Update the "Play own landing sound" checkbox state.
*/
//=============================================================================

function UpdateOwnLandingSound(GUIComponent Sender)
{
  if ( c_PlayOwnFootsteps.IsChecked() )
    DisableComponent(c_PlayOwnLandingSound);
  else
    EnableComponent(c_PlayOwnLandingSound);
}


//== CompensationTypeChanged ==================================================
/**
Update the hint of the ping compensation combo box.
*/
//=============================================================================

function CompensationTypeChanged(GUIComponent Sender)
{
  cb_PingCompensationPlayers.SetHint(PingCompensationHint @ PingCompensationDescription[cb_PingCompensationPlayers.GetIndex()]);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  WindowName = "UT Plus Misc Configuration Options"
  
  WinWidth  = 0.8
  WinHeight = 0.75
  WinLeft   = 0.1
  WinTop    = 0.15
  
  DefaultWidth  = 0.8
  DefaultHeight = 0.75
  DefaultLeft   = 0.1
  DefaultTop    = 0.15
  
  
  // DeRes
  Begin Object Class=moCheckbox Name=cInstantDeRes
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.08
    TabOrder  = 11
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Instant DeRes"
    Hint            = "Players DeRes immediately when killed."
  End Object
  c_InstantDeRes = moCheckbox'cInstantDeRes'
  
  Begin Object Class=moCheckbox Name=cRisingDeRes
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.16
    TabOrder  = 12
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Rising DeRes"
    Hint            = "DeRes-ing players' bodies float upwards."
  End Object
  c_RisingDeRes = moCheckbox'cRisingDeRes'
  
  Begin Object Class=moCheckbox Name=cTeamDeRes
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.24
    TabOrder  = 13
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Team-colored DeRes"
    Hint            = "DeRes effects use team colors."
  End Object
  c_TeamDeRes = moCheckbox'cTeamDeRes'
  
  Begin Object Class=moCheckbox Name=cSkipLeaveBodyEffect
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.32
    TabOrder  = 14
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Skip Linkgun death effect"
    Hint            = "Do not play the 'body leave effect' for a kill with the Linkgun secondary firing mode."
  End Object
  c_SkipLeaveBodyEffect = moCheckbox'cSkipLeaveBodyEffect'
  
  
  // other display and sound options
  Begin Object Class=moCheckbox Name=cHideSpectatorBeacons
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.40
    TabOrder  = 21
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Hide spectator beacons"
    Hint            = "Disable the team beacons in spectator mode."
  End Object
  c_HideSpectatorBeacons = moCheckbox'cHideSpectatorBeacons'
  
  Begin Object Class=moCheckbox Name=cConstantCrosshairSize
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.48
    TabOrder  = 22
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Constant crosshair size"
    Hint            = "Prevents the crosshair from resizing when you pick up items."
  End Object
  c_ConstantCrosshairSize = moCheckbox'cConstantCrosshairSize'
  
  Begin Object Class=moCheckbox Name=cNoDamageIndicators
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.56
    TabOrder  = 23
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "No damage indicators"
    Hint            = "Disables the red screen flashes indicating when you receive damage."
  End Object
  c_NoDamageIndicators = moCheckbox'cNoDamageIndicators'
  
  
  // auto-documenting
  Begin Object Class=moCheckbox Name=cAutoScreenshot
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.64
    TabOrder  = 31
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Scoreboard screenshots"
    Hint            = "Automatically take a screenshot of the scoreboard at the end of each round and at the end of the match."
  End Object
  c_AutoScreenshot = moCheckbox'cAutoScreenshot'
  
  Begin Object Class=moEditbox Name=eAutoScreenshotName
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.72
    TabOrder  = 32
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Screenshot file name"
    Hint            = "Enter the file name for automatically taken screenshots."
  End Object
  e_AutoScreenshotName = moEditbox'eAutoScreenshotName'
  
  Begin Object Class=moCheckbox Name=cAutoDemoRecording
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.80
    TabOrder  = 33
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Auto demo recording"
    Hint            = "Automatically record client-side demos."
  End Object
  c_AutoDemoRecording = moCheckbox'cAutoDemoRecording'
  
  Begin Object Class=moEditbox Name=eAutoDemoName
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.88
    TabOrder  = 34
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Demo file name"
    Hint            = "Enter the file name for automatically recorded client-side demos."
  End Object
  e_AutoDemoName = moEditbox'eAutoDemoName'
  
  
  // ping compensation
  Begin Object Class=moComboBox Name=cbPingCompensationPlayers
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.08
    TabOrder  = 41
    bBoundToParent  = True 
    bScaleToParent  = True
    bReadOnly       = True
    Caption         = "Ping Compensation"
    OnChange        = CompensationTypeChanged
  End Object
  cb_PingCompensationPlayers = moComboBox'cbPingCompensationPlayers'
  
  PingCompensationTypes[0] = "Off"
  PingCompensationTypes[1] = "Prediction Only"
  PingCompensationTypes[2] = "Prediction Prefered"
  PingCompensationTypes[3] = "Server Prefered"
  PingCompensationTypes[4] = "Server Only"
  
  PingCompensationHint           = "Ping compensation method for players:"
  PingCompensationDescription[0] = "Neither client-side ping prediction, nor server-side ping compensation."
  PingCompensationDescription[1] = "Only client-side ping prediction. (server-side ping compensation is ignored)"
  PingCompensationDescription[2] = "Client-side ping prediction, but use server-side compensation if no prediction allowed."
  PingCompensationDescription[3] = "Server-side ping compensation, but use client-side prediction if no compensation available."
  PingCompensationDescription[4] = "Only server-side ping compensation. (client-side ping prediction is ignored)"
  
  Begin Object Class=moCheckbox Name=cPingCompensationProjectiles
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.16
    TabOrder  = 42
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Shockball Ping Prediction"
    Hint            = "Adjust display location of shock balls to compensate ping."
  End Object
  c_PingCompensationProjectiles = moCheckbox'cPingCompensationProjectiles'
  
  Begin Object Class=moCheckbox Name=cDisplayConnectionStatus
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.24
    TabOrder  = 43
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Display connection status"
    Hint            = "Adjust display location of shock balls and redeemer rockets to compensate ping."
  End Object
  c_DisplayConnectionStatus = moCheckbox'cDisplayConnectionStatus'
  
  Begin Object Class=moSlider Name=sPingSmoothing
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.32
    TabOrder  = 44
    bBoundToParent  = True 
    bScaleToParent  = True
    bIntSlider      = True
    MinValue        = 1
    MaxValue        = 100
    Caption         = "Ping display smoothing"
    Hint            = "Select the number of recent pings to be used for calculating the value for connection status display and client-side ping compensation. Note: Setting this to 1 will turn off ping smoothing, which is not useful for ping prediction!"
  End Object
  s_PingSmoothing = moSlider'sPingSmoothing'
  
  
  Begin Object Class=moCheckbox Name=cPlayOwnFootsteps
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.40
    TabOrder  = 54
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Play own footsteps"
    Hint            = "Select, whether you want to hear your own player model's footsteps."
    OnChange        = UpdateOwnLandingSound
  End Object
  c_PlayOwnFootsteps = moCheckbox'cPlayOwnFootsteps'
  
  Begin Object Class=moCheckbox Name=cPlayOwnLandingSound
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.48
    TabOrder  = 55
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Play own landing sound"
    Hint            = "Select, whether you want to hear your own player model's landing sound after jumping or dodging."
  End Object
  c_PlayOwnLandingSound = moCheckbox'cPlayOwnLandingSound'
  
  Begin Object Class=moFloatEdit Name=eRolloffFactor
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.56
    TabOrder  = 55
    bBoundToParent  = True 
    bScaleToParent  = True
    MinValue        = 0.4
    MaxValue        = 1.0
    Step            = 0.05
    Caption         = "Rolloff Factor"
    Hint            = "Enter the desired rolloff factor. Lower values make distant sounds louder. This setting is also used when UTPlus is not active."
  End Object
  e_RolloffFactor = moFloatEdit'eRolloffFactor'
  

  // other things
  Begin Object Class=moCheckbox Name=cEnableClientPlayerIDDB
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.64
    TabOrder  = 61
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable client player ID DB"
    Hint            = "Disable this checkbox if you do not wish to collect the IDs of other players on the server."
  End Object
  c_EnableClientPlayerIDDB = moCheckbox'cEnableClientPlayerIDDB'
  
  Begin Object Class=moCheckbox Name=cLogUTPlusConsoleActivity
    WinWidth  = 0.45
    WinLeft   = 0.52
    WinTop    = 0.72
    TabOrder  = 62
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Log UTPlus console activity"
    Hint            = "When enabled, all UTPlus console output is logged to the file 'UT2004\\UserLogs\\UTP_ClientConsole.log'."
  End Object
  c_LogUTPlusConsoleActivity = moCheckbox'cLogUTPlusConsoleActivity'
}
