/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Configuration page for UTPlus player skin settings.
******************************************************************************/


class UTPGUI_ConfigPagePlayerSkins extends UTPGUI_ConfigPageBase;


//=============================================================================
// Localization
//=============================================================================

var           GUITabItem  TeamColorEnemy, TeamColorFriend;
var           GUITabItem  TeamColors[5];
var localized string      ForcedTeamColors[4];
var localized string      ForcedNeutralColor, OriginalColor;


//=============================================================================
// Variables
//=============================================================================

// overlays
var() automated moCheckbox c_UseSkinOverlay/*, c_AlphaBlendOverlay*/;
var() automated GUITabControl t_OverlayColors;
var() noexport array<UTPGUI_RGBSliderPanel> p_TeamColors;

// forced models
var() automated moCheckbox c_BrightSkins, c_ForceEnemyModel, c_ForceTeamModel;
var() automated moCombobox cb_ForcedEnemyModel, cb_ForcedEnemyColor;
var() automated moCombobox cb_ForcedTeamModel, cb_ForcedTeamColor;

// map-spcific
var() automated moCheckbox c_MapSpecific;

// player mesh preview
var noexport UTP_PlayerMeshActor  MeshActor;
var noexport xUtil.PlayerRecord   Rec;
var noexport UTP_MapSettings      Settings;

var() string  CharacterName;
var() vector  MeshLoc;
var() rotator MeshRot;
var() color   OverlayColor;
var() byte    TeamNum;
var() bool    bEnemy;

var byte EnemyTeam, FriendlyTeam;
var bool bLocalSpectator;


//== LoadSettings =============================================================
/**
Load the settings and create the preview mesh actor.
*/
//=============================================================================

function LoadSettings()
{
  local int i;
  local array<xUtil.PlayerRecord> PlayerRecordList;
  local array<string> PlayerRecords;
  
  // load map-specific settings
  Settings = class'UTP_MapSettings'.static.GetClientConfiguration(PlayerOwner(), string(PlayerOwner().Outer));
  
  CharacterName = PlayerOwner().GetURLOption("Character");
  
  // find team numbers
  if ( PlayerOwner().PlayerReplicationInfo != None && !PlayerOwner().PlayerReplicationInfo.bOnlySpectator ) {
    if ( PlayerOwner().PlayerReplicationInfo.Team != None && PlayerOwner().PlayerReplicationInfo.Team.TeamIndex != 255 ) {
      FriendlyTeam = PlayerOwner().PlayerReplicationInfo.Team.TeamIndex;
      EnemyTeam = (FriendlyTeam + 1) % 2;
    }
    else {
      FriendlyTeam = 255;
      EnemyTeam = 255;
    }
  }
  else if ( PlayerOwner().PlayerReplicationInfo != None && PlayerOwner().PlayerReplicationInfo.bOnlySpectator ) {
    bLocalSpectator = True;
  }
  
  // create skin overlay panels
  for (i = 0; i < ArrayCount(TeamColors); ++i) {
    TeamColors[i].ClassName = string(class'UTPGUI_RGBSliderPanel');
  }
  TeamColorEnemy.ClassName  = string(class'UTPGUI_RGBSliderPanel');
  TeamColorFriend.ClassName = string(class'UTPGUI_RGBSliderPanel');
  
  p_TeamColors[0] = UTPGUI_RGBSliderPanel(t_OverlayColors.AddTabItem(TeamColors[0]));
  p_TeamColors[0].SetColor(Settings.TeamColors[0]);
  p_TeamColors[1] = UTPGUI_RGBSliderPanel(t_OverlayColors.AddTabItem(TeamColors[1]));
  p_TeamColors[1].SetColor(Settings.TeamColors[1]);
  p_TeamColors[2] = UTPGUI_RGBSliderPanel(t_OverlayColors.AddTabItem(TeamColors[4]));
  p_TeamColors[2].SetColor(Settings.TeamColors[4]);
  p_TeamColors[3] = UTPGUI_RGBSliderPanel(t_OverlayColors.AddTabItem(TeamColorEnemy));
  p_TeamColors[3].SetColor(Settings.TeamColorEnemy);
  p_TeamColors[4] = UTPGUI_RGBSliderPanel(t_OverlayColors.AddTabItem(TeamColorFriend));
  p_TeamColors[4].SetColor(Settings.TeamColorFriend);
  
  for (i = 0; i < p_TeamColors.Length; ++i) {
    p_TeamColors[i].OnColorChanged    = OverlayColorChanged;
    p_TeamColors[i].OnPanelActivated  = OverlayPanelActivated;
  }
  
  // fill Force Color combo boxes
  if ( !bLocalSpectator ) {
    if ( EnemyTeam != 255 )
      cb_ForcedEnemyColor.AddItem(OriginalColor @ "(" $ ForcedTeamColors[EnemyTeam] $ ")",, string(254));
    else
      cb_ForcedEnemyColor.AddItem(OriginalColor @ "(" $ ForcedNeutralColor $ ")",, string(254));
  }
  else
    cb_ForcedEnemyColor.AddItem(OriginalColor,, string(254));
  
  if ( !bLocalSpectator ) {
    if ( FriendlyTeam != 255 )
      cb_ForcedTeamColor.AddItem(OriginalColor @ "(" $ ForcedTeamColors[FriendlyTeam] $ ")",, string(254));
    else
      cb_ForcedTeamColor.AddItem(OriginalColor @ "(" $ ForcedNeutralColor $ ")",, string(254));
  }
  else
    cb_ForcedTeamColor.AddItem(OriginalColor,, string(254));
  
  for (i = 0; i < 2; ++i) {
    cb_ForcedEnemyColor.AddItem(ForcedTeamColors[i],, string(i));
    cb_ForcedTeamColor.AddItem(ForcedTeamColors[i],, string(i));
  }
  cb_ForcedEnemyColor.AddItem(ForcedNeutralColor,, string(255));
  cb_ForcedTeamColor.AddItem(ForcedNeutralColor,, string(255));
  
  // fill Force Model combo boxes
  class'xUtil'.static.GetPlayerList(PlayerRecordList);
  for (i = 0; i < PlayerRecordList.Length; ++i) {
    if ( PlayerRecordList[i].DefaultName == "" || PlayerRecordList[i].MeshName == "" || PlayerRecordList[i].BodySkinName == "" || PlayerRecordList[i].FaceSkinName == "" || PlayerRecordList[i].Menu ~= "DUP" )
      continue;
    PlayerRecords[PlayerRecords.Length] = PlayerRecordList[i].DefaultName;
    
    if ( PlayerRecordList[i].DefaultName ~= CharacterName )
      Rec = PlayerRecordList[i];
  }
  class'UTP_Util'.static.SortS(PlayerRecords, True);
  for (i = 0; i < PlayerRecords.Length; ++i) {
    cb_ForcedEnemyModel.AddItem(PlayerRecordList[i].DefaultName,, PlayerRecordList[i].DefaultName);
    cb_ForcedTeamModel.AddItem(PlayerRecordList[i].DefaultName,, PlayerRecordList[i].DefaultName);
  }
    
  // reset non-existant character names in configuration
  if ( cb_ForcedTeamModel.FindExtra(Settings.ForcedTeamModel) == -1 )
    Settings.ClearConfig("ForcedTeamModel");
  
  // Epic bright skins 
  c_BrightSkins.Checked(class'DMMutator'.default.bBrightSkins);
  
  // skin overlay colors
  c_UseSkinOverlay.SetComponentValue(Settings.bUseSkinOverlay, True);
  //c_AlphaBlendOverlay.SetComponentValue(Settings.bAlphaBlendOverlay, True);
  
  // enemy model
  c_ForceEnemyModel.SetComponentValue(Settings.bForceEnemyModel, True);
  cb_ForcedEnemyModel.SetComponentValue(CharacterName, True);
  cb_ForcedEnemyColor.SetComponentValue(Settings.ForcedEnemySkinColor, True);
  
  // team model
  c_ForceTeamModel.SetComponentValue(Settings.bForceTeamModel, True);
  cb_ForcedTeamModel.SetComponentValue(Settings.ForcedTeamModel, True);
  cb_ForcedTeamColor.SetComponentValue(Settings.ForcedTeamSkinColor, True);
  
  // map-specific
  c_MapSpecific.Checked(Settings.bMapSpecific);
  
  // spawn preview actor
  if ( MeshActor == None )
    MeshActor = PlayerOwner().Spawn(class'UTP_PlayerMeshActor', None, '', MeshLoc, MeshRot);
  
  // initialize the components
  ForceModelChanged(None);
  SkinColorChanged(None);
  SkinOverlayChanged(None);
  SetEnemy(None);
}


//== SaveSettings =============================================================
/**
Save the settings here.
*/
//=============================================================================

function SaveSettings()
{
  local int i;
  
  // enemy models
  Settings.bForceEnemyModel = c_ForceEnemyModel.IsChecked();
  Settings.ForcedEnemyModel = cb_ForcedEnemyModel.GetComponentValue();
  if ( c_ForceEnemyModel.IsChecked() && !(Settings.ForcedEnemyModel ~= PlayerOwner().GetURLOption("Character")) ) {
    // force model is enabled, update player character
    PlayerOwner().UpdateURL("Character", Settings.ForcedEnemyModel, true);
    PlayerOwner().ConsoleCommand("ChangeCharacter" @ Settings.ForcedEnemyModel);
  }
  Settings.ForcedEnemySkinColor = byte(cb_ForcedEnemyColor.GetComponentValue());
  
  // team models
  Settings.bForceTeamModel = c_ForceTeamModel.IsChecked();
  Settings.ForcedTeamModel = cb_ForcedTeamModel.GetComponentValue();
  Settings.ForcedTeamSkinColor = byte(cb_ForcedTeamColor.GetComponentValue());
  
  // skin overlays
  Settings.bUseSkinOverlay     = c_UseSkinOverlay.IsChecked();
  //Settings.bAlphaBlendOverlay  = c_AlphaBlendOverlay.IsChecked();
  Settings.TeamColors[0]       = p_TeamColors[0].GetColor();
  Settings.TeamColors[1]       = p_TeamColors[1].GetColor();
  Settings.TeamColors[4]       = p_TeamColors[2].GetColor();
  Settings.TeamColorEnemy      = p_TeamColors[3].GetColor();
  Settings.TeamColorFriend     = p_TeamColors[4].GetColor();
    
  // save configuration
  if ( !c_MapSpecific.IsChecked() ) {
    class'UTP_MapSettings'.default.bForceEnemyModel      = Settings.bForceEnemyModel;
    class'UTP_MapSettings'.default.ForcedEnemyModel      = Settings.ForcedEnemyModel;
    class'UTP_MapSettings'.default.ForcedEnemySkinColor  = Settings.ForcedEnemySkinColor;
    
    class'UTP_MapSettings'.default.bForceTeamModel     = Settings.bForceTeamModel;
    class'UTP_MapSettings'.default.ForcedTeamModel     = Settings.ForcedTeamModel;
    class'UTP_MapSettings'.default.ForcedTeamSkinColor = Settings.ForcedTeamSkinColor;
    
    class'UTP_MapSettings'.default.bUseSkinOverlay     = Settings.bUseSkinOverlay;
    //class'UTP_MapSettings'.default.bAlphaBlendOverlay  = Settings.bAlphaBlendOverlay;
    for (i = 0; i < ArrayCount(Settings.TeamColors); ++i)
      class'UTP_MapSettings'.default.TeamColors[i] = Settings.TeamColors[i];
    
    class'UTP_MapSettings'.default.TeamColorEnemy  = Settings.TeamColorEnemy;
    class'UTP_MapSettings'.default.TeamColorFriend = Settings.TeamColorFriend;
    
    class'UTP_MapSettings'.static.StaticSaveConfig();
    
    if ( Settings.bMapSpecific ) {
      Settings.ClearConfig();
      Settings.bMapSpecific = False;
    }
  }
  else if ( c_MapSpecific.IsChecked() ) {
    Settings.bMapSpecific = True;
    Settings.SaveConfig();
  }
  
  // player skins
  class'DMMutator'.default.bBrightSkins = c_BrightSkins.IsChecked();
  class'DMMutator'.static.StaticSaveConfig();
}


//== Closed ===================================================================
/**
Destroys the preview mesh.
*/
//=============================================================================

function Closed(GUIComponent Sender, bool bCancelled)
{
  local xUtil.PlayerRecord EmptyRecord;
  
  if ( MeshActor != None )
    MeshActor.Destroy();
  MeshActor = None;
  
  Super.Closed(Sender, bCancelled);
  
  Rec = EmptyRecord;
  Settings = None;
}


//== DrawMesh =================================================================
/**
Draws the preview mesh.
*/
//=============================================================================

function DrawMesh(Canvas C)
{
  if ( MeshActor != None ) {
    if ( MeshActor.Location != MeshLoc )
      MeshActor.SetLocation(MeshLoc);
    if ( MeshActor.Rotation != MeshRot )
      MeshActor.SetRotation(MeshRot);
    if ( !(MeshActor.Rec.DefaultName ~= CharacterName) )
      MeshActor.SetCharacter(CharacterName);
    MeshActor.UpdateSkins(TeamNum, OverlayColor, c_BrightSkins.IsChecked());
    C.DrawScreenActor(MeshActor, 30, false, true);
  }
}


//== SetEnemy =================================================================
/**
Display the enemy model.
*/
//=============================================================================

function bool SetEnemy(GUIComponent Sender)
{
  bEnemy = True;
  if ( byte(cb_ForcedEnemyColor.GetComponentValue()) == 254 ) {
    TeamNum = EnemyTeam;
    p_TeamColors[Min(EnemyTeam, 2)].MyButton.OnClick(p_TeamColors[Min(EnemyTeam, 2)].MyButton);
  }
  else {
    TeamNum = byte(cb_ForcedEnemyColor.GetComponentValue());
    p_TeamColors[3].MyButton.OnClick(p_TeamColors[3].MyButton);
  }
  if ( c_ForceEnemyModel.IsChecked() )
    CharacterName = cb_ForcedEnemyModel.GetComponentValue();
  else
    CharacterName = Rec.DefaultName;
  
  return true;
}


//== SetFriendly ==============================================================
/**
Display the team model.
*/
//=============================================================================

function bool SetFriendly(GUIComponent Sender)
{
  bEnemy = False;
  if ( byte(cb_ForcedTeamColor.GetComponentValue()) == 254 ) {
    TeamNum = FriendlyTeam;
    p_TeamColors[Min(FriendlyTeam, 2)].MyButton.OnClick(p_TeamColors[Min(FriendlyTeam, 2)].MyButton);
  }
  else {
    TeamNum = byte(cb_ForcedTeamColor.GetComponentValue());
    p_TeamColors[3].MyButton.OnClick(p_TeamColors[4].MyButton);
  }
  if ( c_ForceTeamModel.IsChecked() )
    CharacterName = cb_ForcedTeamModel.GetComponentValue();
  else
    CharacterName = Rec.DefaultName;
  
  return true;
}


//== ForceModelChanged ========================================================
/**
"Force enemy model" or "Force team model" checkbox changed value.
*/
//=============================================================================

function ForceModelChanged(GUIComponent Sender)
{
  if ( c_ForceEnemyModel.IsChecked() )
    EnableComponent(cb_ForcedEnemyModel);
  else
    DisableComponent(cb_ForcedEnemyModel);
  
  if ( c_ForceTeamModel.IsChecked() )
    EnableComponent(cb_ForcedTeamModel);
  else
    DisableComponent(cb_ForcedTeamModel);
  
  if ( Sender == c_ForceEnemyModel )
    SetEnemy(Sender);
  else if ( Sender == c_ForceTeamModel )
    SetFriendly(Sender);
}


//== EnemyModelChanged ========================================================
/**
An enemy model was changed.
*/
//=============================================================================

function EnemyModelChanged(GUIComponent Sender)
{
  SetEnemy(Sender);
}


//== TeamModelChanged =========================================================
/**
A team model was changed.
*/
//=============================================================================

function TeamModelChanged(GUIComponent Sender)
{
  SetFriendly(Sender);
}


//== SkinColorChanged =========================================================
/**
The skin color options were changed.
*/
//=============================================================================

function SkinColorChanged(GUIComponent Sender)
{
  local int i;
  
  // not only enemy-based overlay colors
  if ( bLocalSpectator || byte(cb_ForcedEnemyColor.GetComponentValue()) == 254 || byte(cb_ForcedTeamColor.GetComponentValue()) == 254 ) {
    for (i = 0; i < 3; ++i) {
      EnableComponent(p_TeamColors[i].MyButton);
    }
  }
  
  if ( byte(cb_ForcedEnemyColor.GetComponentValue()) == 254 )
    DisableComponent(p_TeamColors[3].MyButton);
  else
    EnableComponent(p_TeamColors[3].MyButton);
  
  if ( byte(cb_ForcedTeamColor.GetComponentValue()) == 254 )
    DisableComponent(p_TeamColors[4].MyButton);
  else
    EnableComponent(p_TeamColors[4].MyButton);
  
  // purely enemy-based overlay colors
  if ( !bLocalSpectator && byte(cb_ForcedEnemyColor.GetComponentValue()) != 254 && byte(cb_ForcedTeamColor.GetComponentValue()) != 254 ) {
    for (i = 0; i < 3; ++i) {
      DisableComponent(p_TeamColors[i].MyButton);
    }
  }
  
  if ( Sender == cb_ForcedEnemyColor )
    SetEnemy(Sender);
  else if ( Sender == cb_ForcedTeamColor )
    SetFriendly(Sender);
}


//== SkinOverlayChanged =======================================================
/**
The skin overlay options were changed.
*/
//=============================================================================

function SkinOverlayChanged(GUIComponent Sender)
{
  if ( c_UseSkinOverlay.IsChecked() ) {
    //EnableComponent(c_AlphaBlendOverlay);
    OverlayColorChanged(UTPGUI_RGBSliderPanel(t_OverlayColors.ActiveTab.MyPanel));
  }
  else {
    //DisableComponent(c_AlphaBlendOverlay);
    OverlayColor = default.OverlayColor;
  }
}


//== OverlayPanelActivated ====================================================
/**
An overlay color panel was selected.
*/
//=============================================================================

function OverlayPanelActivated(UTPGUI_RGBSliderPanel Sender)
{
  local byte SenderTeam;
  
  if ( Sender == p_TeamColors[3] ) {
    if ( !bEnemy )
      SetEnemy(Sender);
  }
  else if ( Sender == p_TeamColors[4] ) {
    if ( bEnemy )
      SetFriendly(Sender);
  }
  else if ( Sender != None && !bLocalSpectator ) {
    for (SenderTeam = 0; SenderTeam < p_TeamColors.Length; ++SenderTeam) {
      if ( Sender == p_TeamColors[SenderTeam] )
        break;
    }
    if ( SenderTeam == Min(FriendlyTeam, 2) && bEnemy && SenderTeam != Min(EnemyTeam, 2) )
      SetFriendly(Sender);
    else if ( SenderTeam == Min(EnemyTeam, 2) && !bEnemy && SenderTeam != Min(FriendlyTeam, 2) )
      SetEnemy(Sender);
  }
  OverlayColorChanged(Sender);
}


//== OverlayColorChanged ======================================================
/**
Updates the preview mesh actor.
*/
//=============================================================================

function OverlayColorChanged(UTPGUI_RGBSliderPanel Sender)
{
  if ( Sender != None && c_UseSkinOverlay.IsChecked() )
    OverlayColor = class'UTP_Util'.static.ToColor(Sender.GetColor());
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  WindowName  = "UT Plus Player Visibility Options"
  
  OnRendered    = DrawMesh
  MeshLoc       = (X=580,Y=65,Z=42)
  MeshRot       = (Pitch=-2000,Yaw=34000)
  OverlayColor  = (A=255)
  TeamNum       = 0
  
  EnemyTeam     = 1
  FriendlyTeam  = 0
  
  WinWidth  = 0.9
  WinHeight = 0.85
  WinLeft   = 0.05
  WinTop    = 0.05
  
  DefaultWidth  = 0.9
  DefaultHeight = 0.85
  DefaultLeft   = 0.05
  DefaultTop    = 0.05
  
  ForcedTeamColors[0] = "Red"
  ForcedTeamColors[1] = "Blue"
  ForcedTeamColors[2] = "Green"
  ForcedTeamColors[3] = "Gold"
  ForcedNeutralColor  = "Neutral"
  OriginalColor       = "Original"
  
  
  // bright skins
  Begin Object Class=moCheckbox Name=cBrightSkins
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.06
    TabOrder  = 11
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Epic Bright Skins"
    Hint            = "Enables the more colorful team skins for UT2003 models. (This is a built-in UT2004 option and may affect the game even without UT Plus.)"
  End Object
  c_BrightSkins = moCheckbox'cBrightSkins'
  
  // enemy models
  Begin Object Class=moCheckbox Name=cForceEnemyModel
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.18
    TabOrder  = 21
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Force Enemy Model"
    Hint            = "Force opponents to be displayed with the models specified below."
    OnChange        = ForceModelChanged
    OnClick         = SetEnemy
  End Object
  c_ForceEnemyModel = moCheckbox'cForceEnemyModel'
  
  Begin Object Class=moCombobox Name=cbForcedEnemyModel
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.24
    TabOrder  = 22
    bBoundToParent  = True 
    bScaleToParent  = True
    bReadOnly       = True
    bAlwaysNotify   = True
    Caption         = "Enemy Model"
    Hint            = "Select the prefered model for opponents. This setting will also affect your own model."
    OnChange        = EnemyModelChanged
    OnClick         = SetEnemy
  End Object
  cb_ForcedEnemyModel = moCombobox'cbForcedEnemyModel'
  
  Begin Object Class=moCombobox Name=cbForcedEnemyColor
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.30
    TabOrder  = 23
    bBoundToParent  = True 
    bScaleToParent  = True
    bReadOnly       = True
    bAlwaysNotify   = True
    Caption         = "Enemy Skin Color"
    Hint            = "Select the prefered skin color for opponents."
    OnChange        = SkinColorChanged
    OnClick         = SetEnemy
  End Object
  cb_ForcedEnemyColor = moCombobox'cbForcedEnemyColor'
  
  // team models
  Begin Object Class=moCheckbox Name=cForceTeamModel
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.42
    TabOrder  = 31
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Force Team Model"
    Hint            = "Force teammates to be displayed with the models specified below. This setting will not apply to your own model."
    OnChange        = ForceModelChanged
    OnClick         = SetFriendly
  End Object
  c_ForceTeamModel = moCheckbox'cForceTeamModel'
  
  Begin Object Class=moCombobox Name=cbForcedTeamModel
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.48
    TabOrder  = 32
    bBoundToParent  = True 
    bScaleToParent  = True
    bReadOnly       = True
    bAlwaysNotify   = True
    Caption         = "Team Model"
    Hint            = "Select the prefered model for teammates."
    OnChange        = TeamModelChanged
    OnClick         = SetFriendly
  End Object
  cb_ForcedTeamModel = moCombobox'cbForcedTeamModel'
  
  Begin Object Class=moCombobox Name=cbForcedTeamColor
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.54
    TabOrder  = 33
    bBoundToParent  = True 
    bScaleToParent  = True
    bReadOnly       = True
    bAlwaysNotify   = True
    Caption         = "Team Skin Color"
    Hint            = "Select the prefered skin color for teammates."
    OnChange        = SkinColorChanged
    OnClick         = SetFriendly
  End Object
  cb_ForcedTeamColor = moCombobox'cbForcedTeamColor'
  
  
  // overlays
  Begin Object Class=moCheckbox Name=cUseSkinOverlay
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.66
    TabOrder  = 41
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Use skin overlay"
    Hint            = "Enable the skin overlay color."
    OnChange        = SkinOverlayChanged
  End Object
  c_UseSkinOverlay = moCheckbox'cUseSkinOverlay'
  
  /*Begin Object Class=moCheckbox Name=cAlphaBlendOverlay
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.72
    TabOrder  = 42
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Alpha-blend overlay"
    Hint            = "When displaying a damage overlay, alpha-blend it with the skin overlay color instead of adding it."
  End Object
  c_AlphaBlendOverlay = moCheckbox'cAlphaBlendOverlay'*/
  
  Begin Object Class=GUITabControl Name=tOverlayColors
	  WinWidth  = 0.45
    WinHeight = 0.05
    WinLeft   = 0.52
    WinTop    = 0.60
    TabOrder  = 44
    TabHeight = 0.0375
    bAcceptsInput   = True
    bDockPanels     = True
    bScaleToParent  = True
    bFillSpace      = True
  End Object
  t_OverlayColors = GUITabControl'tOverlayColors'
  
  Begin Object Class=moCheckbox Name=cMapSpecific
    WinWidth  = 0.45
    WinLeft   = 0.03
    WinTop    = 0.84
    TabOrder  = 51
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Apply only to this map"
    Hint            = "When enabled, the settings on this page are only used when the current map is played."
  End Object
  c_MapSpecific = moCheckbox'cMapSpecific'
  
  TeamColors[0]   = (Caption="Red",Hint="Red team overlay color.")
  TeamColors[1]   = (Caption="Blue",Hint="Blue team overlay color.")
  TeamColors[2]   = (Caption="Green",Hint="Green team overlay color.")
  TeamColors[3]   = (Caption="Gold",Hint="Gold team overlay color.")
  TeamColors[4]   = (Caption="Neutral",Hint="Neutral overlay color.")
  TeamColorEnemy  = (Caption="Enemy",Hint="Opponents' overlay color.")
  TeamColorFriend = (Caption="Friend",Hint="Team mates' overlay color.")
}
