/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Configuration page for UTPlus player skin settings.
******************************************************************************/


class UTPGUI_ConfigPageServer extends UTPGUI_ConfigPageBase;


//=============================================================================
// Localization
//=============================================================================

var localized string NoAdminCaption, NoAdminMessage;


//=============================================================================
// Variables
//=============================================================================

// main options
var() automated moCheckbox c_EnableColoredDeathMessages, c_EnableBrightPlayerSkins, c_EnableHitSounds, c_EnablePingCompensation, c_EnableAdvancedMovement, c_AlwaysEnableClientGUI, c_EnableUTANv1Plugin, c_ShowIPsInWhois, c_LessLockdown, c_FixShieldBug, c_EnableProjectileFix, c_RestartMapWhenServerEmpty;

// bright skin options
var() automated moCheckbox c_AllowDeRes, c_AllowBrighterSkins;
var() automated moSlider s_SkinGlow;
var() automated moNumericEdit e_TeamBeaconMaxDist, e_TeamBeaconPlayerInfoMaxDist;

// spawn grenades and double damage
var() automated moCheckbox c_EnableSpawnGrenades, c_DisableDoubleDamage;
var() automated moNumericEdit e_InitialGrenades;

// advanced movement options
var() automated moCheckbox c_AlwaysAllowDodgeDoubleJump, c_EnableBoostDodge, c_FastDodging, c_PawnsIgnoreTerminalVelocity;
var() automated moNumericEdit e_MaxDodgesPerJump;

// automatic server demos
var() automated moCheckbox c_AutoServerDemoRecording;
var() automated moEditBox e_AutoServerDemoName;

// hit sound options
var() automated moCheckbox c_AllowVariableHitSounds, c_LineOfSightHitSounds;

var() int ReceivedConfigData1, ReceivedConfigData2, ReceivedConfigData3;


//== LoadSettings =============================================================
/**
Load the settings and create the preview mesh actor.
*/
//=============================================================================

function LoadSettings()
{
  local UTP_PlayerReplicationInfo LocalUTPPRI;
  
  DisableComponent(b_Ok);
  
  DisableComponent(c_EnableColoredDeathMessages);
  DisableComponent(c_EnableBrightPlayerSkins);
  DisableComponent(c_EnableHitSounds);
  DisableComponent(c_EnablePingCompensation);
  DisableComponent(c_EnableAdvancedMovement);
  DisableComponent(c_AlwaysEnableClientGUI);
  DisableComponent(c_ShowIPsInWhois);
  DisableComponent(c_AutoServerDemoRecording);
  DisableComponent(e_AutoServerDemoName);
  e_AutoServerDemoName.MyEditBox.AllowedCharSet = class'UTP_Util'.default.AllowedFileNameChars;
  e_AutoServerDemoName.MyEditBox.MaxWidth = 50;
  DisableComponent(c_LessLockdown);
  DisableComponent(c_FixShieldBug);
  
  DisableComponent(c_AllowDeRes);
  DisableComponent(c_AllowBrighterSkins);
  DisableComponent(s_SkinGlow);
  DisableComponent(e_TeamBeaconMaxDist);
  DisableComponent(e_TeamBeaconPlayerInfoMaxDist);
  
  DisableComponent(c_EnableSpawnGrenades);
  DisableComponent(e_InitialGrenades);
  DisableComponent(c_DisableDoubleDamage);
  
  DisableComponent(c_AlwaysAllowDodgeDoubleJump);
  DisableComponent(c_EnableBoostDodge);
  DisableComponent(c_FastDodging);
  DisableComponent(c_PawnsIgnoreTerminalVelocity);
  DisableComponent(e_MaxDodgesPerJump);
  
  DisableComponent(c_AllowVariableHitSounds);
  DisableComponent(c_LineOfSightHitSounds);
  
  DisableComponent(c_EnableUTANv1Plugin);
  DisableComponent(c_EnableProjectileFix);
  DisableComponent(c_RestartMapWhenServerEmpty);
  
  LocalUTPPRI = class'UTP_PlayerReplicationInfo'.static.FindFor(PlayerOwner());
  if ( LocalUTPPRI != None ) {
    LocalUTPPRI.OnClientReplyConfig = ReceivedConfiguration;
    LocalUTPPRI.ServerRequestConfig();
  }
}


//== ReceivedConfiguration ====================================================
/**
Can be used to get the results of the server config request.
*/
//=============================================================================

function ReceivedConfiguration(int ConfigData1, int ConfigData2, int ConfigData3, string AutoDemoName)
{
  local UTP_PlayerReplicationInfo LocalUTPPRI;
  local int MaxDodgesPerJump;
  
  LocalUTPPRI = class'UTP_PlayerReplicationInfo'.static.FindFor(PlayerOwner());
  if ( LocalUTPPRI != None ) {
    LocalUTPPRI.OnClientReplyConfig = None;
  }
  if ( ConfigData1 == 0 && ConfigData2 == 0 && AutoDemoName == "" ) {
    Controller.ReplaceMenu(string(class'UT2K4GenericMessageBox'), NoAdminCaption, NoAdminMessage, True);
    return;
  }
  EnableComponent(b_Ok);
  
  EnableComponent(c_EnableColoredDeathMessages);
  EnableComponent(c_EnableBrightPlayerSkins);
  EnableComponent(c_EnableHitSounds);
  EnableComponent(c_EnablePingCompensation);
  EnableComponent(c_EnableAdvancedMovement);
  EnableComponent(c_AlwaysEnableClientGUI);
  EnableComponent(c_ShowIPsInWhois);
  EnableComponent(c_AutoServerDemoRecording);
  EnableComponent(e_AutoServerDemoName);
  EnableComponent(c_LessLockdown);
  EnableComponent(c_FixShieldBug);
  
  EnableComponent(c_EnableSpawnGrenades);
  EnableComponent(e_InitialGrenades);
  EnableComponent(c_DisableDoubleDamage);
  
  EnableComponent(c_AllowDeRes);
  EnableComponent(c_AllowBrighterSkins);
  EnableComponent(s_SkinGlow);
  EnableComponent(e_TeamBeaconMaxDist);
  EnableComponent(e_TeamBeaconPlayerInfoMaxDist);
  
  EnableComponent(c_AlwaysAllowDodgeDoubleJump);
  EnableComponent(c_EnableBoostDodge);
  EnableComponent(c_FastDodging);
  EnableComponent(c_PawnsIgnoreTerminalVelocity);
  EnableComponent(e_MaxDodgesPerJump);
  
  EnableComponent(c_AllowVariableHitSounds);
  EnableComponent(c_LineOfSightHitSounds);
  
  EnableComponent(c_EnableUTANv1Plugin);
  EnableComponent(c_EnableProjectileFix);
  EnableComponent(c_RestartMapWhenServerEmpty);
  
  s_SkinGlow.MySlider.bShowCaption      = True;
  s_SkinGlow.MySlider.bShowValueTooltip = False;
  
  ReceivedConfigData1 = ConfigData1;
  ReceivedConfigData2 = ConfigData2;
  ReceivedConfigData3 = ConfigData3;
  
  
  // main options
  c_EnableColoredDeathMessages.Checked(bool(ConfigData1 & 0x00000001));
  c_EnableBrightPlayerSkins   .Checked(bool(ConfigData1 & 0x00000002));
  c_EnableHitSounds           .Checked(bool(ConfigData1 & 0x00000004));
  c_EnablePingCompensation    .Checked(bool(ConfigData1 & 0x00000008));
  c_EnableAdvancedMovement    .Checked(bool(ConfigData1 & 0x00000010));
  c_AlwaysEnableClientGUI     .Checked(bool(ConfigData1 & 0x00000020));
  c_ShowIPsInWhois            .Checked(bool(ConfigData1 & 0x00000040));
  c_AutoServerDemoRecording   .Checked(bool(ConfigData1 & 0x00000080));
  e_AutoServerDemoName        .SetText(AutoDemoName);
  c_EnableSpawnGrenades       .Checked(bool(ConfigData1 & 0x00000100));
  c_DisableDoubleDamage       .Checked(bool(ConfigData1 & 0x00000200));
  c_LessLockdown              .Checked(bool(ConfigData1 & 0x00000400));
  c_FixShieldBug              .Checked(bool(ConfigData1 & 0x00000800));
  
  c_EnableProjectileFix       .Checked(bool(ConfigData2 & 0x00001000));
  c_RestartMapWhenServerEmpty .Checked(bool(ConfigData2 & 0x00002000));
  
  // spawn grenades
  e_InitialGrenades.SetValue((ConfigData1 >>> 12) & 0xF);
  
  // hit sound options
  c_AllowVariableHitSounds.Checked(bool(ConfigData1 & 0x00010000));
  c_LineOfSightHitSounds  .Checked(bool(ConfigData1 & 0x00020000));
  
  // bright skin options
  c_AllowDeRes                  .Checked(bool(ConfigData1 & 0x0040000));
  c_AllowBrighterSkins          .Checked(bool(ConfigData1 & 0x0080000));
  s_SkinGlow                    .SetValue(byte(ConfigData1 >>> 24));
  e_TeamBeaconMaxDist           .SetValue(ConfigData3 & 0xFFFF);
  e_TeamBeaconPlayerInfoMaxDist .SetValue(ConfigData3 >>> 16);
  
  // advanced movement options
  MaxDodgesPerJump = ConfigData2 & 0x3FF;
  if ( MaxDodgesPerJump > 999 )
    e_MaxDodgesPerJump.SetValue(-1);
  else
    e_MaxDodgesPerJump.SetValue(MaxDodgesPerJump);
  c_AlwaysAllowDodgeDoubleJump  .Checked(bool(ConfigData1 & 0x00100000));
  c_EnableBoostDodge            .Checked(bool(ConfigData1 & 0x00200000));
  c_FastDodging                 .Checked(bool(ConfigData1 & 0x00400000));
  c_PawnsIgnoreTerminalVelocity .Checked(bool(ConfigData1 & 0x00800000));
  
  // UTAN extension
  c_EnableUTANv1Plugin   .Checked(bool(ConfigData2 & 0x0400));
}


//== SaveSettings =============================================================
/**
Send the configuration to the server.
*/
//=============================================================================

function SaveSettings()
{
  local UTP_PlayerReplicationInfo LocalUTPPRI;
  
  LocalUTPPRI = class'UTP_PlayerReplicationInfo'.static.FindFor(PlayerOwner());
  if ( LocalUTPPRI != None ) {
    
    // int #1
    ReceivedConfigData1 = 0;
    if (c_EnableColoredDeathMessages.IsChecked())   ReceivedConfigData1 += 0x00000001;
    if (c_EnableBrightPlayerSkins.IsChecked()   )   ReceivedConfigData1 += 0x00000002;
    if (c_EnableHitSounds.IsChecked()           )   ReceivedConfigData1 += 0x00000004;
    if (c_EnablePingCompensation.IsChecked()    )   ReceivedConfigData1 += 0x00000008;
    if (c_EnableAdvancedMovement.IsChecked()    )   ReceivedConfigData1 += 0x00000010;
    if (c_AlwaysEnableClientGUI.IsChecked()     )   ReceivedConfigData1 += 0x00000020;
    if (c_ShowIPsInWhois.IsChecked()            )   ReceivedConfigData1 += 0x00000040;
    if (c_AutoServerDemoRecording.IsChecked()   )   ReceivedConfigData1 += 0x00000080;
    if (c_EnableSpawnGrenades.IsChecked()       )   ReceivedConfigData1 += 0x00000100;
    if (c_DisableDoubleDamage.IsChecked()       )   ReceivedConfigData1 += 0x00000200;
    if (c_LessLockdown.IsChecked()              )   ReceivedConfigData1 += 0x00000400;
    if (c_FixShieldBug.IsChecked()              )   ReceivedConfigData1 += 0x00000800;
    ReceivedConfigData1 += ((e_InitialGrenades.GetValue()   << 12)       & 0x0000F000);
    if (c_AllowVariableHitSounds.IsChecked()    )   ReceivedConfigData1 += 0x00010000;
    if (c_LineOfSightHitSounds.IsChecked()      )   ReceivedConfigData1 += 0x00020000;
    if (c_AllowDeRes.IsChecked()                )   ReceivedConfigData1 += 0x00040000;
    if (c_AllowBrighterSkins.IsChecked()        )   ReceivedConfigData1 += 0x00080000;
    if (c_AlwaysAllowDodgeDoubleJump.IsChecked())   ReceivedConfigData1 += 0x00100000;
    if (c_EnableBoostDodge.IsChecked()          )   ReceivedConfigData1 += 0x00200000;
    if (c_FastDodging.IsChecked()               )   ReceivedConfigData1 += 0x00400000;
    if (c_PawnsIgnoreTerminalVelocity.IsChecked())  ReceivedConfigData1 += 0x00800000;
    ReceivedConfigData1 += ((s_SkinGlow.GetValue()          << 24)       & 0xFF000000);
    
    // int #2 (not entirely used, so make sure the unknown bits are kept the way they were received)
    ReceivedConfigData2 = (ReceivedConfigData2 & 0xFFFFFC00) | (e_MaxDodgesPerJump.GetValue() & 0x3FF);
    if (c_EnableUTANv1Plugin.IsChecked())       ReceivedConfigData2 = ReceivedConfigData2 | 0x00000400;
    else                                        ReceivedConfigData2 = ReceivedConfigData2 & 0xFFFFFBFF;
    if (c_EnableProjectileFix.IsChecked())      ReceivedConfigData2 = ReceivedConfigData2 | 0x00001000;
    else                                        ReceivedConfigData2 = ReceivedConfigData2 & 0xFFFFEFFF;
    if (c_RestartMapWhenServerEmpty.IsChecked())ReceivedConfigData2 = ReceivedConfigData2 | 0x00002000;
    else                                        ReceivedConfigData2 = ReceivedConfigData2 & 0xFFFFDFFF;
    
    // int #3
    ReceivedConfigData3 = e_TeamBeaconMaxDist.GetValue() + (e_TeamBeaconPlayerInfoMaxDist.GetValue() << 16);
    
    // send the data
    LocalUTPPRI.ServerSetConfig(ReceivedConfigData1, ReceivedConfigData2, ReceivedConfigData3, e_AutoServerDemoName.GetText());
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  WindowName = "UT Plus Server-side Settings"
  
  WinWidth  = 0.9
  WinHeight = 0.85
  WinLeft   = 0.05
  WinTop    = 0.05
  
  DefaultWidth  = 0.9
  DefaultHeight = 0.85
  DefaultLeft   = 0.05
  DefaultTop    = 0.05
  
  NoAdminCaption  = "No Admin"
  NoAdminMessage  = "Please log in as admin before modifying the server settings."
  
  // main options
  Begin Object Class=moCheckbox Name=cEnableColoredDeathMessages
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.05
    TabOrder        = 11
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable colored death messages"
    Hint            = "Enables team-colored death messages."
  End Object
  c_EnableColoredDeathMessages = moCheckbox'cEnableColoredDeathMessages'
  
  Begin Object Class=moCheckbox Name=cEnableProjectileFix
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.10
    TabOrder        = 12
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable projectile fix"
    Hint            = "Enable this option to activate the netcode fix for assault grenades, flak shells and rockets."
  End Object
  c_EnableProjectileFix = moCheckbox'cEnableProjectileFix'
  
  Begin Object Class=moCheckbox Name=cEnableBrightPlayerSkins
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.15
    TabOrder        = 13
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable bright player skins"
    Hint            = "Enables the 'UTPlus Bright Skins' mutator. (This makes your server non-standard.)"
  End Object
  c_EnableBrightPlayerSkins = moCheckbox'cEnableBrightPlayerSkins'
  
  Begin Object Class=moCheckbox Name=cEnableHitSounds
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.20
    TabOrder        = 14
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable hit sounds"
    Hint            = "Enables the 'UTPlus Hit Sounds' mutator. (This makes your server non-standard.)"
  End Object
  c_EnableHitSounds = moCheckbox'cEnableHitSounds'
  
  Begin Object Class=moCheckbox Name=cEnablePingCompensation
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.25
    TabOrder        = 15
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable ping compensation"
    Hint            = "Enables the 'UTPlus Ping Compensation' mutator. (This makes your server non-standard.)"
  End Object
  c_EnablePingCompensation = moCheckbox'cEnablePingCompensation'
  
  Begin Object Class=moCheckbox Name=cEnableAdvancedMovement
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.30
    TabOrder        = 16
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable advanced movement"
    Hint            = "Enables the 'UTPlus Advanced Movement' mutator. (This makes your server non-standard.)"
  End Object
  c_EnableAdvancedMovement = moCheckbox'cEnableAdvancedMovement'
  
  Begin Object Class=moCheckbox Name=cAlwaysEnableClientGUI
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.35
    TabOrder        = 17
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Always enable client-side GUI"
    Hint            = "Always enable the client-side UTPlus HUD and menu extensions."
  End Object
  c_AlwaysEnableClientGUI = moCheckbox'cAlwaysEnableClientGUI'
  
  Begin Object Class=moCheckbox Name=cShowIPsInWhois
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.40
    TabOrder        = 18
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Show IPs in WhoIs"
    Hint            = "This will show other players' IPs in the WhoIs and WhoWas reply. The IP's last part will only be visible for admins."
  End Object
  c_ShowIPsInWhois = moCheckbox'cShowIPsInWhois'
  
  Begin Object Class=moCheckbox Name=cAutoServerDemoRecording
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.45
    TabOrder        = 18
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Auto demo recording"
    Hint            = "Automatically record server-side demos. (only on dedicated servers)"
  End Object
  c_AutoServerDemoRecording = moCheckbox'cAutoServerDemoRecording'
  
  Begin Object Class=moEditbox Name=eAutoServerDemoName
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.50
    TabOrder        = 20
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Demo file name"
    Hint            = "Enter the file name for automatically recorded server-side demos."
  End Object
  e_AutoServerDemoName = moEditbox'eAutoServerDemoName'
  
  Begin Object Class=moCheckbox Name=cLessLockdown
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.55
    TabOrder        = 21
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Less Lockdown"
    Hint            = "Prevents players from going into a falling motion if they are hit with very little momentum. Enabling this option will make dodging easier when getting hit by a high-rate-of-fire weapon."
  End Object
  c_LessLockdown = moCheckbox'cLessLockdown'
  
  Begin Object Class=moCheckbox Name=cFixShieldBug
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.60
    TabOrder        = 22
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Fix Shield Bug"
    Hint            = "Adjusts the way damage is absorbed by shield powerups. The bug in UT2004 prevents the small shield's energy from being used to absorb damage as long as the big shield's energy is not drained."
  End Object
  c_FixShieldBug = moCheckbox'cFixShieldBug'
  
  // UTAN plugin
  Begin Object Class=moCheckbox Name=cEnableUTANv1Plugin
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.67
    TabOrder        = 30
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable plugin for UTAN v1"
    Hint            = "Enables the UTPlus UTAN v1 plugin, which enforces bans entered into the UTAN system."
  End Object
  c_EnableUTANv1Plugin = moCheckbox'cEnableUTANv1Plugin'
  
  // spawn grenades and double damage
  Begin Object Class=moCheckbox Name=cEnableSpawnGrenades
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.74
    TabOrder        = 31
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable Spawn Grenades"
    Hint            = "Enables the 'UTPlus Spawn Grenades' mutator. (This makes your server non-standard.)"
  End Object
  c_EnableSpawnGrenades = moCheckbox'cEnableSpawnGrenades'
  
  Begin Object Class=moNumericEdit Name=eInitialGrenades
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.79
    TabOrder        = 32
    bBoundToParent  = True 
    bScaleToParent  = True
    MinValue        = 0
    MaxValue        = 8
    Step            = 1
    Caption         = "Initial grenades"
    Hint            = "The initial amount of grenades for the Assault Rifle."
  End Object
  e_InitialGrenades = moNumericEdit'eInitialGrenades'
  
  Begin Object Class=moCheckbox Name=cDisableDoubleDamage
    WinWidth        = 0.45
    WinLeft         = 0.03
    WinTop          = 0.86
    TabOrder        = 33
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Disable Double Damage"
    Hint            = "Enables the 'UTPlus No Double Damage' mutator. (This makes your server non-standard.)"
  End Object
  c_DisableDoubleDamage = moCheckbox'cDisableDoubleDamage'
  
  
  // bright skin options
  Begin Object Class=moCheckbox Name=cAllowDeRes
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.05
    TabOrder        = 41
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Allow custom DeRes"
    Hint            = "Allow clients to enable instant and/or team-colored DeRes and disable rising DeRes."
  End Object
  c_AllowDeRes = moCheckbox'cAllowDeRes'
  
  Begin Object Class=moCheckbox Name=cAllowBrighterSkins
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.10
    TabOrder        = 42
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Allow custom-colored skins"
    Hint            = "Allow clients to enable a custom-colored player skin overlay."
  End Object
  c_AllowBrighterSkins = moCheckbox'cAllowBrighterSkins'
  
  Begin Object Class=moSlider Name=sSkinGlow
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.15
    TabOrder        = 43
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Skin Glow"
    Hint            = "Overall player brightness in darker map areas. The default value without UT Plus is 40."
    bIntSlider      = True
    MinValue        = 40
    MaxValue        = 128
  End Object
  s_SkinGlow = moSlider'sSkinGlow'
  
  Begin Object Class=moNumericEdit Name=eTeamBeaconMaxDist
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.20
    TabOrder        = 44
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Team Beacon Distance"
    Hint            = "Maximum distance the small triangle about teammates' heads are visible. (UT2004: 6000)"
    MinValue        = 2000
    MaxValue        = 20000
    Step            = 100
  End Object
  e_TeamBeaconMaxDist = moNumericEdit'eTeamBeaconMaxDist'
  
  Begin Object Class=moNumericEdit Name=eTeamBeaconPlayerInfoMaxDist
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.25
    TabOrder        = 45
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Team Player Info Dist."
    Hint            = "Maximum distance the player's name and health is displayed next to the team beacon. (UT2004: 1800)"
    MinValue        = 1000
    MaxValue        = 10000
    Step            = 100
  End Object
  e_TeamBeaconPlayerInfoMaxDist = moNumericEdit'eTeamBeaconPlayerInfoMaxDist'
  
  
  // advanced movement options
  Begin Object Class=moNumericEdit Name=eMaxDodgesPerJump
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.32
    TabOrder        = 51
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Maximum dodges per jump"
    Hint            = "The maximum number of additional dodges per jumping move. (-1 = unlimited)"
    MinValue        = -1
    MaxValue        = 999
  End Object
  e_MaxDodgesPerJump = moNumericEdit'eMaxDodgesPerJump'
  
  Begin Object Class=moCheckbox Name=cAlwaysAllowDodgeDoubleJump
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.37
    TabOrder        = 52
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Always allow dodge-doublejumps"
    Hint            = "Whether every dodging move can be followed up by a double-jump."
  End Object
  c_AlwaysAllowDodgeDoubleJump = moCheckbox'cAlwaysAllowDodgeDoubleJump'
  
  Begin Object Class=moCheckbox Name=cEnableBoostDodge
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.42
    TabOrder        = 53
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Enable boost dodge"
    Hint            = "Adds a jump's Z velocity to the wall dodge's upward velocity like in UT2003. Allowes for much greater distance if jump and walldodge are executed simultaneously but also cuts off a series of walldodges much earlier due to the increasing downward velocity."
  End Object
  c_EnableBoostDodge = moCheckbox'cEnableBoostDodge'
  
  Begin Object Class=moCheckbox Name=cFastDodging
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.47
    TabOrder        = 54
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Fast dodging"
    Hint            = "Allowes players to buffer in the first click of a dodge double click while they are still in the air. (Only works with at least one walldodge remaining for the jump.)"
  End Object
  c_FastDodging = moCheckbox'cFastDodging'
  
  Begin Object Class=moCheckbox Name=cPawnsIgnoreTerminalVelocity
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.52
    TabOrder        = 55
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Disable movement speed limitation"
    Hint            = "Disables the upper limit of 2500 UU/s for a player's movement speed while jumping."
  End Object
  c_PawnsIgnoreTerminalVelocity = moCheckbox'cPawnsIgnoreTerminalVelocity'
  
  
  // hit sound options
  Begin Object Class=moCheckbox Name=cAllowVariableHitSounds
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.59
    TabOrder        = 61
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Allow variable hit sounds"
    Hint            = "Allow variable hit sounds pitch based on damage."
  End Object
  c_AllowVariableHitSounds = moCheckbox'cAllowVariableHitSounds'
  
  Begin Object Class=moCheckbox Name=cLineOfSightHitSounds
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.64
    TabOrder        = 62
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Use line-of-sight hit sounds"
    Hint            = "Hit sounds will only play if the attacker has a line of sight to the victim."
  End Object
  c_LineOfSightHitSounds = moCheckbox'cLineOfSightHitSounds'
  
  /*Begin Object Class=moCheckbox Name=cCheckInputConsistency
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.71
    TabOrder        = 71
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Check input consistency"
    Hint            = "Disable this option if you experience problems in combination with SafeGame v1.01 beta 7+."
  End Object
  c_CheckInputConsistency = moCheckbox'cCheckInputConsistency'*/
  
  Begin Object Class=moCheckbox Name=cRestartMapWhenServerEmpty
    WinWidth        = 0.45
    WinLeft         = 0.52
    WinTop          = 0.76
    TabOrder        = 72
    bBoundToParent  = True 
    bScaleToParent  = True
    Caption         = "Restart map when server empty"
    Hint            = "Automatically restart the map when the last player leaves the game."
  End Object
  c_RestartMapWhenServerEmpty = moCheckbox'cRestartMapWhenServerEmpty'
}
