/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Menu for better demo playback management.
******************************************************************************/


class UTPGUI_DemoPlaybackMenu extends PopupPageBase;


//=============================================================================
// Import
//=============================================================================

#exec obj load file=..\Textures\UTPlusTextures.utx
#exec obj load file=..\Textures\2K4Menus.utx


//=============================================================================
// Localization
//=============================================================================

var localized string DemoEndedText;
var localized string JumpForwardText;
var localized string JumpBackwardText;
var localized string JumpCaption;
var localized string DumpSequenceCaption;
var localized string DumpSequenceText;


//=============================================================================
// Variables
//=============================================================================

var automated StateButton b_PlayPause, b_Stop, b_Restart, b_FastForward, b_DumpSequence;
var automated UTPGUI_SpeedSlider s_PlaybackSpeed;
var automated GUILabel l_DemoName, l_MapName, l_TimeIndex;

var bool bIsRestarting, bRestartPending, bIsPaused, bDemoEnded, bIsClosing;
var bool bSeeking, bRecording;
var float OriginalGameSpeed;
var string CurrentDemoName;

var int TargetTimeIndex;
var UTP_ReplicationInfo UTPlusRI;


//== InitComponent ============================================================
/**
Initializes the menu.
*/
//=============================================================================

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
  Super.InitComponent(MyController, MyOwner);
  i_FrameBG.WinTop = 0.0;
  i_FrameBG.WinHeight = 1.0;
  i_FrameBG.Image = Texture'Display1';
  i_FrameBG.ImageColor.A = 160;
  
  CurrentDemoName = GetLevelName();
  l_DemoName.Caption = CurrentDemoName;
  l_MapName.Caption = PlayerOwner().Level.Title;
}


//== Opened ===================================================================
/**
Menu was opened. Check for pause and find a reference to the UTPlus RI.
*/
//=============================================================================

function Opened(GUIComponent Sender)
{
  if ( UTPlusRI == None )
    UTPlusRI = class'UTP_ReplicationInfo'.static.FindRI(PlayerOwner());
  if ( UTPlusRI != None && UTPlusRI.MyInteraction != None ) {
    if ( UTPlusRI.MyInteraction.bFrameDumping )
      PlayerOwner().Level.Pauser = PlayerOwner().PlayerReplicationInfo;
    UTPlusRI.MyInteraction.StopDump();
  }
  
  bIsPaused = PlayerOwner().Level.Pauser != None;
  SetPlayPauseImage();
}

//== SetPlayPauseImage ========================================================
/**
Update the image of the Play/Pause button.
*/
//=============================================================================

function SetPlayPauseImage()
{
  if ( bIsPaused ) { // We are paused
    bIsPaused = true;
    b_PlayPause.Images[0] = Texture'PlayBlurry';
    b_PlayPause.Images[1] = Texture'UTPlusTextures.Menu.PlayWatched';
    b_PlayPause.Images[2] = Texture'UTPlusTextures.Menu.PlayWatched';
    b_PlayPause.Images[3] = Texture'PlayPressed';
    b_PlayPause.Images[4] = Texture'PlayBlurry';
  }
  else {
    bIsPaused = false;
    b_PlayPause.Images[0] = Texture'PauseBlurry';
    b_PlayPause.Images[1] = Texture'UTPlusTextures.Menu.PauseWatched';
    b_PlayPause.Images[2] = Texture'UTPlusTextures.Menu.PauseWatched';
    b_PlayPause.Images[3] = Texture'PausePressed';
    b_PlayPause.Images[4] = Texture'PauseBlurry';
  }
}


//== InternalCanClose =========================================================
/**
Prevents the menu from being closed during seeking or a demo restart.
*/
//=============================================================================

function bool InternalCanClose(optional bool bCancelled)
{
  if ( bIsClosing ) {
    TargetTimeIndex = 0;
    bIsRestarting = False;
    bRestartPending = False;
  }
  
  if ( TargetTimeIndex > 0 || bIsRestarting || bRestartPending )
    return false;
  
  return True;
}


//== NotifyLevelChange ========================================================
/**
Handles demo restart.
*/
//=============================================================================

event bool NotifyLevelChange()
{
  UTPlusRI = None;
  
	if ( !bIsClosing && bIsRestarting ) {
	  if ( !bRestartPending ) {
	    SetTimer(0.2, False);
	    PlayerOwner().ClearProgressMessages();
      PlayerOwner().SetProgressTime(6);
      PlayerOwner().SetProgressMessage(0, "Restarting demo...", class'Canvas'.Static.MakeColor(255,255,255));
	    bRestartPending = True;
	    log("NotifyLevelChange - restarting demo");
	  }
	  return false;
	}
	//else if ( !bIsClosing && IsOpen() ) {
    //  return false;
	//}
	log("NotifyLevelChange - playback ended");
	bPersistent = False;
	return Super.NotifyLevelChange();
}


//== Timer ====================================================================
/**
Handles demo restart and forward seeking.
*/
//=============================================================================

function Timer()
{
  local float CurrentTimeIndex;
  
  if ( bIsRestarting ) {
    bIsRestarting = False;
    bDemoEnded = False;
    bIsClosing = True;
    PlayerOwner().ClearProgressMessages();
    PlayerOwner().SetProgressTime(6);
    PlayerOwner().SetProgressMessage(0, "Restarting demo...", class'Canvas'.Static.MakeColor(255,255,255));
    PlayerOwner().Player.Console.DelayedConsoleCommand("demoplay" @ CurrentDemoName);
    Controller.CloseMenu();
    Controller.NotifyLevelChange();
  }
  else {
    CurrentTimeIndex = UTPlusRI.GetCurrentTimeIndex();
    if ( !bDemoEnded && TargetTimeIndex > 0 && CurrentDemoName ~= GetLevelName() ) {
      if ( CurrentTimeIndex + 0.1 < TargetTimeIndex ) {
        PlayerOwner().Level.TimeDilation = UTPlusRI.RealTimeDilation * Clamp(3 * (TargetTimeIndex - CurrentTimeIndex), 1, 4);
      }
      else {
        PlayerOwner().Level.Pauser = PlayerOwner().PlayerReplicationInfo;
        if ( !bIsPaused )
          PlayPauseClick(None);
        TargetTimeIndex = 0;
        KillTimer();
      }
    }
    else if ( bDemoEnded && TargetTimeIndex > 0 ) {
      PlayerOwner().SetProgressTime(6);
      PlayerOwner().SetProgressMessage(1, "Target time index after end of demo",
      class'Canvas'.Static.MakeColor(255,255,255));
      TargetTimeIndex = 0;
    }
  }
}

//== GetLevelName =============================================================
/**
Returns the demo's name.
*/
//=============================================================================

function string GetLevelName()
{
  local string s;
  local int i;
  
  s = PlayerOwner().Level.GetLocalURL();
  i = InStr(Caps(s), ".DEMO4");
  if ( i == -1 )
    i = InStr(s, "?");
  if ( i != -1 )
    s = Left(s, i);
  
  if ( Left(Repl(s, "\\", "/"), 9) ~= "../Demos/" )
    s = Mid(s, 9);
  
  return s;
}


//== StopClick ================================================================
/**
Stops the demo and goes back to the main menu.
*/
//=============================================================================

function bool StopClick(GUIComponent Sender)
{
  if ( Controller.bModAuthor )
    log(Self@"- StopClick"@bDemoEnded);
  
  if ( bIsPaused )
    PlayPauseClick(None);
  bIsClosing = True;
  KillTimer();
  PlayerOwner().Level.TimeDilation = OriginalGameSpeed;
  if ( !bDemoEnded )
    PlayerOwner().ConsoleCommand("StopDemo");
  else {
    bAllowedAsLast = False;
    Controller.CloseMenu();
  }
  return true;
}


//== FFClick ==================================================================
/**
Opens an input box for a destination time index.
*/
//=============================================================================

function bool FFClick(GUIComponent Sender)
{
  if ( Controller.bModAuthor )
    log(Self@"- FFClick"@bDemoEnded);
  
  if ( bDemoEnded || bIsRestarting )
    return false;
  
  TargetTimeIndex = 0;
  KillTimer();
  if ( !bIsPaused )
    PlayPauseClick(None);
  
  if ( UT2K4GetDataMenu(Controller.ActivePage) == None ) {
    if ( Controller.OpenMenu(string(class'UT2K4GetDataMenu'),
    Eval(bIsRestarting, JumpBackwardText, JumpForwardText), JumpCaption) ) {
      UT2K4GetDataMenu(Controller.ActivePage).SetDataString(l_TimeIndex.Caption);
      Controller.ActivePage.OnClose = TimeIndexInputClosed;
      if ( Controller.bModAuthor )
        log(Self@"- FFClick; opened input menu");
    }
    else if ( Controller.bModAuthor )
      log(Self@"- FFClick; OpenMenu failed");
  }
  return true;
}


//== RestartClick =============================================================
/**
Restart the demo.
*/
//=============================================================================

function bool RestartClick(GUIComponent Sender)
{
  if ( Controller.bModAuthor )
    log(Self@"- RestartClick"@bDemoEnded);
  bIsRestarting = True;
  PlayerOwner().Player.Console.DelayedConsoleCommand("disconnect");
  return true;
}


//== TimeIndexInputClosed =====================================================
/**
The destination time index input box was closed.
*/
//=============================================================================

function TimeIndexInputClosed(optional bool bCancelled)
{
  local string TimeIndexString;
  local int i;
  
  if ( Controller.bModAuthor )
    log(Self@"- TimeIndexInputClosed"@bCancelled);
  
  if ( bCancelled || UT2K4GetDataMenu(Controller.ActivePage) == None )
    return;
  
  TimeIndexString = UT2K4GetDataMenu(Controller.ActivePage).GetDataString();
  i = InStr(TimeIndexString, " ");
  if ( i != -1 )
    TimeIndexString = Left(TimeIndexString, i);
  i = InStr(TimeIndexString, ":");
  if ( i != -1 )
    TargetTimeIndex = 60 * float(Left(TimeIndexString, i)) + float(Mid(TimeIndexString, i + 1));
  else
    TargetTimeIndex = float(TimeIndexString);
  
  if ( TargetTimeIndex <= UTPlusRI.GetCurrentTimeIndex() ) {
    TargetTimeIndex = 0;
    return;
  }
  
  if ( bIsPaused )
    PlayPauseClick(None);
  
  if ( TargetTimeIndex > UTPlusRI.GetCurrentTimeIndex() ) {
    SetTimer(0.1, True);
    return;
  }
  
  bIsRestarting = True;
  PlayerOwner().Player.Console.DelayedConsoleCommand("disconnect");
  PlayerOwner().Level.TimeDilation = UTPlusRI.RealTimeDilation;
}


//== PlayPauseClick ===========================================================
/**
The Play/Pause button was clicked.
*/
//=============================================================================

function bool PlayPauseClick(GUIComponent Sender)
{
  if ( bDemoEnded )
    return RestartClick(Sender);
  if ( bIsRestarting )
    return false;
  
  if ( bIsPaused ) {
    bIsPaused = false;
    PlayerOwner().Level.Pauser = None;
  }
  else {
    bIsPaused = true;
    PlayerOwner().Level.Pauser = PlayerOwner().PlayerReplicationInfo;
  }
  SetPlayPauseImage();
  return true;
}


//== RecClick =================================================================
/**
The Record button was clicked.
*/
//=============================================================================

function bool RecClick(GUIComponent Sender)
{
  if ( Controller.bModAuthor )
    log(Self@"- RecClick"@bDemoEnded);
  
  if ( bDemoEnded || bIsRestarting )
    return false;
  
  TargetTimeIndex = 0;
  KillTimer();
  if ( !bIsPaused )
    PlayPauseClick(None);
  
  if ( UTPGUI_FrameRateInputPage(Controller.ActivePage) == None ) {
    if ( Controller.OpenMenu(string(class'UTPGUI_FrameRateInputPage')) ) {
      Controller.ActivePage.OnCanClose = DumpSequenceInputClosing;
      if ( Controller.bModAuthor )
        log(Self@"- RecClick; opened input menu");
    }
    else if ( Controller.bModAuthor )
      log(Self@"- RecClick; OpenMenu failed");
  }
  return true;
}


//== DumpSequenceInputClosing ==================================================
/**
The dump sequence name input box is closing.
*/
//=============================================================================

function bool DumpSequenceInputClosing(optional bool bCancelled)
{
  local int DumpFramerate;
  
  if ( Controller.bModAuthor )
    log(Self@"- DumpSequenceInputClosing"@bCancelled);
  
  if ( bCancelled || UTPGUI_FrameRateInputPage(Controller.ActivePage) == None || UTPlusRI == None )
    return true;
  
  DumpFramerate = int(UTPGUI_FrameRateInputPage(Controller.ActivePage).GetDataString());
  if ( DumpFramerate < 10 ) {
    log("Invalid dump framerate specified:"@DumpFramerate, 'UTPlus');
    return true;  // sanity check: at least 10 frames per second required
  }
  if ( UTPlusRI.MyInteraction != None && UTPlusRI.MyInteraction.StartDump(DumpFramerate) ) {
    if ( bIsPaused )
      PlayPauseClick(None); // unpause
    Controller.CloseAll(True, True);  // the first True here prevents this function infinitely recursing!
    return false; // the menu was already force-closed through CloseAll(), prevent GUIController.CloseMenu() from continuing
  }
  return true;
}


//== StartSliding =============================================================
/**
Start of GameSpeed slider dragging.
*/
//=============================================================================

function StartSliding()
{
  if ( !bIsPaused && !bIsRestarting )
    PlayerOwner().Level.Pauser = PlayerOwner().PlayerReplicationInfo;
}


//== EndSliding ===============================================================
/**
GameSpeed slider released.
*/
//=============================================================================

function EndSliding()
{
  if ( !bIsPaused && !bIsRestarting )
    PlayerOwner().Level.Pauser = None;
  
  PlayerOwner().Level.TimeDilation = UTPlusRI.RealTimeDilation * 2 ** s_PlaybackSpeed.Value;
}


//== UpdateTimeIndex ==========================================================
/**
Updates the displayed time index.
*/
//=============================================================================

function bool UpdateTimeIndex(Canvas C)
{
  local float CurrentTimeIndex;
  
  if ( !bDemoEnded && CurrentDemoName != "" && CurrentDemoName != GetLevelName() ) {
    bDemoEnded = True;
    //l_TimeIndex.Caption @= DemoEndedText;
  }
  if ( !bDemoEnded && UTPlusRI != None ) {
    CurrentTimeIndex = UTPlusRI.GetCurrentTimeIndex();
    if ( CurrentTimeIndex % 60 < 10 )
      l_TimeIndex.Caption = int(CurrentTimeIndex / 60) $ ":0" $ CurrentTimeIndex % 60;
    else
      l_TimeIndex.Caption = int(CurrentTimeIndex / 60) $ ":" $ CurrentTimeIndex % 60;
  }
  
  return false;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=StateButton Name=btnStop
    WinWidth=0.04
    WinHeight=0.053333
    WinLeft=0.03
    WinTop=0.01
    Images(0)=Material'2K4Menus.AVPlayer.StopBlurry'
    Images(1)=Material'UTPlusTextures.Menu.StopWatched'
    Images(2)=Material'UTPlusTextures.Menu.StopWatched'
    Images(3)=Material'2K4Menus.AVPlayer.StopPressed'
    Images(4)=Material'2K4Menus.AVPlayer.StopBlurry'
    TabOrder=0
    Hint="Return to main menu."
    OnClick=StopClick
  End Object
  b_Stop=StateButton'btnStop'

  Begin Object Class=StateButton Name=btnRestart
    WinWidth=0.04
    WinHeight=0.053333
    WinLeft=0.075
    WinTop=0.01
    Images(0)=Material'2K4Menus.AVPlayer.PreviousTrackBlurry'
    Images(1)=Material'UTPlusTextures.Menu.PreviousTrackWatched'
    Images(2)=Material'UTPlusTextures.Menu.PreviousTrackWatched'
    Images(3)=Material'2K4Menus.AVPlayer.PreviousTrackPressed'
    Images(4)=Material'2K4Menus.AVPlayer.PreviousTrackBlurry'
    TabOrder=1
    Hint="Restart demo."
    OnClick=RestartClick
  End Object
  b_Restart=StateButton'btnRestart'

  Begin Object Class=StateButton Name=btnFastForward
    WinWidth=0.04
    WinHeight=0.053333
    WinLeft=0.12
    WinTop=0.01
    Images(0)=Material'2K4Menus.AVPlayer.NextTrackBlurry'
    Images(1)=Material'UTPlusTextures.Menu.NextTrackWatched'
    Images(2)=Material'UTPlusTextures.Menu.NextTrackWatched'
    Images(3)=Material'2K4Menus.AVPlayer.NextTrackPressed'
    Images(4)=Material'2K4Menus.AVPlayer.NextTrackBlurry'
    TabOrder=2
    Hint="Jump forward to target time index."
    OnClick=FFClick
  End Object
  b_FastForward=StateButton'btnFastForward'

  Begin Object Class=StateButton Name=btnPlayPause
    WinWidth=0.040000
    WinHeight=0.053333
    WinLeft=0.165
    WinTop=0.01
    Images(0)=Material'2K4Menus.AVPlayer.PauseBlurry'
    Images(1)=Material'UTPlusTextures.Menu.PauseWatched'
    Images(2)=Material'UTPlusTextures.Menu.PauseWatched'
    Images(3)=Material'2K4Menus.AVPlayer.PausePressed'
    Images(4)=Material'2K4Menus.AVPlayer.PauseBlurry'
    TabOrder=3
    Hint="Toggle play/pause.."
    OnClick=PlayPauseClick
  End Object
  b_PlayPause=StateButton'btnPlayPause'

  Begin Object Class=StateButton Name=btnDumpSequence
    WinWidth=0.040000
    WinHeight=0.053333
    WinLeft=0.21
    WinTop=0.01
    Images(0)=Material'UTPlusTextures.Menu.RecBlurry'
    Images(1)=Material'UTPlusTextures.Menu.RecWatched'
    Images(2)=Material'UTPlusTextures.Menu.RecWatched'
    Images(3)=Material'UTPlusTextures.Menu.RecPressed'
    Images(4)=Material'UTPlusTextures.Menu.RecBlurry'
    TabOrder=4
    Hint="Dump screenshot sequence..."
    OnClick=RecClick
  End Object
  b_DumpSequence=StateButton'btnDumpSequence'

  Begin Object Class=UTPGUI_SpeedSlider Name=sPlaybackSpeed
    WinWidth=0.2
    WinHeight=0.04
    WinLeft=0.265
    WinTop=0.02
    TabOrder=5
    Hint="Set the relative playback speed."
    CaptionStyleName="TextLabel"
    OnStartSliding=StartSliding
    OnEndSliding=EndSliding
  End Object
  s_PlaybackSpeed=UTPGUI_SpeedSlider'sPlaybackSpeed'

  Begin Object Class=GUILabel Name=lbTimeIndex
    WinWidth=0.15
    WinHeight=0.07
    WinLeft=0.03
    WinTop=0.06
    Caption="0:00.00"
    Hint="Current time index"
    StyleName="TextLabel"
    TextColor=(R=255,G=255,B=0,A=255)
    TextAlign=TXTA_Right
    OnDraw=UpdateTimeIndex
  End Object
  l_TimeIndex=GUILabel'lbTimeIndex';

  Begin Object Class=GUILabel Name=lbMapName
    WinWidth=0.525
    WinHeight=0.05
    WinLeft=0.45
    WinTop=0.01
    Caption=""
    FontScale=FNS_Large
    StyleName="TextLabel"
    TextColor=(R=255,G=255,B=0,A=255)
    TextAlign=TXTA_Right
  End Object
  l_MapName=GUILabel'lbMapName';

  Begin Object Class=GUILabel Name=lbDemoName
    WinWidth=0.775
    WinHeight=0.07
    WinLeft=0.2
    WinTop=0.06
    Caption=""
    StyleName="TextLabel"
    TextColor=(R=255,G=255,B=0,A=255)
    TextAlign=TXTA_Right
  End Object
  l_DemoName=GUILabel'lbDemoName';
  
  WinTop=0.0;
  WinLeft=0.0
  WinWidth=1.0
  WinHeight=0.15
  
  OnCanClose=InternalCanClose
  
  bAllowedAsLast=true
  bPersistent=True
  
  DemoEndedText = "(demo ended)"
  
  JumpForwardText  = "Jump To Time Index"
  JumpBackwardText = "Restart And Jump To Time Index"
  JumpCaption      = "Enter destination time index: "
  
  DumpSequenceText    = "Enter dump sequence name: "
  DumpSequenceCaption = "Dump screenshot sequence"
}
