/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Prompts the user for a framerate value.
******************************************************************************/


class UTPGUI_FrameRateInputPage extends UT2K4GenericMessageBox;


//=============================================================================
// Variables
//=============================================================================

var automated GUIButton b_Cancel;
var automated moComboBox c_Data;


//== InitComponent ============================================================
/**
Initializes the controls.
*/
//=============================================================================

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
  local int i;
  
  c_Data.TabOrder   = 0;
  b_OK.TabOrder     = 1;
  b_Cancel.TabOrder = 2;
  
  Super.InitComponent(MyController, MyOwner);
  
  class'UTP_Util'.static.SortI(class'UTP_DemoPlaybackSettings'.default.DumpFrameRatePreset.Presets);
  c_Data.MyComboBox.Edit.AllowedCharSet = "0123456789";
  c_Data.MyComboBox.Edit.MaxWidth       = 2;
  for (i = 0; i < class'UTP_DemoPlaybackSettings'.default.DumpFrameRatePreset.Presets.Length; ++i) {
    c_Data.AddItem(string(class'UTP_DemoPlaybackSettings'.default.DumpFrameRatePreset.Presets[i]));
  }
  c_Data.SetText(string(class'UTP_DemoPlaybackSettings'.default.DumpFrameRatePreset.LastValue));
  
  RemoveComponent(l_Text2);
  
  b_Ok.bBoundToParent = true;
  b_Ok.bStandardized  = true;
  b_Ok.bScaleToParent = true;
  b_Cancel.bBoundToParent = true;
  b_Cancel.bStandardized  = true;
  b_Cancel.bScaleToParent = true;
}


//== HandleParameters =========================================================
/**
Sets the captions.
*/
//=============================================================================

function HandleParameters(string A, string B)
{
  if ( A != "" )
    l_Text.Caption = A;
  if ( B != "" )
	  c_Data.SetCaption(B);
}


//== GetDataString ============================================================
/**
Returns the framerate value as string.
*/
//=============================================================================

function string GetDataString()
{
	return c_Data.GetText();
}


//== SetDataString ============================================================
/**
Sets the framerate value.
*/
//=============================================================================

function SetDataString(string Str)
{
	Super.SetDataString(Str);
	c_Data.SetText(Str);
}


//== InternalOnClick ==========================================================
/**
Closes the menu.
*/
//=============================================================================

function bool InternalOnClick(GUIComponent Sender)
{
	Controller.CloseMenu(Sender == b_Cancel);
  
  if ( Sender == b_Ok )
    class'UTP_DemoPlaybackSettings'.default.DumpFrameRatePreset.LastValue = int(c_Data.GetText());
  class'UTP_DemoPlaybackSettings'.static.StaticSaveConfig();
  
	return true;
}


//== InternalOnKeyEvent =======================================================
/**
Cancels when Escape is pressed.
*/
//=============================================================================

function bool InternalOnKeyEvent(out byte Key, out byte State, float Delta)
{
	if ( State == 3 && Key == 0x0B )	// Escape
		return InternalOnClick(b_Cancel);
  
	return Super.InternalOnKeyEvent(Key, State, Delta);
}


//== InternalOnPreDraw ========================================================
/**
Aligns the buttons.
*/
//=============================================================================

function bool InternalOnPreDraw(Canvas C)
{
	local bool bResult;
	local float width, left, top;
  
	bResult = Super.InternalOnPreDraw(C);
  
	top = 1 - 64 / ActualHeight() - b_Cancel.WinHeight;
	width = 0.15 + 2 * b_Cancel.WinWidth;
	left = 0.5 - 0.5 * width;
  b_Cancel.SetPosition(left, top, b_Cancel.WinWidth, b_Cancel.WinHeight);
	
  left += 0.15 + b_Cancel.WinWidth;
	b_Ok.SetPosition(left, top, b_Cancel.WinWidth, b_Cancel.WinHeight);
  
	return bResult;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
	WinWidth  = 0.6
	WinHeight = 0.42
	WinLeft   = 0.2
	WinTop    = 0.29
  
	Begin Object class=GUILabel Name=lbDialogText
    WinWidth  = 0.5
    WinHeight = 0.09
    WinLeft   = 0.25
    WinTop    = 0.38
    Caption   = "DumpFrames with constant FPS"
    TextALign = TXTA_Center
    StyleName = "TextButton"
    FontScale = FNS_Large
	End Object
  l_Text = GUILabel'lbDialogText'
  
	Begin Object Class=GUIButton Name=btnCancel
    WinWidth  = 0.3
    WinHeight = 0.04
    WinLeft   = 0.2
    WinTop    = 0.55
    Caption   = "CANCEL"
    Hint      = "Close this menu, discarding changes."
    OnClick   = InternalOnClick
	End Object
	b_Cancel = GUIButton'btnCancel'
  
	Begin Object class=moComboBox Name=cData
    WinWidth      = 0.45
    WinHeight     = 0.05
    WinLeft       = 0.25
    WinTop        = 0.5
    Caption       = "Desired Framerate"
    CaptionWidth  = 0.5
    TabOrder      = 0
	End Object
	c_Data = moComboBox'cData'
}
