/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Ingame settings panel for UTPlus.
******************************************************************************/


class UTPGUI_MidGamePanel extends MidGamePanel;


//=============================================================================
// Imports
//=============================================================================

#exec obj load file="..\Textures\UTPlusTextures.utx"


//=============================================================================
// Structs
//=============================================================================

struct TProperty {
  var           string PropName;
  var localized string PropDesc;
  var           string Extra;
};

//=============================================================================
// Variables
//=============================================================================

var() automated GUIImage              i_UTPlusLogo;
var() automated AltSectionBackground  sb_Settings;
var() automated GUIMultiColumnListBox lb_Settings;
var   noexport  GUIMultiColumnList    li_Settings;

var() automated GUIButton b_ConfigPlayerSkins, b_ConfigHitSounds, b_ConfigMisc, b_ConfigServer;
var() automated GUIButton b_AdminLogin, b_RestartMap;

var editconst editconstarray array<TProperty> PropertyList;
var localized string UnusedValue;
var localized string lblAdminLogin, lblAdminLogout;

var class<UTPGUI_ConfigPageBase> ConfigPagePlayerSkins;
var class<UTPGUI_ConfigPageBase> ConfigPageHitSounds;
var class<UTPGUI_ConfigPageBase> ConfigPageMisc;
var class<UTPGUI_ConfigPageBase> ConfigPageServer;
var class<UTPGUI_ConfigPageBase> AdminLoginPage;


//== InitComponent ============================================================
/**
Initialize the list of active settings.
*/
//=============================================================================

function InitComponent(GUIController InController, GUIComponent InOwner)
{
  local int i;
  local UTP_ReplicationInfo UTPlusRI;
  local bool bRemoteAdmin;
  
  Super.InitComponent(InController, InOwner);
  
  li_Settings = lb_Settings.List;
  li_Settings.SortColumn = -1;
  li_Settings.OnDrawItem = DrawServerSetting;
  
  for (i = 0; i < PropertyList.Length; ++i) {
    li_Settings.AddedItem();
  }
  
  sb_Settings.ManageComponent(lb_Settings);
  lb_Settings.Header.bAcceptsInput = False;
  
  b_ConfigPlayerSkins.OnClick = OpenConfigPage;
  b_ConfigHitSounds.OnClick   = OpenConfigPage;
  b_ConfigMisc.OnClick        = OpenConfigPage;
  b_ConfigServer.OnClick      = OpenConfigPage;
  b_RestartMap.OnClick        = RestartMap;
  
  if ( PlayerOwner() != None ) {
    foreach PlayerOwner().AllActors(class'UTP_ReplicationInfo', UTPlusRI) {
      bRemoteAdmin = UTPlusRI.bRemoteAdmin && UTPlusRI.LocalUTPPRI != None;
      break;
    }
  }
  if ( !bRemoteAdmin )
    DisableComponent(b_ConfigServer);
  
  if ( PlayerOwner().Level.NetMode == NM_Standalone ) {
    RemoveComponent(b_AdminLogin);
  }
  else {
    b_AdminLogin.OnClick = AdminLogin; 
    UpdateAdminButtons();
    SetTimer(0.1, True);
  }
}


//== UpdateAdminButtons =======================================================
/**
Initialize the list of active settings.
*/
//=============================================================================

function UpdateAdminButtons()
{
  local UTP_ReplicationInfo UTPlusRI;
  local bool bAdmin;
  
  if ( PlayerOwner() != None ) {
    foreach PlayerOwner().AllActors(class'UTP_ReplicationInfo', UTPlusRI)
      break;
  }
  bAdmin = PlayerOwner().PlayerReplicationInfo.bAdmin || UTPlusRI != None && UTPlusRI.LocalUTPPRI != None && UTPlusRI.LocalUTPPRI.bAdmin;
  if ( bAdmin && b_AdminLogin.Caption != lblAdminLogout ) {
    b_AdminLogin.Caption = lblAdminLogout;
  }
  else if ( !bAdmin && b_AdminLogin.Caption != lblAdminLogin ) {
    b_AdminLogin.Caption = lblAdminLogin;
  }
  
  if ( bAdmin )
    EnableComponent(b_RestartMap);
  else
    DisableComponent(b_RestartMap);
}


//== OpenConfigPage ===========================================================
/**
Open the settings page associated with the button.
*/
//=============================================================================

function bool OpenConfigPage(GUIComponent Sender)
{
  switch (Sender) {
    case b_ConfigPlayerSkins: Controller.OpenMenu(string(ConfigPagePlayerSkins)); return true;
    case b_ConfigHitSounds:   Controller.OpenMenu(string(ConfigPageHitsounds));   return true;
    case b_ConfigMisc:        Controller.OpenMenu(string(ConfigPageMisc));        return true;
    case b_ConfigServer:      Controller.OpenMenu(string(ConfigPageServer));      return true;
  }
  return false;
}


//== RestartMap ===============================================================
/**
Restart the current map.
*/
//=============================================================================

function bool RestartMap(GUIComponent Sender)
{
  local PlayerController LocalPlayer;
  
  LocalPlayer = PlayerOwner();
  Controller.CloseMenu(False);
  LocalPlayer.Mutate("UTP Restart");
  return true;
}


//== AdminLogin ===============================================================
/**
Initialize the list of active settings.
*/
//=============================================================================

function bool AdminLogin(GUIComponent Sender)
{
  if ( b_AdminLogin.Caption == lblAdminLogin ) {
    Controller.OpenMenu(string(AdminLoginPage));
  }
  else {
    PlayerOwner().ConsoleCommand("ADMINLOGOUT");
  }
  return true;
}


//== Timer ====================================================================
/**
Find the UTPlusRI and update the admin login status.
*/
//=============================================================================

function Timer()
{
  if ( PlayerOwner().bDemoOwner )
    Controller.CloseMenu();
  else
    UpdateAdminButtons();
}


//== Free =====================================================================
/**
Free object references.
*/
//=============================================================================

function Free()
{
  KillTimer();
  Super.Free();
}

//== LevelChanged =============================================================
/**
Avoid potentially buggy implementation of MidGamePanel.LevelChanged().
*/
//=============================================================================

function LevelChanged()
{
  Super(UT2K4TabPanel).LevelChanged();
}


//== DrawServerSetting ========================================================
/**
Draws an entry of the Settings list box.
*/
//=============================================================================

function DrawServerSetting(Canvas Canvas, int i, float X, float Y, float W, float H, bool bSelected, bool bPending)
{
  local float CellLeft, CellWidth;
  local UTP_ReplicationInfo UTPlusRI;
  
  if ( PlayerOwner() != None ) {
    foreach PlayerOwner().AllActors(class'UTP_ReplicationInfo', UTPlusRI)
      break;
  }
  
  if ( UTPlusRI == None )
    return;
  
  if ( i < PropertyList.Length && PropertyList[i].PropName != "" ) {
    li_Settings.GetCellLeftWidth(0, CellLeft, CellWidth);
    if ( PropertyList[i].Extra == "" || InStr(PropertyList[i].Extra, ".") != -1 )
      li_Settings.Style.DrawText(Canvas, li_Settings.MenuState, CellLeft + 5, Y, CellWidth, H, TXTA_Left, PropertyList[i].PropDesc, li_Settings.FontScale);
    else
      li_Settings.Style.DrawText(Canvas, li_Settings.MenuState, CellLeft + 5, Y, CellWidth, H, TXTA_Left, " " @ PropertyList[i].PropDesc, li_Settings.FontScale);
    
    li_Settings.GetCellLeftWidth(1, CellLeft, CellWidth);
    if ( PropertyList[i].Extra == "" || InStr(PropertyList[i].Extra, ".") != -1 )
      li_Settings.Style.DrawText(Canvas, li_Settings.MenuState, CellLeft + 5, Y, CellWidth, H, TXTA_Left, UTPlusRI.GetPropertyText(PropertyList[i].PropName), li_Settings.FontScale);
    else if ( bool(UTPlusRI.GetPropertyText(PropertyList[i].Extra)) )
      li_Settings.Style.DrawText(Canvas, li_Settings.MenuState, CellLeft + 5, Y, CellWidth, H, TXTA_Left, " " @ UTPlusRI.GetPropertyText(PropertyList[i].PropName), li_Settings.FontScale);
    else
      li_Settings.Style.DrawText(Canvas, li_Settings.MenuState, CellLeft + 5, Y, CellWidth, H, TXTA_Left, " " @ UnusedValue, li_Settings.FontScale);
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=GUIImage Name=iUTPlusLogo
    WinWidth  = 0.4
    WinHeight = 0.45
    WinLeft   = 0.05
    WinTop    = 0.49
    ImageStyle = ISTY_Justified
    Image = Material'UTPlusLogo'
  End Object
  i_UTPlusLogo = GUIImage'iUTPlusLogo'
  
  Begin Object class=AltSectionBackground Name=sbSettings
    WinWidth        = 0.96
    WinHeight       = 0.45
    WinLeft         = 0.02
    WinTop          = 0.02
    LeftPadding     = 0.0
    RightPadding    = 0.0
    bBoundToParent  = true
    bScaleToParent  = true
    bFillClient     = true
  End Object
  sb_Settings = AltSectionBackground'sbSettings'
  
  Begin Object Class=GUIMultiColumnListBox Name=lbSettings
    WinWidth  = 1.0
    WinHeight = 1.0
    WinLeft   = 0.0
    WinTop    = 0.0
    bScaleToParent      = true
    bBoundToParent      = true
    bInitializeList     = false
    bVisibleWhenEmpty   = true
    StyleName           = "ServerBrowserGrid"
    SelectedStyleName   = "BrowserListSelection"
    ColumnHeadings(0)   = "Current Server Settings"
    ColumnHeadings(1)   = "Values"
    HeaderColumnPerc(0) = 0.75
    HeaderColumnPerc(1) = 0.25
    TabOrder = 1
  End Object
  lb_Settings = GUIMultiColumnListBox'lbSettings'
  
  Begin Object Class=GUIButton Name=btnConfigPlayerSkins
    WinWidth  = 0.45
    WinHeight = 0.06
    WinLeft   = 0.51
    WinTop    = 0.5
    StyleName = "SquareButton"
    Caption   = "Player Visibility Settings"
    TabOrder  = 2
    bAutoSize = True
    bAutoShrink = False
  End Object
  b_ConfigPlayerSkins = GUIButton'btnConfigPlayerSkins'
  
  Begin Object Class=GUIButton Name=btnConfigHitSounds
    WinWidth  = 0.45
    WinHeight = 0.06
    WinLeft   = 0.51
    WinTop    = 0.57
    StyleName = "SquareButton"
    Caption   = "Hit Sound Settings"
    TabOrder  = 3
    bAutoSize = True
    bAutoShrink = False
  End Object
  b_ConfigHitSounds = GUIButton'btnConfigHitSounds'
  
  Begin Object Class=GUIButton Name=btnConfigMisc
    WinWidth  = 0.45
    WinHeight = 0.06
    WinLeft   = 0.51
    WinTop    = 0.64
    StyleName = "SquareButton"
    Caption   = "Misc Settings"
    TabOrder  = 5
    bAutoSize = True
    bAutoShrink = False
  End Object
  b_ConfigMisc = GUIButton'btnConfigMisc'
  
  Begin Object Class=GUIButton Name=btnConfigServer
    WinWidth  = 0.45
    WinHeight = 0.06
    WinLeft   = 0.51
    WinTop    = 0.71
    StyleName = "SquareButton"
    Caption   = "Server Settings"
    TabOrder  = 6
    bAutoSize = True
    bAutoShrink = False
  End Object
  b_ConfigServer = GUIButton'btnConfigServer'
  
  Begin Object Class=GUIButton Name=btnRestartMap
    WinWidth  = 0.45
    WinHeight = 0.06
    WinLeft   = 0.51
    WinTop    = 0.78
    StyleName = "SquareButton"
    Caption   = "Restart Map"
    TabOrder  = 7
    bAutoSize = True
    bAutoShrink = False
  End Object
  b_RestartMap = GUIButton'btnRestartMap'
  
  Begin Object Class=GUIButton Name=btnAdminLogin
    WinWidth  = 0.45
    WinHeight = 0.06
    WinLeft   = 0.51
    WinTop    = 0.85
    StyleName = "SquareButton"
    TabOrder  = 8
    bAutoSize = True
    bAutoShrink = False
  End Object
  b_AdminLogin = GUIButton'btnAdminLogin'
  lblAdminLogin   = "Admin Login"
  lblAdminLogout  = "Admin Logout"
  
  ConfigPagePlayerSkins = UTPGUI_ConfigPagePlayerSkins
  ConfigPageHitSounds   = UTPGUI_ConfigPageHitSounds
  ConfigPageMisc        = UTPGUI_ConfigPageMisc
  ConfigPageServer      = UTPGUI_ConfigPageServer
  AdminLoginPage        = UTPGUI_AdminLoginPage
  
  PropertyList(0)   = (PropName="bAllowColors",PropDesc="Custom player colors enabled",Extra="")
  PropertyList(1)   = (PropName="bAllowDeRes",PropDesc="De-Res modifications allowed",Extra="bAllowColors")
  PropertyList(2)   = (PropName="bAllowBrighterSkins",PropDesc="Brighter player skins allowed",Extra="bAllowColors")
  PropertyList(3)   = (PropName="SkinGlow",PropDesc="Skin Glow",Extra="bAllowColors")
  PropertyList(4)   = (PropName="TeamBeaconMaxDist",PropDesc="Maximum Team Beacon Distance",Extra="bAllowColors")
  PropertyList(5)   = (PropName="TeamBeaconPlayerInfoMaxDist",PropDesc="Max. Team Beacon Player Info Dist.",Extra="bAllowColors")
  PropertyList(7)   = (PropName="bHitSoundsAvailable",PropDesc="Hit sounds enabled",Extra="")
  PropertyList(8)   = (PropName="bAllowVariableHitSounds",PropDesc="Variable hit sound pitch allowed",Extra="bHitSoundsAvailable")
  PropertyList(9)   = (PropName="bLineOfSightHitSounds",PropDesc="Line-of-sight hit sounds",Extra="bHitSoundsAvailable")
  
  PropertyList(11)  = (PropName="bAllowPingPrediction",PropDesc="Client ping prediction allowed",Extra="")
  PropertyList(12)  = (PropName="bServerPingCompensationAvailable",PropDesc="Server ping compensation available",Extra="")
  
  PropertyList(14)  = (PropName="bMultiDodgingEnabled",PropDesc="Advanced Movement enabled",Extra="")
  PropertyList(15)  = (PropName="bBoostDodgeEnabled",PropDesc="Boostdodge enabled",Extra="bMultiDodgingEnabled")
  PropertyList(16)  = (PropName="MaxDodgesPerJump",PropDesc="Max. additional dodges per jump",Extra="bMultiDodgingEnabled")
  PropertyList(17)  = (PropName="bAlwaysAllowDodgeDoubleJump",PropDesc="Dodge-jumps always allowed",Extra="bMultiDodgingEnabled")
  PropertyList(18)  = (PropName="bPawnsIgnoreTerminalVelocity",PropDesc="Movement speed limitation disabled",Extra="bMultiDodgingEnabled")
  PropertyList(19)  = (PropName="bFastDodging",PropDesc="Fast dodging enabled",Extra="bMultiDodgingEnabled")
  PropertyList(20)  = (PropName="bImprovedSlopeDodging",PropDesc="Improved slope dodging",Extra="bMultiDodgingEnabled")
  
  PropertyList(22)  = (PropName="bLessLockdown",PropDesc="Less lockdown",Extra="")
  PropertyList(23)  = (PropName="bFixShieldBug",PropDesc="Fix shield bug",Extra="")
  PropertyList(24)  = (PropName="bEnableProjectileFix",PropDesc="Projectile simulation fix",Extra="")
  
  UnusedValue = "---"
}
