/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Contains three sliders for setting RGB color values.
******************************************************************************/


class UTPGUI_RGBSliderPanel extends GUITabPanel;


//=============================================================================
// Variables
//=============================================================================

var() automated moSlider s_Red, s_Green, s_Blue;


//== SetColor =================================================================
/**
Sets the initial color.
*/
//=============================================================================

function SetColor(UTP_Pawn.TSkinColor Color)
{
  s_Red.SetComponentValue(Color.R, True);
  s_Green.SetComponentValue(Color.G, True);
  s_Blue.SetComponentValue(Color.B, True);
}


//== GetColor =================================================================
/**
Returns the selected color.
*/
//=============================================================================

function UTP_Pawn.TSkinColor GetColor()
{
  local UTP_Pawn.TSkinColor Color;
  
  Color.R = s_Red.GetValue();
  Color.G = s_Green.GetValue();
  Color.B = s_Blue.GetValue();
  
  return Color;
}


//== ColorChanged =================================================================
/**
The color has changed.
*/
//=============================================================================

function ColorChanged(GUIComponent Sender)
{
  OnColorChanged(Self);
}


//== delegate OnColorChanged ==================================================
/**
Called when the selected color has changed.
*/
//=============================================================================

delegate OnColorChanged(UTPGUI_RGBSliderPanel Sender);


//== InternalOnShow ===========================================================
/**
The panel was selected.
*/
//=============================================================================

function InternalOnShow()
{
  OnPanelActivated(Self);
}


//== delegate OnPanelActivated ================================================
/**
Called when this panel has been activated.
*/
//=============================================================================

delegate OnPanelActivated(UTPGUI_RGBSliderPanel Sender);


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=moSlider Name=sRed
    WinWidth  = 0.9
    WinLeft   = 0.05
    WinTop    = 0.1
    TabOrder  = 1
    bBoundToParent  = True 
    bScaleToParent  = True
    bIntSlider      = True
    MinValue        = 0
    MaxValue        = 255
    Caption         = "Red"
    Hint            = "Red component of the overlay color."
    OnChange        = ColorChanged
  End Object
  s_Red = moSlider'sRed'
  
  Begin Object Class=moSlider Name=sGreen
    WinWidth  = 0.9
    WinLeft   = 0.05
    WinTop    = 0.3
    TabOrder  = 2
    bBoundToParent  = True 
    bScaleToParent  = True
    bIntSlider      = True
    MinValue        = 0
    MaxValue        = 255
    Caption         = "Green"
    Hint            = "Green component of the overlay color."
    OnChange        = ColorChanged
  End Object
  s_Green = moSlider'sGreen'
  
  Begin Object Class=moSlider Name=sBlue
    WinWidth  = 0.9
    WinLeft   = 0.05
    WinTop    = 0.5
    TabOrder  = 3
    bBoundToParent  = True 
    bScaleToParent  = True
    bIntSlider      = True
    MinValue        = 0
    MaxValue        = 255
    Caption         = "Blue"
    Hint            = "Blue component of the overlay color."
    OnChange        = ColorChanged
  End Object
  s_Blue = moSlider'sBlue'
  
  OnShow = InternalOnShow
}
