/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Client-side player GUID database.
******************************************************************************/


class UTP_ClientPlayerIDs extends UTP_PlayerIDsBase
    transient;


//== ImportDatabase ===========================================================
/**
Imports a the client player ID database from the specified object.
*/
//=============================================================================

function ImportDatabase(UTP_ClientPlayerIDsExport Database)
{
  local int i, j;
  local byte hash;
  
  if ( Database == None ) {
    log("No database object available for loading the player ID database!", 'UTPlusError');
    return;
  }
  
  log("Reading" @ PlayerList.Length @ "entries (" $ Names.Length @ "names) from client player ID database...", 'UTPlus');
  
  // preallocate memory for the players and names lists
  PlayerList.Length = Database.PlayerList.Length;
  Names.Length      = Database.NameList.Length;
  
  // clear hash table
  for (i = 0; i < ArrayCount(HashTable); ++i) {
    HashTable[i].iIDs.Length = 0;
    HashTable[i].iIPs.Length = 0;
    HashTable[i].iNames.Length = 0;
  }
  
  // add names
  for (i = 0; i < Names.Length; ++i) {
    Names[i].PlayerName = Database.NameList[i];
    hash = HashString(Database.NameList[i]);
    HashTable[hash].iNames[HashTable[hash].iNames.Length] = i;
  }
  
  // add IDs/IPs and fill player list
  for (i = 0; i < PlayerList.Length; ++i) {
    PlayerList[i].iID = FindID(Database.PlayerList[i].ID, True);    // also updates the ID hashtable
    PlayerList[i].iIP = FindIP(Database.PlayerList[i].IP, True);    // also updates the IP hashtable
    PlayerList[i].iNames = Database.PlayerList[i].iNames;
    
    // fill in iPlayer references
    IDs[PlayerList[i].iID].iPlayers[IDs[PlayerList[i].iID].iPlayers.Length] = i;
    IPs[PlayerList[i].iIP].iPlayers[IPs[PlayerList[i].iIP].iPlayers.Length] = i;
    for (j = 0; j < PlayerList[i].iNames.Length; ++j) {
      Names[
        PlayerList[i].iNames[j]
      ].iPlayers[
        Names[
          PlayerList[i].iNames[j]
        ].iPlayers.Length
      ] = i;
    }
  }
}


//== ExportDatabase ===========================================================
/**
Exports the client player ID database to the specified object.
*/
//=============================================================================

function ExportDatabase(UTP_ClientPlayerIDsExport Database)
{
  local int i;
  
  if ( Database == None ) {
    log("No database object available for saving the player ID database!", 'UTPlusError');
    return;
  }
  
  // pre-allocate memory
  Database.PlayerList.Length  = PlayerList.Length;
  Database.NameList.Length    = Names.Length;
  
  log("Storing" @ PlayerList.Length @ "entries (" $ Names.Length @ "names) in client player ID database...", 'UTPlus');
  for (i = 0; i < PlayerList.Length; ++i) {
    Database.PlayerList[i].ID     = IDs[PlayerList[i].iID].ID;
    Database.PlayerList[i].IP     = IPs[PlayerList[i].iIP].IP;
    Database.PlayerList[i].iNames = PlayerList[i].iNames;
  }
  for (i = 0; i < Names.Length; ++i) {
    Database.NameList[i] = Names[i].PlayerName;
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
}
