/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Displays team-colored player names in death messages.
******************************************************************************/


class UTP_ColorDeathMessage extends xDeathMessage;


//=============================================================================
// Variables
//=============================================================================

var() color TeamColors[5];


//== static GetConsoleColor ===================================================
/**
Returns the main draw color for the death message.
*/
//=============================================================================

static function color GetConsoleColor(PlayerReplicationInfo RelatedPRI_1)
{
  return Default.DrawColor;
}


//== static GetString =========================================================
/**
Returns the message string.
*/
//=============================================================================

static function string GetString(optional int Switch, optional PlayerReplicationInfo RelatedPRI_1, optional PlayerReplicationInfo RelatedPRI_2, optional Object OptionalObject)
{
  if ( Class<DamageType>(OptionalObject) == None )
    return "";
  
  if ( Switch == 1 )
    return class'UTP_Util'.static.ColorCode(Default.DrawColor) $ class'GameInfo'.static.ParseKillMessage(GetPlayerName(RelatedPRI_1), GetPlayerName(RelatedPRI_2), class'UTP_Util'.static.StripCodes(class<DamageType>(OptionalObject).static.SuicideMessage(RelatedPRI_2)));
  
  return class'UTP_Util'.static.ColorCode(Default.DrawColor) $ class'GameInfo'.static.ParseKillMessage(GetPlayerName(RelatedPRI_1), GetPlayerName(RelatedPRI_2), class'UTP_Util'.static.StripCodes(class<DamageType>(OptionalObject).static.DeathMessage(RelatedPRI_1, RelatedPRI_2)));
}


//== static GetPlayerName =====================================================
/**
Returns the player name with the color codes for the player's team.
*/
//=============================================================================

static function string GetPlayerName(PlayerReplicationInfo PRI)
{
  local int TeamNum;
  
  if ( PRI == None )
    return Default.SomeoneString;
  else if ( PRI.Team != None )
    TeamNum = PRI.Team.TeamIndex;
  else
    TeamNum = 255;
  
  return class'UTP_Util'.static.ColorCode(default.TeamColors[Min(TeamNum, ArrayCount(default.TeamColors) - 1)]) $ class'UTP_Util'.static.StripCodes(PRI.PlayerName) $ class'UTP_Util'.static.ColorCode(Default.DrawColor);
}


//== static ClientReceive =====================================================
/**
Generate the additional multi-kill, killer and victim messages.
*/
//=============================================================================

static function ClientReceive(PlayerController P, optional int Switch, optional PlayerReplicationInfo RelatedPRI_1, optional PlayerReplicationInfo RelatedPRI_2, optional Object OptionalObject)
{
  local PlayerReplicationInfo SpectatorPRI;
  local UTP_PlayerReplicationInfo KillerUTPPRI;
  
  if ( P == None || P != P.Level.GetLocalPlayerController() && MessagingSpectator(P) == None )
    return;
  
  if ( !class'xDeathMessage'.default.bNoConsoleDeathMessages ) 
    Super(LocalMessage).ClientReceive(P, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject);
  
  if ( Switch == 1 )
    return; // suicide
  
  KillerUTPPRI = class'UTP_PlayerReplicationInfo'.static.GetFor(RelatedPRI_1);
  
  if ( KillerUTPPRI == None )
    return;
  
  SpectatorPRI = P.PlayerReplicationInfo;
  
  if ( UTP_DemoCameraViewer(P.ViewTarget) != None && UTP_DemoCameraViewer(P.ViewTarget).Camera != None && UTP_DemoCameraViewer(P.ViewTarget).Camera.TargetPRI != None )
    SpectatorPRI = UTP_DemoCameraViewer(P.ViewTarget).Camera.TargetPRI;
  else if ( Pawn(P.ViewTarget) != None && Pawn(P.ViewTarget).PlayerReplicationInfo != None )
    SpectatorPRI = Pawn(P.ViewTarget).PlayerReplicationInfo;
  else if ( Controller(P.ViewTarget) != None && Controller(P.ViewTarget).PlayerReplicationInfo != None )
    SpectatorPRI = Controller(P.ViewTarget).PlayerReplicationInfo;
  
  if ( RelatedPRI_1 == SpectatorPRI ) {
    // viewtarget or own pawn killed someone
    P.myHUD.LocalizedMessage(default.ChildMessage, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject);
  }
  if ( RelatedPRI_2 == SpectatorPRI ) {
    // viewtarget or own pawn was killed
    P.myHUD.LocalizedMessage(class'xVictimMessage', 0, RelatedPRI_1);
  }
  
  // check for multi kills
  if ( RelatedPRI_1 != RelatedPRI_2 && RelatedPRI_2 != None && (RelatedPRI_2.Team == None || RelatedPRI_1.Team != RelatedPRI_2.Team) ) {
    if ( P.Level.TimeSeconds - KillerUTPPRI.LastKillTime < 4 ) {
      KillerUTPPRI.MultiKillLevel++;
      if ( RelatedPRI_1 == SpectatorPRI ) {
        P.myHUD.LocalizedMessage(class'MultiKillMessage', KillerUTPPRI.MultiKillLevel);
        if ( class'PlayerController'.default.bNoMatureLanguage )
          P.PlayRewardAnnouncement(class'MultiKillMessage'.default.KillSoundName[Min(KillerUTPPRI.MultiKillLevel - 1, 5)], 1, true);
        else
          P.PlayRewardAnnouncement(class'MultiKillMessage'.default.KillSoundName[Min(KillerUTPPRI.MultiKillLevel - 1, 6)], 1, true);
      }
    } 
    else
      KillerUTPPRI.MultiKillLevel = 0;
    KillerUTPPRI.LastKillTime = P.Level.TimeSeconds;
  }
  else
    KillerUTPPRI.MultiKillLevel = 0;
}


//=============================================================================
// Default configuration
//=============================================================================

defaultproperties
{
  DrawColor     = (R=0,G=255,B=0,A=255)
  TeamColors[0] = (R=255,G=64,B=64,A=255)
  TeamColors[1] = (R=64,G=192,B=255,A=255)
  TeamColors[2] = (R=96,G=255,B=96,A=255)
  TeamColors[3] = (R=255,G=255,B=64,A=255)
  TeamColors[4] = (R=255,G=192,B=64,A=255)
}