/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Camera actor for demo playback.
******************************************************************************/


class UTP_DemoCamera extends Keypoint;


//=============================================================================
// Variables
//=============================================================================

var vector  FreeViewLocation, CurrentCameraLocation;
var rotator FreeViewRotation, CurrentCameraRotation;

var vector  LastViewTargetLocation, PrevLastViewTargetLocation;
var rotator LastViewTargetRotation, LastViewTargetViewRotation;
var vector  LastVelocity;
var vector  ChaseViewDestination;

var bool bTemporaryViewMode;
var bool bNoOtherViewMode;
var name RealCameraViewMode;

var UTP_ReplicationInfo   UTPlusRI;
var UTP_CamPathManager    PathManager;
var UTP_DemoCameraViewer  ViewerDummy;
var PlayerController      DemoSpectator;
var UTP_Interaction       MyInteraction;

var Actor                     ViewTarget;
var PlayerReplicationInfo     TargetPRI;
var UTP_PlayerReplicationInfo TargetUTPPRI;
var Pawn                      TargetPawn, LastTargetPawn;
var Projectile                TargetProjectile;
var array<Teleporter>         Teleporters;
var array<WarpZoneMarker>     WarpZoneMarkers;

var bool bViewNextProjectile, bWasProjectileCam;
var array<Projectile> IgnoredProj;
var float ProjectileImpactSwitchDelay;

var UTP_ValueFader  ValueFader;
var int             VelocityFaderIndex;
var int             DestinationFaderIndex;

// free flight input button states
var transient bool bForward, bBackward, bUp, bDown, bLeft, bRight;
var float TurnRate, PitchRate;

var transient float ScreenFadeValue;


//== PostBeginPlay ============================================================
/**
Get a reference to a value fader, the path manager and the UTP_ReplicationInfo and spawn the dummy viewtarget for the demo spectator.
*/
//=============================================================================

event PostBeginPlay()
{
  local Teleporter T;
  local WarpZoneMarker W;
  
  Super.PostBeginPlay();
  
  ValueFader = class'UTP_ValueFader'.static.GetValueFader(Self);
  UTPlusRI = class'UTP_ReplicationInfo'.static.FindRI(Self);
  PathManager = class'UTP_CamPathManager'.static.GetPathManager(Self);
  
  foreach AllActors(class'Teleporter', T)
    Teleporters[Teleporters.Length] = T;
  foreach AllActors(class'WarpZoneMarker', W)
    WarpZoneMarkers[WarpZoneMarkers.Length] = W;
}


//== FadeIn ===================================================================
/**
Start a fade-in-from-black effect.
*/
//=============================================================================

function FadeIn()
{
  ScreenFadeValue = 1.0;
}


//== PostRender ===============================================================
/**
Render a fade-in effect when desired.
*/
//=============================================================================

function PostRender(Canvas C)
{
  if ( ScreenFadeValue > 0 && class'UTP_DemoPlaybackSettings'.default.bFadeOnViewChange ) {
    C.Style = ERenderStyle.STY_Alpha;
    C.DrawColor.A = FClamp(255.0 * Square(ScreenFadeValue), 0, 255);
    C.ColorModulate.W = 1.0;
    C.SetPos(0,0);
    C.DrawRect(Texture'BlackTexture', C.ClipX, C.ClipY);
  }
}


//== GetPawnForPRI ============================================================
/**
Returns a Pawn for a specified PRI.
*/
//=============================================================================

function Pawn GetPawnForPRI(PlayerReplicationInfo PRI)
{
  local Pawn P, PickedPawn;
  
  if ( PRI == None )
    return None;
  
  if (ViewerDummy == None)
    ViewerDummy = Spawn(class'UTP_DemoCameraViewer', Self);
  foreach DynamicActors(class'Pawn', P) {
    if ( P == ViewerDummy || P.PlayerReplicationInfo != PRI )
      continue;
    if ( PickedPawn == None || P.IsA('RedeemerWarhead') && P.IsInState('Flying') || PickedPawn.Controller == None )
      PickedPawn = P;
  }
  
  return PickedPawn;
}


//== ToggleViewMode ===========================================================
/**
Global version only disables projectile chase view.
*/
//=============================================================================

function bool ToggleViewMode()
{
  if ( TargetProjectile != None ) {
    // switch back to player
    TargetProjectile = None;
    return true;
  }
  return false;
}


//== ViewNextPlayer ===========================================================
/**
Switches to the next player.
*/
//=============================================================================

function ViewNextPlayer()
{
  local PlayerReplicationInfo PRI, PickedPRI;
  local Pawn PickedPawn, PickedPRIPawn;
  local bool bFound;
  
  //log(Level.TimeSeconds@"ViewNextPlayer"@bViewNextProjectile@bWasProjectileCam);
  
  bViewNextProjectile = False;
  
  if ( TargetProjectile != None ) {
    // switch back to player
    TargetProjectile = None;
    return;
  }
  
  foreach DynamicActors(class'PlayerReplicationInfo', PRI) {
    if ( /*PRI == DemoSpectator.PlayerReplicationInfo ||*/ !PRI.bOnlySpectator && !PRI.bOutOfLives && !PRI.bWaitingPlayer ) {
      if ( bFound || PickedPRI == None ) {
        PickedPawn = GetPawnForPRI(PRI);
        
        if ( PickedPawn != None ) {
          PickedPRI = PRI;
          PickedPRIPawn = PickedPawn;
          if ( bFound )
            break;
        }
      }
      if ( PRI == TargetPRI )
        bFound = True;
    }
  }
  
  if ( PickedPawn == None ) {
    TargetPRI = DemoSpectator.PlayerReplicationInfo;
    SetViewTarget(DemoSpectator);
  }
  else {
    TargetPRI = PickedPRI;
    SetViewTarget(PickedPawn);
    SwitchToOriginalMode();
  }
}


//== ViewNextProjectile =======================================================
/**
Prepares to switch to the next projectile fired by the player.
*/
//=============================================================================

function ViewNextProjectile()
{
  local Projectile P;
  
  if ( TargetPawn != None ) {
    IgnoredProj.Length = 0;
    foreach DynamicActors(class'Projectile', P) {
      if ( P.Instigator == TargetPawn )
        IgnoredProj[IgnoredProj.Length] = P;
    }
    bViewNextProjectile = true;
  }
  else
    bViewNextProjectile = false;
  
  //log(Level.TimeSeconds@"ViewNextProjectile"@bViewNextProjectile@TargetPawn@IgnoredProj.Length);
}


//== FellOutOfWorld ===========================================================
/**
The camera shouldn't care about falling out of the world.
*/
//=============================================================================

event FellOutOfWorld(eKillZType KillType);


//== Destroyed ================================================================
/**
Clean up objects used only by the demo camera.
*/
//=============================================================================

event Destroyed()
{
  Super.Destroyed();
  
  if (ViewerDummy != None)
    ViewerDummy.Destroy();
  if (MyInteraction != None) {
    if (MyInteraction.Camera == Self)
      MyInteraction.Camera = None;
  }
  MyInteraction = None;
}


//== CenterView ===============================================================
/**
Resets the view rotation. (Should be implemented in view mode states.)
*/
//=============================================================================

function CenterView();


//== TemporarilySwitchToMode ==================================================
/**
Temporarily switch to a diffrent camera mode.
*/
//=============================================================================

function TemporarilySwitchToMode(name TempMode, optional bool bFixedViewMode)
{
  if ( !bTemporaryViewMode )
    RealCameraViewMode = GetStateName();
  if ( bFixedViewMode )
    bNoOtherViewMode = True;
  
  bTemporaryViewMode = True;
  if ( RealCameraViewMode != TempMode ) {
    log("Temporarily switching to"@TempMode, 'UTPlusDebug');
    GotoState(TempMode);
  }
}


//== SwitchToOriginalMode =====================================================
/**
Switch back to the original camera mode while in a temporary mode.
*/
//=============================================================================

function SwitchToOriginalMode()
{
  if ( !bTemporaryViewMode )
    return;
  
  bNoOtherViewMode = False;
  bTemporaryViewMode = False;
  if ( GetStateName() != RealCameraViewMode ) {
    log("Returning to"@RealCameraViewMode, 'UTPlusDebug');
    GotoState(RealCameraViewMode);
  }
}


//== SetViewTarget ============================================================
/**
Smoothly updates the current view source and destination locations.
*/
//=============================================================================

function SetViewTarget(Actor NewViewTarget)
{
  if ( NewViewTarget == None )
    NewViewTarget = DemoSpectator;
  if ( Controller(NewViewTarget) != None && Controller(NewViewTarget).Pawn != None )
    NewViewTarget = Controller(NewViewTarget).Pawn;
  
  ViewTarget = NewViewTarget;
  if ( Pawn(NewViewTarget) != None ) {
    TargetPawn = Pawn(NewViewTarget);
  }
  else if ( NewViewTarget.Instigator != None ) {
    TargetPawn = NewViewTarget.Instigator;
  }
  else if ( Pawn(NewViewTarget.Owner) != None ) {
    TargetPawn = Pawn(NewViewTarget.Owner);
  }
  
  if ( TargetUTPPRI == None || TargetUTPPRI.AssociatedPRI != TargetPRI )
    TargetUTPPRI = class'UTP_PlayerReplicationInfo'.static.GetFor(TargetPRI);
  
  if ( Projectile(ViewTarget) != None ) {
    TargetProjectile = Projectile(ViewTarget);
  }
  else {
    TargetProjectile = None;
  }
  
  if (Vehicle(ViewTarget) != None)
    ViewTarget.StaticClearConfig("TPCamDistance");
  
  //log(Level.TimeSeconds@"SetViewTarget"@ViewTarget@TargetPawn@TargetProjectile);
}


//== state CameraViewMode =====================================================
/**
Base state for all camera view modes.
*/
//=============================================================================

state CameraViewMode
{
  //== Tick ===================================================================
  /**
  Turns off projectile chase view or switches to next view mode.
  */
  //===========================================================================
  
  event Tick(float DeltaTime)
  {
    local Projectile P;
    local Pawn NewPawn;
    local int i;
    local bool bFoundProj;
    
    if ( ScreenFadeValue > 0 ) {
      ScreenFadeValue -= 3 * DeltaTime;
      if ( ScreenFadeValue < 0 )
        ScreenFadeValue = 0;
    }
    if ( DemoSpectator == None )
      DemoSpectator = Level.GetLocalPlayerController();
    if ( DemoSpectator == None )
      return; // no local controller? who recorded this thing?
    
    if ( DemoSpectator.IsA('DemoRecSpectator') && DemoSpectator.IsInState('Spectating') )
      DemoSpectator.GotoState('');
    
    /*if (!DemoSpectator.bDemoOwner && DemoRecSpectator(DemoSpectator) == None && (Pawn(DemoSpectator.ViewTarget) == None && Controller(DemoSpectator.ViewTarget) == None && DemoSpectator.ViewTarget != Self || !DemoSpectator.PlayerReplicationInfo.bOnlySpectator)) {
      //log(DemoSpectator.ViewTarget @ DemoSpectator.PlayerReplicationInfo.bOnlySpectator);
      Destroy();
      return;
    }*/
    
    if ( ViewTarget == DemoSpectator || !bWasProjectileCam && TargetPRI == None && TargetPawn == None && TargetProjectile == None ) {
      ViewNextPlayer();
    }
    else if ( TargetPRI != None && !bWasProjectileCam && (bTemporaryViewMode || !IsInState('FreeFlightView')) && (TargetPawn == None || TargetPawn.Health <= 0 || TargetPawn.PlayerReplicationInfo != TargetPRI) ) {
      NewPawn = GetPawnForPRI(TargetPRI);
      if ( NewPawn != None && (ViewTarget == None || ViewTarget == TargetPawn) )
        SetViewTarget(NewPawn);
    }
    
    if (ViewerDummy == None)
      ViewerDummy = Spawn(class'UTP_DemoCameraViewer', Self);
    if ( DemoSpectator.ViewTarget != ViewerDummy )
      DemoSpectator.SetViewTarget(ViewerDummy);
    if ( DemoSpectator.bBehindView )
      DemoSpectator.bBehindView = False;
    
    if ( bViewNextProjectile ) {
      if ( TargetPawn != None ) {
        foreach RadiusActors(class'Projectile', P, 1500, TargetPawn.Location) {
          if ( P.Instigator != TargetPawn )
            continue;
          for (i = 0; i < IgnoredProj.Length; ++i) {
            if ( P == IgnoredProj[i] )
              break;
          }
          if ( i != IgnoredProj.Length )
            continue;
          if ( P.Velocity dot (P.Location - TargetPawn.Location) > 0 ) {
            bFoundProj = True;
            break;
          }
        }
        if ( bFoundProj ) {
          SetViewTarget(P);
          bViewNextProjectile = false;
          IgnoredProj.Length = 0;
          TemporarilySwitchToMode('ChaseProjectile', True);
        }
      }
      else
        bViewNextProjectile = false;
    }
    
    if ( PrevLastViewTargetLocation != LastViewTargetLocation )
      PrevLastViewTargetLocation = LastViewTargetLocation;
    if ( ViewTarget != None ) {
      if (Vehicle(ViewTarget) != None)
        LastViewTargetLocation = Vehicle(ViewTarget).GetCameraLocationStart();
      else
        LastViewTargetLocation = ViewTarget.Location;
      LastViewTargetRotation = ViewTarget.Rotation;
      if ( Pawn(ViewTarget) != None && (TargetUTPPRI == None || PlayerController(Pawn(ViewTarget).Controller) != None) )
        LastViewTargetViewRotation = Pawn(ViewTarget).GetViewRotation();
      else if ( TargetUTPPRI != None )
        LastViewTargetViewRotation = TargetUTPPRI.GetViewRotation();
      else if ( VSize(ViewTarget.Velocity) > 0 )
        LastViewTargetViewRotation = rotator(ViewTarget.Velocity);
      /*else
        LastViewTargetViewRotation = LastViewTargetRotation;*/
    }
    else if ( !bWasProjectileCam && TargetUTPPRI != None ) {
      LastViewTargetLocation = TargetUTPPRI.LastKnownLocation;
      LastViewTargetRotation = TargetUTPPRI.GetViewRotation();
      LastViewTargetViewRotation = TargetUTPPRI.GetViewRotation();
    }
  }
  
  
  //== ToggleViewMode =========================================================
  /**
  Turns off projectile chase view or switches to next view mode.
  */
  //===========================================================================
  
  function bool ToggleViewMode()
  {
    if ( Global.ToggleViewMode() ) {
      return true;
    }
    
    if ( !bTemporaryViewMode || !bNoOtherViewMode )
      SwitchToNextMode();
    return true;
  }
  
  
  //== SwitchToNextMode =======================================================
  /**
  Switch to next view mode. (Should be implemented in derived states.)
  */
  //===========================================================================
  
  function SwitchToNextMode();
  
  
  //== BeginState =============================================================
  /**
  The next call to Tick() is the first one since switching to this view mode.
  */
  //===========================================================================
  
  function BeginState()
  {
    //log(Level.TimeSeconds@"BeginState"@GetStateName());
    Super.BeginState();
  }
}


//== state FirstPersonView ====================================================
/**
The camera does not function as viewtarget by itself, but only follows the current view and switches to 3rd-person when the tracked player dies.
*/
//=============================================================================

auto state FirstPersonView extends CameraViewMode
{
  function Tick(float DeltaTime)
  {
    Super.Tick(DeltaTime);
    
    if (bDeleteMe)
      return;
    
    if ( TargetPawn != None ) {
      if ( TargetPawn.Health > 0 && !TargetPawn.IsInState('Dying') ) {
        FreeViewRotation = LastViewTargetViewRotation;
        if ( Rotation != FreeViewRotation )
          SetRotation(FreeViewRotation);
        
        FreeViewLocation = TargetPawn.Location + TargetPawn.EyePosition();
        if ( Location != FreeViewLocation )
          SetLocation(FreeViewLocation);
      }
      else {
        TemporarilySwitchToMode('FreeBehindView');
      }
    }
    else {
      TemporarilySwitchToMode('FreeFlightView');
    }
  }
  
  
  //== SwitchToNextMode =======================================================
  /**
  Switch to fixed behind view mode.
  */
  //===========================================================================
  
  function SwitchToNextMode()
  {
    GotoState('FixedBehindView');
  }
  
  
  //== BeginState =============================================================
  /**
  Fade in when switching the view to 1st person.
  */
  //===========================================================================
  
  function BeginState()
  {
    FadeIn();
    Super.BeginState();
  }
  
  
  //== EndState ===============================================================
  /**
  The camera uses the player's velocity when it leaves this camera mode.
  */
  //===========================================================================
  
  event EndState()
  {
    Super.EndState();
    Acceleration = vect(0,0,0);
    if ( ViewTarget != None )
      Velocity = ViewTarget.Velocity;
  }
}


//== state FixedBehindView ====================================================
/**
The camera stays behind the player.
*/
//=============================================================================

state FixedBehindView extends CameraViewMode
{
  function Tick(float DeltaTime)
  {
    local vector NewCamLocation, HL, HN;
    local vector CamViewDestination;
    local vector X, Y, Z, CamOffset, LookAt, WorldOffset;
    
    Super.Tick(DeltaTime);
    
    if (bDeleteMe)
      return;
    
    if (Vehicle(TargetPawn) != None) {
      LookAt = Vehicle(TargetPawn).TPCamLookat;
      CamOffset = LookAt - vect(1,0,0) * Vehicle(TargetPawn).default.TPCamDistance;
      WorldOffset = Vehicle(TargetPawn).TPCamWorldOffset;
    }
    else {
      LookAt = class'UTP_DemoPlaybackSettings'.default.Player3rdPersonViewOffset;
      CamOffset = class'UTP_DemoPlaybackSettings'.default.Player3rdPersonOffset;
    }
    if ( TargetPawn != None && TargetPawn.Health > 0 && !TargetPawn.IsInState('Dying') ) {
      GetAxes(LastViewTargetViewRotation, X, Y, Z);
      CamViewDestination = LastViewTargetLocation + WorldOffset + X * LookAt.X + Y * LookAt.Y + Z * LookAt.Z;
      if ( Trace(HL, HN, CamViewDestination, LastViewTargetLocation, False) != None )
        CamViewDestination = HL;
      
      NewCamLocation = LastViewTargetLocation + WorldOffset + X * CamOffset.X + Y * CamOffset.Y + Z * CamOffset.Z;
      if ( Trace(HL, HN, NewCamLocation, LastViewTargetLocation, False) != None )
        NewCamLocation = HL;
      
      FreeViewLocation = NewCamLocation;
      if ( Location != FreeViewLocation )
        SetLocation(FreeViewLocation);
      
      FreeViewRotation = rotator(CamViewDestination - NewCamLocation);
      if ( Rotation != FreeViewRotation )
        SetRotation(FreeViewRotation);
    }
    else
      TemporarilySwitchToMode('FreeBehindView');
  }
  
  
  //== SwitchToNextMode =======================================================
  /**
  Switch to next view mode.
  */
  //===========================================================================
  
  function SwitchToNextMode()
  {
    GotoState('FreeBehindView');
  }
  
  
  //== BeginState =============================================================
  /**
  Fade in when switching the view to fixed 3rd person.
  */
  //===========================================================================
  
  function BeginState()
  {
    FadeIn();
    Super.BeginState();
  }
  
  
  //== EndState ===============================================================
  /**
  The camera uses the player's velocity when it leaves this camera mode.
  */
  //===========================================================================
  
  event EndState()
  {
    Super.EndState();
    Acceleration = vect(0,0,0);
    if ( ViewTarget != None )
      Velocity = ViewTarget.Velocity;
  }
}


//== state FreeBehindView ====================================================
/**
The camera rotation is controlled by the spectator, but the cam always faces the player.
*/
//=============================================================================

state FreeBehindView extends CameraViewMode
{
  function Tick(float DeltaTime)
  {
    local vector NewCamLocation, HL, HN;
    local vector CamViewDestination;
    local vector X, Y, Z, CamOffset, LookAt, WorldOffset;
    
    Super.Tick(DeltaTime);
    
    if (bDeleteMe)
      return;
    
    if ( bTemporaryViewMode && TargetPawn != None && TargetPawn.Health > 0 ) {
      SwitchToOriginalMode();
      return;
    }
    
    if (Vehicle(TargetPawn) != None) {
      LookAt = Vehicle(TargetPawn).TPCamLookat;
      CamOffset = LookAt - vect(1,0,0) * Vehicle(TargetPawn).default.TPCamDistance;
      WorldOffset = Vehicle(TargetPawn).TPCamWorldOffset;
    }
    else {
      LookAt = class'UTP_DemoPlaybackSettings'.default.Player3rdPersonViewOffset;
      CamOffset = class'UTP_DemoPlaybackSettings'.default.Player3rdPersonOffset;
    }
    GetAxes(FreeViewRotation, X, Y, Z);
    CamViewDestination = LastViewTargetLocation + WorldOffset + X * LookAt.X + Y * LookAt.Y + Z * LookAt.Z;
    if ( Trace(HL, HN, CamViewDestination, LastViewTargetLocation, False) != None )
      CamViewDestination = HL;
    
    NewCamLocation = LastViewTargetLocation + WorldOffset + X * CamOffset.X + Y * CamOffset.Y + Z * CamOffset.Z;
    if ( Trace(HL, HN, NewCamLocation, LastViewTargetLocation, False) != None )
      NewCamLocation = HL;
    
    FreeViewLocation = NewCamLocation;
    if ( Location != FreeViewLocation )
      SetLocation(FreeViewLocation);
    
    if ( Rotation != rotator(CamViewDestination - NewCamLocation) )
      SetRotation(rotator(CamViewDestination - NewCamLocation));
  }
  
  
  //== CenterView =============================================================
  /**
  Resets the view rotation.
  */
  //===========================================================================
  
  function CenterView()
  {
    FreeViewRotation.Pitch = 0;
  }
  
  
  //== SwitchToNextMode =======================================================
  /**
  Switch to next view mode.
  */
  //===========================================================================
  
  function SwitchToNextMode()
  {
    GotoState('ChaseCamView');
  }
  
  
  //== BeginState =============================================================
  /**
  Set the turn and pitch rates.
  */
  //===========================================================================
  
  event BeginState()
  {
    Super.BeginState();
    TurnRate  = class'UTP_DemoPlaybackSettings'.default.Free3rdPersonViewTurnRate;
    PitchRate = class'UTP_DemoPlaybackSettings'.default.Free3rdPersonViewPitchRate;
  }
  
  
  //== EndState ===============================================================
  /**
  The camera uses the player's velocity when it leaves this camera mode.
  */
  //===========================================================================
  
  event EndState()
  {
    Super.EndState();
    Acceleration = vect(0,0,0);
    if ( ViewTarget != None )
      Velocity = ViewTarget.Velocity;
  }
}


//== state ChaseCamView =======================================================
/**
The camera loosely follows the player.
*/
//=============================================================================

state ChaseCamView extends CameraViewMode
{
  function Tick(float DeltaTime)
  {
    local vector NewCamLocation, PrevLocation, HL, HN;
    local vector CamViewDestination, CamAcceleration;
    local vector X, Y, Z, Dest, CamOffset, LookAt, WorldOffset;
    local float MaxSpeed;
    local int i, j;
    
    if ( VSize(Velocity) > 0 ) {
      PrevLocation = Location;
      for (i = 0; i < Teleporters.Length; ++i) {
        if ( Teleporters[i].NearSpot(Location) && Teleporters[i].URL != "" && Velocity dot (Teleporters[i].Location - Location) > 0 ) {
          for (j = 0; j < Teleporters.Length; ++j) {
            if ( i != j && Teleporters[i].URL ~= string(Teleporters[j].Tag) && SetLocation(Teleporters[j].Location) ) {
              SetRotation(Teleporters[j].Rotation);
              FadeIn();
              break;
            }
          }
          if ( Location != PrevLocation ) {
            Velocity = vect(0,0,0);
            break;
          }
        }
      }
    }
    
    Super.Tick(DeltaTime);
    
    if (bDeleteMe)
      return;
    
    if ( GetStateName() != 'ChaseCamView' )
      return; // there's no way to specify a super state name for function calls, I think
    
    if ( bTemporaryViewMode && TargetPawn != None && TargetPawn.Health > 0 ) {
      SwitchToOriginalMode();
      return;
    }
    
    if (Vehicle(TargetPawn) != None) {
      LookAt = Vehicle(TargetPawn).TPCamLookat;
      CamOffset = LookAt - vect(1,0,0) * Vehicle(TargetPawn).default.TPCamDistance;
      WorldOffset = Vehicle(TargetPawn).TPCamWorldOffset;
    }
    else {
      LookAt = class'UTP_DemoPlaybackSettings'.default.PlayerChaseViewOffset;
      CamOffset = class'UTP_DemoPlaybackSettings'.default.PlayerChaseOffset;
    }
    if ( TargetPawn != None && (TargetPawn.Health > 0 || FastTrace(LastViewTargetLocation, Location)) ) {
      GetAxes(LastViewTargetViewRotation, X, Y, Z);
      CamViewDestination = LastViewTargetLocation + WorldOffset + X * LookAt.X + Y * LookAt.Y + Z * LookAt.Z;
      if ( Trace(HL, HN, CamViewDestination, LastViewTargetLocation, False) != None )
        CamViewDestination = HL;
      
      if ( ScreenFadeValue == 0.0 && VSize(LastViewTargetLocation - PrevLastViewTargetLocation) > 160 )
        ValueFader.AddRotatorFadeValue(DestinationFaderIndex, rotator(CamViewDestination - Location));
      else
        ValueFader.AddRotatorFadeValue(DestinationFaderIndex, rotator(CamViewDestination - Location), True);
      
      NewCamLocation = LastViewTargetLocation + WorldOffset + X * CamOffset.X + Y * CamOffset.Y + Z * CamOffset.Z;
      
      if ( Trace(HL, HN, NewCamLocation, LastViewTargetLocation, False, vect(10,10,10)) != None ) {
        NewCamLocation = HL - 20 * Normal(NewCamLocation - LastViewTargetLocation);
      }
      
      if ( VSize(Location - NewCamLocation) < 3 * FMax(FMax(TargetPawn.GroundSpeed, TargetPawn.AirSpeed) * 5, class'UTP_DemoPlaybackSettings'.default.MaxChaseCamSpeed) && (PathManager.FindPath(Location, NewCamLocation, Dest) || FastTrace(Dest, Location)) ) {
        CamAcceleration = class'UTP_DemoPlaybackSettings'.default.ChaseCamAcceleration * Normal(Dest - Location);
      }
      else {
        FadeIn();
        CamAcceleration = vect(0,0,0);
        Velocity = vect(0,0,0);
        SetLocation(NewCamLocation);
      }
    }
    else {
      CamAcceleration = vect(0,0,0);
      NewCamLocation = Location + Velocity;
    }
    
    Velocity = Normal(Velocity) * VSize(Velocity) * (1 - DeltaTime);
    Velocity += DeltaTime * CamAcceleration;
    if (TargetPawn != None)
      MaxSpeed = FMin(FMax(FMax(TargetPawn.GroundSpeed, TargetPawn.AirSpeed) * 5, class'UTP_DemoPlaybackSettings'.default.MaxChaseCamSpeed), class'UTP_DemoPlaybackSettings'.default.PlayerChaseSpeedFalloff * VSize(NewCamLocation - Location));
    else
      MaxSpeed = FMin(class'UTP_DemoPlaybackSettings'.default.MaxChaseCamSpeed, class'UTP_DemoPlaybackSettings'.default.PlayerChaseSpeedFalloff * VSize(NewCamLocation - Location));
    if ( VSize(Velocity) > MaxSpeed )
      Velocity = Normal(Velocity) * MaxSpeed;
    
    SetRotation(ValueFader.GetFadedRotatorValue(DestinationFaderIndex));
  }
  
  
  //== CenterView =============================================================
  /**
  Resets the view rotation.
  */
  //===========================================================================
  
  function CenterView()
  {
    SetLocation(ViewTarget.Location);
  }
  
  
  //== SwitchToNextMode =======================================================
  /**
  Switch to next view mode.
  */
  //===========================================================================
  
  function SwitchToNextMode()
  {
    GotoState('FreeFlightView');
  }
  
  
  //== BeginState =============================================================
  /**
  Initialize the value fader.
  */
  //===========================================================================
  
  event BeginState()
  {
    Super.BeginState();
    
    DestinationFaderIndex = ValueFader.AddRotatorFader(CurrentCameraRotation, 1.0, False, True, True);
    
    bCollideWorld = True;
    bCanTeleport = True;
    SetCollision(True, False, False);
    SetPhysics(PHYS_Projectile);
    ProjectileImpactSwitchDelay = class'UTP_DemoPlaybackSettings'.default.ProjectileImpactSwitchDelay;
  }
  
  
  //== EndState ===============================================================
  /**
  Remove the value faders.
  */
  //===========================================================================
  
  event EndState()
  {
    Super.EndState();
    Acceleration = vect(0,0,0);
    FreeViewRotation = Rotation;
    ValueFader.RemoveRotatorFader(DestinationFaderIndex);
    
    bCollideWorld = False;
    bCanTeleport = False;
    SetCollision(False, False, False);
    SetPhysics(PHYS_None);
  }
}


//== state FreeFlightView =====================================================
/**
The camera movement is controlled entirely by the spectator.
*/
//=============================================================================

state FreeFlightView extends CameraViewMode
{
  function Tick(float DeltaTime)
  {
    local vector X, Y, Z;
    local vector NewCamVelocity;
    local rotator NewCamRotation;
    
    Super.Tick(DeltaTime);
    
    if ( bDeleteMe || GetStateName() != 'FreeFlightView')
      return;
    
    if ( bTemporaryViewMode && TargetPawn != None && TargetPawn.Health > 0 ) {
      SwitchToOriginalMode();
      return;
    }
    else if ( TargetPawn != None )
      TargetPawn = None;
    
    ValueFader.AddRotatorFadeValue(DestinationFaderIndex, FreeViewRotation, True);
    NewCamRotation = ValueFader.GetFadedRotatorValue(DestinationFaderIndex);
    
    GetAxes(NewCamRotation, X, Y, Z);
    
    if ( class'UTP_DemoPlaybackSettings'.default.bVerticalUpDown )
      Z = vect(0,0,1);
    
    if ( bForward && !bBackward )
      NewCamVelocity += X * class'UTP_DemoPlaybackSettings'.default.FreeFlightAxisSpeeds.X;
    else if ( bBackward && !bForward )
      NewCamVelocity -= X * class'UTP_DemoPlaybackSettings'.default.FreeFlightAxisSpeeds.X;
    
    if ( bRight && !bLeft )
      NewCamVelocity += Y * class'UTP_DemoPlaybackSettings'.default.FreeFlightAxisSpeeds.Y;
    else if ( bLeft && !bRight )
      NewCamVelocity -= Y * class'UTP_DemoPlaybackSettings'.default.FreeFlightAxisSpeeds.Y;
    
    if ( bUp && !bDown )
      NewCamVelocity += Z * class'UTP_DemoPlaybackSettings'.default.FreeFlightAxisSpeeds.Z;
    else if ( bDown && !bUp )
      NewCamVelocity -= Z * class'UTP_DemoPlaybackSettings'.default.FreeFlightAxisSpeeds.Z;
    
    if ( LastVelocity != NewCamVelocity ) {
      ValueFader.AddVectorFadeValue(VelocityFaderIndex, NewCamVelocity);
      LastVelocity = NewCamVelocity;
    }
    Velocity = ValueFader.GetFadedVectorValue(VelocityFaderIndex);
    
    /*if ( VSize(Velocity) > 0 )
      MoveSmooth(Velocity * DeltaTime);*/
    
    if ( Rotation != NewCamRotation )
      SetRotation(NewCamRotation);
  }
  
  
  //== CenterView =============================================================
  /**
  Resets the view rotation.
  */
  //===========================================================================
  
  function CenterView()
  {
    ValueFader.AddRotatorFadeValue(DestinationFaderIndex, FreeViewRotation);
    FreeViewRotation.Pitch = 0;
  }
  
  
  //== BeginState =============================================================
  /**
  Initialize the value fader.
  */
  //===========================================================================
  
  event BeginState()
  {
    Super.BeginState();
    
    TurnRate  = class'UTP_DemoPlaybackSettings'.default.FreeFlightTurnRate;
    PitchRate = class'UTP_DemoPlaybackSettings'.default.FreeFlightPitchRate;
    
    FreeViewRotation = CurrentCameraRotation;
    FreeViewLocation = CurrentCameraLocation;
    LastVelocity = Velocity;
    if ( !bTemporaryViewMode )
      TargetPawn = None;
    
    DestinationFaderIndex = ValueFader.AddRotatorFader(FreeViewRotation, 0.5, False, True);
    VelocityFaderIndex = ValueFader.AddVectorFader(Velocity, class'UTP_DemoPlaybackSettings'.default.VelocityFadeTime);
    
    bCollideWorld = True;
    bCanTeleport = True;
    SetCollision(True, False, False);
    SetPhysics(PHYS_Projectile);
  }
  
  
  //== EndState ===============================================================
  /**
  Remove the value faders.
  */
  //===========================================================================
  
  event EndState()
  {
    Super.EndState();
    TargetPawn = GetPawnForPRI(TargetPRI);
    Acceleration = vect(0,0,0);
    Velocity = ValueFader.GetFadedVectorValue(VelocityFaderIndex);
    FreeViewRotation = Rotation;
    ValueFader.RemoveVectorFader(VelocityFaderIndex);
    ValueFader.RemoveRotatorFader(DestinationFaderIndex);
    bCollideWorld = False;
    bCanTeleport = False;
    SetCollision(False, False, False);
    SetPhysics(PHYS_None);
  }
  
  
  //== SwitchToNextMode =======================================================
  /**
  Switch to next view mode.
  */
  //===========================================================================
  
  function SwitchToNextMode()
  {
    GotoState('FirstPersonView');
  }
}


//== state ChaseProjectile ====================================================
/**
The camera follows a projectile.
*/
//=============================================================================

state ChaseProjectile extends ChaseCamView
{
  function Tick(float DeltaTime)
  {
    local vector NewCamLocation, HL, HN;
    local vector CamViewDestination, CamAcceleration;
    local vector X, Y, Z, Dest;
    local float MaxSpeed;
    
    Super.Tick(DeltaTime);
    
    if (bDeleteMe)
      return;
    
    if ( TargetProjectile == None && ProjectileImpactSwitchDelay < 0) {
      if ( TargetPawn == None || TargetPawn.Health <= 0 )
        TargetPawn = GetPawnForPRI(TargetPRI);
      if ( TargetPawn != None && TargetPawn.Health > 0 ) {
        SwitchToOriginalMode();
        return;
      }
    }
    else if ( TargetProjectile == None ) {
      ProjectileImpactSwitchDelay -= DeltaTime;
      //log(Level.TimeSeconds@"ProjectileImpactSwitchDelay"@ProjectileImpactSwitchDelay@bWasProjectileCam);
    }
    
    GetAxes(LastViewTargetViewRotation, X, Y, Z);
    CamViewDestination = LastViewTargetLocation + X * class'UTP_DemoPlaybackSettings'.default.ProjectileChaseViewOffset.X + Y * class'UTP_DemoPlaybackSettings'.default.ProjectileChaseViewOffset.Y + Z * class'UTP_DemoPlaybackSettings'.default.ProjectileChaseViewOffset.Z;
    if ( Trace(HL, HN, CamViewDestination, LastViewTargetLocation, False) != None )
      CamViewDestination = HL;
    
    if ( VSize(LastViewTargetLocation - PrevLastViewTargetLocation) > 100 )
      ValueFader.AddRotatorFadeValue(DestinationFaderIndex, rotator(CamViewDestination - Location));
    else
      ValueFader.AddRotatorFadeValue(DestinationFaderIndex, rotator(CamViewDestination - Location), True);
    
    NewCamLocation = LastViewTargetLocation + X * class'UTP_DemoPlaybackSettings'.default.ProjectileChaseOffset.X + Y * class'UTP_DemoPlaybackSettings'.default.ProjectileChaseOffset.Y + Z * class'UTP_DemoPlaybackSettings'.default.ProjectileChaseOffset.Z;
    if ( Trace(HL, HN, NewCamLocation, LastViewTargetLocation, False) != None )
      NewCamLocation = HL;
    
    if ( PathManager.FindPath(Location, NewCamLocation, Dest) || FastTrace(Dest, Location) ) {
      CamAcceleration = class'UTP_DemoPlaybackSettings'.default.ProjectileCamAcceleration * Normal(Dest - Location);
    }
    else {
      FadeIn();
      CamAcceleration = vect(0,0,0);
      Velocity = vect(0,0,0);
      SetLocation(NewCamLocation);
    }
    
    Velocity = Normal(Velocity) * VSize(Velocity) * (1 - DeltaTime);
    Velocity += DeltaTime * CamAcceleration;
    MaxSpeed = FMin(class'UTP_DemoPlaybackSettings'.default.MaxProjectileCamSpeed, class'UTP_DemoPlaybackSettings'.default.ProjectileChaseSpeedFalloff * VSize(NewCamLocation - Location));
    if ( VSize(Velocity) > MaxSpeed )
      Velocity = Normal(Velocity) * MaxSpeed;
    
    /*if ( VSize(Velocity) > 0 )
      MoveSmooth(Velocity * DeltaTime);*/
    SetRotation(ValueFader.GetFadedRotatorValue(DestinationFaderIndex));
  }
  
  
  //== BeginState =============================================================
  /**
  Initialize the value fader.
  */
  //===========================================================================
  
  event BeginState()
  {
    Super.BeginState();
    ProjectileImpactSwitchDelay = class'UTP_DemoPlaybackSettings'.default.ProjectileImpactSwitchDelay;
    bWasProjectileCam = True;
  }
  
  
  //== EndState ===============================================================
  /**
  Remove the value faders.
  */
  //===========================================================================
  
  event EndState()
  {
    Super.EndState();
    bWasProjectileCam = False;
  }
}


//== HitWall ==================================================================
/**
Slide along walls.
*/
//=============================================================================

event HitWall(vector HitNormal, Actor Wall)
{
  Velocity -= (Velocity dot HitNormal) * HitNormal;
}


//== DisplayDebug =============================================================
/**
Let the DemoCamera draw the debug information.
*/
//=============================================================================

simulated function DisplayDebug(Canvas Canvas, out float YL, out float YPos)
{
  local string T;
  local float XL;
  
  Canvas.Style = ERenderStyle.STY_Normal;
  Canvas.StrLen("TEST", XL, YL);
  YPos = YPos + YL;
  Canvas.SetPos(4,YPos);
  Canvas.SetDrawColor(255,0,0);
  T = GetDebugName();
  if ( bDeleteMe )
    T = T$" DELETED (bDeleteMe == true)";
  
  Canvas.DrawText(T, false);
  YPos += YL;
  Canvas.SetPos(4,YPos);
  Canvas.SetDrawColor(255,255,255);
  
  Canvas.DrawText("Location: "$Location$" Rotation "$Rotation, false);
  YPos += YL;
  Canvas.SetPos(4,YPos);
  Canvas.DrawText("Velocity: "$Velocity$" Speed "$VSize(Velocity)$" Speed2D "$VSize(Velocity-Velocity.Z*vect(0,0,1)), false);
  YPos += YL;
  Canvas.SetPos(4,YPos);
  Canvas.DrawText("Acceleration: "$Acceleration, false);
  YPos += YL;
  Canvas.SetPos(4,YPos);
  
  Canvas.DrawColor.B = 255;
  Canvas.DrawText("STATE: "$GetStateName(), false);
  YPos += YL;
  Canvas.SetPos(4,YPos);
  
  Canvas.DrawText("TargetPawn "$GetItemName(string(TargetPawn))$" TargetUTPPRI "$TargetUTPPRI);
  YPos += YL;
  Canvas.SetPos(4,YPos);
  
  if (TargetUTPPRI != None)
    Canvas.DrawText("TargetRotation "$TargetUTPPRI.GetViewRotation());
  YPos += YL;
  Canvas.SetPos(4,YPos);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bStatic           = False
  //bAlwaysTick       = True
  bBounce           = True
  bCollideActors    = False
  bCollideWorld     = False
  bIgnoreOutOfWorld = True
  CollisionHeight   = 15.0
  CollisionRadius   = 15.0
  InitialState      = FirstPersonView
}
