/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Dummy pawn used as viewtarget for the demo playback spectator.
******************************************************************************/


class UTP_DemoCameraViewer extends Pawn;


//=============================================================================
// Variables
//=============================================================================

var UTP_DemoCamera Camera;


//== PostBeginPlay ============================================================
/**
Store the reference to the demo camera before it gets overridden by the controller.
*/
//=============================================================================

event PostBeginPlay()
{
  Super.PostBeginPlay();
  Camera = UTP_DemoCamera(Owner);
}


//== Died =====================================================================
/**
The scout pawn should never die.
*/
//=============================================================================

function Died(Controller Killer, class<DamageType> DamageType, vector HitLocation)
{
  Health = Default.Health;
}


//== DrawHUD ==================================================================
/**
Draw the 1st person or 3rd person HUD, when applicable.
*/
//=============================================================================

function DrawHUD(Canvas C)
{
  local HUD H;
  
  if ( Camera != None && Camera.DemoSpectator != None )
    H = Camera.DemoSpectator.MyHud;
  if ( H == None )
    return;
  
  if ( Camera.TargetPawn != None )
    H.PawnOwner = Camera.TargetPawn;
  if ( Camera.TargetPRI != None )
    H.PawnOwnerPRI = Camera.TargetPRI;
  
	if ( H.PawnOwner != None && H.PawnOwner != Self && H.PawnOwner.bSpecialHUD )
		H.PawnOwner.DrawHud(C);
  if ( H.bShowDebugInfo || H.bHideHUD || H.bShowLocalStats || H.bShowScoreBoard )
    return;
  
  if ( H.PlayerOwner == None || H.PawnOwner == None || H.PawnOwner == Self || H.PawnOwnerPRI == None || !Camera.IsInState('FirstPersonView') )
    H.DrawSpectatingHud(C);
  else if ( !H.PawnOwner.bHideRegularHUD )
    H.DrawHud(C);
  
  H.PawnOwner = Self;
}


//== Tick =====================================================================
/**
Update the location of the scout pawn.
*/
//=============================================================================

event Tick(float DeltaTime)
{
  if ( Camera != None ) {
    if ( Camera.IsInState('FreeFlightView') )
      PlayerReplicationInfo = Level.GetLocalPlayerController().PlayerReplicationInfo;
    else
      PlayerReplicationInfo = Camera.TargetPRI;
    
    if ( Location != Camera.Location )
      SetLocation(Camera.Location);
    if ( Rotation != Camera.Rotation )
      SetRotation(Camera.Rotation);
  }
  else {
    if ( Controller != None )
      Controller.Destroy();
    Destroy();
  }
}


//== SpectatorSpecialCalcView =================================================
/**
Calculate the camera parameters.
*/
//=============================================================================

function bool SpectatorSpecialCalcView(PlayerController Viewer, out Actor ViewActor, out vector CameraLocation, out rotator CameraRotation)
{
  if ( Camera.IsInState('FirstPersonView') && Camera.TargetPawn != None )
    ViewActor = Camera.TargetPawn;
  else
    ViewActor = Camera;
  CameraLocation = Camera.Location;
  CameraRotation = Camera.Rotation;
  return true;
}


//== GetDemoRecordingWeapon ===================================================
/**
Update the Weapon and PendingWeapon references for demo playback.
*/
//=============================================================================

function Weapon GetDemoRecordingWeapon()
{
  if ( Camera != None && Camera.TargetPawn != None ) {
    Inventory = Camera.TargetPawn.Inventory;
    return Camera.TargetPawn.GetDemoRecordingWeapon();
  }
  
  Inventory = None;
  Weapon = None;
  PendingWeapon = None;
  
  return Super.GetDemoRecordingWeapon();
}


//== CanTrigger ===============================================================
/**
The scout pawn should not trigger anything.
*/
//=============================================================================

function bool CanTrigger(Trigger T)
{
  return false;
}


//== FellOutOfWorld ===========================================================
/**
The camera scout shouldn't care about falling out of the world.
*/
//=============================================================================

event FellOutOfWorld(eKillZType KillType);


//== DisplayDebug =============================================================
/**
Let the DemoCamera draw the debug information.
*/
//=============================================================================

simulated function DisplayDebug(Canvas C, out float YL, out float YPos)
{
  Super.DisplayDebug(C, YL, YPos);
  if (Camera != None)
    Camera.DisplayDebug(C, YL, YPos);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bCanBeDamaged         = False
  bCanFly               = True
  bCanSwim              = True
  Physics               = PHYS_Flying
  CollisionHeight       = 1.0
  CollisionRadius       = 1.0
  CrouchHeight          = 1.0
  CrouchRadius          = 1.0
  bHidden               = True
  bShouldBaseAtStartup  = False
  bDontPossess          = True
  bHideRegularHUD       = True
  bSpecialHUD           = True
  bSpecialCalcView      = True
  bNoTeamBeacon         = True
  bCanUse               = False
  Visibility            = 0
  bStasis               = False
  RemoteRole            = ROLE_None
  bGameRelevant         = True
  bCollideActors        = False
  bCollideWorld         = True
  bIgnoreOutOfWorld     = True
}
