/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Draws the UTPlus HUD additions.
******************************************************************************/


class UTP_HudOverlay extends HudOverlay;


//=============================================================================
// Imports
//=============================================================================

#exec obj load file=..\Textures\UTPlusTextures.utx


//=============================================================================
// Structs
//=============================================================================

struct RotatedWidget {
  var() HudBase.SpriteWidget SpriteWidget;      // SpriteWidget to rotate
  
  var() bool bFixedAngle;               // DrawRotatedWidget should only offset the widget, not actually rotate it
  var() int Angle;                      // rotation angle in Unreal units
  var() float OffsetCenterX;            // offset of rotation center, relative
  var() float OffsetCenterY;            // to unrotated upper-left corner
  var() float OffsetRotatedX;           // horizontal and vertical displacement
  var() float OffsetRotatedY;           // scaled and rotated with the sprite
};

struct TWidgetLocation {
  var() bool bFlipH, bFlipV;
  
  var() EDrawPivot DrawPivot;
  var() float PosX, PosY;
  var() int OffsetX, OffsetY;
};

struct TProceduralScriptedTexture {
  var() FinalBlend.EFrameBufferBlending FrameBufferBlending;
  var() Actor Client;
  var() int SizeX, SizeY;
  var() array<int> Values;
  var() editinline ScriptedTexture Tex1, Tex2;
  var() editinline FinalBlend DisplayTexture;
};


//=============================================================================
// Variables
//=============================================================================

var UTP_ReplicationInfo       UTPlusRI;
var UTP_PlayerReplicationInfo UTPlusPRI;
var Hud                       Hud;
var HudBase                   HudBase;

// welcome message
var() float WelcomeTime;
var   float InitTime;

// connection status
//var HudBase.SpriteWidget  BadConnectionIcon;
var bool bAlternateDisplayLocation;  // true if connection status was moved to alternate location
var() HudBase.SpriteWidget  ConnectionIcon;
var() HudBase.SpriteWidget  ConnectionBackground;
var() HudBase.SpriteWidget  ConnectionBackgroundDisc;
var() HudBase.SpriteWidget  ConnectionAlert;
var() HudBase.NumericWidget ConnectionPing;
var() HudBase.NumericWidget ConnectionLoss;

var() TWidgetLocation LocConnectionIcon[2];
var() TWidgetLocation LocConnectionBackground[2];
var() TWidgetLocation LocConnectionBackgroundDisc[2];
var() TWidgetLocation LocConnectionAlert[2];
var() TWidgetLocation LocConnectionPing[2];
var() TWidgetLocation LocConnectionLoss[2];

// internal variables for DrawRotatedWidget() function
var private int iTexRotatorPool;        // index into TexRotator object pool
var private float TimeTexRotatorPool;   // time of last TexRotator pool reset
var private array<TexRotator> ListTexRotatorPool;  // TexRotator object pool


//== PostBeginPlay ============================================================
/**
Initialize the Timer for gathering player timing statistics.
*/
//=============================================================================

event PostBeginPlay()
{
  Super.PostBeginPlay();
  
  SetTimer(0.2, True);
  //PingGraph = Spawn(class'UTP_PingGraph');
  //ConnectionGraph.WidgetTexture = PingGraph.DisplayMaterial;
}


//== Destroyed ================================================================
/**
Destroy the ping graph updater and free materials into ObjectPool.
*/
//=============================================================================

event Destroyed()
{
  Super.Destroyed();
  
  //ConnectionGraph.WidgetTexture = None;
  //PingGraph.Destroy();
  
  while (ListTexRotatorPool.Length > 0) {
    Level.ObjectPool.FreeObject(ListTexRotatorPool[0]);
    ListTexRotatorPool.Remove(0, 1);
  }
}


//== Timer ====================================================================
/**
Update the player timing statistics.
*/
//=============================================================================

event Timer()
{
  if ( UTPlusPRI == None && Hud != None ) {
    UTPlusPRI = class'UTP_PlayerReplicationInfo'.static.FindFor(Hud);
    if ( UTPlusPRI != None ) {
      UTPlusPRI.HudOverlay = Self;
      SetTimer(0.0, False);
    }
  }
}


//== NotifyRespawn ============================================================
/**
Called when a pickup respawned.
*/
//=============================================================================

/*function NotifyRespawn(class<Pickup> PickupClass, int PickupID)
{
  if ( PickupGraphs.Length > PickupID && PickupID >= 0 )
    PickupGraphs[PickupID].AddMark(PickupRespawnColor);
}*/


//== NotifyPickup =============================================================
/**
Called when a pickup was picked up by a player.
*/
//=============================================================================

/*function NotifyPickup(class<Pickup> PickupClass, int PickupID, PlayerReplicationInfo Taker)
{
  if ( PickupGraphs.Length > PickupID && PickupID >= 0 ) {
    if ( Taker == Hud.PlayerOwner.PlayerReplicationInfo )
      PickupGraphs[PickupID].AddMark(PickupTakenSelfColor);
    else
      PickupGraphs[PickupID].AddMark(PickupTakenOtherColor);
  }
}*/


//== Render ===================================================================
/**
Main rendering function.
*/
//=============================================================================

function Render(Canvas C)
{
  local Plane OldColorModulate;
  local color OldDrawColor;
  
  if ( InitTime == 0 ) {
    InitTime = Level.TimeSeconds;
    SetConnectionLocation(False);
  }
  
  OldColorModulate  = C.ColorModulate;
  OldDrawColor      = C.DrawColor;
  
  C.ColorModulate.X = 1.0; 
  C.ColorModulate.Y = 1.0; 
  C.ColorModulate.Z = 1.0; 
  C.ColorModulate.W = 1.0;
  C.DrawColor = C.MakeColor(255, 255, 255);
  
  if ( Hud.PlayerOwner == None || Hud.PawnOwner == None || Hud.PawnOwnerPRI == None || Hud.PlayerOwner.IsSpectating() && Hud.PlayerOwner.bBehindView ) {
    DrawSpectatingHud(C);
  }
  else if ( !Hud.PawnOwner.bHideRegularHUD ) {
    DrawHud(C);
  }
  
  if ( Level.TimeSeconds - InitTime < WelcomeTime )
    DisplayWelcomeMessage(C);
  
  C.DrawColor     = OldDrawColor;
  C.ColorModulate = OldColorModulate;
}


//== SetConnectionLocation ====================================================
/**
Adjusts the connection status widget locations.
*/
//=============================================================================

function SetConnectionLocation(bool bUseAlternateDisplayLocation)
{
  bAlternateDisplayLocation = bUseAlternateDisplayLocation;
  if ( bUseAlternateDisplayLocation ) {
    GetSpriteWidgetLocation(ConnectionIcon,           default.ConnectionIcon,           LocConnectionIcon[1]);
    GetSpriteWidgetLocation(ConnectionBackground,     default.ConnectionBackground,     LocConnectionBackground[1]);
    GetSpriteWidgetLocation(ConnectionBackgroundDisc, default.ConnectionBackgroundDisc, LocConnectionBackgroundDisc[1]);
    GetSpriteWidgetLocation(ConnectionAlert,          default.ConnectionAlert,          LocConnectionAlert[1]);
    
    GetNumericWidgetLocation(ConnectionPing, LocConnectionPing[1]);
    GetNumericWidgetLocation(ConnectionLoss, LocConnectionLoss[1]);
  }
  else {
    GetSpriteWidgetLocation(ConnectionIcon,           default.ConnectionIcon,           LocConnectionIcon[0]);
    GetSpriteWidgetLocation(ConnectionBackground,     default.ConnectionBackground,     LocConnectionBackground[0]);
    GetSpriteWidgetLocation(ConnectionBackgroundDisc, default.ConnectionBackgroundDisc, LocConnectionBackgroundDisc[0]);
    GetSpriteWidgetLocation(ConnectionAlert,          default.ConnectionAlert,          LocConnectionAlert[0]);
    
    GetNumericWidgetLocation(ConnectionPing, LocConnectionPing[0]);
    GetNumericWidgetLocation(ConnectionLoss, LocConnectionLoss[0]);
  }
}


//== DrawSpectatingHud ========================================================
/**
Draw HUD elements for spectator HUD.
*/
//=============================================================================

function DrawSpectatingHud(Canvas C)
{
  if ( Level.NetMode == NM_Client && class'UTP_ClientSettings'.default.bDisplayConnectionStatus )
    DrawConnectionStatus(C, False);
}


//== DrawHud ==================================================================
/**
Draw HUD elements for player HUD.
*/
//=============================================================================

function DrawHud(Canvas C)
{
  if ( Level.NetMode == NM_Client && class'UTP_ClientSettings'.default.bDisplayConnectionStatus )
    DrawConnectionStatus(C, Hud.PlayerOwner.bAdrenalineEnabled);
  
  /*if ( PickupGraphs.Length > 0 ) {
    C.DrawColor = Hud.WhiteColor;
    C.SetPos(0, 0.5 * C.ClipY);
    if ( PickupGraphs[0].Values.Length > 0 )
      PickupGraphs[0].UpdateGraph();
    C.DrawTile(PickupGraphs[0].DisplayMaterial, C.ClipX, 0.25 * C.ClipY, 0, 0, PickupGraphs[0].DisplayMaterial.MaterialUSize(), PickupGraphs[0].DisplayMaterial.MaterialVSize());
  }*/
}


//== DisplayWelcomeMessage ====================================================
/**
Draw the welcome message right after the player connected.
*/
//=============================================================================

function DisplayWelcomeMessage(Canvas C);


//== DrawConnectionStatus =====================================================
/**
Draw the connection status.
*/
//=============================================================================

function DrawConnectionStatus(Canvas C, bool bUseAlternateDisplayLocation)
{
  local float Ping;
  local byte Loss;
  
  if ( HudBase != None ) {
    if ( bAlternateDisplayLocation != bUseAlternateDisplayLocation )
      SetConnectionLocation(bUseAlternateDisplayLocation);
    
    HudBase.DrawSpriteWidget(C, ConnectionBackground);
    HudBase.DrawSpriteWidget(C, ConnectionBackgroundDisc);
    if ( Hud.bShowBadConnectionAlert )
      HudBase.DrawSpriteWidget(C, ConnectionAlert);
    HudBase.DrawSpriteWidget(C, ConnectionIcon);
    
    /*if ( class'UTP_ClientSettings'.default.bDisplayPingGraph ) {
      if ( PingGraph.Values.Length > 0 )
        PingGraph.UpdateGraph();
      HudBase.DrawSpriteWidget(C, ConnectionGraph);
    }*/
    
    if ( HudCDeathMatch(Hud) != None ) {
      if ( UTPlusPRI != None && UTPlusPRI.RealPing > 0 )
        Ping = UTPlusPRI.RealPing;
      else if ( Hud.PlayerOwner != None )
        Ping = 1000 * Hud.PlayerOwner.ExactPing;
      
      ConnectionPing.Value = Ping;
      ConnectionPing.Tints[0].G = 255 - FClamp(4 * (Ping - 100), 0, 255);
      ConnectionPing.Tints[0].R = FMin((Ping - 10) * 51 / 18, 255);
      ConnectionPing.Tints[1] = ConnectionPing.Tints[0];
      HudBase.DrawNumericWidget(C, ConnectionPing, HudCDeathMatch(Hud).DigitsBig);
      
      if ( Hud.PlayerOwner != None && Hud.PlayerOwner.PlayerReplicationInfo != None )
        Loss = Hud.PlayerOwner.PlayerReplicationInfo.PacketLoss;
      ConnectionLoss.Value = Loss;
      if ( Loss == 0 )
        ConnectionLoss.Tints[0] = Hud.WhiteColor;
      else
        ConnectionLoss.Tints[0] = Hud.RedColor;
      ConnectionLoss.Tints[1] = ConnectionLoss.Tints[0];
      HudBase.DrawNumericWidget(C, ConnectionLoss, HudCDeathMatch(Hud).DigitsBig);
    }
  }
}


//== DrawRotatedWidget ========================================================
/**
Draws a RotatedWidget sprite on the given Canvas.
*/
//=============================================================================

final function DrawRotatedWidget(Canvas C, RotatedWidget RotatedWidget)
{
  local float AngleInRadian;
  local float SinAngle;
  local float CosAngle;
  local TexRotator TexRotator;
  local HudBase.SpriteWidget RotatedSpriteWidget;
  
  if ( TimeTexRotatorPool != Level.TimeSeconds )
    iTexRotatorPool = 0;
  else
    iTexRotatorPool++;
  
  TimeTexRotatorPool = Level.TimeSeconds;
  if ( iTexRotatorPool >= ListTexRotatorPool.Length )
    ListTexRotatorPool[iTexRotatorPool] = TexRotator(Level.ObjectPool.AllocateObject(class'TexRotator'));
  
  TexRotator = ListTexRotatorPool[iTexRotatorPool];
  RotatedSpriteWidget = RotatedWidget.SpriteWidget;
  if ( !RotatedWidget.bFixedAngle ) {
    RotatedSpriteWidget.WidgetTexture = TexRotator;
    
    TexRotator.Material        = RotatedWidget.SpriteWidget.WidgetTexture;
    TexRotator.UOffset         = RotatedWidget.OffsetCenterX + RotatedWidget.SpriteWidget.TextureCoords.X1;
    TexRotator.VOffset         = RotatedWidget.OffsetCenterY + RotatedWidget.SpriteWidget.TextureCoords.Y1;
    TexRotator.Rotation.Yaw    = RotatedWidget.Angle;
    TexRotator.Rotation.Pitch  = 0;
    TexRotator.Rotation.Roll   = 0;
    TexRotator.TexRotationType = TR_FixedRotation;
  }
  if ( RotatedWidget.OffsetRotatedX != 0 || RotatedWidget.OffsetRotatedY != 0 ) {
    AngleInRadian = Pi * RotatedWidget.Angle / 32768;
    SinAngle = Sin(AngleInRadian);
    CosAngle = Cos(AngleInRadian);
    
    RotatedSpriteWidget.OffsetX += RotatedWidget.OffsetRotatedX * CosAngle + RotatedWidget.OffsetRotatedY * SinAngle;
    RotatedSpriteWidget.OffsetY += RotatedWidget.OffsetRotatedY * CosAngle - RotatedWidget.OffsetRotatedX * SinAngle;
  }
  
  HudBase.DrawSpriteWidget(C, RotatedSpriteWidget);
}


//== GetSpriteWidgetLocation ==================================================
/**
Applies a WidgetLocation to a SpriteWidget.
*/
//=============================================================================

final function GetSpriteWidgetLocation(out HudBase.SpriteWidget W, HudBase.SpriteWidget WDefault, TWidgetLocation L)
{
  W.PosX = L.PosX;
  W.PosY = L.PosY;
  W.OffsetX = L.OffsetX;
  W.OffsetY = L.OffsetY;
  W.DrawPivot = L.DrawPivot;
  
  if ( L.bFlipH ) {
    W.TextureCoords.X1 = WDefault.TextureCoords.X2;
    W.TextureCoords.X2 = WDefault.TextureCoords.X1;
  }
  if ( L.bFlipV ) {
    W.TextureCoords.Y1 = WDefault.TextureCoords.Y2;
    W.TextureCoords.Y2 = WDefault.TextureCoords.Y1;
  }
}


//== GetRotatedWidgetLocation =================================================
/**
Applies a WidgetLocation to a RotatedWidget's SpriteWidget.
*/
//=============================================================================

final function GetRotatedWidgetLocation(out RotatedWidget W, RotatedWidget WDefault, TWidgetLocation L)
{
  GetSpriteWidgetLocation(W.SpriteWidget, WDefault.SpriteWidget, L);
}


//== GetNumericWidgetLocation =================================================
/**
Applies a WidgetLocation to a NumericWidget.
*/
//=============================================================================

final function GetNumericWidgetLocation(out HudBase.NumericWidget W, TWidgetLocation L)
{
  W.PosX = L.PosX;
  W.PosY = L.PosY;
  W.OffsetX = L.OffsetX;
  W.OffsetY = L.OffsetY;
  W.DrawPivot = L.DrawPivot;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  WelcomeTime = 15.0
  
  ConnectionIcon            = (WidgetTexture=Texture'ConnectionIcon',RenderStyle=STY_Alpha,TextureCoords=(X1=0,Y1=0,X2=128,Y2=128),TextureScale=0.21,ScaleMode=SM_Right,Scale=1.000000,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  ConnectionBackground      = (WidgetTexture=Texture'HudContent.Generic.HUD',RenderStyle=STY_Alpha,TextureCoords=(X1=168,Y1=211,X2=334,Y2=255),TextureScale=0.53,ScaleMode=SM_Right,Scale=1.000000,Tints[0]=(R=0,G=0,B=0,A=150),Tints[1]=(R=0,G=0,B=0,A=150))
  ConnectionBackgroundDisc  = (WidgetTexture=Texture'HudContent.Generic.HUD',RenderStyle=STY_Alpha,TextureCoords=(X1=119,Y1=258,X2=173,Y2=313),TextureScale=0.53,ScaleMode=SM_Right,Scale=1.000000,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  ConnectionAlert           = (WidgetTexture=Material'HudContent.Generic.fb_Pulse001',RenderStyle=STY_Alpha,TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.53,ScaleMode=SM_Right,Scale=1.000000,Tints[0]=(R=255,G=192,B=0,A=255),Tints[1]=(R=255,G=192,B=0,A=255))
  ConnectionPing            = (RenderStyle=STY_Alpha,TextureScale=0.4,Tints[0]=(A=255),Tints[1]=(A=255))
  ConnectionLoss            = (RenderStyle=STY_Alpha,TextureScale=0.3,Tints[0]=(A=255),Tints[1]=(A=255))
  
  LocConnectionIcon(0)            = (PosX=1.0,PosY=0.0,OffsetX=5,OffsetY=20,DrawPivot=DP_UpperRight)
  LocConnectionBackground(0)      = (PosX=1.0,PosY=0.0,OffsetX=0,OffsetY=10,DrawPivot=DP_UpperRight)
  LocConnectionBackgroundDisc(0)  = (PosX=1.0,PosY=0.0,OffsetX=0,OffsetY=5,DrawPivot=DP_UpperRight)
  LocConnectionAlert(0)           = (PosX=1.0,PosY=0.0,OffsetX=6,OffsetY=1,DrawPivot=DP_UpperRight)
  LocConnectionPing(0)            = (PosX=1.0,PosY=0.0,OffsetX=-74,OffsetY=32,DrawPivot=DP_MiddleRight)
  LocConnectionLoss(0)            = (PosX=1.0,PosY=0.0,OffsetX=-100,OffsetY=78,DrawPivot=DP_MiddleRight)
  
  LocConnectionIcon(1)            = (PosX=1.0,PosY=0.0,OffsetX=5,OffsetY=184,DrawPivot=DP_UpperRight)
  LocConnectionBackground(1)      = (PosX=1.0,PosY=0.0,OffsetX=0,OffsetY=74,DrawPivot=DP_UpperRight)
  LocConnectionBackgroundDisc(1)  = (PosX=1.0,PosY=0.0,OffsetX=0,OffsetY=69,DrawPivot=DP_UpperRight)
  LocConnectionAlert(1)           = (PosX=1.0,PosY=0.0,OffsetX=6,OffsetY=65,DrawPivot=DP_UpperRight)
  LocConnectionPing(1)            = (PosX=1.0,PosY=0.0,OffsetX=-74,OffsetY=116,DrawPivot=DP_MiddleRight)
  LocConnectionLoss(1)            = (PosX=1.0,PosY=0.0,OffsetX=-100,OffsetY=192,DrawPivot=DP_MiddleRight)
}
