/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Catches map changes and hacks clientdemo playback to add some features.
******************************************************************************/


class UTP_Interaction extends Interaction;


//=============================================================================
// Structs
//=============================================================================

struct TAliasData {
  var string Alias;
  var array<string> Commands;
  var array<string> Parameters;
  var array<byte> bOnRelease;
};

struct TKeybindData {
  var string RawKeyBind;
  var array<string> Commands;
  var array<string> Parameters;
  var array<byte> bOnRelease;
  var array<byte> MatchingAlias;
};


//=============================================================================
// Variables
//=============================================================================

var bool    bClientDemoPlayback;
var Actor   ViewTarget;
var vector  ViewLocation;
var rotator ViewRotation;

var UTP_DemoCamera Camera;
var UTP_CamPathManager PathManager;

// auto-screenshots
var bool bPendingScreenshot, bPendingScoresScreenshot;
var bool bPrevShowScores, bPrevShowStats, bPrevShowDebug;

var bool bFrameDumping;

var bool bPrimary, bSecondary;
var TKeybindData KeyBindCache[256];
var array<TAliasData> Aliases;


//== Initialized ==============================================================
/**
Called whe the interaction is created.
*/
//=============================================================================

event Initialized()
{
  local Object O;
  local string AliasesString, Command, Parameters;
  local array<string> AliasesList;
  local int i, j;
  
  Super.Initialized();
  if ( ViewportOwner != None && ViewportOwner.Actor != None && ViewportOwner.Actor.bDemoOwner ) {
    log("Demo playback.", 'UTPlus');
    bClientDemoPlayback = True;
    Camera = ViewportOwner.Actor.Spawn(class'UTP_DemoCamera', ViewportOwner.Actor);
    Camera.MyInteraction = Self;
  }
  PathManager = class'UTP_CamPathManager'.static.GetPathManager(ViewportOwner.Actor);
  
  foreach AllObjects(class'Object', O) {
    if ( O.IsA('Input') ) {
      //log(O.GetPropertyText("Aliases"));
      AliasesString = O.GetPropertyText("Aliases");
      break;
    }
  }
  
  Split(Mid(AliasesString, 9, Len(AliasesString) - 10), "),(Command=", AliasesList);
  // fill Aliases array
  Aliases.Length = 40;
  for (i = 0; i < 40; ++i) {
    if ( Left(AliasesList[i], 1) == "\"" ) {
      Divide(Mid(AliasesList[i], 1), "\",Alias=", Command, Aliases[i].Alias);
      Command = "|" $ Repl(Command, Chr(9), " ") $ "|";
      while (InStr(Command, "  ") != -1) {
        Command = Repl(Command, "  ", " ");
      }
      Command = Repl(Command, " |", "|");
      Command = Repl(Command, "| ", "|");
      while (InStr(Command, "||") != -1) {
        Command = Repl(Command, "||", "|");
      }
      
      Split(Command, "|", Aliases[i].Commands);
      for (j = 0; j < Aliases[i].Commands.Length; ++j) {
        if ( Aliases[i].Commands[j] == "" )
          Aliases[i].Commands.Remove(j--, 1);
        else {
          if ( Right(Aliases[i].Commands[j], 1) != " " )
            Aliases[i].Commands[j] $= " ";
          //log(i@j@Aliases[i].Commands[j], 'Alias');
        }
      }
    }
    else {
      Aliases[i].Alias = Mid(AliasesList[i], 7);
    }
    if ( Left(Aliases[i].Alias, 1) == "\"" ) {
      Aliases[i].Alias = Mid(Aliases[i].Alias, 1, Len(Aliases[i].Alias) - 2);
    }
  }
  // remove unused alias entries
  for (i = 0; i < Aliases.Length; ++i) {
    if (Aliases[i].Alias == "") {
      Aliases.Remove(i--, 1);
      continue;
    }
    for (j = 0; j < Aliases[i].Commands.Length; j++) {
      if ( !Divide(Aliases[i].Commands[j], " ", Command, Parameters) ) {
        Command = Aliases[i].Commands[j];
        Parameters = "";
      }
      if ( Command ~= "OnRelease" || Command ~= "OnRelease " ) {
        Aliases[i].bOnRelease[j] = 1;
        if ( !Divide(Parameters, " ", Command, Parameters) ) {
          Command = Parameters;
          Parameters = "";
        }
      }
      else
        Aliases[i].bOnRelease[j] = 0;
      if (Right(Command, 1) == " ")
        Command = Left(Command, Len(Command) - 1);
      if (Command == "") {
        Aliases[i].Commands.Remove(j--, 1);
        continue;
      }
      Aliases[i].Commands[j] = Command;
      Aliases[i].Parameters[j] = Parameters;
    }
  }
}


//== OpenUTPlusMenu ===========================================================
/**
Open the UTPlus menu.
*/
//=============================================================================

function OpenUTPlusMenu()
{
  local GUITabControl LoginMenuTabs;
  local UTP_ReplicationInfo UTPlusRI;
  local int i;
  
  if ( ViewportOwner != None && ViewportOwner.Actor != None && UnrealPlayer(ViewportOwner.Actor) != None && UnrealPlayer(ViewportOwner.Actor).LoginMenuClass != "" ) {
    ViewportOwner.Actor.ClientOpenMenu(UnrealPlayer(ViewportOwner.Actor).LoginMenuClass);
    
    foreach ViewportOwner.Actor.DynamicActors(class'UTP_ReplicationInfo', UTPlusRI) {
      UTPlusRI.Tick(0.0);
    }
    
    if ( GUIController(ViewportOwner.GUIController) != None && UT2K4PlayerLoginMenu(GUIController(ViewportOwner.GUIController).ActivePage) != None ) {
      LoginMenuTabs = UT2K4PlayerLoginMenu(GUIController(ViewportOwner.GUIController).ActivePage).c_Main;
      if ( LoginMenuTabs != None ) {
        for (i = 0; i < LoginMenuTabs.TabStack.Length; ++i) {
          if ( UTPGUI_MidGamePanel(LoginMenuTabs.TabStack[i].MyPanel) != None ) {
            LoginMenuTabs.TabStack[i].OnClick(LoginMenuTabs.TabStack[i]);
            break;
          }
        }
      }
    }
  }
}


//== OpenDemoMenu =============================================================
/**
Open the demo playback menu.
*/
//=============================================================================

function bool OpenDemoMenu()
{
  if ( ViewportOwner != None && ViewportOwner.Actor != None ) {
    if ( class'UTP_DemoPlaybackSettings'.default.DemoPlaybackMenu != None ) {
      ViewportOwner.Actor.ClientOpenMenu(string(class'UTP_DemoPlaybackSettings'.default.DemoPlaybackMenu));
      return true;
    }
    else
      log("No UTPlus Demo Playback Menu configured!", 'UTPlusError');
  }
  return false;
}


//== KeyIsBoundTo =============================================================
/**
Returns, whether a command is bound to a key.
*/
//=============================================================================

function bool KeyIsBoundTo(EInputKey Key, string BoundCommand, optional bool bOnRelease)
{
  local string Keybind;
  local int CmdLen, i, j, k;
  local string Command, Parameters;
  
  if ( BoundCommand == "" || Key == IK_None )
    return false;
  
  Keybind = ViewportOwner.Actor.ConsoleCommand("KeyName" @ Key);
  Keybind = ViewportOwner.Actor.ConsoleCommand("KeyBinding" @ Keybind);
  
  if (KeyBindCache[Key].RawKeyBind != Keybind) {
    // refresh the keybind data cache for this key
    KeyBindCache[Key].RawKeyBind = Keybind;
    KeyBindCache[Key].Commands.Length = 0;
    KeyBindCache[Key].Parameters.Length = 0;
    KeyBindCache[Key].bOnRelease.Length = 0;
    KeyBindCache[Key].MatchingAlias.Length = 0;
    
    // remove multiple whitespaces
    Keybind = "|" $ Repl(Keybind, Chr(9), " ") $ "|";
    while (InStr(Keybind, "  ") != -1) {
      Keybind = Repl(Keybind, "  ", " ");
    }
    Keybind = Repl(Keybind, " |", "|");
    Keybind = Repl(Keybind, "| ", "|");
    while (InStr(Command, "||") != -1) {
      Command = Repl(Command, "||", "|");
    }
    
    Split(Keybind, "|", KeyBindCache[Key].Commands);
    
    for (i = 0; i < KeyBindCache[Key].Commands.Length; i++) {
      if (KeyBindCache[Key].Commands[i] == "") {
        KeyBindCache[Key].Commands.Remove(i--, 1);
        continue;
      }
      if ( !Divide(KeyBindCache[Key].Commands[i], " ", Command, Parameters) ) {
        Command = KeyBindCache[Key].Commands[i];
        Parameters = "";
      }
      if ( Command ~= "OnRelease" || Command ~= "OnRelease " ) {
        KeyBindCache[Key].bOnRelease[i] = 1;
        if ( !Divide(Parameters, " ", Command, Parameters) ) {
          Command = Parameters;
          Parameters = "";
        }
      }
      else
        KeyBindCache[Key].bOnRelease[i] = 0;
      if (Command == "") {
        KeyBindCache[Key].Commands.Remove(i--, 1);
        continue;
      }
      if (Parameters != "" && Right(Parameters, 1) != " ")
        Parameters $= " ";
      KeyBindCache[Key].Commands[i] = Command;
      KeyBindCache[Key].Parameters[i] = Parameters;
      KeyBindCache[Key].MatchingAlias[i] = -1;
      for (j = 0; j < Aliases.Length; j++) {
        if (Aliases[j].Alias ~= Command) {
          KeyBindCache[Key].MatchingAlias[i] = j;
          break;
        }
      }
    }
  }
  
  CmdLen = Len(BoundCommand);
  for (i = 0; i < KeyBindCache[Key].Commands.Length; i++) {
    if (bOnRelease == bool(KeyBindCache[Key].bOnRelease[i])) {
      // try exact match
      if (KeyBindCache[Key].Commands[i] ~= BoundCommand)
        return true;
      
      // try match with parameters
      if (KeyBindCache[Key].Parameters[i] != "" && Left(KeyBindCache[Key].Commands[i] @ KeyBindCache[Key].Parameters[i], CmdLen + 1) ~= (BoundCommand $ " "))
        return true;
    }
    
    k = KeyBindCache[Key].MatchingAlias[i];
    if (k < Aliases.Length) {
      // try alias match
      for (j = 0; j < Aliases[k].Commands.Length; j++) {
        if (bOnRelease == bool(Aliases[k].bOnRelease[j])) {
          // try exact match
          if (Aliases[k].Commands[j] ~= BoundCommand)
            return true;
          
          // try match with parameters
          if (Aliases[k].Parameters[j] != "" && Left(Aliases[k].Commands[j] @ Aliases[k].Parameters[j], CmdLen + 1) ~= (BoundCommand $ " "))
            return true;
        }
      }
    }
  }
  return false;
}


//== KeyEvent =================================================================
/**
Captures key events for ingame-config menu and demo playback commands.
*/
//=============================================================================

function bool KeyEvent(out EInputKey Key, out EInputAction Action, float Delta)
{ 
  if ( ViewportOwner == None || ViewportOwner.Console != None && ViewportOwner.Console.bTyping )
    return false;
  
  if ( bClientDemoPlayback || Camera != None ) {
    // client demo playback
    if ( Action == IST_Press ) {
      if ( bClientDemoPlayback && KeyIsBoundTo(Key, "ShowMenu") ) {
        return OpenDemoMenu();
      }
      else if (!bClientDemoPlayback  && KeyIsBoundTo(Key, "ToggleDemoCamera")) {
        Camera.Destroy();
        Camera = None;
      }
      else if ( KeyIsBoundTo(Key, "Fire") || KeyIsBoundTo(Key, "DemoViewNextPlayer") ) {
        // view next player
        if ( Camera != None )
          Camera.ViewNextPlayer();
        return true;
      }
      else if ( KeyIsBoundTo(Key, "AltFire") || KeyIsBoundTo(Key, "ToggleBehindView") ) {
        // toggle view mode
        if ( Camera != None )
          Camera.ToggleViewMode();
        return true;
      }
      else if ( KeyIsBoundTo(Key, "Use") || KeyIsBoundTo(Key, "ViewNextProjectile") ) {
        if ( Camera != None )
          Camera.ViewNextProjectile();
        return true;
      }
      else if ( KeyIsBoundTo(Key, "CenterView") ) {
        if ( Camera != None )
          Camera.CenterView();
        return true;
      }
      else if ( KeyIsBoundTo(Key, "MoveForward") ) {
        if ( Camera != None )
          Camera.bForward = true;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "MoveBackward") ) {
        if ( Camera != None )
          Camera.bBackward = true;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "StrafeLeft") ) {
        if ( Camera != None )
          Camera.bLeft = true;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "StrafeRight") ) {
        if ( Camera != None )
          Camera.bRight = true;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "Jump") ) {
        if ( Camera != None )
          Camera.bUp = true;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "Duck") ) {
        if ( Camera != None )
          Camera.bDown = true;
        return true;
      }
    }
    else if ( Action == IST_Release ) {
      if ( bClientDemoPlayback && KeyIsBoundTo(Key, "ShowMenu", True) ) {
        return OpenDemoMenu();
      }
      else if (!bClientDemoPlayback  && KeyIsBoundTo(Key, "ToggleDemoCamera", True)) {
        Camera.Destroy();
        Camera = None;
      }
      else if ( KeyIsBoundTo(Key, "DemoViewNextPlayer", True) ) {
        // view next player
        if ( Camera != None )
          Camera.ViewNextPlayer();
        return true;
      }
      else if ( KeyIsBoundTo(Key, "ToggleBehindView", True) ) {
        // toggle view mode
        if ( Camera != None )
          Camera.ToggleViewMode();
        return true;
      }
      else if ( KeyIsBoundTo(Key, "Use", True) || KeyIsBoundTo(Key, "ViewNextProjectile", True) ) {
        if ( Camera != None )
          Camera.ViewNextProjectile();
        return true;
      }
      else if ( KeyIsBoundTo(Key, "MoveForward") ) {
        if ( Camera != None )
          Camera.bForward = false;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "MoveBackward") ) {
        if ( Camera != None )
          Camera.bBackward = false;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "StrafeLeft") ) {
        if ( Camera != None )
          Camera.bLeft = false;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "StrafeRight") ) {
        if ( Camera != None )
          Camera.bRight = false;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "Jump") ) {
        if ( Camera != None )
          Camera.bUp = false;
        return true;
      }
      else if ( KeyIsBoundTo(Key, "Duck") ) {
        if ( Camera != None )
          Camera.bDown = false;
        return true;
      }
    }
    else if ( Action == IST_Axis && Camera != None ) {
      if ( Key == IK_MouseX ) {
        Delta *= Camera.TurnRate;
        Camera.FreeViewRotation.Yaw += Delta;
        //log(ViewRotation);
        return true;
      }
      else if ( Key == IK_MouseY ) {
        Delta *= Camera.PitchRate;
        Camera.FreeViewRotation.Pitch += Delta;
        Camera.FreeViewRotation.Pitch = Clamp(Camera.FreeViewRotation.Pitch, -16380, 16380);
        //log(ViewRotation);
        return true;
      }
    }
  }
  else if ( Action == IST_Press || Action == IST_Release ) {
    if ( KeyIsBoundTo(Key, "MyMenu", Action == IST_Release) ) {
      OpenUTPlusMenu();
    }
    else if (ViewportOwner.Actor.PlayerReplicationInfo.bOnlySpectator && KeyIsBoundTo(Key, "ToggleDemoCamera", Action == IST_Release)) {
      Camera = ViewportOwner.Actor.Spawn(class'UTP_DemoCamera', ViewportOwner.Actor);
      Camera.MyInteraction = Self;
      Camera.GotoState('ChaseCamView');
    }
  }
  
  //log("KeyEvent" @ GetEnum(enum'EInputKey', Key) @ GetEnum(enum'EInputAction', Action) @ Delta);
  return false;
}


//== PreRender ================================================================
/**
If a screenshot should be taken, this function enables the scoreboard.
*/
//=============================================================================

function PreRender(Canvas C)
{
  if ( !bFrameDumping && bPendingScreenshot && ViewportOwner.Actor != None && ViewportOwner.Actor.MyHud != None ) {
    bPrevShowScores = ViewportOwner.Actor.MyHud.bShowScoreboard;
    bPrevShowStats = ViewportOwner.Actor.MyHud.bShowLocalStats;
    bPrevShowDebug = ViewportOwner.Actor.MyHud.bShowDebugInfo;
    
    ViewportOwner.Actor.MyHud.bShowScoreboard = bPendingScoresScreenshot;
    ViewportOwner.Actor.MyHud.bShowLocalStats = False;
    ViewportOwner.Actor.MyHud.bShowDebugInfo = False;
  }
  if ( !bClientDemoPlayback && Camera != None && ViewportOwner.Actor.PlayerReplicationInfo != None && !ViewportOwner.Actor.PlayerReplicationInfo.bOnlySpectator ) {
    Camera.Destroy();
    Camera = None;
  }
}


//== PostRender ===============================================================
/**
Update the camera location in 1st person and fixed behind view modes.
*/
//=============================================================================

function PostRender(Canvas C)
{
  local vector CamLocation;
  local rotator CamRotation;
  local string AutoShotName;
  local UTP_ReplicationInfo UTPlusRI;
  
  if ( !bFrameDumping && bPendingScreenshot && ViewportOwner.Actor != None && ViewportOwner.Actor.MyHud != None ) {
    //log("Drawing UTPlus logo to" @ 0 $"/"$ C.ClipY - 0.5 * C.ClipX @"/"@ C.ClipX $"/"$ 0.5 * C.ClipX);
    C.Reset();
    C.Style = 5; // ERenderStyle.STY_Alpha
    C.SetDrawColor(255, 255, 255, 32);
    C.SetPos(0, C.ClipY - 0.5 * C.ClipX);
    C.DrawRect(Texture'UTPlusText', C.ClipX, 0.5 * C.ClipX);
    //C.SetDrawColor(255, 255, 255, 32);
    //C.SetPos(0, C.ClipY - 0.5 * C.ClipX);
    //C.DrawRect(Texture'UTPlusTextGlow', C.ClipX, 0.5 * C.ClipX);
    AutoShotName = class'UTP_Util'.static.ParseFileName(class'UTP_ClientSettings'.default.AutoScreenshotName, ViewportOwner.Actor);
    if ( bPendingScoresScreenshot ) {
      if ( AutoShotName != "" )
        ViewportOwner.Console.DelayedConsoleCommand("shot UTPScoresShot_" $ AutoShotName);
      else
        ViewportOwner.Console.DelayedConsoleCommand("shot");
    }
    else {
      foreach ViewportOwner.Actor.DynamicActors(class'UTP_ReplicationInfo', UTPlusRI) break;
      if ( UTPlusRI != None )
        ViewportOwner.Console.DelayedConsoleCommand("shot UTPAutoShot" $ UTPlusRI.GameScreenshotCount $ "_" $ class'UTP_Util'.static.ParseFileName("%d_%t_%g_%m", ViewportOwner.Actor));
      else
        ViewportOwner.Console.DelayedConsoleCommand("shot UTPAutoShot_" $ class'UTP_Util'.static.ParseFileName("%d_%t_%g_%m", ViewportOwner.Actor));
    }
    bPendingScreenshot = False;
    bPendingScoresScreenshot = False;
    ViewportOwner.Actor.MyHud.bShowScoreboard = bPrevShowScores || ViewportOwner.Actor.IsInState('RoundEnded') || ViewportOwner.Actor.IsInState('GameEnded');
    ViewportOwner.Actor.MyHud.bShowLocalStats = bPrevShowStats;
    ViewportOwner.Actor.MyHud.bShowDebugInfo = bPrevShowDebug;
  }
  if ( Camera != None ) {
    // client demo playback, update camera location
    C.GetCameraLocation(CamLocation, CamRotation);
    Camera.CurrentCameraLocation = CamLocation;
    Camera.CurrentCameraRotation = CamRotation;
    Camera.PostRender(C);
  }
  if ( (ViewportOwner.Actor.PlayerReplicationInfo == None || ViewportOwner.Actor.PlayerReplicationInfo.bOnlySpectator) && class'UTP_ClientSettings'.default.bDrawPathStatus && PathManager != None )
    PathManager.DrawStatus(C);
}


//== StartDump ================================================================
/**
Start dumping each frame with the desired framerate.
*/
//=============================================================================

function bool StartDump(int FrameRate)
{
  StopDump();
  
  bFrameDumping = True;
  log("Start dumping frames with" @ FrameRate @ "FPS.", 'UTPlus');
  if ( !ConsoleCommand("FixedFPS"@FrameRate) || !ConsoleCommand("DumpFrames Start") ) {
    StopDump();
    return false;
  }
  return true;
}


//== StopDump =================================================================
/**
Stop dumping frames.
*/
//=============================================================================

function StopDump()
{
  if ( bFrameDumping ) {
    ConsoleCommand("DumpFrames Stop");
    ConsoleCommand("FixedFPS");
    log("DumpFrames stopped.", 'UTPlus');
    bFrameDumping = False;
  }
}


//== NotifyLevelChange ========================================================
/**
Notify the UTP_ReplicationInfo that the level is about to change.
*/
//=============================================================================

event NotifyLevelChange()
{
  local UTP_ReplicationInfo UTPlusRI;
  
  log("Map change detected.", 'UTPlusDebug');
  StopDump();
  
  foreach ViewportOwner.Actor.DynamicActors(class'UTP_ReplicationInfo', UTPlusRI) {
    UTPlusRI.NotifyLevelChange();
  }
  
  if ( Camera != None )
    Camera.Destroy();
  Camera = None;
  if (PathManager != None)
    PathManager.Destroy();
  PathManager = None;
  
  Master.RemoveInteraction(Self);
  
  // HACKFIX: This sucks, but it's the only way to reliably repair the "damage"
  //          done to the default properties by the Spawn Grenades mutator in
  //          single player.
  // (It's Epic's fault because they *require* the change to be made to the defaults.)
  class'GrenadeAmmo'.default.InitialAmount = 4;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bActive   = True
  bVisible  = True
}
