/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Map-specific clientside settings for UTPlus.
******************************************************************************/


class UTP_MapSettings extends Object
    dependson(UTP_Pawn)
    transient
    config(UTPlusClientSettings)
    perobjectconfig;


//=============================================================================
// Configuration
//=============================================================================

// Player visibility
var(EnemyModel) config bool   bForceEnemyModel;
var(EnemyModel) config string ForcedEnemyModel;
var(EnemyModel) config byte   ForcedEnemySkinColor;

var(TeamModel) config bool   bForceTeamModel;
var(TeamModel) config string ForcedTeamModel;
var(TeamModel) config byte   ForcedTeamSkinColor;

var(SkinOverlay) config bool                bUseSkinOverlay;
//var(SkinOverlay) config bool                bAlphaBlendOverlay;
var(SkinOverlay) config UTP_Pawn.TSkinColor TeamColorEnemy;
var(SkinOverlay) config UTP_Pawn.TSkinColor TeamColorFriend;
var(SkinOverlay) config UTP_Pawn.TSkinColor TeamColors[5]   "Index: 0 Red, 1 Blue, 2 Green, 3 Gold, 4 Neutral";


//=============================================================================
// Variables
//=============================================================================

var() editconst bool bMapSpecific;
var   private LevelInfo Level;


//== static GetClientConfiguration ============================================
/**
Returns a client configuration object for the specified map.
*/
//=============================================================================

static function UTP_MapSettings GetClientConfiguration(Actor Requester, string MapName)
{
  local UTP_MapSettings Settings, UTPMS;
  local array<string> SettingsList;
  local int i;
  local bool bTryGarbage;
  local array<Object> Refs;
  
  if ( Requester == None )
    return None;
  
  foreach Requester.AllObjects(class'UTP_MapSettings', UTPMS) {
    if ( UTPMS.Level != Requester.Level ) {
      log(UTPMS.Class.Name $ "'" $ UTPMS.Name $ "' belongs to a different level!", 'Warning');
      log("", 'UTPlusDebugGarbage');
      log("Referencers:", 'UTPlusDebug');
      Requester.GetReferencers(UTPMS, Refs);
      while (Refs.Length > 0) {
        log("Ref:"@Refs[0].Class @ Refs[0], 'UTPlusDebug');
        Requester.ConsoleCommand("obj refs class=" $ Refs[0].Class @ "name=" $ Refs[0], True);
        Refs.Remove(0, 1);
      }
      bTryGarbage = True;
    }
    else {
      Settings = UTPMS;
    }
  }
  UTPMS = None;
  
  if ( bTryGarbage ) {
    Requester.ConsoleCommand("obj list", True);
    Requester.ConsoleCommand("obj garbage", True);
    Requester.ConsoleCommand("obj list class=" $ class'UTP_MapSettings', True);
  }
  if ( Settings == None ) {
    Settings = new(None, MapName) default.Class;
    if ( Settings != None ) {
      Settings.Level = Requester.Level;
      SettingsList = GetPerObjectNames("UTPlusClientSettings");
      for (i = 0; i < SettingsList.Length; ++i) {
        //log(SettingsList[i]);
        if ( SettingsList[i] ~= MapName ) {
          Settings.bMapSpecific = True;
          break;
        }
      }
    }
  }
  return Settings;
}


//== Free =====================================================================
/**
Free references to other objects.
*/
//=============================================================================

function Free()
{
  Level = None;
}


//=============================================================================
// Default configuration
//=============================================================================

defaultproperties
{
  bForceEnemyModel      = False
  ForcedEnemyModel      = "Arclite"
  ForcedEnemySkinColor  = 254
  
  bForceTeamModel     = False
  ForcedTeamModel     = "Tamika"
  ForcedTeamSkinColor = 254
  
  bUseSkinOverlay     = True
  //bAlphaBlendOverlay  = True
  TeamColorEnemy  = (R=255,G=255,B=0)
  TeamColorFriend = (R=255,G=0,B=0)
  TeamColors(0)   = (R=255,G=0,B=0)
  TeamColors(1)   = (R=0,G=0,B=255)
  TeamColors(2)   = (R=0,G=255,B=0)
  TeamColors(3)   = (R=255,G=255,B=0)
  TeamColors(4)   = (R=255,G=0,B=255)
}