/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


UTPlus Pawn class for bright skins.
******************************************************************************/


class UTP_Pawn extends xMutantPawn;


//=============================================================================
// Structs
//=============================================================================

struct TPawnLocation {
  var InterpCurve LocX, LocY, LocZ, CollRadius, CollHeight;
};

struct TRepOverlay {
  var Material  Material;
  var float     Duration;   // TODO: change to byte to save bandwidth
  var float     EndTime;
};

struct TSkinColor {
  var() config byte R;
  var() config byte G;
  var() config byte B;
};


//=============================================================================
// Variables
//=============================================================================

var UTP_ReplicationInfo       UTPlusRI;
var UTP_PlayerReplicationInfo UTPlusPRI;
var GameReplicationInfo       GRI;
var byte                      RealTeam;
var xUtil.PlayerRecord        PlayerRecord;
var UTP_MapSettings           MapSettings;

var() Material MutantInvisMaterial;

// bright skin overlays
var() editinline Material       BodySkin, FaceSkin, OverlayTexture;
var() editinline ConstantColor  SkinColorTeam, OverlayColorFade;
var() editinline Combiner       SkinCombinerBody, SkinCombinerHead, OverlayCombiner;
var() editinline Shader         SkinShaderBody, SkinShaderHead;
var() editinline FinalBlend     SkinFinalBody, SkinFinalHead;
var() editinline ColorModifier  DeResColorBody;
var() editinline ColorModifier  DeResColorHead;
var() editinline FinalBlend     DeResFinalBody;
var() editinline FinalBlend     DeResFinalHead;

var() string    DeResBaseBody;
var() string    DeResBaseHead;
var() color     DeResColor;
var() float     OverlayDuration;
var() float     OverlayEndTime;
var TRepOverlay RepOverlay, LastRepOverlay;
var TSkinColor  MySkinColor;

var bool bBrightSkins, bDemoPlayback, bLessLockdown, bFixShieldBug;
var vector PredictedLocation;
var TPawnLocation LocationHistory;
var bool bImprovedSlopeDodging;
var float MaxUnlagTime, LastLocationUpdateTime;
var UTP_PawnCollision UnlaggedCollision;

var float HealthDrainTime;
var int HealthDrain, ShieldDrain;
var bool bDrainingHealth;
var bool bContinousShield;


//=============================================================================
// Replication
//=============================================================================

replication
{
  unreliable if ( Role == ROLE_Authority )
    RepOverlay;
}


//== PostBeginPlay ============================================================
/**
Fix the completely weird implementation of PostBeginPlay() in xMutantPawn.
*/
//=============================================================================

simulated event PostBeginPlay()
{
  if ( Role == ROLE_Authority && Level.Game != None ) {
    GRI = Level.Game.GameReplicationInfo;
    MutantGRI = MutantGameReplicationInfo(GRI);
  }
  else {
    foreach DynamicActors(class'GameReplicationInfo', GRI)
      break;
  }
  
  foreach DynamicActors(class'UTP_ReplicationInfo', UTPlusRI)
    break;
  
  Super(xPawn).PostBeginPlay();
  
  MapSettings = class'UTP_MapSettings'.static.GetClientConfiguration(Self, string(Outer));
  
  if ( Level.NetMode != NM_DedicatedServer && Level.GetLocalPlayerController().bDemoOwner )
    bDemoPlayback = True;
  
  AllocateOverlayTextures();
}


//== Destroyed ================================================================
/**
Free the overlay materials.
*/
//=============================================================================

simulated event Destroyed()
{
  Super.Destroyed();
  if (UnlaggedCollision != None)
    UnlaggedCollision.Destroy();
  Skins.Length = 0;
  FreeTextures();
}


//== AllocateOverlayTextures ==================================================
/**
Create the overlay materials.
*/
//=============================================================================

simulated function AllocateOverlayTextures()
{
  if ( Level.NetMode != NM_DedicatedServer ) {
    // allocate overlay modifier materials
    
    SkinColorTeam = ConstantColor(Level.ObjectPool.AllocateObject(class'ConstantColor'));
    SkinColorTeam.Color.R          = 0;
    SkinColorTeam.Color.G          = 0;
    SkinColorTeam.Color.B          = 0;
    SkinColorTeam.Color.A          = 255;
    SkinColorTeam.FallbackMaterial = None;
    SkinColorTeam.SurfaceType      = EST_Default;
    
    SkinCombinerBody = Combiner(Level.ObjectPool.AllocateObject(class'Combiner'));
    SkinCombinerBody.CombineOperation  = CO_Add;
    SkinCombinerBody.AlphaOperation    = AO_Use_Alpha_From_Material2;
    SkinCombinerBody.Material1         = SkinColorTeam;
    SkinCombinerBody.Material2         = BodySkin;
    SkinCombinerBody.Mask              = None;
    SkinCombinerBody.InvertMask        = False;
    SkinCombinerBody.Modulate2X        = False;
    SkinCombinerBody.Modulate4X        = False;
    SkinCombinerBody.FallbackMaterial  = None;
    SkinCombinerBody.SurfaceType       = EST_Default;
    
    SkinCombinerHead = Combiner(Level.ObjectPool.AllocateObject(class'Combiner'));
    SkinCombinerHead.CombineOperation  = CO_Add;
    SkinCombinerHead.AlphaOperation    = AO_Use_Alpha_From_Material2;
    SkinCombinerHead.Material1         = SkinColorTeam;
    SkinCombinerHead.Material2         = FaceSkin;
    SkinCombinerHead.Mask              = None;
    SkinCombinerHead.InvertMask        = False;
    SkinCombinerHead.Modulate2X        = False;
    SkinCombinerHead.Modulate4X        = False;
    SkinCombinerHead.FallbackMaterial  = None;
    SkinCombinerHead.SurfaceType       = EST_Default;
    
    OverlayColorFade = ConstantColor(Level.ObjectPool.AllocateObject(class'ConstantColor'));
    OverlayColorFade.Color.R          = 0;
    OverlayColorFade.Color.G          = 0;
    OverlayColorFade.Color.B          = 0;
    OverlayColorFade.Color.A          = 0;
    OverlayColorFade.FallbackMaterial = None;
    OverlayColorFade.SurfaceType      = EST_Default;
    
    OverlayCombiner = Combiner(Level.ObjectPool.AllocateObject(class'Combiner'));
    OverlayCombiner.CombineOperation  = CO_AlphaBlend_With_Mask;
    OverlayCombiner.AlphaOperation    = AO_Use_Mask;
    OverlayCombiner.Material1         = OverlayColorFade;
    OverlayCombiner.Material2         = OverlayTexture;
    OverlayCombiner.Mask              = OverlayColorFade;
    OverlayCombiner.InvertMask        = False;
    OverlayCombiner.Modulate2X        = False;
    OverlayCombiner.Modulate4X        = False;
    OverlayCombiner.FallbackMaterial  = None;
    OverlayCombiner.SurfaceType       = EST_Default;
    
    // body skin shader
    SkinShaderBody = Shader(Level.ObjectPool.AllocateObject(class'Shader'));
    SkinShaderBody.Diffuse                       = SkinCombinerBody;
    SkinShaderBody.Opacity                       = None;
    SkinShaderBody.Specular                      = None;
    SkinShaderBody.SpecularityMask               = None;
    SkinShaderBody.SelfIllumination              = None;
    SkinShaderBody.SelfIlluminationMask          = None;
    SkinShaderBody.Detail                        = None;
    SkinShaderBody.OutputBlending                = OB_Normal;
    SkinShaderBody.TwoSided                      = True;
    SkinShaderBody.WireFrame                     = False;
    SkinShaderBody.PerformLightingOnSpecularPass = False;
    SkinShaderBody.ModulateSpecular2X            = False;
    SkinShaderBody.FallbackMaterial              = None;
    SkinShaderBody.SurfaceType                   = EST_Default;
    
    SkinFinalBody = FinalBlend(Level.ObjectPool.AllocateObject(class'FinalBlend'));
    SkinFinalBody.FrameBufferBlending = FB_Overwrite;
    SkinFinalBody.ZWrite              = True;
    SkinFinalBody.ZTest               = True;
    SkinFinalBody.AlphaTest           = False;
    SkinFinalBody.TwoSided            = True;
    SkinFinalBody.AlphaRef            = 0;
    SkinFinalBody.Material            = SkinShaderBody;
    SkinFinalBody.FallbackMaterial    = SkinShaderBody;
    SkinFinalBody.SurfaceType         = EST_Default;
    
    // head skin shader
    SkinShaderHead = Shader(Level.ObjectPool.AllocateObject(class'Shader'));
    SkinShaderHead.Diffuse                       = SkinCombinerHead;
    SkinShaderHead.Opacity                       = None;
    SkinShaderHead.Specular                      = None;
    SkinShaderHead.SpecularityMask               = None;
    SkinShaderHead.SelfIllumination              = None;
    SkinShaderHead.SelfIlluminationMask          = None;
    SkinShaderHead.Detail                        = None;
    SkinShaderHead.OutputBlending                = OB_Normal;
    SkinShaderHead.TwoSided                      = True;
    SkinShaderHead.WireFrame                     = False;
    SkinShaderHead.PerformLightingOnSpecularPass = False;
    SkinShaderHead.ModulateSpecular2X            = False;
    SkinShaderHead.FallbackMaterial              = None;
    SkinShaderHead.SurfaceType                   = EST_Default;
    
    SkinFinalHead = FinalBlend(Level.ObjectPool.AllocateObject(class'FinalBlend'));
    SkinFinalHead.FrameBufferBlending = FB_Overwrite;
    SkinFinalHead.ZWrite              = True;
    SkinFinalHead.ZTest               = True;
    SkinFinalHead.AlphaTest           = False;
    SkinFinalHead.TwoSided            = True;
    SkinFinalHead.AlphaRef            = 0;
    SkinFinalHead.Material            = SkinShaderHead;
    SkinFinalHead.FallbackMaterial    = SkinShaderHead;
    SkinFinalHead.SurfaceType         = EST_Default;
  }
}


//== AllocateDeResTextures ====================================================
/**
Create the DeRes materials.
*/
//=============================================================================

simulated function AllocateDeResTextures()
{
  local PlayerController LocalController;
  local bool bLocalSpectator, bOnLocalTeam;
  
  if ( Level.NetMode != NM_DedicatedServer ) {
    if ( FinalBlend(default.DeResMat0) != None ) 
      default.DeResMat0 = Material(DynamicLoadObject(DeResBaseBody, class'Material'));
    if ( FinalBlend(default.DeResMat1) != None ) 
      default.DeResMat1 = Material(DynamicLoadObject(DeResBaseHead, class'Material'));
    
    if ( UTPlusRI != None && UTPlusRI.bAllowDeRes && class'UTP_ClientSettings'.default.bTeamDeRes ) {
      LocalController = Level.GetLocalPlayerController();
      bLocalSpectator = LocalController.PlayerReplicationInfo != None && LocalController.PlayerReplicationInfo.bOnlySpectator;
      
      if ( OldController != None ) {
        bOnLocalTeam = LocalController == OldController;
        if ( LocalController.PlayerReplicationInfo != None && LocalController.PlayerReplicationInfo.Team != None )
          bOnLocalTeam = bOnLocalTeam || LocalController.PlayerReplicationInfo.Team.TeamIndex == RealTeam;
      }
      
      if ( bOnLocalTeam ) {
        if ( MapSettings.ForcedTeamSkinColor == 254 || bDemoPlayback && class'UTP_DemoPlaybackSettings'.default.bForceRegularModels )
          DeResColor = class'UTP_Util'.static.ToColor(MapSettings.TeamColors[Min(RealTeam, ArrayCount(MapSettings.TeamColors) - 1)]);
        else
          DeResColor = class'UTP_Util'.static.ToColor(MapSettings.TeamColorFriend);
      }
      else {
        if ( MapSettings.ForcedEnemySkinColor == 254 || bDemoPlayback && class'UTP_DemoPlaybackSettings'.default.bForceRegularModels )
          DeResColor = class'UTP_Util'.static.ToColor(MapSettings.TeamColors[Min(RealTeam, ArrayCount(MapSettings.TeamColors) - 1)]);
        else
          DeResColor = class'UTP_Util'.static.ToColor(MapSettings.TeamColorEnemy);
      }
      DeResColor = 4 * DeResColor;
    }
    
    if ( DeResColor.R < 128 && DeResColor.G < 128 && DeResColor.B < 128 ) {
      DeResColor.R = 0;
      DeResColor.G = 255;
      DeResColor.B = 0;
    }
    DeResColorBody = ColorModifier(Level.ObjectPool.AllocateObject(class'ColorModifier'));
    DeResColorBody.Material       = default.DeResMat0;
    DeResColorBody.Color          = DeResColor;
    DeResColorBody.RenderTwoSided = True;
    DeResColorBody.AlphaBlend     = True;
    
    DeResColorHead = ColorModifier(Level.ObjectPool.AllocateObject(class'ColorModifier'));
    DeResColorHead.Material       = default.DeResMat1;
    DeResColorHead.Color          = DeResColor;
    DeResColorHead.RenderTwoSided = True;
    DeResColorHead.AlphaBlend     = True;
    
    DeResFinalBody = FinalBlend(Level.ObjectPool.AllocateObject(class'FinalBlend'));
    DeResFinalBody.Material             = DeResColorBody;
    DeResFinalBody.FallbackMaterial     = DeResColorBody;
    DeResFinalBody.TwoSided             = True;
    DeResFinalBody.ZWrite               = True;
    DeResFinalBody.ZTest                = True;
    DeResFinalBody.AlphaTest            = False;
    DeResFinalBody.AlphaRef             = 0;
    DeResFinalBody.FrameBufferBlending  = FB_Brighten;
    
    DeResFinalHead = FinalBlend(Level.ObjectPool.AllocateObject(class'FinalBlend'));
    DeResFinalHead.Material             = DeResColorHead;
    DeResFinalHead.FallbackMaterial     = DeResColorHead;
    DeResFinalHead.TwoSided             = True;
    DeResFinalHead.ZWrite               = True;
    DeResFinalHead.ZTest                = True;
    DeResFinalHead.AlphaTest            = False;
    DeResFinalHead.AlphaRef             = 0;
    DeResFinalHead.FrameBufferBlending  = FB_Brighten;
  }
}


//== FreeTextures =============================================================
/**
Free the overlay and DeRes materials.
*/
//=============================================================================

simulated function FreeTextures()
{
  // overlays
  
  // body shader
  if ( SkinFinalBody != None ) {
    SkinFinalBody.Material         = None;
    SkinFinalBody.FallbackMaterial = None;
    Level.ObjectPool.FreeObject(SkinFinalBody);
    SkinFinalBody = None;
  }
  if ( SkinShaderBody != None ) {
    SkinShaderBody.Diffuse   = None;
    SkinShaderBody.Opacity   = None;
    SkinShaderBody.Specular  = None;
    Level.ObjectPool.FreeObject(SkinShaderBody);
    SkinShaderBody = None;
  }
  
  // head shader
  if ( SkinFinalHead != None ) {
    SkinFinalHead.Material         = None;
    SkinFinalHead.FallbackMaterial = None;
    Level.ObjectPool.FreeObject(SkinFinalHead);
    SkinFinalHead = None;
  }
  if ( SkinShaderHead != None ) {
    SkinShaderHead.Diffuse   = None;
    SkinShaderHead.Opacity   = None;
    SkinShaderHead.Specular  = None;
    Level.ObjectPool.FreeObject(SkinShaderHead);
    SkinShaderHead = None;
  }
  
  // overlay combiner
  if ( OverlayCombiner != None ) {
    OverlayCombiner.Material1         = None;
    OverlayCombiner.Material2         = None;
    OverlayCombiner.Mask              = None;
    OverlayCombiner.FallbackMaterial  = None;
    Level.ObjectPool.FreeObject(OverlayCombiner);
    OverlayCombiner = None;
  }
  if ( OverlayColorFade != None ) {
    Level.ObjectPool.FreeObject(OverlayColorFade);
    OverlayColorFade = None;
  }
  
  // body combiner
  if ( SkinCombinerBody != None ) {
    SkinCombinerBody.Material1         = None;
    SkinCombinerBody.Material2         = None;
    SkinCombinerBody.Mask              = None;
    SkinCombinerBody.FallbackMaterial  = None;
    Level.ObjectPool.FreeObject(SkinCombinerBody);
    SkinCombinerBody = None;
  }
  
  // head combiner
  if ( SkinCombinerHead != None ) {
    SkinCombinerHead.Material1         = None;
    SkinCombinerHead.Material2         = None;
    SkinCombinerHead.Mask              = None;
    SkinCombinerHead.FallbackMaterial  = None;
    Level.ObjectPool.FreeObject(SkinCombinerHead);
    SkinCombinerHead = None;
  }
  if ( SkinColorTeam != None ) {
    Level.ObjectPool.FreeObject(SkinColorTeam);
    SkinColorTeam = None;
  }
  
  
  // DeRes
  if ( DeResFinalBody != None ) {
    DeResFinalBody.Material = None;
    DeResFinalBody.FallbackMaterial = None;
    Level.ObjectPool.FreeObject(DeResFinalBody);
    DeResFinalBody = None;
  }
  if ( DeResFinalHead != None ) {
    DeResFinalHead.Material = None;
    DeResFinalHead.FallbackMaterial = None;
    Level.ObjectPool.FreeObject(DeResFinalHead);
    DeResFinalHead = None;
  }
  if ( DeResColorBody != None ) {
    DeResColorBody.Material = None;
    Level.ObjectPool.FreeObject(DeResColorBody);
    DeResColorBody = None;
  }
  if ( DeResColorHead != None ) {
    DeResColorHead.Material = None;
    Level.ObjectPool.FreeObject(DeResColorHead);
    DeResColorHead = None;
  }
}


//== ForceDefaultCharacter ====================================================
/**
Checks, whether a default model should be forced for this pawn.
*/
//=============================================================================

simulated function bool ForceDefaultCharacter()
{
  local PlayerController LocalPlayer;
  local PlayerReplicationInfo MyPRI;
  local bool bIsLocalPlayer, bOnLocalTeam;
  local xUtil.PlayerRecord Rec;
  
  if ( Level.NetMode == NM_DedicatedServer || bDemoPlayback && class'UTP_DemoPlaybackSettings'.default.bForceRegularModels )
    return false;
  
  if ( PlayerReplicationInfo == None ) {
    if ( DrivenVehicle == None || DrivenVehicle.PlayerReplicationInfo == None )
      return false;
    else
      MyPRI = DrivenVehicle.PlayerReplicationInfo;
  }
  else
    MyPRI = PlayerReplicationInfo;
  
  // validate and use player's model for enemies
  LocalPlayer = Level.GetLocalPlayerController();
  if ( LocalPlayer != None && LocalPlayer.PlayerReplicationInfo != None && !LocalPlayer.PlayerReplicationInfo.bOnlySpectator ) {
    bIsLocalPlayer = MyPRI == LocalPlayer.PlayerReplicationInfo;
    if ( GRI != None && GRI.bTeamGame && MyPRI.Team != None && MyPRI.Team.TeamIndex != 255 )
      bOnLocalTeam = LocalPlayer.PlayerReplicationInfo.Team != None && LocalPlayer.PlayerReplicationInfo.Team.TeamIndex != 255 && LocalPlayer.PlayerReplicationInfo.Team.TeamIndex == MyPRI.Team.TeamIndex;
  }
  else
    return false;
  
  if ( bOnLocalTeam && !bIsLocalPlayer ) {
    if ( MapSettings.bForceTeamModel ) {
      PlacedCharacterName = MapSettings.ForcedTeamModel;
      Rec = class'xUtil'.static.FindPlayerRecord(PlacedCharacterName);
      if ( Rec.Species == None ) {
        MapSettings.ClearConfig("ForcedTeamModel");
        PlacedCharacterName = MapSettings.ForcedTeamModel;
      }
      return true;
    }
  }
  else {
    if ( MapSettings.bForceEnemyModel ) {
      PlacedCharacterName = GetURLOption("Character");
      Rec = class'xUtil'.static.FindPlayerRecord(PlacedCharacterName);
      if ( Rec.Species == None ) {
        PlacedCharacterName = MapSettings.ForcedEnemyModel;
        Rec = class'xUtil'.static.FindPlayerRecord(PlacedCharacterName);
        if ( Rec.Species == None ) {
          class'UTP_ClientSettings'.static.StaticClearConfig("ForcedEnemyModel");
          PlacedCharacterName = MapSettings.ForcedEnemyModel;
        }
      }
      return true;
    }
  }
  
  return false;
}


//== GetDefaultCharacter ======================================================
/**
Returns the default character name for this pawn.
*/
//=============================================================================

simulated function string GetDefaultCharacter()
{
  if ( Level.IsDemoBuild() )
    PlacedCharacterName = "Jakob";
  
  return PlacedCharacterName;
}


//== Setup ====================================================================
/**
Set up mesh, skins, etc.
*/
//=============================================================================

simulated function Setup(xUtil.PlayerRecord rec, optional bool bLoadNow)
{
  if ( rec.Species == None || ForceDefaultCharacter() )
    rec = class'xUtil'.static.FindPlayerRecord(GetDefaultCharacter());
  
  PlayerRecord = rec;
  Species = PlayerRecord.Species;
  RagdollOverride = PlayerRecord.Ragdoll;
  if ( !RealSetup() ) {
    PlayerRecord = class'xUtil'.static.FindPlayerRecord(GetDefaultCharacter());
    if ( !RealSetup() )
      return;
  }
  ResetPhysicsBasedAnim();
}


//== RealSetup ================================================================
/**
Apply the specified team skins.
*/
//=============================================================================

simulated function bool RealSetup()
{
  local Mesh NewMesh, CustomSkel;
  local string VoicePackName, SkelName;
  local class<VoicePack> VoicePackClass;
  local int TeamNum, i, j;
  
  if ( bAlreadySetup ) {
    // make sure correct teamskin
    if ( Level.NetMode == NM_Client ) {
      if ( PlayerReplicationInfo != None && PlayerReplicationInfo.Team != None )
        TeamNum = PlayerReplicationInfo.Team.TeamIndex;
      else if ( DrivenVehicle != None && DrivenVehicle.PlayerReplicationInfo != None && DrivenVehicle.PlayerReplicationInfo.Team != None )
        TeamNum = DrivenVehicle.PlayerReplicationInfo.Team.TeamIndex;
      if ( TeamSkin == TeamNum && bBrightSkins == class'DMMutator'.default.bBrightSkins )
        return true;
      
      SetTeamSkin(TeamNum);
    }		
    return true;
  }
  NewMesh = Mesh(DynamicLoadObject(PlayerRecord.MeshName, class'Mesh'));
  if ( NewMesh == None ) {
    log("Failed to load player mesh" @ PlayerRecord.MeshName);
    return false;
  }
  
  bAlreadySetup = true;
  LinkMesh(NewMesh);
  AssignInitialPose();
  
  bIsFemale = PlayerRecord.Sex ~= "Female";
  if ( PlayerReplicationInfo != None )
    PlayerReplicationInfo.bIsFemale = bIsFemale;
  if ( Level.NetMode != NM_DedicatedServer && PlayerRecord.Skeleton != "" )
    CustomSkel = Mesh(DynamicLoadObject(PlayerRecord.Skeleton, class'Mesh'));
  
  if ( bIsFemale ) {
    SkelName = Species.Default.FemaleSkeleton;
    if ( Level.bLowSoundDetail )
      SoundGroupClass = class<xPawnSoundGroup>(DynamicLoadObject("XGame.xJuggFemaleSoundGroup", class'Class'));
    else
      SoundGroupClass = class<xPawnSoundGroup>(DynamicLoadObject(Species.Default.FemaleSoundGroup, class'Class'));
  }
  else {
    SkelName = Species.Default.MaleSkeleton;
    if ( Level.bLowSoundDetail )
      SoundGroupClass = class<xPawnSoundGroup>(DynamicLoadObject("XGame.xJuggMaleSoundGroup", class'Class'));
    else
      SoundGroupClass = class<xPawnSoundGroup>(DynamicLoadObject(Species.Default.MaleSoundGroup, class'Class'));
  }
  
  if ( Level.NetMode != NM_DedicatedServer ) {
    if ( CustomSkel != None )
      SkeletonMesh = CustomSkel;
    else if ( SkelName != "" )
      SkeletonMesh = Mesh(DynamicLoadObject(SkelName, class'Mesh'));
    
    TeamNum = 255;
    if ( PlayerReplicationInfo != None && PlayerReplicationInfo.Team != None )
      TeamNum = PlayerReplicationInfo.Team.TeamIndex;
    else if ( DrivenVehicle != None && DrivenVehicle.PlayerReplicationInfo != None && DrivenVehicle.PlayerReplicationInfo.Team != None )
      TeamNum = DrivenVehicle.PlayerReplicationInfo.Team.TeamIndex;
    else if ( Level.GRI != None && Level.GRI.bForceTeamSkins )
      TeamNum = Species.Default.DMTeam;
    
    SetTeamSkin(TeamNum);
  }
  GibGroupClass = class<xPawnGibGroup>(DynamicLoadObject(Species.Default.GibGroup, class'Class'));
  
  if ( Level.NetMode == NM_DedicatedServer ) {
    if ( PlayerRecord.Sex ~= "Female" )
      VoicePackName = "XGame.JuggFemaleVoice";
    else
      VoicePackName = "XGame.JuggMaleVoice";
    VoicePackClass = class<VoicePack>(DynamicLoadObject(VoicePackName, class'Class'));
    VoiceType = VoicePackName;
    if ( PlayerReplicationInfo != None )
      PlayerReplicationInfo.VoiceType = VoicePackClass;
    VoiceClass = class<TeamVoicePack>(VoicePackClass);
  }
  else {
    if ( !Level.bLowSoundDetail ) {
      if ( PlayerReplicationInfo != None && PlayerReplicationInfo.VoiceTypeName != "" )
        VoicePackName = PlayerReplicationInfo.VoiceTypeName;
      else
        VoicePackName = PlayerRecord.VoiceClassName;
      if ( VoicePackName != "" )
        VoicePackClass = class<VoicePack>(DynamicLoadObject(VoicePackName, class'Class'));
    }
    if ( VoicePackClass == None ) {
      VoicePackName = Species.static.GetVoiceType(bIsFemale, Level);
      VoicePackClass = class<VoicePack>(DynamicLoadObject(VoicePackName, class'Class'));
    }
    VoiceType = VoicePackName;
    if ( PlayerReplicationInfo != None )
      PlayerReplicationInfo.VoiceType = VoicePackClass;
    VoiceClass = class<TeamVoicePack>(VoicePackClass);
  }
  
  // add unique taunts
  for (i = 0; i < 16; ++i) {
    if ( Species.Default.TauntAnims[i] != '' ) {
      j = TauntAnims.Length;
      TauntAnims[j] = Species.Default.TauntAnims[i];
      TauntAnimNames[j] = Species.Default.TauntAnimNames[i];
      if ( j == 15 )
        break;
    }
  }
  return true;
}


//== SetTeamSkin ==============================================================
/**
Cet the player's skins for the specified team.
*/
//=============================================================================

simulated function SetTeamSkin(int TeamNum)
{
  local bool bBodyAlpha, bHeadAlpha;
  
  TeamSkin = TeamNum;
  bClearWeaponOffsets = PlayerRecord.ZeroWeaponOffsets;
  
  if ( TeamNum == 0 )
    Texture = Texture'RedMarker_t';
  else
    Texture = Texture'BlueMarker_t';
  
  // load the skin materials
  GetTeamSkin(TeamNum, BodySkin, FaceSkin);
  
  // set up the body skin shader
  bBodyAlpha = IsAlphaTexture(BodySkin);
  if ( SkinCombinerBody != None ) {
    SkinCombinerBody.Material2 = GetSkinTexture(BodySkin);
    if ( SkinShaderBody != None ) {
      if ( bBodyAlpha )
        SkinShaderBody.Opacity = GetSkinTexture(BodySkin);
      else
        SkinShaderBody.Opacity = None;
    }
  }
  if ( SkinFinalBody != None ) {
    if ( bBodyAlpha )
      SkinFinalBody.FrameBufferBlending = FB_AlphaBlend;
    else
      SkinFinalBody.FrameBufferBlending = FB_Overwrite;
  }
  
  // set up the head skin shader
  bHeadAlpha = IsAlphaTexture(FaceSkin);
  if ( SkinCombinerHead != None ) {
    SkinCombinerHead.Material2 = GetSkinTexture(FaceSkin);
    if ( SkinShaderHead != None ) {
      if ( bHeadAlpha )
        SkinShaderHead.Opacity = GetSkinTexture(FaceSkin);
      else
        SkinShaderHead.Opacity = None;
    }
  }
  if ( SkinFinalHead != None ) {
    if ( bHeadAlpha )
      SkinFinalHead.FrameBufferBlending = FB_AlphaBlend;
    else
      SkinFinalHead.FrameBufferBlending = FB_Overwrite;
  }
  if ( SkinShaderHead != None ) {
    if ( PlayerRecord.TeamFace && SkinCombinerHead != None )
      SkinShaderHead.Diffuse = SkinCombinerHead;
    else
      SkinShaderHead.Diffuse = GetSkinTexture(FaceSkin);
  }
  
  // set the mesh skins
  if ( UTPlusRI != None && UTPlusRI.bAllowColors && MapSettings.bUseSkinOverlay ) {
    Skins[0] = SkinFinalBody;
    Skins[1] = SkinFinalHead;
  }
  else {
    Skins[0] = BodySkin;
    Skins[1] = FaceSkin;
  }
}


//== GetTeamSkin ==============================================================
/**
Get the character's skins for the specified team.
*/
//=============================================================================

simulated function GetTeamSkin(int TeamNum, out Material NewBodySkin, out Material NewFaceSkin)
{
  local string BodySkinName, FaceSkinName;
  local Material TeamFaceSkin;
  
  NewFaceSkin = Material(DynamicLoadObject(PlayerRecord.FaceSkinName, class'Material'));
  NewBodySkin = None;
  
  if ( TeamNum != 255 && (Level.GRI == None || !Level.GRI.bNoTeamSkins) ) {
    if ( class'DMMutator'.Default.bBrightSkins && (Left(PlayerRecord.BodySkinName, 12) ~= "PlayerSkins.") ) {
      BodySkinName = "Bright" $ PlayerRecord.BodySkinName $ "_" $ TeamNum $ "B";
      NewBodySkin = Material(DynamicLoadObject(BodySkinName, class'Material',true));
    }
    if ( NewBodySkin == None ) {
      BodySkinName = PlayerRecord.BodySkinName $ "_" $ TeamNum;
      NewBodySkin = Material(DynamicLoadObject(BodySkinName, class'Material'));
      
      // allow team head skins with new skins
      if ( PlayerRecord.TeamFace ) {
        FaceSkinName = PlayerRecord.FaceSkinName $ "_" $ TeamNum;
        TeamFaceSkin = Material(DynamicLoadObject(FaceSkinName, class'Material'));
        if ( TeamFaceSkin != None )
          NewFaceSkin = TeamFaceSkin;
      }
    }
    if ( NewBodySkin == None ) {
      log("TeamSkin not found" @ NewBodySkin @ "for" @ Mesh);
      NewBodySkin = Material(DynamicLoadObject(PlayerRecord.BodySkinName, class'Material'));
    }
  }
  else
    NewBodySkin = Material(DynamicLoadObject(PlayerRecord.BodySkinName, class'Material'));
}


//== SetOverlayMaterial =======================================================
/**
Cool new bright skin overlay code!
*/
//=============================================================================

simulated function SetOverlayMaterial(Material NewOverlay, float Time, bool bOverride)
{
  local float CurrentTimeIndex;
  
  //log(UTPlusRI.GetCurrentTimeIndex(True)@"SetOverlayMaterial"@NewOverlay@Time@bOverride, Name);
  if ( UTPlusRI != None && UTPlusRI.bAllowColors ) {
    CurrentTimeIndex = UTPlusRI.GetCurrentTimeIndex(True);
    if ( RepOverlay.Material == None || RepOverlay.EndTime <= CurrentTimeIndex || RepOverlay.Material == NewOverlay || bOverride ) {
      RepOverlay.Material   = NewOverlay;
      RepOverlay.Duration   = Time;
      RepOverlay.EndTime    = CurrentTimeIndex + Time * Level.TimeDilation;
      RepOverlay = RepOverlay;    // force replication
      NetUpdateTime = Level.TimeSeconds - 1;
    }
  }
  else
    Super.SetOverlayMaterial(NewOverlay, Time, bOverride);
}


//== TickFX ===================================================================
/**
Swap out the InvisMaterial if neccessary.
*/
//=============================================================================

simulated function TickFX(float DeltaTime)
{
  local PlayerController LocalController;
  local PlayerReplicationInfo LocalPRI;
  local Pawn TargetPawn;
  local bool bOnLocalTeam, bLocalSpectator;
  local float CurrentTimeIndex;
  
  LocalController = Level.GetLocalPlayerController();
  bLocalSpectator = LocalController.PlayerReplicationInfo != None && LocalController.PlayerReplicationInfo.bOnlySpectator || LocalController.IsA('DemoRecSpectator');
  
  if ( LocalController.bDemoOwner ) {
    if ( Pawn(LocalController.ViewTarget) != None ) {
      TargetPawn = Pawn(LocalController.ViewTarget);
    }
    else if ( LocalController.ViewTarget.Instigator != None ) {
      TargetPawn = LocalController.ViewTarget.Instigator;
    }
    else if ( Pawn(LocalController.ViewTarget.Owner) != None ) {
      TargetPawn = Pawn(LocalController.ViewTarget.Owner);
    }
  }
  if ( TargetPawn == None )
    LocalPRI = LocalController.PlayerReplicationInfo;
  else
    LocalPRI = TargetPawn.PlayerReplicationInfo;
  
  bOnLocalTeam = LocalPRI == PlayerReplicationInfo;
  
  // find GRI
  if ( GRI == None ) {
    if ( MutantGRI != None )
      GRI = MutantGRI;
    else {
      foreach DynamicActors(class'GameReplicationInfo', GRI)
        break;
    }
  }
  
  // find real team index
  if ( PlayerReplicationInfo != None ) {
    if ( PlayerReplicationInfo.Team != None ) {
      RealTeam = PlayerReplicationInfo.Team.TeamIndex;
      bOnLocalTeam = bOnLocalTeam || PlayerReplicationInfo.Team == LocalPRI.Team;
    }
    else if ( PlayerReplicationInfo.Team == None ) {
      if ( GRI != None && GRI.bForceTeamSkins && Species != None )
        RealTeam = Species.default.DMTeam;
      else
        RealTeam = 255;
    }
    if ( Controller(PlayerReplicationInfo.Owner) != None && OldController == None )
      OldController = Controller(PlayerReplicationInfo.Owner);
  }
  
  // xMutantPawn.TickFX() stuff
  if ( MutantGRI != None ) {
    if ( InvisMaterial != MutantInvisMaterial )
      InvisMaterial = MutantInvisMaterial;
    
    // See if there is a mutant, and this is the mutant pawn
    bMutantEffect = MutantGRI.MutantPRI != None && MutantGRI.MutantPRI == PlayerReplicationInfo;
    bScriptPostRender = MutantGRI != None && PlayerReplicationInfo == MutantGRI.BottomFeederPRI;
    
    if( bMutantEffect && !bOldMutantEffect ) {
      // Spawn funky glowy effect
      MutantFX = Spawn(class'BonusPack.MutantGlow', self, , Location);
      if ( MutantFX != None ) {
        MutantFX.Emitters[0].SkeletalMeshActor = self;
        MutantFX.SetLocation(Location - vect(0, 0, 49));
        MutantFX.SetRotation(Rotation + rot(0, -16384, 0));
        MutantFX.SetBase(self);
      }
    }
    else if ( !bMutantEffect && bOldMutantEffect ) {
      // Remove funky glowy effect
      if ( MutantFX != None ) {
        MutantFX.Emitters[0].SkeletalMeshActor = None;
        MutantFX.Kill();
        MutantFX = None;
      }
    }
    
    bOldMutantEffect = bMutantEffect;
  }
  else if ( InvisMaterial != default.InvisMaterial ) {
    InvisMaterial = default.InvisMaterial;
  }
  
  // set desired player skins
  if ( Level.NetMode != NM_DedicatedServer && !bInvis && Species != None && xPlayerReplicationInfo(PlayerReplicationInfo) != None && bAlreadySetup && LocalController != None && !LocalController.IsInState('GameEnded') ) {
    if ( UTPlusRI != None && UTPlusRI.bAllowColors && !bLocalSpectator ) {
      if ( bOnLocalTeam ) {
        if ( (MapSettings.ForcedTeamSkinColor == 254 || bDemoPlayback && class'UTP_DemoPlaybackSettings'.default.bForceRegularModels) && (TeamSkin != RealTeam || bBrightSkins != class'DMMutator'.default.bBrightSkins) )
          SetTeamSkin(RealTeam);
        else if ( MapSettings.ForcedTeamSkinColor != 254 && (!bDemoPlayback || !class'UTP_DemoPlaybackSettings'.default.bForceRegularModels) && (TeamSkin != MapSettings.ForcedTeamSkinColor || bBrightSkins != class'DMMutator'.default.bBrightSkins) )
          SetTeamSkin(MapSettings.ForcedTeamSkinColor);
      }
      else {
        if ( (MapSettings.ForcedEnemySkinColor == 254 || bDemoPlayback && class'UTP_DemoPlaybackSettings'.default.bForceRegularModels) && (TeamSkin != RealTeam || bBrightSkins != class'DMMutator'.default.bBrightSkins) )
          SetTeamSkin(RealTeam);
        else if ( MapSettings.ForcedEnemySkinColor != 254 && (!bDemoPlayback || !class'UTP_DemoPlaybackSettings'.default.bForceRegularModels) && (TeamSkin != MapSettings.ForcedEnemySkinColor || bBrightSkins != class'DMMutator'.default.bBrightSkins) )
          SetTeamSkin(MapSettings.ForcedEnemySkinColor);
      }
    }
    else {
      if ( TeamSkin != RealTeam || bBrightSkins != class'DMMutator'.default.bBrightSkins )
        SetTeamSkin(RealTeam);
    }
    
    // set overlay color to desired team color
    if ( UTPlusRI != None && UTPlusRI.bAllowColors && MapSettings.bUseSkinOverlay && SkinColorTeam != None ) {
      if ( bOnLocalTeam ) {
        if ( bLocalSpectator || MapSettings.ForcedTeamSkinColor == 254 || bDemoPlayback && class'UTP_DemoPlaybackSettings'.default.bForceRegularModels )
          SkinColorTeam.Color = class'UTP_Util'.static.ToColor(MapSettings.TeamColors[Min(RealTeam, ArrayCount(MapSettings.TeamColors) - 1)]);
        else
          SkinColorTeam.Color = class'UTP_Util'.static.ToColor(MapSettings.TeamColorFriend);
      }
      else {
        if ( bLocalSpectator || MapSettings.ForcedEnemySkinColor == 254 || bDemoPlayback && class'UTP_DemoPlaybackSettings'.default.bForceRegularModels )
          SkinColorTeam.Color = class'UTP_Util'.static.ToColor(MapSettings.TeamColors[Min(RealTeam, ArrayCount(MapSettings.TeamColors) - 1)]);
        else
          SkinColorTeam.Color = class'UTP_Util'.static.ToColor(MapSettings.TeamColorEnemy);
      }
    }
    
    bBrightSkins = class'DMMutator'.default.bBrightSkins;
  }
  
  Super(xPawn).TickFX(DeltaTime);
  
  // set skin overlay
  if ( Level.NetMode != NM_DedicatedServer && !bDeRes && !bInvis && Health > 0 && UTPlusRI != None && UTPlusRI.bAllowColors && MapSettings.bUseSkinOverlay ) {
    
    CurrentTimeIndex = UTPlusRI.GetCurrentTimeIndex(True);
    
    // make sure the OverlayMaterial is empty - we handle the overlay ourselves
    if ( OverlayMaterial != None ) {
      //log("Setting OverlayShader" @ OverlayShader @ ClientOverlayCounter @ ClientOverlayTimer @ OverlayTimer);
      OverlayMaterial       = None;
      OverlayTimer          = 0.0;
      ClientOverlayTimer    = 0.0;
      ClientOverlayCounter  = 0.0;
    }
    if ( HighDetailOverlay != None )
      HighDetailOverlay = None;
    
    // overlay changed
    if ( LastRepOverlay != RepOverlay ) {
      LastRepOverlay = RepOverlay;
      //log(CurrentTimeIndex@"New overlay:"@RepOverlay.Material@RepOverlay.Duration@RepOverlay.EndTime, Name);
      OverlayTexture = RepOverlay.Material;
      if ( OverlayTexture != None && RepOverlay.EndTime > CurrentTimeIndex ) {
        OverlayEndTime  = RepOverlay.EndTime;
        OverlayDuration = RepOverlay.Duration;
        OverlayCombiner.Material2 = GetOverlayTexture();
        SkinShaderBody.Specular = OverlayCombiner;
        SkinShaderHead.Specular = OverlayCombiner;
      }
      else {
        OverlayEndTime = CurrentTimeIndex;
        OverlayDuration = 0;
        SkinShaderBody.Specular = None;
        SkinShaderHead.Specular = None;
      }
    }
    
    // if an additional overlay is active...
    if ( OverlayEndTime > CurrentTimeIndex ) {
      if ( OverlayDuration < 1 )
        OverlayColorFade.Color.A = FMin((OverlayEndTime - CurrentTimeIndex) * 255 / OverlayDuration, 255);
      else
        OverlayColorFade.Color.A = FMin((OverlayEndTime - CurrentTimeIndex) * 255, 255);
    }
    else if ( SkinShaderBody != None && SkinShaderBody.Specular != None ) {
      //log(CurrentTimeIndex@"Overlay timed out:"@OverlayCombiner.Material2, Name);
      SkinShaderBody.Specular = None;
      SkinShaderHead.Specular = None;
    }
  }
  else {
    if ( LastRepOverlay != RepOverlay ) {
      LastRepOverlay = RepOverlay;
      //log(UTPlusRI.GetCurrentTimeIndex(True)@"New overlay:"@RepOverlay.Material@RepOverlay.Duration@"(alternative)", Name);
      OverlayTexture    = RepOverlay.Material;
      OverlayDuration   = RepOverlay.Duration;
      OverlayEndTime    = RepOverlay.EndTime;
      if ( OverlayTimer == OverlayDuration )
        OverlayTimer = OverlayDuration + 0.001;
      else
        OverlayTimer = OverlayDuration;
      OverlayMaterial       = OverlayTexture;
      ClientOverlayTimer    = OverlayTimer;
      ClientOverlayCounter  = OverlayTimer;
    }
    if ( PlayerRecord.UseSpecular && Level.DetailMode != DM_Low && HighDetailOverlay == None ) {
      HighDetailOverlay = Material'UT2004Weapons.WeaponShader';
    }
  }
  
  // set skin glow
  if ( !bInvis && !bDeRes && Health > 0 && UTPlusRI != None && UTPlusRI.SkinGlow != 0 )
    AmbientGlow = Clamp(UTPlusRI.SkinGlow, 40, 128);
}


//== TickDeRes ================================================================
/**
Returns the texture of the overlay.
*/
//=============================================================================

simulated function TickDeRes(float DeltaTime)
{
  if ( LifeSpan < 3.0 ) {
    AmbientGlow = byte(254.0 * (LifeSpan / 3.0)); // Scale down over time.
    if ( DeResColorBody != None ) {
      DeResColorBody.Color = DeResColor * (LifeSpan / 3.0);
    }
    if ( DeResColorHead != None ) {
      DeResColorHead.Color = DeResColor * (LifeSpan / 3.0);
    }
  }
}


//== GetOverlayTexture ========================================================
/**
Returns the texture of the overlay.
*/
//=============================================================================

simulated final function Material GetOverlayTexture(optional Material Material)
{
  if ( Material == None )
    Material = OverlayTexture;
  
  if ( Shader(Material) != None ) {
    if ( Shader(Material).Specular != None )
      Material = GetOverlayTexture(Shader(Material).Specular);
    else if ( Shader(Material).Diffuse != None )
      Material = GetOverlayTexture(Shader(Material).Diffuse);
  }
  else if ( FinalBlend(Material) != None )
    Material = GetOverlayTexture(FinalBlend(Material).Material);
  
  if ( Material != None )
    return Material;
  else
    return default.OverlayTexture;
}


//== GetSkinTexture ===========================================================
/**
Returns the actual skin texture without additional modifiers.
*/
//=============================================================================

static final function Material GetSkinTexture(Material Material)
{
  if ( Shader(Material) != None )
    Material = GetSkinTexture(Shader(Material).Diffuse);
  else if ( FinalBlend(Material) != None )
    Material = GetSkinTexture(FinalBlend(Material).Material);
  
  return Material;
}


//== IsAlphaTexture ===========================================================
/**
Returns, whether the material has an active alpha channel.
*/
//=============================================================================

static final function bool IsAlphaTexture(Material Material)
{
  if ( Material == None )
    return false;
  if ( Texture(Material) != None )
    return Texture(Material).bAlphaTexture;
  if ( Shader(Material) != None )
    return Shader(Material).Opacity != None;
  if ( FinalBlend(Material) != None )
    return FinalBlend(Material).FrameBufferBlending == FB_AlphaBlend;
  
  return false;
}


//== StartDeRes ===============================================================
/**
Start the DeRes effect.
*/
//=============================================================================

simulated function StartDeRes()
{
  local KarmaParamsSkel skelParams;
  local int i;
  
  if ( Level.NetMode == NM_DedicatedServer )
    return;
  
  if ( UTPlusRI != None && UTPlusRI.bAllowDeRes )
    AllocateDeResTextures();
  AmbientGlow = 254;
  MaxLights   = 0;
  
  DeResFX = Spawn(class'DeResPart', self,, Location);
  if ( DeResFX != None ) {
    DeResFX.Emitters[0].SkeletalMeshActor = self;
    if ( UTPlusRI != None && UTPlusRI.bAllowDeRes ) {
      DeResFX.Emitters[0].ColorMultiplierRange.Z.Min = 1.0;
      DeResFX.Emitters[0].ColorMultiplierRange.Z.Max = 1.0;
      DeResFX.Emitters[0].ColorScale[0].Color = DeResColor;
      DeResFX.Emitters[0].ColorScale[1].Color = DeResColor * 0.745;
      DeResFX.Emitters[0].ColorScale[2].Color = DeResColor * 0.745;
    }
    DeResFX.SetBase(self);
  }
  
  if ( UTPlusRI != None && UTPlusRI.bAllowDeRes ) {
    Skins[0] = DeResFinalBody;
    Skins[1] = DeResFinalHead;
    for (i = 2; i < Max(4, Skins.Length); ++i) {
      Skins[i] = DeResFinalBody;
    }
  }
  else {
    Skins[0] = DeResMat0;
    Skins[1] = DeResMat1;
    for (i = 2; i < Max(4, Skins.Length); ++i) {
      Skins[i] = DeResMat0;
    }
  }
  if( Physics == PHYS_KarmaRagdoll ) {
    if ( UTPlusRI == None || !UTPlusRI.bAllowDeRes || class'UTP_ClientSettings'.default.bRisingDeRes ) {
      // Attach bone lifter to raise body
      KAddBoneLifter('bip01 Spine',   DeResLiftVel, DeResLateralFriction, DeResLiftSoftness);
      KAddBoneLifter('bip01 Spine2',  DeResLiftVel, DeResLateralFriction, DeResLiftSoftness);
      
      // Turn off gravity while de-res-ing
      KSetActorGravScale(DeResGravScale);
    }
    
    // Turn off collision with the world for the ragdoll.
    KSetBlockKarma(false);
    
    // Turn off convulsions during de-res
    skelParams = KarmaParamsSkel(KParams);
    if ( skelParams != None )
      skelParams.bKDoConvulsions = false;
    
    if ( !KIsAwake() )
      KWake();
  }
  
  AmbientSound = Sound'GeneralAmbience.Texture19';
  SoundRadius = 40.0;
  
  // Turn off collision when we de-res (avoids rockets etc. hitting corpse!)
  SetCollision(false, false, false);
  
  // Remove/disallow projectors
  Projectors.Remove(0, Projectors.Length);
  bAcceptsProjectors = false;
  
  // Remove shadow
  if ( PlayerShadow != None )
    PlayerShadow.bShadowActive = false;
  
  // Remove flames
  RemoveFlamingEffects();
  
  // Turn off any overlays
  SetOverlayMaterial(None, 0.0, true);
  OverlayMaterial = None;
  
  bDeRes = true;
}


//== PlayDying ================================================================
/**
Reset the skins and play the dying animations and effects, skiping the body-leave-effect, if desired and allowed.
*/
//=============================================================================

simulated function PlayDying(class<DamageType> DamageType, vector HitLoc)
{
  local float frame, rate;
  local name seq;
  local LavaDeath LD;
  local BodyEffect BE;
  
  AmbientSound = None;
  bCanTeleport = false; // sjs - fix karma going crazy when corpses land on teleporters
  bReplicateMovement = false;
  bTearOff = true;
  bPlayedDeath = true;
  if ( Level.NetMode != NM_DedicatedServer ) {
    Skins[0] = BodySkin;
    Skins[1] = FaceSkin;
  }
  
  if ( CurrentCombo != None )
    CurrentCombo.Destroy();
  
  HitDamageType = DamageType; // these are replicated to other clients
  TakeHitLocation = HitLoc;
  
  if ( DamageType != None ) {
    if ( DamageType.default.bSkeletize ) {
      SetOverlayMaterial(DamageType.Default.DamageOverlayMaterial, 4.0, true);
      if ( !bSkeletized ) {
        if ( Level.NetMode != NM_DedicatedServer && SkeletonMesh != None ) {
          if ( DamageType.default.bLeaveBodyEffect && !class'UTP_ClientSettings'.default.bSkipLeaveBodyEffect && UTPlusRI != None && UTPlusRI.bAllowDeRes ) {
            BE = spawn(class'BodyEffect', self);
            if ( BE != None ) {
              BE.DamageType = DamageType;
              BE.HitLoc = HitLoc;
              bFrozenBody = true;
            }
          }
          GetAnimParams(0, seq, frame, rate);
          LinkMesh(SkeletonMesh, true);
          Skins.Length = 0;
          PlayAnim(seq, 0, 0);
          SetAnimFrame(frame);
        }
        if ( Physics == PHYS_Walking )
          Velocity = vect(0,0,0);
        TearOffMomentum *= 0.25;
        bSkeletized = true;
        if ( Level.NetMode != NM_DedicatedServer && DamageType == class'FellLava' ) {
          LD = spawn(class'LavaDeath',,, Location + vect(0,0,10), Rotation);
          if ( LD != None )
            LD.SetBase(self);
          PlaySound(Sound'WeaponSounds.BExplosion5', SLOT_None, 1.5 * TransientSoundVolume);
        }
      }
    }
    else if ( DamageType.Default.DeathOverlayMaterial != None )
      Super.SetOverlayMaterial(DamageType.Default.DeathOverlayMaterial, DamageType.default.DeathOverlayTime, true);
    else if ( DamageType.Default.DamageOverlayMaterial != None && Level.DetailMode != DM_Low && !Level.bDropDetail )
      Super.SetOverlayMaterial(DamageType.Default.DamageOverlayMaterial, 2 * DamageType.default.DamageOverlayTime, true);
  }
  
  // stop shooting
  AnimBlendParams(1, 0.0);
  FireState = FS_None;
  LifeSpan = RagdollLifeSpan;
  
  GotoState('Dying');
  if ( BE != None )
    return;
  
  PlayDyingAnimation(DamageType, HitLoc);
}


//== state Dying ==============================================================
/**
Immediately start DeRes-ing, if desired and allowed.
*/
//=============================================================================

simulated state Dying
{
  //== Timer ==================================================================
  /**
  Determine when to start deres.
  */
  //===========================================================================
  
  simulated function Timer()
  {
    local KarmaParamsSkel skelParams;
    local PlayerController LocalPlayer;
    
    if ( !PlayerCanSeeMe() ) {
			Destroy();
    }
    // If we are running out of life, bute we still haven't come to rest, force the de-res.
    // unless pawn is the viewtarget of a player who used to own it
    else if ( LifeSpan <= DeResTime && bDeRes == false ) {
			skelParams = KarmaParamsSkel(KParams);
      
			// check not viewtarget
      LocalPlayer = Level.GetLocalPlayerController();
			if ( LocalPlayer != None && LocalPlayer.ViewTarget == self ) {
        skelParams.bKImportantRagdoll = true;
        LifeSpan = FMax(LifeSpan, DeResTime + 2.0);
        SetTimer(1.0, false);
        return;
      }
      else {
        skelParams.bKImportantRagdoll = false;
      }
      // spawn derez
      StartDeRes();
    }
    else {
      SetTimer(1.0, false);
    }
  }
  
Begin:
  AmbientGlow = default.AmbientGlow;
	PlayDyingSound();
  if ( Level.NetMode != NM_DedicatedServer && class'UTP_ClientSettings'.default.bInstantDeRes && UTPlusRI != None && UTPlusRI.bAllowDeRes && Level.GetLocalPlayerController().ViewTarget != Self && (UTP_DemoCamera(Level.GetLocalPlayerController().ViewTarget) == None || UTP_DemoCamera(Level.GetLocalPlayerController().ViewTarget).TargetPawn != Self) ) {
    while (bFrozenBody)
      Sleep(0.0);
    
    StartDeRes();
    LifeSpan = FMin(LifeSpan, DeResTime);
    Stop;
  }
	Sleep(0.2);
	bInvulnerableBody = false;
}


//== TakeDamage ===============================================================
/**
Add hit sounds for being hit during spawn protection.
*/
//=============================================================================

function TakeDamage(int Damage, Pawn InstigatedBy, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
  local int ActualDamage;
  local Controller Killer;
  
  if ( DamageType == None ) {
    if ( InstigatedBy != None )
      warn("No damagetype for damage by "$InstigatedBy$" with weapon "$InstigatedBy.Weapon);
    DamageType = class'DamageType';
  }
  
  if ( Role < ROLE_Authority ) {
    log(self$" client damage type "$DamageType$" by "$InstigatedBy);
    return;
  }
  
  if ( Health <= 0 )
    return;
  
  if ( (InstigatedBy == None || InstigatedBy.Controller == None) && DamageType.default.bDelayedDamage && DelayedDamageInstigatorController != None )
    InstigatedBy = DelayedDamageInstigatorController.Pawn;
  
  if ( Physics == PHYS_None && DrivenVehicle == None )
    SetMovementPhysics();
  if ( Physics == PHYS_Walking && damageType.default.bExtraMomentumZ )
    Momentum.Z = FMax(Momentum.Z, 0.4 * VSize(Momentum));
  if ( instigatedBy == self )
    Momentum *= 0.6;
  Momentum = Momentum / Mass;
  
  if ( Weapon != None )
    Weapon.AdjustPlayerDamage(Damage, InstigatedBy, HitLocation, Momentum, DamageType);
  if ( DrivenVehicle != None )
    DrivenVehicle.AdjustDriverDamage(Damage, InstigatedBy, HitLocation, Momentum, DamageType);
  if ( InstigatedBy != None && InstigatedBy.HasUDamage() ) {
    if ( UTPlusRI != None )
      Damage *= UTPlusRI.AmpDamageMultiplier;
    else
      Damage *= 2;
  }
  ActualDamage = Level.Game.ReduceDamage(Damage, self, InstigatedBy, HitLocation, Momentum, DamageType);
  
  if ( InstigatedBy != None && InstigatedBy != Self && InstigatedBy.Health > 0 )
    OnCalcHitSoundsFor(class'UTP_PlayerReplicationInfo'.static.FindFor(InstigatedBy), Damage, Self);
  
  if ( DamageType.default.bArmorStops && ActualDamage > 0 )
    ActualDamage = ShieldAbsorb(ActualDamage);
  
  Health -= ActualDamage;
  if ( HitLocation == vect(0,0,0) )
    HitLocation = Location;
  
  PlayHit(ActualDamage, InstigatedBy, HitLocation, DamageType, Momentum);
  if ( Health <= 0 ) {
    // pawn died
    if (UnlaggedCollision != None && UnlaggedCollision.bCollideActors) {
      DisableUnlag();
      SetLocation(UnlaggedCollision.Location);
    }
    if ( DamageType.default.bCausedByWorld && (InstigatedBy == None || InstigatedBy == self) && LastHitBy != None )
      Killer = LastHitBy;
    else if ( InstigatedBy != None )
      Killer = InstigatedBy.GetKillerController();
    if ( Killer == None && DamageType.Default.bDelayedDamage )
      Killer = DelayedDamageInstigatorController;
    if ( bPhysicsAnimUpdate )
      TearOffMomentum = momentum;
    Died(Killer, DamageType, HitLocation);
  }
  else {
    AddVelocity(Momentum);
    if ( Controller != None )
      Controller.NotifyTakeHit(InstigatedBy, HitLocation, ActualDamage, DamageType, Momentum);
    if ( InstigatedBy != None && InstigatedBy != self )
      LastHitBy = InstigatedBy.Controller;
  }
  MakeNoise(1.0);
}


//== AddVelocity ==============================================================
/**
Adding changes to lockdown.
*/
//=============================================================================

function AddVelocity(vector NewVelocity)
{
  if ( bIgnoreForces || NewVelocity == vect(0,0,0) )
    return;
  if ( Physics == PHYS_Falling && AIController(Controller) != None )
    ImpactVelocity += NewVelocity;
  if ( Physics == PHYS_Walking && (!bLessLockdown || NewVelocity dot vect(0,0,1) > 20) || (Physics == PHYS_Ladder || Physics == PHYS_Spider) && NewVelocity.Z > Default.JumpZ )
    SetPhysics(PHYS_Falling);
  if ( Velocity.Z > 380 && NewVelocity.Z > 0 )
    NewVelocity.Z *= 0.5;
  Velocity += NewVelocity;
}


//== delegate OnCalcHitSoundsFor ==============================================
/**
This is assigned to the UTP_GameRules.CalcHitSoundsFor function for playing spawn-protect hit sounds.
*/
//=============================================================================

delegate OnCalcHitSoundsFor(UTP_PlayerReplicationInfo Player, int Damage, Pawn Victim);


//== FootStepping =============================================================
/**
Really turn off own footsteps if configured.
*/
//=============================================================================

simulated function FootStepping(int Side)
{
  local PlayerController LocalPlayer;
  
  if ( !bPlayOwnFootsteps ) {
    LocalPlayer = Level.GetLocalPlayerController();
    if ( LocalPlayer == None || LocalPlayer.ViewTarget != Self ) {
      Super.FootStepping(Side);
    }
  }
  else
    Super.FootStepping(Side);
}


//== Landed ===================================================================
/**
Turn off own landing sound if configured.
*/
//=============================================================================

event Landed(vector HitNormal)
{
  local PlayerController LocalPlayer;
  
  Super(UnrealPawn).Landed(HitNormal);
  MultiJumpRemaining = MaxMultiJump;
  
  if ( !bPlayOwnFootsteps && !class'UTP_ClientSettings'.default.bPlayOwnLandingSound )
    LocalPlayer = Level.GetLocalPlayerController();
  if ( Health > 0 && (LocalPlayer == None || LocalPlayer.ViewTarget != Self) && Level.TimeSeconds - SplashTime > 0.25 )
    PlayOwnedSound(GetSound(EST_Land), SLOT_Interact, FMin(1,-0.3 * Velocity.Z / JumpZ));
}


//== PerformDodge =============================================================
/**
Perform a dodging move in the desired direction.
*/
//=============================================================================

function bool PerformDodge(eDoubleClickDir DoubleClickMove, vector X, vector Y)
{
  local vector HitLoc, HitNorm;
  local float VelocityZ;
  local name Anim;
  
  if ( Physics == PHYS_Falling ) {
    if (DoubleClickMove == DCLICK_Forward)
      Anim = WallDodgeAnims[0];
    else if (DoubleClickMove == DCLICK_Back)
      Anim = WallDodgeAnims[1];
    else if (DoubleClickMove == DCLICK_Left)
      Anim = WallDodgeAnims[2];
    else if (DoubleClickMove == DCLICK_Right)
      Anim = WallDodgeAnims[3];
    
    if ( PlayAnim(Anim, 1.0, 0.1) )
      bWaitForAnim = true;
    AnimAction = Anim;
    
    TakeFallingDamage();
    if (Velocity.Z < -DodgeSpeedZ * 0.5)
      Velocity.Z += DodgeSpeedZ * 0.5;
    HitNorm = vect(0,0,1);
  }
  else if (bImprovedSlopeDodging) {
    // adjust dodge dir to make dodging more efficient on slopes
    if (Y != vect(0,0,0) && Base != None && Trace(HitLoc, HitNorm, Location - vect(0,0,1) * (CollisionHeight + 20), Location - vect(0,0,1) * (CollisionHeight - 10), False, vect(0.7,0.7,0) * CollisionRadius + vect(0,0,1)) != None && X Dot HitNorm < 0) {
      Y = vect(0,0,1) Cross X;  // fix the cross vector so it actually points 90 to the right, not the left
      X = Y Cross HitNorm;
    }
    else
      HitNorm = vect(0,0,1);
  }
  
  VelocityZ = Velocity Dot HitNorm;
  Velocity = DodgeSpeedFactor * GroundSpeed * X + (Velocity Dot Y) * Y + vect(0,0,1);
  
  if ( !bCanDodgeDoubleJump )
    MultiJumpRemaining = 0;
  if ( bCanBoostDodge || (VelocityZ < -100) )
    Velocity += HitNorm * (VelocityZ + DodgeSpeedZ);
  else
    Velocity += HitNorm * DodgeSpeedZ;
  
  CurrentDir = DoubleClickMove;
  SetPhysics(PHYS_Falling);
  PlayOwnedSound(GetSound(EST_Dodge), SLOT_Pain, GruntVolume,,80);
  return true;
}


//== GiveWeapon ===============================================================
/**
Allow mutators to modify the weapon class.
*/
//=============================================================================

function GiveWeapon(string aClassName )
{
  local class<Weapon> WeaponClass;
  local Weapon NewWeapon;
  
  WeaponClass = class<Weapon>(Level.Game.BaseMutator.GetInventoryClass(aClassName));
  
  if ( FindInventoryType(WeaponClass) != None )
    return;
  newWeapon = Spawn(WeaponClass);
  if ( newWeapon != None )
    newWeapon.GiveTo(self);
}


//== DoCombo ==================================================================
/**
Start the specific adrenaline combo.
*/
//=============================================================================

function DoCombo(class<Combo> ComboClass)
{
  local int i;
  
  if (ComboClass != None && CurrentCombo == None) {
    if (bDrainingHealth && ComboClass == class'ComboDefensive')
      CurrentCombo = Spawn(class'UTP_ComboDefensive', self);
    else
      CurrentCombo = Spawn(ComboClass, self);
    
    // Record stats for using the combo
    UnrealMPGameInfo(Level.Game).SpecialEvent(PlayerReplicationInfo, string(CurrentCombo.Class));
    if ( ComboClass.Name == 'ComboSpeed' )
      i = 0;
    else if ( ComboClass.Name == 'ComboBerserk' )
      i = 1;
    else if ( ComboClass.Name == 'ComboDefensive' )
      i = 2;
    else if ( ComboClass.Name == 'ComboInvis' )
      i = 3;
    else
      i = 4;
    TeamPlayerReplicationInfo(PlayerReplicationInfo).Combos[i]++;
  }
}


//== Tick =====================================================================
/**
Compensate ping, if desired.
*/
//=============================================================================

simulated event Tick(float DeltaTime)
{
  Super.Tick(DeltaTime);
  
  if ( bDrainingHealth && Health > 0 ) {
    HealthDrainTime += DeltaTime / 1.1;
    if ( HealthDrainTime >= 1.0 ) {
      HealthDrainTime -= 1.0;
      if ( HealthDrain < 0 && Health > HealthMax || HealthDrain > 0 && Health < SuperHealthMax ) {
        Health += HealthDrain;
        if ( HealthDrain < 0 && Health < HealthMax )
          Health = HealthMax;
        else if ( HealthDrain > 0 && Health > SuperHealthMax )
          Health = SuperHealthMax;
      }
      
      if ( ShieldDrain < 0 && ShieldStrength > ShieldStrengthMax / 2 || ShieldDrain > 0 && ShieldStrength < ShieldStrengthMax ) {
        ShieldStrength += ShieldDrain;
        if ( ShieldDrain < 0 && ShieldStrength < ShieldStrengthMax / 2 )
          ShieldStrength = ShieldStrengthMax / 2;
        else if ( ShieldDrain > 0 && ShieldStrength > ShieldStrengthMax )
          ShieldStrength = ShieldStrengthMax;
      }
    }
  }
  
  if ( UTPlusRI != None ) {
    if ( UTPlusPRI == None )
      UTPlusPRI = class'UTP_PlayerReplicationInfo'.static.FindFor(Self);
    if ( Level.NetMode == NM_Client && UTPlusRI.LocalUTPPRI != None ) {
      if ( UTPlusRI.bPawnsIgnoreTerminalVelocity && !bIgnoreTerminalVelocity )
        bIgnoreTerminalVelocity = True;
      if ( !bDemoPlayback && Level.NetMode == NM_Client && UTPlusRI.bAllowPingPrediction && (class'UTP_ClientSettings'.default.PingCompensationPlayers == PredictionOnly || class'UTP_ClientSettings'.default.PingCompensationPlayers == PredictionPreferred || class'UTP_ClientSettings'.default.PingCompensationPlayers == ServerPreferred && !UTPlusRI.bServerPingCompensationAvailable) ) {
        if ( Level.TimeSeconds - LastRenderTime < 0.3 && Health > 0 && Role < ROLE_AutonomousProxy ) {
          //PredictedLocation = PredictLocation(UTPlusRI.LocalUTPPRI.GamePing, 1);
          PredictedLocation = UTPlusRI.PredictPawnMovement(Self, UTPlusRI.LocalUTPPRI.GamePing);
        }
        else if ( PredictedLocation != Location ) {
          PredictedLocation = Location;
        }
        PrePivot += PredictedLocation - Location + vect(0,0,1) * (default.CollisionHeight - CollisionHeight);
        PrePivot *= 0.5;
      }
    }
    if (Level.NetMode != NM_Client && Level.NetMode != NM_Standalone && UTPlusRI.bServerPingCompensationAvailable)
      UpdateUnlagLocation();
  }
  if ( UTPlusPRI != None && Health > 0 ) {
    UTPlusPRI.LastKnownLocation = Location;
  }
}


//== UpdateUnlagLocation ======================================================
/**
Remember the last ticks' locations for serverside ping compensation.
*/
//=============================================================================

function UpdateUnlagLocation()
{
  local int i;
  
  if (LastLocationUpdateTime == Level.TimeSeconds)
    return;
  if (Health <= 0 || !bCollideActors || Physics != PHYS_Walking && Physics != PHYS_Falling && Physics != PHYS_Flying)
    return;
  LastLocationUpdateTime = Level.TimeSeconds;
  
  while (LocationHistory.LocX.Points.Length > 1 && LocationHistory.LocX.Points[1].InVal < LastLocationUpdateTime - MaxUnlagTime) {
    //log(LastLocationUpdateTime@"- removing"@LocationHistory.LocX.Points[0].InVal);
    LocationHistory.LocX.Points.Remove(0, 1);
    LocationHistory.LocY.Points.Remove(0, 1);
    LocationHistory.LocZ.Points.Remove(0, 1);
    LocationHistory.CollRadius.Points.Remove(0, 1);
    LocationHistory.CollHeight.Points.Remove(0, 1);
  }
  i = LocationHistory.LocX.Points.Length;
  
  LocationHistory.LocX.Points.Length = i + 1;
  LocationHistory.LocX.Points[i].InVal = LastLocationUpdateTime;
  LocationHistory.LocX.Points[i].OutVal = Location.X;
  
  LocationHistory.LocY.Points.Length = i + 1;
  LocationHistory.LocY.Points[i].InVal = LastLocationUpdateTime;
  LocationHistory.LocY.Points[i].OutVal = Location.Y;
  
  LocationHistory.LocZ.Points.Length = i + 1;
  LocationHistory.LocZ.Points[i].InVal = LastLocationUpdateTime;
  LocationHistory.LocZ.Points[i].OutVal = Location.Z;
  
  LocationHistory.CollRadius.Points.Length = i + 1;
  LocationHistory.CollRadius.Points[i].InVal = LastLocationUpdateTime;
  LocationHistory.CollRadius.Points[i].OutVal = CollisionRadius;
  
  LocationHistory.CollHeight.Points.Length = i + 1;
  LocationHistory.CollHeight.Points[i].InVal = LastLocationUpdateTime;
  LocationHistory.CollHeight.Points[i].OutVal = CollisionHeight;
}


//== EnableUnlag ==============================================================
/**
Spawn and enable the unlagged collision cylinder.
*/
//=============================================================================

function EnableUnlag(float PingTime)
{
  local vector UnlaggedLocation;
  local float UnlagTime;
  local range UnlagTimeRange;
  local float UnlaggedRadius, UnlaggedHeight;
  
  if (Level.NetMode == NM_Standalone || !bCollideActors || Health <= 0)
    return;
  UpdateUnlagLocation();
  
  InterpCurveGetInputDomain(LocationHistory.LocX, UnlagTimeRange.Min, UnlagTimeRange.Max);
  //log(Name @ "Unlagging:" @ PingTime @ UnlagTimeRange.Min @ UnlagTimeRange.Max);
  UnlagTime = FClamp(Level.TimeSeconds - PingTime, Level.TimeSeconds - MaxUnlagTime, UnlagTimeRange.Max);
  UnlaggedRadius = InterpCurveEval(LocationHistory.CollRadius, UnlagTime);
  UnlaggedHeight = InterpCurveEval(LocationHistory.CollHeight, UnlagTime);
  // should result in 0/0 if too early, i.e. no chance to hit with trace weapon
  
  UnlagTime = FClamp(Level.TimeSeconds - PingTime, UnlagTimeRange.Min, UnlagTimeRange.Max);
  UnlaggedLocation.X = InterpCurveEval(LocationHistory.LocX, UnlagTime);
  UnlaggedLocation.Y = InterpCurveEval(LocationHistory.LocY, UnlagTime);
  UnlaggedLocation.Z = InterpCurveEval(LocationHistory.LocZ, UnlagTime);
  
  if (UnlaggedCollision == None) {
    UnlaggedCollision = Spawn(class'UTP_PawnCollision', self,, UnlaggedLocation);
    UnlaggedCollision.UnlaggedPawn = Self;
  }
  else
    UnlaggedCollision.SetLocation(UnlaggedLocation);
  UnlaggedCollision.SetCollisionSize(UnlaggedRadius, UnlaggedHeight);
  UnlaggedCollision.SetCollision(bCollideActors, bBlockActors, bBlockPlayers);
  SetCollision(false, false, false);
  //log(Name @ "Diff:" @ Location @"/"@ UnlaggedCollision.Location @"|"@ UnlaggedRadius @"/"@ UnlaggedHeight);
}


//== DisableUnlag =============================================================
/**
Disable the unlagged collision cylinder for a UTP_Pawn.
*/
//=============================================================================

function DisableUnlag()
{
  if (Level.NetMode == NM_Standalone || UnlaggedCollision == None || !UnlaggedCollision.bCollideActors)
    return;
  
  SetCollision(UnlaggedCollision.bCollideActors, UnlaggedCollision.bBlockActors, UnlaggedCollision.bBlockPlayers);
  UnlaggedCollision.SetCollision(false, false, false);
}


//== GetViewRotation ==========================================================
/**
Return the player's view rotation, possibly interpolated on clients from server data.
*/
//=============================================================================

simulated function rotator GetViewRotation()
{
  if ( Controller == None ) {
    if ( Level.NetMode == NM_Client && UTPlusPRI != None )
      return UTPlusPRI.GetViewRotation();
    return Rotation;
  }
  return Controller.GetViewRotation();
}


//== GetCollisionExtent =======================================================
/**
Why is this thing non-simulated in Actor? o_O
*/
//=============================================================================

simulated function vector GetCollisionExtent()
{
  return CollisionRadius * vect(1,1,0) + CollisionHeight * vect(0,0,1);
}


//== ShieldAbsorb =============================================================
/**
Fixing Epic's shield absorption inconsistency.
*/
//=============================================================================

function int ShieldAbsorb(int InDamage)
{
  local int BigShieldStrength;
  local float Interval, Damage, Remaining;
  
  if ( ShieldStrength == 0 )
    return InDamage;
  else if ( !bFixShieldBug )
    return Super.ShieldAbsorb(InDamage);
  
  Damage = InDamage;    // "floatify" integer damage value
  
  SetOverlayMaterial(ShieldHitMat, ShieldHitMatTime, False);
  PlaySound(Sound'WeaponSounds.ArmorHit', SLOT_Pain, 2 * TransientSoundVolume,, 400);
  
  if ( bContinousShield ) {
    // continous shield for Q-Mode
    Interval = ShieldStrength / 0.75;
    if ( Interval >= Damage ) {
      // remaining damage entirely processed by the shield
      ShieldStrength -= Round(0.75 * Damage);
      if ( SmallShieldStrength > ShieldStrength )
        SmallShieldStrength = ShieldStrength;   // make sure the value is reduced as well (just in case)
      return Round(0.25 * Damage);
    }
    
    // more damage than the shield can handle - shield is used up
    ShieldStrength = 0;
    SmallShieldStrength = 0;
    Damage -= Interval;
    Remaining = 0.25 * Interval;
    return Damage + Remaining;
  }
  
  // calculate the virtual amount of big shield energy left
  BigShieldStrength = ShieldStrength - SmallShieldStrength;
  //log(Level.TimeSeconds @ "Big shield:" @ BigShieldStrength @ "Small shield:" @ SmallShieldStrength @ "Damage:" @ InDamage, Name);
  
  // the shield points above 100 absorb 100%
  if ( ShieldStrength > 100 ) {
    Interval = ShieldStrength - 100;
    if ( Interval >= Damage ) {
      // damage entirely absorbed by the shield points above 100
      ShieldStrength -= Damage;
      //log(Level.TimeSeconds @ "Shields fully absorbed all of" @ Damage @ "damage with" @ ShieldStrength @ "shield left", Name);
      return 0;
    }
    else {
      // more damage than the shield points above 100 can handle
      ShieldStrength = 100;
      BigShieldStrength -= Interval;
      Damage -= Interval;
      // Remaining = 0.00 * Interval;
      //log(Level.TimeSeconds @ "Shields fully absorbed" @ Interval @ "damage with" @ Damage @ "damage and" @ ShieldStrength @ "shield left", Name);
    }
  }
  
  // the remaining big shield only absorbs 75% of the damage
  if ( BigShieldStrength > 0 ) {
    Interval = BigShieldStrength / 0.75;
    // This is, how much damage the big shield can still process if it uses up all its energy.
    // 75% of that damage is absorbed, the remaining part goes through.
    // Only damage not processed here will be handed down to the small shield.
    
    if ( Interval >= Damage ) {
      // remaining damage entirely processed by the big shield
      ShieldStrength -= Round(0.75 * Damage);
      /*if (ShieldStrength < SmallShieldStrength) {
        warn("ShieldStrength < SmallShieldStrength"@ShieldStrength@SmallShieldStrength);
        Assert(false);
      }*/
      //log(Level.TimeSeconds @ "Big shield partially absorbed" @ Round(0.75 * Damage) @ "damage with" @ Round(0.25 * Damage) @ "damage going to health and" @ ShieldStrength @ "shield left", Name);
      return Round(0.25 * Damage);
    }
    else {
      // more damage than the big shield can handle - big shield is used up, only small shield left 
      ShieldStrength = SmallShieldStrength;
      // BigShieldStrength = 0;
      Damage -= Interval;
      Remaining = 0.25 * Interval;
      //log(Level.TimeSeconds @ "Big shield partially absorbed" @ Interval @ "damage with" @ Remaining @ "damage going to health and" @ Damage @ "damage and" @ ShieldStrength @ "shield left", Name);
    }
  }
  
  // let the small shield absorb 50% of the remaining damage
  if ( SmallShieldStrength > 0 ) {
    Interval = 2 * SmallShieldStrength;
    // The small shield processes twice the damage it absorbs.
    
    if ( Interval >= Damage ) {
      // remaining damage entirely processed by the small shield
      ShieldStrength -= Round(0.5 * Damage);
      SmallShieldStrength = ShieldStrength;
      //log(Level.TimeSeconds @ "Small shield partially absorbed" @ Damage @ "damage with" @ 0.5 * Damage @ "damage going to health (" $ Round(Remaining + 0.5 * Damage) @ "total) and" @ ShieldStrength @ "shield left", Name);
      return Round(Remaining + 0.5 * Damage);
    }
    else {
      // more damage than the small shield can handle - small shield is used up
      ShieldStrength = 0;
      SmallShieldStrength = 0;
      Damage -= Interval;
      Remaining += 0.5 * Interval;
      //log(Level.TimeSeconds @ "Small shield partially absorbed" @ Interval @ "damage with" @ Remaining @ "damage going to health and" @ Damage @ "damage and" @ ShieldStrength @ "shield left", Name);
    }
  }
  //log(Level.TimeSeconds @ "Damage not absorbed:" @ Round(Damage + Remaining), Name);
  return Round(Damage + Remaining);
}


//== CanUseShield =============================================================
/**
Returns, how much of the shield amount offered can be used.
*/
//=============================================================================

function int CanUseShield(int ShieldAmount)
{
  if ( !bContinousShield )
    return Super.CanUseShield(ShieldAmount);
  
  if ( ShieldStrength < 0 )
    ShieldStrength = 0;
  if ( ShieldStrength < ShieldStrengthMax )
    return Min(ShieldStrengthMax, ShieldStrength + ShieldAmount) - ShieldStrength;
  return 0;
}


//== AddShieldStrength ========================================================
/**
Adds to shield and returns whether successful.
*/
//=============================================================================

function bool AddShieldStrength(int ShieldAmount)
{
  local int OldShieldStrength;
  
  if ( !bContinousShield )
    return Super.AddShieldStrength(ShieldAmount);
  
  OldShieldStrength = ShieldStrength;
  ShieldStrength += CanUseShield(ShieldAmount);
  return (ShieldStrength != OldShieldStrength);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  MutantInvisMaterial = FinalBlend'MutantSkins.Shaders.MutantGlowFinal'
  InvisMaterial       = FinalBlend'XEffectMat.InvisOverlayFB'
  OverlayTexture      = Texture'BlackTexture'
  DeResBaseBody       = "UTPlusDeResTextures.DeResBaseBody"
  DeResBaseHead       = "UTPlusDeResTextures.DeResBaseHead"
  RealTeam            = 255
  MaxUnlagTime        = 1.0
  HealthDrain         = -1
  ShieldDrain         = -1
}
