/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Predicts the movement of a UTP_Pawn.
******************************************************************************/


class UTP_PawnPredicter extends Actor;


//=============================================================================
// Variables
//=============================================================================

var transient float MaxGroundSpeed;


//== Landed ===================================================================
/**
Switch to walking.
*/
//=============================================================================

event Landed(vector HitNormal)
{
  SetPhysics(PHYS_Walking);
  Velocity -= (Velocity dot HitNormal) * HitNormal; // slide along the wall
  if (VSize(Velocity) > MaxGroundSpeed)
    Velocity *= MaxGroundSpeed / VSize(Velocity);
}


//== HitWall ===================================================================
/**
Bounce off ceiling or slide along wall.
*/
//=============================================================================

event HitWall(vector HitNormal, Actor Wall)
{
  if ( HitNormal.Z < -0.7 )
    Velocity -= 2 * (Velocity dot HitNormal) * HitNormal; // bounce off a ceiling
  else
    Velocity -= (Velocity dot HitNormal) * HitNormal; // slide along the wall
}


//== FellOutOfWorld ===========================================================
/**
The prediction actor shouldn't care about falling out of the world.
*/
//=============================================================================

event FellOutOfWorld(eKillZType KillType);


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bShouldBaseAtStartup  = True
  bCollideWorld         = True
  bIgnoreOutOfWorld     = True
  bCollideActors        = False
  bBlockActors          = False
  bHidden               = True
  bBounce               = True
  Physics               = PHYS_Falling
  RemoteRole            = ROLE_None
}
