/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Configuration page for UTPlus player skin settings.
******************************************************************************/


class UTP_PlayerMeshActor extends Actor
    notplaceable;


//=============================================================================
// Variables
//=============================================================================

var() float       TauntDelay  "seconds before playing the taunt anim";
var() array<name> TauntAnims  "taunt animations";
var() array<name> IdleAnims   "idle animations while not taunting";

var noexport xUtil.PlayerRecord  Rec;
var int TeamSkin;
var bool bBrightSkins;

var() noexport editinline ConstantColor  OverlayColor;
var() noexport editinline Shader         OverlayShaderBody, OverlayShaderHead;
var() noexport editinline FinalBlend     OverlayFinalBody, OverlayFinalHead;


//== PostBeginPlay ============================================================
/**
Initialize the overlay textures.
*/
//=============================================================================

function PostBeginPlay()
{
  Super.PostBeginPlay();
  AllocateOverlayTextures();
}


//== Destroyed ================================================================
/**
Free references.
*/
//=============================================================================

function Destroyed()
{
  local xUtil.PlayerRecord EmptyRecord;
  
  Skins.Length = 0;
  FreeTextures();
  Rec = EmptyRecord;
  
  Super.Destroyed();
}


//== AllocateOverlayTextures ==================================================
/**
Create the overlay materials.
*/
//=============================================================================

function AllocateOverlayTextures()
{
  // allocate overlay modifier materials
  OverlayColor = ConstantColor(Level.ObjectPool.AllocateObject(class'ConstantColor'));
  OverlayColor.Color.R = 0;
  OverlayColor.Color.G = 0;
  OverlayColor.Color.B = 0;
  OverlayColor.Color.A = 255;
  OverlayColor.FallbackMaterial = None;
  OverlayColor.SurfaceType = EST_Default;
  
  OverlayShaderBody = Shader(Level.ObjectPool.AllocateObject(class'Shader'));
  OverlayShaderBody.Diffuse                       = None;
  OverlayShaderBody.Opacity                       = None;
  OverlayShaderBody.Specular                      = OverlayColor;
  OverlayShaderBody.SpecularityMask               = None;
  OverlayShaderBody.SelfIllumination              = None;
  OverlayShaderBody.SelfIlluminationMask          = None;
  OverlayShaderBody.Detail                        = None;
  OverlayShaderBody.OutputBlending                = OB_Normal;
  OverlayShaderBody.TwoSided                      = True;
  OverlayShaderBody.WireFrame                     = False;
  OverlayShaderBody.PerformLightingOnSpecularPass = False;
  OverlayShaderBody.ModulateSpecular2X            = False;
  OverlayShaderBody.FallbackMaterial              = None;
  OverlayShaderBody.SurfaceType                   = EST_Default;
  
  OverlayFinalBody = FinalBlend(Level.ObjectPool.AllocateObject(class'FinalBlend'));
  OverlayFinalBody.FrameBufferBlending = FB_AlphaBlend;
  OverlayFinalBody.ZWrite              = True;
  OverlayFinalBody.ZTest               = True;
  OverlayFinalBody.AlphaTest           = False;
  OverlayFinalBody.TwoSided            = True;
  OverlayFinalBody.AlphaRef            = 0;
  OverlayFinalBody.Material            = OverlayShaderBody;
  OverlayFinalBody.FallbackMaterial    = OverlayShaderBody;
  OverlayFinalBody.SurfaceType         = EST_Default;
  
  OverlayShaderHead = Shader(Level.ObjectPool.AllocateObject(class'Shader'));
  OverlayShaderHead.Diffuse                       = None;
  OverlayShaderHead.Opacity                       = None;
  OverlayShaderHead.Specular                      = OverlayColor;
  OverlayShaderHead.SpecularityMask               = None;
  OverlayShaderHead.SelfIllumination              = None;
  OverlayShaderHead.SelfIlluminationMask          = None;
  OverlayShaderHead.Detail                        = None;
  OverlayShaderHead.OutputBlending                = OB_Normal;
  OverlayShaderHead.TwoSided                      = True;
  OverlayShaderHead.WireFrame                     = False;
  OverlayShaderHead.PerformLightingOnSpecularPass = False;
  OverlayShaderHead.ModulateSpecular2X            = False;
  OverlayShaderHead.FallbackMaterial              = None;
  OverlayShaderHead.SurfaceType                   = EST_Default;
  
  OverlayFinalHead = FinalBlend(Level.ObjectPool.AllocateObject(class'FinalBlend'));
  OverlayFinalHead.FrameBufferBlending = FB_AlphaBlend;
  OverlayFinalHead.ZWrite              = True;
  OverlayFinalHead.ZTest               = True;
  OverlayFinalHead.AlphaTest           = False;
  OverlayFinalHead.TwoSided            = True;
  OverlayFinalHead.AlphaRef            = 0;
  OverlayFinalHead.Material            = OverlayShaderHead;
  OverlayFinalHead.FallbackMaterial    = OverlayShaderHead;
  OverlayFinalHead.SurfaceType         = EST_Default;
}


//== FreeTextures =============================================================
/**
Free the overlay materials.
*/
//=============================================================================

function FreeTextures()
{
  if ( OverlayFinalBody != None ) {
    OverlayFinalBody.Material = None;
    OverlayFinalBody.FallbackMaterial = None;
    Level.ObjectPool.FreeObject(OverlayFinalBody);
    OverlayFinalBody = None;
  }
  if ( OverlayShaderBody != None ) {
    OverlayShaderBody.Diffuse = None;
    OverlayShaderBody.Specular = None;
    Level.ObjectPool.FreeObject(OverlayShaderBody);
    OverlayShaderBody = None;
  }
  if ( OverlayFinalHead != None ) {
    OverlayFinalHead.Material = None;
    OverlayFinalHead.FallbackMaterial = None;
    Level.ObjectPool.FreeObject(OverlayFinalHead);
    OverlayFinalHead = None;
  }
  if ( OverlayShaderHead != None ) {
    OverlayShaderHead.Diffuse = None;
    OverlayShaderHead.Specular = None;
    Level.ObjectPool.FreeObject(OverlayShaderHead);
    OverlayShaderHead = None;
  }
  if ( OverlayColor != None ) {
    Level.ObjectPool.FreeObject(OverlayColor);
    OverlayColor = None;
  }
}


//== SetCharacter =============================================================
/**
Links a new mesh and resets the animation list. Call UpdateSkins afterwards.
*/
//=============================================================================

function SetCharacter(string CharacterName)
{
  local xUtil.PlayerRecord  NewRec;
  local Mesh NewMesh;
  
  if ( Rec.DefaultName ~= CharacterName )
    return;
  
  NewRec = class'xUtil'.static.FindPlayerRecord(CharacterName);
  
  if ( NewRec.DefaultName == "" || NewRec.DefaultName ~= Rec.DefaultName )
    return;
  
  NewMesh = Mesh(DynamicLoadObject(NewRec.MeshName, class'Mesh'));
  
  if ( NewMesh == None )
    return;
  
  Rec = NewRec;
  
  LinkMesh(NewMesh);
  
  Skins.Length = 0;
  TauntAnims  = default.TauntAnims;
  TeamSkin    = default.TeamSkin;
  
  GotoState('Taunting', 'BeginTaunt');
}


//== UpdateSkins ==============================================================
/**
Updates the player skin parameters.
*/
//=============================================================================

function UpdateSkins(int TeamNum, color SkinOverlayColor, bool bNewBrightSkins)
{
  if ( TeamNum != TeamSkin || bNewBrightSkins != bBrightSkins )
    SetTeamSkin(TeamNum, bNewBrightSkins);
  
  if ( OverlayColor != None )
    OverlayColor.Color = SkinOverlayColor;
}


//== SetTeamSkin ==============================================================
/**
Load and assign the team skins. (modified from xGame.SpeciesType)
*/
//=============================================================================

function SetTeamSkin(int TeamNum, bool bNewBrightSkins)
{
  local Material BodySkin, FaceSkin;
  local bool bBodyAlpha, bHeadAlpha;
  
  TeamSkin = TeamNum;
  bBrightSkins = bNewBrightSkins;
  
  GetTeamSkin(TeamNum, BodySkin, FaceSkin);
  
  bBodyAlpha = class'UTP_Pawn'.static.IsAlphaTexture(BodySkin);
  if ( OverlayShaderBody != None ) {
    OverlayShaderBody.Diffuse = class'UTP_Pawn'.static.GetSkinTexture(BodySkin);
    if ( bBodyAlpha )
      OverlayShaderBody.Opacity = OverlayShaderBody.Diffuse;
    else
      OverlayShaderBody.Opacity = None;
  }
  if ( OverlayFinalBody != None ) {
    if ( bBodyAlpha )
      OverlayFinalBody.FrameBufferBlending = FB_AlphaBlend;
    else
      OverlayFinalBody.FrameBufferBlending = FB_Overwrite;
  }
  bHeadAlpha = class'UTP_Pawn'.static.IsAlphaTexture(FaceSkin);
  if ( OverlayShaderHead != None ) {
    OverlayShaderHead.Diffuse = class'UTP_Pawn'.static.GetSkinTexture(FaceSkin);
    
    if ( Rec.TeamFace )
      OverlayShaderHead.Specular = OverlayColor;
    else
      OverlayShaderHead.Specular = None;
    
    if ( bHeadAlpha )
      OverlayShaderHead.Opacity = OverlayShaderHead.Diffuse;
    else
      OverlayShaderHead.Opacity = None;
  }
  if ( OverlayFinalHead != None ) {
    if ( bHeadAlpha )
      OverlayFinalHead.FrameBufferBlending = FB_AlphaBlend;
    else
      OverlayFinalHead.FrameBufferBlending = FB_Overwrite;
  }
  
  Skins[0] = OverlayFinalBody;
  Skins[1] = OverlayFinalHead;
}


//== GetTeamSkin ==============================================================
/**
Get the character's skins for the specified team.
*/
//=============================================================================

function GetTeamSkin(int TeamNum, out Material NewBodySkin, out Material NewFaceSkin)
{
  local string BodySkinName, FaceSkinName;
  local Material TeamFaceSkin;
  
  NewFaceSkin = Material(DynamicLoadObject(Rec.FaceSkinName, class'Material'));
  
  if ( TeamNum != 255 ) {
    if ( bBrightSkins && (Left(Rec.BodySkinName, 12) ~= "PlayerSkins.") ) {
      BodySkinName = "Bright" $ Rec.BodySkinName $ "_" $ TeamNum $ "B";
      NewBodySkin = Material(DynamicLoadObject(BodySkinName, class'Material',true));
    }
    if ( NewBodySkin == None ) {
      BodySkinName = Rec.BodySkinName $ "_" $ TeamNum;
      NewBodySkin = Material(DynamicLoadObject(BodySkinName, class'Material'));
      
      // allow team head skins with new skins
      if ( Rec.TeamFace ) {
        FaceSkinName = Rec.FaceSkinName $ "_" $ TeamNum;
        TeamFaceSkin = Material(DynamicLoadObject(FaceSkinName, class'Material'));
        if ( TeamFaceSkin != None )
          NewFaceSkin = TeamFaceSkin;
      }
    }
    if ( NewBodySkin == None ) {
      log("TeamSkin not found" @ NewBodySkin @ "for" @ Mesh);
      NewBodySkin = Material(DynamicLoadObject(Rec.BodySkinName, class'Material'));
    }
  }
  else
    NewBodySkin = Material(DynamicLoadObject(Rec.BodySkinName, class'Material'));
}


//== state Taunting ===========================================================
/**
Automatically play some taunt animations.
*/
//=============================================================================

state Taunting
{
  //== GetRandomTauntAnim =====================================================
  /**
  Returns a random taunt animation name.
  */
  //===========================================================================
  
  function name GetRandomTauntAnim()
  {
    local int i;
    
    while (TauntAnims.Length > 0) {
      i = Rand(TauntAnims.Length);
      if ( HasAnim(TauntAnims[i]) )
        return TauntAnims[i];
      else
        TauntAnims.Remove(i, 1);
    }
    
    return '';
  }
  
  
  //== PlayTauntAnim ==========================================================
  /**
  Play the next taunt animation.
  */
  //===========================================================================
  
  function PlayTauntAnim()
  {
    local name TauntName;
    
    TauntName = GetRandomTauntAnim();
    if ( HasAnim(TauntName) )
      PlayAnim(TauntName,, 0.2);
  }
  
BeginTaunt:
  LoopAnim(IdleAnims[Rand(IdleAnims.Length)]);
Taunt:
  Sleep(TauntDelay);
  PlayTauntAnim();
  FinishAnim();
  LoopAnim(IdleAnims[Rand(IdleAnims.Length)],, 0.3);
  Sleep(1.5 * FRand() + 1);
  Goto('Taunt');
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  TauntAnims(0)=PThrust
  TauntAnims(1)=AssSmack
  TauntAnims(2)=ThroatCut
  TauntAnims(3)=Specific_1
  TauntAnims(4)=gesture_cheer
  TauntAnims(5)=Gesture_Taunt01
  TauntAnims(6)=Gesture_Taunt02
  TauntAnims(7)=Gesture_Taunt03
  TauntAnims(8)=gesture_beckon
  TauntAnims(9)=Idle_Character01
  TauntAnims(10)=Idle_Character02
  TauntAnims(11)=Idle_Character03
  IdleAnims(0)=Idle_Rest
  IdleAnims(1)=Idle_Biggun
  TauntDelay=5.0
  
  TeamSkin = -1
  
  RemoteRole=ROLE_None
  bUnlit=True
  bHidden=True
  bAcceptsProjectors=false
  AmbientGlow=128
  DrawType=DT_Mesh
  Style=STY_Alpha
  DrawScale=1.0
  DrawScale3D=(X=1.0,Y=1.0,Z=1.0)
  bAlwaysTick=true
  LODBias=100000
}