/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Generic holder class for replicated functions.
******************************************************************************/


class UTP_PlayerReplicationInfo extends ReplicationInfo
    dependson(UTP_ClientSettings)
    config;


//=============================================================================
// Imports
//=============================================================================

#exec obj load file=..\Sounds\AnnouncerMain.uax


//=============================================================================
// Variables
//=============================================================================

var globalconfig  array<string> HitSoundNames     "List of hit sound names.";
var               array<Sound>  HitSounds         "List of loaded hit sounds.";
var localized     string        descHitSoundNames "Description for PlayInfo";

// TODO: find a way to lower the bandwidth requirements of hit sounds
var int   AccumulatedEnemyDamage,       LastEnemyDamage;
var byte  AccumulatedEnemyHits,         LastEnemyHits;
var int   AccumulatedVehicleDamage,     LastVehicleDamage;
var byte  AccumulatedVehicleHits,       LastVehicleHits;
var byte  AccumulatedFriendlyHits,      LastFriendlyHits;
var byte  AccumulatedProtectedHits,     LastProtectedHits;
var byte  AccumulatedEmptyVehicleHits,  LastEmptyVehicleHits;

var bool bPostNetBeginPlay;
var UTP_ClientSettings.EPingCompensationType PingCompensationType;

var UTP_ReplicationInfo   UTPlusRI      "Reference to the global UTP_ReplicationInfo actor.";
var PlayerReplicationInfo AssociatedPRI "Replicated reference to the owner's PRI.";
var PlayerController      MyOwner       "Replicated reference to owning PlayerController.";
var UTP_HudOverlay        HudOverlay;
var UTP_ValueFader        ValueFader;
var Console               LocalConsole;
var FileLog               ClientLog;
var int                   MyGUID, MyIP, MyLocalID, MyServerID;
var string                LastName;
var array<string>         LastNames;
var bool                  bAdmin        "Admin state replicated only to the owner.";

// Multi-Dodging
var PlayerInput     PlayerInput;
var EDoubleClickDir LastDoubleClickDir;
var int             NumDodgesPerJump;

var array<float> LastPings;
var float RealPing  "Exact ping in real-time milliseconds";
var float GamePing  "Exact ping in game seconds.";
var byte  RepPing   "Replicated ping in real-time milliseconds, capped at 1020 and divided by 4.";

var array<string> PendingClientMessages, BufferedConsoleMessages;
var array<byte>   NoLogMessages;

var transient vector LastKnownLocation;
var transient rotator ViewRotation, LastViewRotation;
var int   MultiKillLevel;
var float LastKillTime, LastViewRotationUpdate;
var int RotationFaderIndex;
var transient bool bMapChange;


//=============================================================================
// Replication
//=============================================================================

replication
{
  // variable replication
  reliable if ( bNetInitial && Role == ROLE_Authority )
    AssociatedPRI, MyGUID, MyIP, MyServerID;
  
  reliable if ( bNetInitial && bNetOwner && Role == ROLE_Authority )
    MyOwner;
  
  reliable if ( bNetDirty && Role == Role_Authority && (!bNetOwner || bDemoRecording) )
    RepPing;
  
  reliable if ( (bNetInitial || bNetDirty) && bNetOwner && Role == ROLE_Authority )
    bAdmin;
  
  reliable if ( bNetDirty && Role == ROLE_Authority )
    ViewRotation;
  
  // hit sound data
  //reliable if ( bNetInitial && Role == ROLE_Authority )
  //  LastEnemyDamage, LastEnemyHits, LastVehicleDamage, LastVehicleHits, LastFriendlyHits, LastProtectedHits, LastEmptyVehicleHits;
  reliable if ( (bNetInitial || bNetDirty) && Role == ROLE_Authority )
    AccumulatedEnemyDamage, AccumulatedEnemyHits, AccumulatedVehicleDamage, AccumulatedVehicleHits, AccumulatedFriendlyHits, AccumulatedProtectedHits, AccumulatedEmptyVehicleHits;
  
  // function replication
  unreliable if ( Role == ROLE_Authority )
    ClientReplyPing;
  
  unreliable if ( Role < ROLE_Authority )
    ServerRequestPing, ServerUpdatePing;
  
  reliable if ( Role == ROLE_Authority )
    /*ClientPlayHitSound,*/ ClientReplyServerConfig, ClientMessage, ClientReset;
  
  reliable if ( Role < ROLE_Authority )
    ServerRequestConfig, ServerSetConfig, ServerSetPingCompensationType;
}


//== static GetFor ===========================================================
/**
Returns a UTP_PlayerReplicationInfo actor for a player.
*/
//=============================================================================

static function UTP_PlayerReplicationInfo GetFor(Actor Player)
{
  local PlayerReplicationInfo PRI;
  local UTP_PlayerReplicationInfo UTP_PRI;
  
  if ( Player == None )
    return None;
  
  if ( UTP_Pawn(Player) != None && UTP_Pawn(Player).UTPlusPRI != None )
    return UTP_Pawn(Player).UTPlusPRI;
  PRI = PlayerReplicationInfo(Player);
  if ( PRI == None && Controller(Player) != None )
    PRI = Controller(Player).PlayerReplicationInfo;
  if ( PRI == None && Pawn(Player) != None )
    PRI = Pawn(Player).PlayerReplicationInfo;
  if ( PRI == None && Controller(Player.Owner) != None )
    PRI = Controller(Player.Owner).PlayerReplicationInfo;
  if ( PRI == None && Pawn(Player.Owner) != None )
    PRI = Pawn(Player.Owner).PlayerReplicationInfo;
  
  if ( PRI == None )
    return None;
  
  foreach Player.DynamicActors(class'UTP_PlayerReplicationInfo', UTP_PRI) {
    if ( UTP_PRI.AssociatedPRI == PRI )
      return UTP_PRI;
  }
  return None;
}


//== static FindFor ===========================================================
/**
Returns a UTP_PlayerReplicationInfo actor for a player. If none is found, a new one is spawned and returned.
*/
//=============================================================================

static function UTP_PlayerReplicationInfo FindFor(Actor Player)
{
  local Controller PC;
  local UTP_PlayerReplicationInfo UTP_PRI;
  
  if ( Player == None )
    return None;
  
  UTP_PRI = GetFor(Player);
  if ( UTP_PRI != None )
    return UTP_PRI;
  
  PC = Controller(Player);
  if ( PC == None && Controller(Player.Owner) != None )
    PC = Controller(Player.Owner);
  if ( PC == None && Pawn(Player) != None )
    PC = Pawn(Player).Controller;
  if ( PC == None && Vehicle(Player) != None && Vehicle(Player).Driver != None )
    PC = Vehicle(Player).Driver.Controller;
  
  if ( PC == None )
    return None;
  
  if ( Player.Level.NetMode != NM_Client ) {
    //log("Spawning" @ default.Class @ "for" @ Player $ "/" $ PC);
    return PC.Spawn(default.Class, PC);
  }
}


//== ServerSetConfig ==========================================================
/**
Called by the client on the server to modify the UTPlus configuration.
*/
//=============================================================================

function ServerSetConfig(int ConfigData1, int ConfigData2, int ConfigData3, string AutoDemoName)
{
  OnServerSetConfig(Self, ConfigData1, ConfigData2, ConfigData3, AutoDemoName);
}


//== delegate OnServerSetConfig ===============================================
/**
Sets the server's configuration.
*/
//=============================================================================

delegate OnServerSetConfig(UTP_PlayerReplicationInfo UTP_PRI, int ConfigData1, int ConfigData2, int ConfigData3, string AutoDemoName);


//== ServerRequestConfig ======================================================
/**
Called by the client on the server to request the current UTPlus configuration.
*/
//=============================================================================

function ServerRequestConfig()
{
  local int ConfigData1, ConfigData2, ConfigData3;
  local string AutoDemoName;
  
  if ( OnServerRequestConfig(Self, ConfigData1, ConfigData2, ConfigData3, AutoDemoName) )
    ClientReplyServerConfig(ConfigData1, ConfigData2, ConfigData3, AutoDemoName);
  else
    ClientReplyServerConfig(0, 0, 0, "");
}


//== delegate OnServerRequestConfig ===========================================
/**
Gets the server's configuration.
*/
//=============================================================================

delegate bool OnServerRequestConfig(UTP_PlayerReplicationInfo UTP_PRI, out int ConfigData1, out int ConfigData2, out int ConfigData3, out string AutoDemoName);


//== ClientReplyServerConfig ==================================================
/**
Called on the client when the server replied to a server config request.
*/
//=============================================================================

simulated function ClientReplyServerConfig(int ConfigData1, int ConfigData2, int ConfigData3, string AutoDemoName)
{
  OnClientReplyConfig(ConfigData1, ConfigData2, ConfigData3, AutoDemoName);
}


//== OnClientReplyConfig ======================================================
/**
Can be used to get the results of the server config request.
*/
//=============================================================================

delegate OnClientReplyConfig(int ConfigData1, int ConfigData2, int ConfigData3, string AutoDemoName);


//== PreBeginPlay =============================================================
/**
Initializes replicated owner and PRI variables.
*/
//=============================================================================

function PreBeginPlay()
{
  Super.PreBeginPlay();
  
  if ( !bDeleteMe ) {
    MyOwner = PlayerController(Owner);
    AssociatedPRI = Controller(Owner).PlayerReplicationInfo;
    if ( MyOwner != None ) {
      MyGUID = class'UTP_Util'.static.HexToInt(Left(MyOwner.GetPlayerIDHash(), 8));
      MyIP = class'UTP_Util'.static.IPToInt(MyOwner.GetPlayerNetworkAddress()) & 0xFFFFFF00;  // partially mask the IP
    }
    //log(MyOwner@AssociatedPRI@MyGUID@MyIP);
  }
}


//== PostNetBeginPlay =========================================================
/**
Find the UTP replication info and register the HUD display.
*/
//=============================================================================

simulated function PostNetBeginPlay()
{
  if ( MyOwner != None ) {
    if (MyOwner == Level.GetLocalPlayerController())
      ServerSetPingCompensationType(class'UTP_ClientSettings'.default.PingCompensationPlayers);
    
    foreach AllObjects(class'PlayerInput', PlayerInput)
      if ( PlayerInput.Outer == MyOwner )
        break;
    if ( PlayerInput != None && PlayerInput.Outer != MyOwner )
      PlayerInput = None;
    if ( MyOwner.Player != None )
      LocalConsole = MyOwner.Player.Console;
    if ( Level.NetMode == NM_Client )
      SetTimer(0.25 + 0.1 * FRand(), True);  // start finding the ping
  }
  foreach DynamicActors(class'UTP_ReplicationInfo', UTPlusRI) {
    UTPlusRI.Register(Self);
    break;
  }
  Super.PostNetBeginPlay();
  
  LastEnemyDamage       = AccumulatedEnemyDamage;
  LastEnemyHits         = AccumulatedEnemyHits;
  LastVehicleDamage     = AccumulatedVehicleDamage;
  LastVehicleHits       = AccumulatedVehicleHits;
  LastFriendlyHits      = AccumulatedFriendlyHits;
  LastProtectedHits     = AccumulatedProtectedHits;
  LastEmptyVehicleHits  = AccumulatedEmptyVehicleHits;
  bPostNetBeginPlay     = True;
  
  if ( Level.NetMode == NM_Client )
    bNetNotify = True;
  if ( Level.NetMode != NM_DedicatedServer )
    ValueFader = class'UTP_ValueFader'.static.GetValueFader(Self);
}


//== ServerSetPingCompensationType ============================================
/**
Called by the client on the server to modify the UTPlus configuration.
*/
//=============================================================================

function ServerSetPingCompensationType(UTP_ClientSettings.EPingCompensationType Type)
{
  PingCompensationType = Type;
}


//== PostNetReceive ===========================================================
/**
Checks, whether hitsounds should be played.
*/
//=============================================================================

simulated function PostNetReceive()
{
  local bool bProtectedHit;
  local bool bTeamHit;
  local bool bEmptyVehicleHit;
  local float HitSoundPitchPlayer;
  local float HitSoundPitchVehicle;
  
  Super.PostNetReceive();
  
  if ( !bPostNetBeginPlay ) {
    log("Ignoring hit sound before PostNetBeginPlay");
    return;
  }
  
  if ( AccumulatedEnemyDamage != LastEnemyDamage ) {
    //HitSoundPitchPlayer = 1.5 - FClamp((float(AccumulatedEnemyDamage - LastEnemyDamage) - 10) / 100, 0.0, 1.3);
    HitSoundPitchPlayer = 0.00005 * Square(FMin(AccumulatedEnemyDamage - LastEnemyDamage, 200) - 200) + 0.1;
    LastEnemyDamage = AccumulatedEnemyDamage;
  }
  else if ( AccumulatedEnemyHits != LastEnemyHits ) {
    HitSoundPitchPlayer = -1;
    LastEnemyHits = AccumulatedEnemyHits;
  }
  
  if ( AccumulatedVehicleDamage != LastVehicleDamage ) {
    //HitSoundPitchVehicle = 1.5 - FClamp((float(AccumulatedVehicleDamage - LastVehicleDamage) - 10) / 100, 0.0, 1.3);
    HitSoundPitchVehicle = 0.00005 * Square(FMin(AccumulatedVehicleDamage - LastVehicleDamage, 200) - 200) + 0.1;
    LastVehicleDamage = AccumulatedVehicleDamage;
  }
  else if ( AccumulatedVehicleHits != LastVehicleHits ) {
    HitSoundPitchVehicle = -1;
    LastVehicleHits = AccumulatedVehicleHits;
  }
  
  if ( AccumulatedFriendlyHits != LastFriendlyHits ) {
    bTeamHit = True;
    LastFriendlyHits = AccumulatedFriendlyHits;
  }
  
  if ( AccumulatedProtectedHits != LastProtectedHits ) {
    bProtectedHit = True;
    LastProtectedHits = AccumulatedProtectedHits;
  }
  
  if ( AccumulatedEmptyVehicleHits != LastEmptyVehicleHits ) {
    bEmptyVehicleHit = True;
    LastEmptyVehicleHits = AccumulatedEmptyVehicleHits;
  }
  
  if ( HitSoundPitchPlayer != 0 || HitSoundPitchVehicle != 0 || bTeamHit || bProtectedHit || bEmptyVehicleHit ) {
    ClientPlayHitSound(HitSoundPitchPlayer, HitSoundPitchVehicle, bTeamHit, bProtectedHit, bEmptyVehicleHit);
  }
}


//== Timer ====================================================================
/**
Update the client's ping.
*/
//=============================================================================

simulated function Timer()
{
  if ( Level.Netmode == NM_Client ) {
    ServerRequestPing(1000 * Level.TimeSeconds);
    SetTimer(0.25 + 0.1 * FRand(), True);
  }
}


//== ServerRequestPing ========================================================
/**
Called by the client on the server to request a ping.
*/
//=============================================================================

function ServerRequestPing(float Timestamp)
{
  ClientReplyPing(Timestamp);
}


//== ClientReplyPing ==========================================================
/**
Called on the client when the server replied to a ping request.
*/
//=============================================================================

simulated function ClientReplyPing(float Timestamp)
{
  local float NewPing;
  local int i;
  
  NewPing = 1000 * Level.TimeSeconds - Timestamp;
  if ( NewPing > 0 ) {
    if ( LastPings.Length > class'UTP_ClientSettings'.default.PingSmoothing )
      LastPings.Length = class'UTP_ClientSettings'.default.PingSmoothing;
    
    GamePing = NewPing;
    LastPings.Insert(0, 1);
    LastPings[0] = GamePing;
    
    for (i = 1; i < LastPings.Length; ++i)
      GamePing += LastPings[i] * (LastPings.Length - i) / LastPings.Length;
    
    GamePing *= 2;
    GamePing /= i;
    RealPing = GamePing / Level.TimeDilation;
    
    /*if ( HudOverlay != None && HudOverlay.PingGraph != None ) {
      HudOverlay.PingGraph.AddValue(0.001 * RealPing);
      if ( AssociatedPRI != None )
        HudOverlay.PingGraph.AddPLValue(0.01 * AssociatedPRI.PacketLoss);
    }*/
    
    ServerUpdatePing(GamePing);
    GamePing *= 0.001;
    RepPing   = FMin(Round(0.25 * RealPing), 255);
  }
}


//== ServerUpdatePing =========================================================
/**
Called by the client on the server to modify the UTPlus configuration.
*/
//=============================================================================

function ServerUpdatePing(float NewPing)
{
  GamePing = NewPing * 0.001;
  RepPing  = FMin(Round(0.25 * NewPing / Level.TimeDilation), 255);
}


//== NotifyLevelChange ========================================================
/**
Close the client log file.
*/
//=============================================================================

simulated function NotifyLevelChange()
{
  if ( ClientLog != None ) {
    ClientLog.Logf("Closing log file at" @ class'UTP_Util'.static.Date(Level.Year, Level.Month, Level.Day) @ class'UTP_Util'.static.Time(Level.Hour, Level.Minute, Level.Second));
    ClientLog.Logf("");
    ClientLog.CloseLog();
    ClientLog.Destroy();
  }
  ClientLog = None;
  PlayerInput = None;
  LocalConsole = None;
  OnClientReplyConfig = None;
  bMapChange = True;
}


//== Destroyed ================================================================
/**
Unregister from the UTP replication info.
*/
//=============================================================================

simulated function Destroyed()
{
  Super.Destroyed();
  
  if ( UTPlusRI != None )
    UTPlusRI.Unregister(Self);
  
  NotifyLevelChange();
}


//== ClientPlayHitSound =======================================================
/**
Plays a hit sound on the client.
*/
//=============================================================================

simulated function ClientPlayHitSound(float PlayerPitch, float VehiclePitch, bool bSameTeam, bool bProtected, bool bEmptyVehicle)
{
  local PlayerController PC;
  
  if ( class'UTP_ClientSettings'.default.HitSoundVolume <= 0 )
    return;
  
  if ( !class'UTP_ClientSettings'.default.bVariableHitSounds && PlayerPitch != 0 || PlayerPitch < 0 )
    PlayerPitch = 1;
  if ( !class'UTP_ClientSettings'.default.bVariableHitSounds && VehiclePitch != 0 || VehiclePitch < 0 )
    VehiclePitch = 1;
  
  PC = Level.GetLocalPlayerController();
  if ( Pawn(PC.ViewTarget) != None && Pawn(PC.ViewTarget).Health > 0 && Pawn(PC.ViewTarget).PlayerReplicationInfo == AssociatedPRI ) {
    if ( bSameTeam )
	  	PlayHitSound(PC.ViewTarget, class'UTP_ClientSettings'.default.HitSoundTeam, 1);
    if ( bProtected )
	  	PlayHitSound(PC.ViewTarget, class'UTP_ClientSettings'.default.HitSoundProtected, 1);
    if ( bEmptyVehicle )
	  	PlayHitSound(PC.ViewTarget, class'UTP_ClientSettings'.default.HitSoundEmptyVehicle, 1);
    if ( PlayerPitch > 0 )
      PlayHitSound(PC.ViewTarget, class'UTP_ClientSettings'.default.HitSoundEnemy, PlayerPitch);
    if ( VehiclePitch > 0 )
      PlayHitSound(PC.ViewTarget, class'UTP_ClientSettings'.default.HitSoundVehicle, VehiclePitch);
  }
}


//== PlayHitSound =============================================================
/**
Plays a certain hit sound.
*/
//=============================================================================

simulated function PlayHitSound(Actor ViewTarget, int Num, float Pitch)
{
  if ( HitSounds.Length <= Num % HitSoundNames.Length || HitSounds[Num % HitSoundNames.Length] == None ) {
    HitSounds[Num % HitSoundNames.Length] = Sound(DynamicLoadObject(HitSoundNames[Num % HitSoundNames.Length], class'Sound'));
  }
  ViewTarget.PlaySound(HitSounds[Num % HitSoundNames.Length],, class'UTP_ClientSettings'.default.HitSoundVolume,,, Pitch, False);
}


//== AddConsoleMessage ========================================================
/**
Adds a console message to the message buffer.
*/
//=============================================================================

simulated function AddConsoleMessage(string Msg)
{
  if ( LocalConsole == None ) {
    BufferedConsoleMessages[BufferedConsoleMessages.Length] = Msg;
  }
  else {
    while (BufferedConsoleMessages.Length > 0) {
      LocalConsole.Chat(BufferedConsoleMessages[0], 10.0, AssociatedPRI);
      BufferedConsoleMessages.Remove(0, 1);
    }
    LocalConsole.Chat(Msg, 10.0, AssociatedPRI);
  }
}


//== LogConsoleMessage ========================================================
/**
Implements Multi-Dodging features.
*/
//=============================================================================

simulated function LogConsoleMessage(string Msg)
{
  local GameReplicationInfo tmpGRI;
  local PlayerController tmpPC;
  
  if ( ClientLog == None && !bMapChange ) {
    ClientLog = Spawn(class'FileLog', Self);
    if ( ClientLog != None ) {
      ClientLog.OpenLog("UTP_ClientConsole");
      ClientLog.Logf("");
      ClientLog.Logf(class'UTP_Util'.static.RepeatChar("=", 79));
      ClientLog.Logf(class'UTP_Util'.static.GetVersionString() @ class'UTP_Util'.static.GetCopyrightString());
      foreach DynamicActors(class'GameReplicationInfo', tmpGRI) {
        break;
      }
      tmpPC = Level.GetLocalPlayerController();
      ClientLog.Logf("Playing on" @ tmpGRI.ServerName @ "(" $ tmpPC.GetServerNetworkAddress() $ ")");
      ClientLog.Logf(class'UTP_Util'.static.RepeatChar("=", 79));
    }
  }
  if ( ClientLog != None ) {
    ClientLog.Logf(class'UTP_Util'.static.Date(Level.Year, Level.Month, Level.Day) @ class'UTP_Util'.static.Time(Level.Hour, Level.Minute, Level.Second) @ class'UTP_Util'.static.StripCodes(Msg));
  }
}


//== ClientMessage ============================================================
/**
Implements Multi-Dodging features.
*/
//=============================================================================

simulated function ClientMessage(string Msg, optional bool bNoLog)
{
  if ( Level.NetMode == NM_Client || Level.GetLocalPlayerController() == MyOwner ) {  
    AddConsoleMessage(Msg);
    if ( !bNoLog && class'UTP_ClientSettings'.default.bLogUTPlusConsoleActivity )
      LogConsoleMessage(Msg);
  }
}


//== QueueClientMessage =======================================================
/**
Adds a client message to the message queue.
*/
//=============================================================================

function QueueClientMessage(string Msg, optional bool bNoLog)
{
  while (Msg != "") {
    PendingClientMessages[PendingClientMessages.Length] = Left(Msg, 100);
    Msg = Mid(Msg, 100);
    NoLogMessages[NoLogMessages.Length] = byte(bNoLog);
  }
}


//== Tick =====================================================================
/**
Implements Multi-Dodging features, stores player names for logging and sends queued client messages.
*/
//=============================================================================

simulated function Tick(float DeltaTime)
{
  local bool bWasActive;
  local WeaponFire WF;
  local vector X, Y, Z;
  local rotator NewRot;
  local Controller ControllerOwner;
  local Pawn OwnerPawn;
  local ONSWeapon Weapon;
  
  if ( AssociatedPRI != None && AssociatedPRI.PlayerName != "" && LastName != AssociatedPRI.PlayerName ) {
    //log("Player changed name from '" $ LastName $ "' to '" $ AssociatedPRI.PlayerName, 'UTPlusDebug');
    LastName = AssociatedPRI.PlayerName;
    if ( LastName != "" )
      LastNames[LastNames.Length] = LastName;
  }
  
  if ( MyOwner != None && Level.NetMode != NM_Client && PendingClientMessages.Length > 0 ) {
    ClientMessage(PendingClientMessages[0], bool(NoLogMessages[0]));
    PendingClientMessages.Remove(0, 1);
    NoLogMessages.Remove(0, 1);
  }
  if ( MyOwner != None ) {
    NewRot = MyOwner.GetViewRotation();
    if ( ViewRotation != NewRot )
      ViewRotation = NewRot;
  }
  else if (Level.NetMode != NM_Client && Level.TimeSeconds - LastViewRotationUpdate > 0.2) {
    ControllerOwner = Controller(Owner);
    if (ControllerOwner != None)
      OwnerPawn = ControllerOwner.Pawn;
    if (OwnerPawn != None) {
      if (xPawn(OwnerPawn) != None && OwnerPawn.Weapon != None) {
        NewRot = OwnerPawn.GetViewRotation();
        if ( ControllerOwner.Enemy != None && ControllerOwner.CanSee(ControllerOwner.Enemy) ) {
          WF = OwnerPawn.Weapon.GetFireMode(OwnerPawn.Weapon.BotMode);
          if (WF == None || !WF.IsFiring())
            WF = OwnerPawn.Weapon.GetFireMode(OwnerPawn.Weapon.BestMode());
          if (WF != None) {
            GetAxes(NewRot, X, Y, Z);
            NewRot = WF.AdjustAim(WF.GetFireStart(X, Y, Z), 0.0);
          }
        }
      }
      else if ( ONSVehicle(OwnerPawn) != None && ONSVehicle(OwnerPawn).ActiveWeapon < ONSVehicle(OwnerPawn).Weapons.Length && ONSVehicle(OwnerPawn).Weapons[ONSVehicle(OwnerPawn).ActiveWeapon] != None) {
        Weapon = ONSVehicle(OwnerPawn).Weapons[ONSVehicle(OwnerPawn).ActiveWeapon];
        Weapon.CalcWeaponFire();
        if (Weapon.bCorrectAim)
          NewRot = Weapon.AdjustAim(False);
        else
          NewRot = Weapon.WeaponFireRotation;
      }
      else if ( ONSWeaponPawn(OwnerPawn) != None && ONSWeaponPawn(OwnerPawn).Gun != None) {
        Weapon = ONSWeaponPawn(OwnerPawn).Gun;
        Weapon.CalcWeaponFire();
        if (Weapon.bCorrectAim)
          NewRot = Weapon.AdjustAim(False);
        else
          NewRot = Weapon.WeaponFireRotation;
      }
      else
        NewRot = OwnerPawn.Rotation;
      
      LastViewRotationUpdate = Level.TimeSeconds;
      if ( ViewRotation != NewRot )
        ViewRotation = NewRot;
    }
  }
  
  if ( MyOwner == None || UTPlusRI == None || !UTPlusRI.bMultiDodgingEnabled )
    return;
  
  if ( MyOwner.Pawn != None && MyOwner.Pawn.Physics == PHYS_Walking )
    NumDodgesPerJump = UTPlusRI.MaxDodgesPerJump;
  
  if ( UTPlusRI.bFastDodging && PlayerInput != None && PlayerInput.DoubleClickTimer < PlayerInput.DoubleClickTime && MyOwner.DoubleClickDir == DCLICK_None && LastDoubleClickDir > DCLICK_None && LastDoubleClickDir < DCLICK_Active ) {
    if ( MyOwner.DoubleClickDir == DCLICK_Active )
      bWasActive = True;
    PlayerInput.DoubleClickTimer = PlayerInput.DoubleClickTime;
    MyOwner.DoubleClickDir = LastDoubleClickDir;
  }
  
  if ( bWasActive || MyOwner.DoubleClickDir == DCLICK_Active ) {
    if ( NumDodgesPerJump != 0 ) {
      NumDodgesPerJump--;
      if ( !UTPlusRI.bFastDodging || !bWasActive )
        MyOwner.ClearDoubleClick();
      MyOwner.DoubleClickDir = DCLICK_None;
    }
    if ( UTPlusRI.bAlwaysAllowDodgeDoubleJump && xPawn(MyOwner.Pawn) != None && xPawn(MyOwner.Pawn).MultiJumpRemaining == 0 )
      xPawn(MyOwner.Pawn).MultiJumpRemaining = 1;
  }
  LastDoubleClickDir = MyOwner.DoubleClickDir;
}


//== GetViewRotation ==========================================================
/**
Remember the last ticks' locations for movement prediction.
*/
//=============================================================================

simulated function rotator GetViewRotation()
{
  if ( LastViewRotation != ViewRotation ) {
    LastViewRotation = ViewRotation;
    if ( RotationFaderIndex == -1 )
      RotationFaderIndex = ValueFader.AddRotatorFader(ViewRotation, 0.21,,, True);
    else
      ValueFader.AddRotatorFadeValue(RotationFaderIndex, ViewRotation);
  }
  if ( RotationFaderIndex == -1 ) {
    //log("ViewRotation:"@ViewRotation, Owner.Name);
    return ViewRotation;
  }
  //log("faded ViewRotation:"@ValueFader.GetFadedRotatorValue(RotationFaderIndex), Owner.Name);
  return ValueFader.GetFadedRotatorValue(RotationFaderIndex);
}


//== ClientReset ==============================================================
/**
Reset the game clientsidely.
*/
//=============================================================================

simulated function ClientReset(int NewElapsedTime, int NewRemainingTime)
{
  local float Atten;
  
  if ( Level.NetMode == NM_Client ) {
    if ( UTPlusRI != None )
      UTPlusRI.ClientReset(NewElapsedTime, NewRemainingTime);
    
    MultiKillLevel = 0;
    LastKillTime = 0.0;
  }
  
  if ( MyOwner == None || Level.NetMode == NM_DedicatedServer || MyOwner.AnnouncerLevel == 0 )
    return;
  MyOwner.LastPlaySound  = Level.TimeSeconds;   // so voice messages won't overlap
  MyOwner.LastPlaySpeech = Level.TimeSeconds;   // don't want chatter to overlap announcements
  
  Atten = 2.0 * FClamp(0.1 + float(MyOwner.AnnouncerVolume) * 0.225, 0.2, 1.0);
  MyOwner.ClientPlaySound(Sound'AnnouncerMain.Reset', true, Atten, SLOT_Talk);
}


//== static GetDescriptionText ================================================
/**
Returns a description for the specified configuration option.
*/
//=============================================================================

static event string GetDescriptionText(string PropName)
{
  switch (PropName) {
    case "HitSoundNames": return default.descHitSoundNames;
  }
  
  return Super.GetDescriptionText(PropName);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bOnlyDirtyReplication = True
  //bNetNotify = True
  descHitSoundNames = "The list of available hit sounds."
  MyLocalID = -1
  NetUpdateFrequency = 10
  RotationFaderIndex = -1
  
  // HitSounds
  HitSoundNames[0]  = "UTPlusSounds.HitSounds.Quake3"
  HitSoundNames[1]  = "UTPlusSounds.HitSounds.Quake3Friendly"
  HitSoundNames[2]  = "UTPlusSounds.HitSounds.TTM"
  HitSoundNames[3]  = "UTPlusSounds.HitSounds.UTPure"
  HitSoundNames[4]  = "UTPlusSounds.HitSounds.Tournament1"
  HitSoundNames[5]  = "UTPlusSounds.HitSounds.Tournament2"
  HitSoundNames[6]  = "UTPlusSounds.HitSounds.TournamentFriendly"
  HitSoundNames[7]  = "UTPlusSounds.HitSounds.TournamentProtected"
  HitSoundNames[8]  = "UTPlusSounds.HitSounds.UTPlus1"
  HitSoundNames[9]  = "UTPlusSounds.HitSounds.UTPlus2"
  HitSoundNames[10] = "UTPlusSounds.HitSounds.UTPlus3"
  HitSoundNames[11] = "UTPlusSounds.HitSounds.UTPlus4"
}