/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Replicates certain information to all clients.
******************************************************************************/


class UTP_ReplicationInfo extends ReplicationInfo;


//=============================================================================
// Localization
//=============================================================================

var localized string StartedDemoRecordingMessage;
var localized string StoppedDemoRecordingMessage;


//=============================================================================
// Variables
//=============================================================================

var PlayerController                  LocalPlayer;
var UTP_PlayerReplicationInfo         LocalUTPPRI;
var array<UTP_PlayerReplicationInfo>  UTPPRIs;
var UTP_HudOverlay                    HudOverlay;
var UTP_PawnPredicter                 LocalPawnPredicter;
var UTP_MapSettings                   MapSettings;
//var UTP_ClientPlayerIDs               PlayerIDs;
var UTP_ScoutController               DemoEndDetector;
var UTP_Interaction                   MyInteraction;
var UTP_CamPathManager                PathManager;
var AudioSubsystem                    AudioSubsystem;

var GUI.GUITabItem UTPMenuPanel;

var bool bAddedPanel, bAddedInteraction;
var bool bUpdatedCharacterName;
var bool bWantDemoRecording;
var bool bLastShouldDemoRecord, bDemoRecInProgress;
var bool bNotFirstCall;
var byte LastRestartCount;
var byte LastGameScreenshotCount, LastScoreScreenshotCount;
var transient bool bNotFirstTick;


//=============================================================================
// Replicated Variables
//=============================================================================

var() editconst bool bAllowPingPrediction;
var() editconst bool bServerPingCompensationAvailable;

// player visibility
var() editconst bool bAllowColors;
var() editconst bool bAllowDeRes;
var() editconst bool bAllowBrighterSkins;
var() editconst byte SkinGlow;
var() editconst int  TeamBeaconMaxDist;
var() editconst int  TeamBeaconPlayerInfoMaxDist;

// hit sounds
var() editconst bool bHitSoundsAvailable;
var() editconst bool bAllowVariableHitSounds;
var() editconst bool bLineOfSightHitSounds;

// multi-dodging
var() editconst bool bMultiDodgingEnabled;
var() editconst bool bBoostDodgeEnabled;
var() editconst int  MaxDodgesPerJump;
var() editconst bool bAlwaysAllowDodgeDoubleJump;
var() editconst bool bPawnsIgnoreTerminalVelocity;
var() editconst bool bFastDodging;
var() editconst bool bImprovedSlopeDodging;

// death messages
var() editconst bool bUDamageInDeathMessages;

// misc
var() editconst bool bAdvancedAdmin, bRemoteAdmin, bSilentAdminLogin;
var() editconst bool bNoDoubleDamage, bLessLockdown, bFixShieldBug, bEnableProjectileFix;
var() editconst float AmpDamageMultiplier;

// game state
var bool bShouldDemoRecord  "Determines, whether a demo should be recorded.";
var byte RestartCount       "When this value changes while a demo is being recorded, a new demo should be started.";
var byte GameScreenshotCount    "When this value changes, clients should take a HUD screenshot.";
var byte ScoreScreenshotCount   "When this value changes, clients should take a scoreboard screenshot.";
var int GlobalTimerCount, StartTimerCount, LastTimerCount;
var float GlobalTimerIncrement;
var float RealTimeDilation;


//=============================================================================
// Replication
//=============================================================================

replication
{
  // client notices for demo recording and screenshots
  reliable if ( Role == ROLE_Authority )
    bShouldDemoRecord, RestartCount, GameScreenshotCount, ScoreScreenshotCount;
  
  // client timer and timing updates
  unreliable if ( Role == ROLE_Authority )
    GlobalTimerCount, RealTimeDilation;
  
  // ping compensation, bright skins, hit sounds
  reliable if ( bNetInitial && Role == ROLE_Authority )
    bAllowPingPrediction, bServerPingCompensationAvailable, bAllowColors, bHitSoundsAvailable, bMultiDodgingEnabled, bAdvancedAdmin, bUDamageInDeathMessages, bRemoteAdmin, bSilentAdminLogin, bNoDoubleDamage, bLessLockdown, bFixShieldBug, AmpDamageMultiplier, bEnableProjectileFix;
  
  // player visibility
  reliable if ( bNetInitial && bAllowColors && Role == ROLE_Authority )
    bAllowDeRes, bAllowBrighterSkins, SkinGlow, TeamBeaconMaxDist, TeamBeaconPlayerInfoMaxDist;
  
  // hit sounds
  reliable if ( bNetInitial && bHitSoundsAvailable && Role == ROLE_Authority )
    bAllowVariableHitSounds, bLineOfSightHitSounds;
  
  // multi-dodging
  reliable if ( bNetInitial && bMultiDodgingEnabled && Role == ROLE_Authority )
    bBoostDodgeEnabled, MaxDodgesPerJump, bAlwaysAllowDodgeDoubleJump, bFastDodging, bPawnsIgnoreTerminalVelocity, bImprovedSlopeDodging;
}


//== static FindRI ============================================================
/**
Returns an existing UTP_ReplicationInfo actor.
*/
//=============================================================================

static function UTP_ReplicationInfo FindRI(Actor Referencer)
{
  local UTP_ReplicationInfo UTP_RI;
  
  if ( Referencer == None )
    return None;
  
  foreach Referencer.DynamicActors(class'UTP_ReplicationInfo', UTP_RI) {
    return UTP_RI;
  }
}


//== static InitRI ============================================================
/**
Returns an existing UTP_ReplicationInfo actor or spawns and returns a new one.
*/
//=============================================================================

static function UTP_ReplicationInfo InitRI(Actor Referencer)
{
  local UTP_ReplicationInfo UTP_RI;
  
  UTP_RI = FindRI(Referencer);
  
  if ( UTP_RI == None && Referencer.Level.NetMode != NM_Client ) {
    //log("Spawning" @ default.Class);
    UTP_RI = Referencer.Spawn(default.Class);
  }
  
  return UTP_RI;
}


//== PredictPawnMovement ======================================================
/**
Predicts a pawn's movement.
*/
//=============================================================================

simulated function vector PredictPawnMovement(Pawn Other, float DeltaTime)
{
  if (Other.Physics == PHYS_None || Other.Physics != PHYS_Falling && Other.Base != None && Other.Velocity == vect(0,0,0))
    return Other.Location;  // never predict local player's or non-moving pawn's location
  
  if ( LocalPawnPredicter == None )
    foreach DynamicActors(class'UTP_PawnPredicter', LocalPawnPredicter) break;
  
  if ( LocalPawnPredicter == None )
    LocalPawnPredicter = Spawn(class'UTP_PawnPredicter', self,, Other.Location);
  
  if ( LocalPawnPredicter == None )
    return Other.Location;
  
  if ( LocalPawnPredicter.MaxGroundSpeed != Other.GroundSpeed )
    LocalPawnPredicter.MaxGroundSpeed = Other.GroundSpeed;
  if ( LocalPawnPredicter.Physics != Other.Physics )
    LocalPawnPredicter.SetPhysics(Other.Physics);
  if ( LocalPawnPredicter.CollisionHeight != Other.CollisionHeight || LocalPawnPredicter.CollisionRadius != Other.CollisionRadius )
    LocalPawnPredicter.SetCollisionSize(LocalPawnPredicter.CollisionRadius, Other.CollisionHeight);
  if ( LocalPawnPredicter.Location != Other.Location )
    LocalPawnPredicter.SetLocation(Other.Location);
  if ( LocalPawnPredicter.Base != Other.Base )
    LocalPawnPredicter.SetBase(Other.Base);
  
  LocalPawnPredicter.Velocity = Other.Velocity;
  /*if ( Other.Base != None )
    LocalPawnPredicter.Velocity += Other.Base.Velocity;*/
  
  LocalPawnPredicter.AutonomousPhysics(DeltaTime);
  LocalPawnPredicter.SetPhysics(PHYS_None);
  if ( Other.Base != None && Other.Base.Physics != PHYS_None )
    return LocalPawnPredicter.Location + Other.Base.Velocity * DeltaTime;
  return LocalPawnPredicter.Location;
}


//== PreBeginPlay =============================================================
/**
Initializes the replication info.
*/
//=============================================================================

event PreBeginPlay()
{
  Super.PreBeginPlay();
  
  if ( !bDeleteMe ) {
    AddToPackageMap();
    RealTimeDilation = Level.TimeDilation;
    bAdvancedAdmin = AccessControlIni(Level.Game.AccessControl) != None;
    DemoEndDetector = Spawn(class'UTP_ScoutController', self);
    if ( DemoEndDetector != None ) {
      DemoEndDetector.OnGameEnded = GameEnded;
      DemoEndDetector.OnRoundEnded = RoundEnded;
    }
  }
}


//== PostBeginPlay ============================================================
/**
Find the UTP replication info.
*/
//=============================================================================

simulated event PostBeginPlay()
{
  local UTP_PlayerReplicationInfo thisUTPPRI;
  local UTP_Pawn thisUTPPawn;
  
  Super.PostBeginPlay();
  
  foreach DynamicActors(class'UTP_Pawn', thisUTPPawn) {
    thisUTPPawn.UTPlusRI = Self;
  }
  foreach DynamicActors(class'UTP_PlayerReplicationInfo', thisUTPPRI) {
    thisUTPPRI.UTPlusRI = Self;
    Register(thisUTPPRI);
  }
  UTPMenuPanel.ClassName = string(class'UTPGUI_MidGamePanel');
  
  if ( Level.NetMode != NM_DedicatedServer ) {
    PathManager = class'UTP_CamPathManager'.static.GetPathManager(Self);
  }
}


//== PostNetBeginPlay =========================================================
/**
Initializes the bPawnsIgnoreTerminalVelocity feature.
*/
//=============================================================================

simulated event PostNetBeginPlay()
{
  local xPawn P;
  local xPickupBase PB;
  
  if ( Level.NetMode == NM_Client && bPawnsIgnoreTerminalVelocity ) {
    foreach DynamicActors(class'xPawn', P)
    P.bIgnoreTerminalVelocity = true;
  }
  StartTimerCount = GlobalTimerCount;
  Super.PostNetBeginPlay();
  
  if ( bNoDoubleDamage && Level.NetMode == NM_Client ) {
    foreach AllActors(class'xPickupBase', PB) {
      if ( class<UDamagePack>(PB.Powerup) != None )  {
        if ( PB.myEmitter != None )
          PB.myEmitter.Destroy();
        PB.bHidden = True;
        PB.ResetStaticFilterState();  // force new bHidden value to be applied for rendering
      }
    }
  }
}


//== Register =================================================================
/**
Registers s UTP player replication info.
*/
//=============================================================================

simulated function Register(UTP_PlayerReplicationInfo NewUTPPRI)
{
  local int i;
  
  if ( NewUTPPRI != None ) {
    /*if ( PlayerIDs != None && NewUTPPRI.MyLocalID == -1 ) {
      NewUTPPRI.MyLocalID = PlayerIDs.AddPlayer(NewUTPPRI.MyGUID, NewUTPPRI.MyIP, NewUTPPRI.LastName);
    }
    if ( NewUTPPRI.MyLocalID != -1 ) {
      while (NewUTPPRI.LastNames.Length > 0) {
        PlayerIDs.AddPlayerName(NewUTPPRI.MyLocalID, NewUTPPRI.LastNames[0]);
        NewUTPPRI.LastNames.Remove(0, 1);
      }
    }*/
  }
  else
    return;
  
  for (i = 0; i < UTPPRIs.Length; ++i) {
    if ( UTPPRIs[i] == None )
      UTPPRIs.Remove(i--, 1);
    else if ( UTPPRIs[i] == NewUTPPRI )
      return;
  }
  
  UTPPRIs[UTPPRIs.Length] = NewUTPPRI;
}


//== Unregister ===============================================================
/**
Registers s UTP player replication info.
*/
//=============================================================================

simulated function Unregister(UTP_PlayerReplicationInfo OldUTPPRI)
{
  local int i;
  
  /*if ( OldUTPPRI != None ) {
    if ( PlayerIDs != None && OldUTPPRI.MyLocalID == -1 ) {
      OldUTPPRI.MyLocalID = PlayerIDs.AddPlayer(OldUTPPRI.MyGUID, OldUTPPRI.MyIP, OldUTPPRI.LastName);
    }
    if ( OldUTPPRI.MyLocalID != -1 ) {
      while (OldUTPPRI.LastNames.Length > 0) {
        PlayerIDs.AddPlayerName(OldUTPPRI.MyLocalID, OldUTPPRI.LastNames[0]);
        OldUTPPRI.LastNames.Remove(0, 1);
      }
    }
  }*/
  for (i = 0; i < UTPPRIs.Length; ++i)
    if ( UTPPRIs[i] == None || UTPPRIs[i] == OldUTPPRI )
      UTPPRIs.Remove(i--, 1);
}


//== NotifyLocalPlayerDead ====================================================
/**
The local player just died, do expensive stuff now.
*/
//=============================================================================

simulated function NotifyLocalPlayerDead(PlayerController PC)
{
  //log("Local player died.", 'UTPlusDebug');
  SavePlayerIDs();
}


//== NotifyLevelChange ========================================================
/**
Notify the UTP_ReplicationInfo that the level is about to change.
*/
//=============================================================================

simulated function NotifyLevelChange()
{
  Disable('Tick');
  
  if (HudOverlay != None)
    HudOverlay.Destroy();
  HudOverlay = None;
  
  if ( PathManager != None )
    PathManager.SavePathNetwork();
  PathManager = None;
  
  SavePlayerIDs();
  //PlayerIDs = None;
  
  MapSettings.Free();
  MapSettings = None;
  
  if ( LocalUTPPRI != None )
    LocalUTPPRI.NotifyLevelChange();
  
  MyInteraction = None;
  AudioSubsystem = None;
}


//== SavePlayerIDs ============================================================
/**
Save the client-side player ID database.
*/
//=============================================================================

simulated function SavePlayerIDs()
{
  /*local UTP_ClientPlayerIDsExport ExportedPlayerIDs;
  local GameInfo Game;
  
  //local int i;
  
  if ( PlayerIDs != None && PlayerIDs.bIsDirty ) {
    Game = Level.Game;
    if ( Game == None )
      Game = Level.GetLocalPlayerController().GetEntryLevel().Game;
    
    if ( Game != None ) {
      log("Saving client player ID database...", 'UTPlus');
      ExportedPlayerIDs = Game.LoadDataObject(class'UTP_ClientPlayerIDsExport', "PlayerIDs", "UTPlusClientIDDatabase");
      if ( ExportedPlayerIDs == None )
        ExportedPlayerIDs = Game.CreateDataObject(class'UTP_ClientPlayerIDsExport', "PlayerIDs", "UTPlusClientIDDatabase");
      if ( ExportedPlayerIDs != None ) {
        PlayerIDs.ExportDatabase(ExportedPlayerIDs);
        if ( !Game.SavePackage("UTPlusClientIDDatabase") )
          log("Error saving player ID database!", 'UTPlusError');
        else
          PlayerIDs.bIsDirty = False;
      }
      else
        log("Error initializing player ID database!", 'UTPlusError');
    }
    else
      log("No GameInfo available - can't save client player ID database", 'UTPlusError');
    
    //log("Saving player IDs...", 'UTPlusDebug');
    /*for (i = 0; i < UTPPRIs.Length; ++i) {
      if ( UTPPRIs[i] != None ) {
        if ( UTPPRIs[i].MyLocalID == -1 ) {
          UTPPRIs[i].MyLocalID = PlayerIDs.AddPlayer(UTPPRIs[i].MyGUID, UTPPRIs[i].MyIP, UTPPRIs[i].LastName);
        }
        if ( UTPPRIs[i].MyLocalID != -1 ) {
          while (UTPPRIs[i].LastNames.Length > 0) {
            PlayerIDs.AddPlayerName(UTPPRIs[i].MyLocalID, UTPPRIs[i].LastNames[0]);
            UTPPRIs[i].LastNames.Remove(0, 1);
          }
        }
      }
    }
    PlayerIDs.SaveDatabase();*/
  }*/
}


//== PostNetReceive ===========================================================
/**
Checks for match start/end events.
*/
//=============================================================================

simulated event PostNetReceive()
{
  local string AutoDemoName;
  local bool bServerDemo;
  
  if ( LocalPlayer == None )
    LocalPlayer = Level.GetLocalPlayerController();
  if ( LocalPlayer != None && LocalPlayer.bDemoOwner )
    return; // demo playback
  
  if ( Level.NetMode != NM_Client )
    bServerDemo = True;
  if ( bLastShouldDemoRecord && bShouldDemoRecord && bNotFirstCall && LastRestartCount != RestartCount ) {
    // should still record a demo, but a restart was requested
    if ( bDemoRecInProgress ) {
      if ( LocalPlayer != None )
        LocalPlayer.Player.Console.DelayedConsoleCommand("StopDemo");
      else
        ConsoleCommand("StopDemo");
      if ( bServerDemo )
        BroadcastLocalizedMessage(class'UTP_DemoMessage', 0);
      else
        LocalPlayer.ClientMessage(StoppedDemoRecordingMessage);
      bDemoRecInProgress = False;
    }
    bLastShouldDemoRecord = False;
  }
  if ( bLastShouldDemoRecord != bShouldDemoRecord ) {
    // demo recording changed
    if ( bShouldDemoRecord && class'UTP_ClientSettings'.default.bAutoDemoRecording ) {
      // should start demo recording
      AutoDemoName = class'UTP_Util'.static.ParseFileName(class'UTP_ClientSettings'.default.AutoDemoName, Self);
      if ( bServerDemo )
        BroadcastLocalizedMessage(class'UTP_DemoMessage', 1);
      else
        LocalPlayer.ClientMessage(StartedDemoRecordingMessage);
      bDemoRecInProgress = True;
      if ( AutoDemoName == "" ) {
        if ( LocalPlayer != None )
          LocalPlayer.Player.Console.DelayedConsoleCommand("DemoRec");
        else
          ConsoleCommand("DemoRec");
      }
      else if ( Level.NetMode == NM_Client ) {
        if ( LocalPlayer != None )
          LocalPlayer.Player.Console.DelayedConsoleCommand("DemoRec UTPAutoClientDemo_" $ AutoDemoName);
        else
          ConsoleCommand("DemoRec UTPAutoClientDemo_" $ AutoDemoName);
      }
      else {
        if ( LocalPlayer != None )
          LocalPlayer.Player.Console.DelayedConsoleCommand("DemoRec UTPAutoDemo_" $ AutoDemoName);
        else
          ConsoleCommand("DemoRec UTPAutoDemo_" $ AutoDemoName);
      }
    }
    else if ( bLastShouldDemoRecord && !bShouldDemoRecord ) {
      // shold stop demo recording
      if ( bDemoRecInProgress ) {
        if ( LocalPlayer != None )
          LocalPlayer.Player.Console.DelayedConsoleCommand("StopDemo");
        else
          ConsoleCommand("StopDemo");
        if ( bServerDemo )
          BroadcastLocalizedMessage(class'UTP_DemoMessage', 0);
        else
          LocalPlayer.ClientMessage(StoppedDemoRecordingMessage);
        bDemoRecInProgress = False;
      }
    }
  }
  if ( bNotFirstCall && MyInteraction != None ) {
    if ( LastScoreScreenshotCount != ScoreScreenshotCount && class'UTP_ClientSettings'.default.bAutoScreenshot ) {
      MyInteraction.bPendingScoresScreenshot = True;
      MyInteraction.bPendingScreenshot = True;
    }
    else if ( LastGameScreenshotCount != GameScreenshotCount ) {
      MyInteraction.bPendingScoresScreenshot = False;
      MyInteraction.bPendingScreenshot = True;
      if ( LocalUTPPRI != None )
        LocalUTPPRI.ClientMessage("Taking automatic game screenshot: UTPAutoShot" $ GameScreenshotCount $ "_" $ class'UTP_Util'.static.ParseFileName("%d_%t_%g_%m", self));
    }
  }
  bNotFirstCall = True;
  bLastShouldDemoRecord = bShouldDemoRecord;
  LastRestartCount = RestartCount;
  LastScoreScreenshotCount = ScoreScreenshotCount;
  LastGameScreenshotCount = GameScreenshotCount;
  Super.PostNetReceive();
}


//== TakeClientScreenshots ====================================================
/**
Take a screenshot of the HUD on all clients. Should be called by e.g. a security actor.
*/
//=============================================================================

function TakeClientScreenshots()
{
  GameScreenshotCount++;
  NetUpdateTime = Level.TimeSeconds - 1;
}


//== MatchStarting ============================================================
/**
Called when the match starts. Tells clients they can start demo recording.
*/
//=============================================================================

function MatchStarting()
{
  if ( !bShouldDemoRecord ) {
    bShouldDemoRecord = True;
    if ( Level.NetMode != NM_DedicatedServer || bWantDemoRecording )
      PostNetReceive();
  }
}


//== GameEnded ================================================================
/**
Called when the game ends. Tells clients to take screenshots and starts a timer for stopping demos.
*/
//=============================================================================

function GameEnded()
{
  ScoreScreenshotCount++;
  SetTimer(5.0, False);
  if ( Level.NetMode != NM_DedicatedServer )
    PostNetReceive();
}


//== MapRestart ===============================================================
/**
Called when the map is manually restarted. Starts a timer for stopping demos.
*/
//=============================================================================

function MapRestart()
{
  SetTimer(1.0, False);
}


//== RoundEnded ===============================================================
/**
Called when a round ends. Tells clients to take screenshots and starts a timer for stopping demos.
*/
//=============================================================================

function RoundEnded()
{
  ScoreScreenshotCount++;
  SetTimer(5.0, False);
  if ( Level.NetMode != NM_DedicatedServer )
    PostNetReceive();
}


//== Timer ====================================================================
/**
Tells clients to stop demo recording.
*/
//=============================================================================

event Timer()
{
  bShouldDemoRecord = False;
  if ( Level.NetMode != NM_DedicatedServer || bWantDemoRecording )
    PostNetReceive();
}


//== Tick =====================================================================
/**
Adjusts all shock balls' visible locations on clients.
*/
//=============================================================================

simulated event Tick(float DeltaTime)
{
  local ShockBall Proj;
  local UT2K4PlayerLoginMenu LoginMenu;
  local MidGamePanel Panel;
  local int i;
  local xUtil.PlayerRecord Rec;
  local float CurrentRolloff;
  //local UTP_ClientPlayerIDsExport ExportedPlayerIDs;
  //local GameInfo Game;
  
  if ( !bNotFirstTick ) {
    bNotFirstTick = True;
    //log(Level.GetLocalURL(), 'UTPlusDebug');
    //log(Level.GetAddressURL(), 'UTPlusDebug');
    //log(Level.GetURLMap(True), 'UTPlusDebug');
  }
  if ( Level.NetMode != NM_Client ) {
    GlobalTimerIncrement += DeltaTime / Level.TimeDilation;
    while (GlobalTimerIncrement > 0.2) {
      //log(Level.TimeSeconds@"GlobalTimerIncrement:"@GlobalTimerIncrement@"GlobalTimerCount:"@GlobalTimerCount);
      GlobalTimerCount++;
      GlobalTimerIncrement -= 0.2;
    }
    
    // check for automatic demo recording
    if ( Level.Game.NumPlayers + Level.Game.NumBots == 0 || Level.Game.IsInState('PendingMatch') && DeathMatch(Level.Game) != None && !DeathMatch(Level.Game).bStartedCountDown ) {
      // no players on the server - stop recording
      if ( bShouldDemoRecord && TimerRate == 0 ) {
        SetTimer(2.0, False);
        /*bShouldDemoRecord = False;
        if ( Level.NetMode != NM_DedicatedServer || bWantDemoRecording )
          PostNetReceive();*/
      }
    }
    else if ( DeathMatch(Level.Game) != None ) {
      if ( !bShouldDemoRecord && (Level.Game.IsInState('MatchInProgress') || Level.Game.IsInState('PendingMatch') && DeathMatch(Level.Game).bStartedCountDown) ) {
        bShouldDemoRecord = True;
        ++RestartCount;
        if ( Level.NetMode != NM_DedicatedServer || bWantDemoRecording )
          PostNetReceive();
      }
      else if ( !bShouldDemoRecord && DeathMatch(Level.Game).ResetCountDown > 0 ) {
        bShouldDemoRecord = True;
        ++RestartCount;
        if ( Level.NetMode != NM_DedicatedServer || bWantDemoRecording )
          PostNetReceive();
      }
    }
  }
  else {
    if ( LastTimerCount != GlobalTimerCount ) {
      //log(Level.TimeSeconds@"GlobalTimerIncrement:"@GlobalTimerIncrement@"GlobalTimerCount:"@GlobalTimerCount@"LastTimerCount:"@LastTimerCount);
      LastTimerCount = GlobalTimerCount;
      GlobalTimerIncrement = 0;
    }
    GlobalTimerIncrement += DeltaTime / Level.TimeDilation;
  }
  if ( Level.NetMode == NM_DedicatedServer )
    return;
  
  if ( LocalPlayer == None )
    LocalPlayer = Level.GetLocalPlayerController();
  if ( LocalPlayer == None )
    return;
  
  if ( !bAddedInteraction ) {
    MyInteraction = UTP_Interaction(LocalPlayer.Player.InteractionMaster.AddInteraction(string(class'UTP_Interaction'), LocalPlayer.Player));
    if (MyInteraction != None && MyInteraction.bClientDemoPlayback) {
      for (i = 0; i < class'UTP_DemoPlaybackSettings'.default.CustomDemoInteractions.Length; i++) {
        LocalPlayer.Player.InteractionMaster.AddInteraction(class'UTP_DemoPlaybackSettings'.default.CustomDemoInteractions[i], LocalPlayer.Player);
      }
    }
    bAddedInteraction = True;
  }
  
  if ( AudioSubsystem == None && Level.NextSwitchCountdown == 0 )
    foreach AllObjects(class'AudioSubsystem', AudioSubsystem) break;
  
  if ( AudioSubsystem != None ) {
    CurrentRolloff = float(AudioSubsystem.GetPropertyText("Rolloff"));
    if ( CurrentRolloff != FMax(class'UTP_ClientSettings'.default.RolloffFactor, 0.4) ) {
      ConsoleCommand("ROLLOFF"@FMax(class'UTP_ClientSettings'.default.RolloffFactor, 0.4));
      log("Current Rolloff:" @ CurrentRolloff @ "New Rolloff:" @ AudioSubsystem.GetPropertyText("Rolloff"));
    }
  }
  
  /*if ( PlayerIDs == None && !LocalPlayer.bDemoOwner ) {
    Game = Level.Game;
    if ( Game == None )
      Game = Level.GetLocalPlayerController().GetEntryLevel().Game;
    
    // load client player ID database
    if ( Game != None ) {
      PlayerIDs = new(None) class'UTP_ClientPlayerIDs';
      if ( PlayerIDs != None ) {
        ExportedPlayerIDs = Game.LoadDataObject(class'UTP_ClientPlayerIDsExport', "Database", "UTPlusClientIDDatabase");
        if ( PlayerIDs == None ) {
          ExportedPlayerIDs = Game.CreateDataObject(class'UTP_ClientPlayerIDsExport', "Database", "UTPlusClientIDDatabase");
          if ( ExportedPlayerIDs == None )
            log("Could not create client ID database!", 'UTPlusError');
        }
        if ( ExportedPlayerIDs != None )
          PlayerIDs.ImportDatabase(ExportedPlayerIDs);
      }
      else
        log("Failed to create client player ID log object!", 'UTPlusError');
    }
  }*/
  
  if ( MapSettings == None ) {
    MapSettings = class'UTP_MapSettings'.static.GetClientConfiguration(Self, string(Outer));
    if ( MapSettings.bForceEnemyModel && !(MapSettings.ForcedEnemyModel ~= GetURLOption("Character")) ) {
      Rec = class'xUtil'.static.FindPlayerRecord(MapSettings.ForcedEnemyModel);
      if ( Rec.Species != None ) {
        // force model is enabled for this map, update player character
        UpdateURL("Character", MapSettings.ForcedEnemyModel, True);
        ConsoleCommand("ChangeCharacter" @ MapSettings.ForcedEnemyModel);
      }
      else
        MapSettings.ForcedEnemyModel = GetURLOption("Character");
    }
  }
  
  if ( bAllowColors ) {
    if ( LocalPlayer.TeamBeaconMaxDist > 0 && int(LocalPlayer.TeamBeaconMaxDist) != TeamBeaconMaxDist )
      LocalPlayer.TeamBeaconMaxDist = TeamBeaconMaxDist;
    if ( LocalPlayer.TeamBeaconPlayerInfoMaxDist > 0 && int(LocalPlayer.TeamBeaconPlayerInfoMaxDist) != TeamBeaconPlayerInfoMaxDist )
      LocalPlayer.TeamBeaconPlayerInfoMaxDist = TeamBeaconPlayerInfoMaxDist;
  }
  
  if ( LocalPlayer.MyHud != None ) {
    if ( class'UTP_ClientSettings'.default.bNoDamageIndicators && HudBase(LocalPlayer.MyHud) != None ) {
      HudBase(LocalPlayer.MyHud).DamageTime[0] = 0;
      HudBase(LocalPlayer.MyHud).DamageTime[1] = 0;
      HudBase(LocalPlayer.MyHud).DamageTime[2] = 0;
      HudBase(LocalPlayer.MyHud).DamageTime[3] = 0;
    }
    if ( class'UTP_ClientSettings'.default.bConstantCrosshairSize && LocalPlayer.MyHud.LastPickupTime > 0 )
      LocalPlayer.MyHud.LastPickupTime = 0;
    
    if ( HudOverlay == None ) {
      HudOverlay = Spawn(class'UTP_HudOverlay');
      if ( HudOverlay != None ) {
        HudOverlay.UTPlusRI = Self;
        LocalPlayer.MyHud.AddHudOverlay(HudOverlay);
        HudOverlay.Hud = LocalPlayer.MyHud;
        HudOverlay.HudBase = HudBase(LocalPlayer.MyHud);
      }
    }
  }
  
  if ( LocalPlayer.bHideSpectatorBeacons != class'UTP_ClientSettings'.default.bHideSpectatorBeacons )
    LocalPlayer.bHideSpectatorBeacons = class'UTP_ClientSettings'.default.bHideSpectatorBeacons;
  
  if ( LocalPlayer.IsA('DemoRecSpectator') ) {
    return;
  }
  
  if ( !bUpdatedCharacterName )
    UpdateCharacterName();
  
  // check for login menu
  if ( !bAddedPanel && GUIController(LocalPlayer.Player.GUIController) != None && UT2K4PlayerLoginMenu(GUIController(LocalPlayer.Player.GUIController).ActivePage) != None ) {
    LoginMenu = UT2K4PlayerLoginMenu(GUIController(LocalPlayer.Player.GUIController).ActivePage);
    if ( LoginMenu != None ) {
      Panel = MidGamePanel(LoginMenu.c_Main.AddTabItem(UTPMenuPanel));
      if ( Panel != None )
        Panel.ModifiedChatRestriction = LoginMenu.UpdateChatRestriction;
    }
    bAddedPanel = True;
  }
  
  if ( LocalPlayer != None && LocalUTPPRI == None ) {
    for (i = 0; i < UTPPRIs.Length; ++i) {
      if ( UTPPRIs[i] != None && UTPPRIs[i].MyOwner == LocalPlayer ) {
        LocalUTPPRI = UTPPRIs[i];
        break;
      }
    }
  }
  
  if ( Level.NetMode != NM_Client )
    return;
  
  if ( bAllowPingPrediction && class'UTP_ClientSettings'.default.bPingCompensationProjectiles && LocalUTPPRI != None ) {
    foreach DynamicActors(class'ShockBall', Proj) {
      if ( Proj.Role < ROLE_AutonomousProxy && Level.TimeSeconds - Proj.LastRenderTime < 0.5 && VSize(Proj.Velocity) > 100 ) {
        if ( Proj.PrePivot != Proj.default.PrePivot + Proj.Velocity * LocalUTPPRI.GamePing )
          Proj.PrePivot = Proj.default.PrePivot + Proj.Velocity * LocalUTPPRI.GamePing;
      }
      else if ( Proj.PrePivot != Proj.default.PrePivot )
        Proj.PrePivot = Proj.default.PrePivot;
    }
  }
}


//== GetCurrentTimeIndex ======================================================
/**
Returns the current time index in real-time seconds.
*/
//=============================================================================

simulated function float GetCurrentTimeIndex(optional bool bServerTime)
{
  local float TimeIndex;
  
  if ( Level.NetMode == NM_Client && LastTimerCount != GlobalTimerCount ) {
    LastTimerCount = GlobalTimerCount;
    GlobalTimerIncrement = 0;
  }
  if ( bServerTime )
    TimeIndex = GlobalTimerCount;
  else
    TimeIndex = GlobalTimerCount - StartTimerCount;
  TimeIndex *= 0.2;
  TimeIndex += GlobalTimerIncrement;  // more precise on server
  
  return TimeIndex;
}


//== UpdateCharacterName ======================================================
/**
Updates the local player's character name.
*/
//=============================================================================

simulated function UpdateCharacterName()
{
  if ( LocalPlayer != None && LocalPlayer.PlayerReplicationInfo != None ) {
    /*if ( bAllowColors && class'UTP_ClientSettings'.default.bForceEnemyModel ) {
      Rec = class'xUtil'.static.FindPlayerRecord(LocalPlayer.PlayerReplicationInfo.CharacterName);
      if ( Rec.Sex ~= "Female" )
        LocalPlayer.SetPawnClass(string(LocalPlayer.PawnClass), class'UTP_ClientSettings'.default.ForcedEnemyFemaleModel);
      else
        LocalPlayer.SetPawnClass(string(LocalPlayer.PawnClass), class'UTP_ClientSettings'.default.ForcedEnemyMaleModel);
    }*/
    bUpdatedCharacterName = True;
  }
}


//== ClientReset ==============================================================
/**
Reset the game clientsidely.
*/
//=============================================================================

simulated function ClientReset(int NewElapsedTime, int NewRemainingTime)
{
  local ReplicationInfo A;
  local int i;
  
  foreach AllActors(class'ReplicationInfo', A) {
    if ( GameReplicationInfo(A) != None ) {
      GameReplicationInfo(A).ElapsedTime = NewElapsedTime;
      GameReplicationInfo(A).RemainingTime = NewRemainingTime;
    }
    else if ( PlayerReplicationInfo(A) != None ) {
      // additional reset
      PlayerReplicationInfo(A).GoalsScored = 0;
      PlayerReplicationInfo(A).bOutOfLives = False;
      PlayerReplicationInfo(A).Kills = 0;
      if ( TeamPlayerReplicationInfo(A) != None ) {
        TeamPlayerReplicationInfo(A).bFirstBlood = False;
        TeamPlayerReplicationInfo(A).WeaponStatsArray.Length = 0;
        TeamPlayerReplicationInfo(A).VehicleStatsArray.Length = 0;
        TeamPlayerReplicationInfo(A).FlagTouches = 0;
        TeamPlayerReplicationInfo(A).FlagReturns = 0;
        TeamPlayerReplicationInfo(A).Suicides = 0;
        TeamPlayerReplicationInfo(A).flakcount = 0;
        TeamPlayerReplicationInfo(A).combocount = 0;
        TeamPlayerReplicationInfo(A).headcount = 0;
        TeamPlayerReplicationInfo(A).ranovercount = 0;
        TeamPlayerReplicationInfo(A).DaredevilPoints = 0;
        for (i = 0; i < ArrayCount(TeamPlayerReplicationInfo(A).Spree); ++i) {
          TeamPlayerReplicationInfo(A).Spree[i] = 0;
        }
        for (i = 0; i < ArrayCount(TeamPlayerReplicationInfo(A).MultiKills); ++i) {
          TeamPlayerReplicationInfo(A).MultiKills[i] = 0;
        }
        for (i = 0; i < ArrayCount(TeamPlayerReplicationInfo(A).Combos); ++i) {
          TeamPlayerReplicationInfo(A).Combos[i] = 0;
        }
      }
    }
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  UTPMenuPanel  = (Caption="UT Plus",Hint="UT Plus Settings")
  bAlwaysTick   = True
  bNetNotify    = True
  
  AmpDamageMultiplier = 2.0
  
  StartedDemoRecordingMessage = "Automatically recording client-side demo..."
  StoppedDemoRecordingMessage = "Automatic client-side demo recording stopped."
}
