/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


UTPlus controller class for in-game round detection and path-related operations during demo playback.
******************************************************************************/


class UTP_ScoutController extends Controller;


//== delegate OnGameEnded =====================================================
/**
Called when the match has ended.
*/
//=============================================================================

delegate OnGameEnded();


//== delegate OnRoundEnded ====================================================
/**
Called when a round has ended.
*/
//=============================================================================

delegate OnRoundEnded();


//== state GameEnded ==========================================================
/**
Calls OnGameEnded delegate.
*/
//=============================================================================

state GameEnded
{
  function BeginState()
  {
    OnGameEnded();
  }

Begin:
  Sleep(0.0);
  GotoState('');
}


//== state RoundEnded =========================================================
/**
Calls OnRoundEnded delegate.
*/
//=============================================================================

state RoundEnded
{
  function BeginState()
  {
    OnRoundEnded();
  }

Begin:
  Sleep(0.0);
  GotoState('');
}


//== GetPathTo ================================================================
/**
Find a path to the specified actor.
*/
//=============================================================================

function Actor GetPathTo(Actor Dest)
{
  local int i;
  local Actor Best;
  local vector Dir;
  
  if ( Dest == None )
    return None;
  
  if ( Pawn.Physics != PHYS_Falling && (RouteGoal != Dest || Level.TimeSeconds - LastRouteFind > 1 + FRand()) ) {
    MoveTarget = FindPathToward(Dest, false);
    if ( MoveTarget == None )
      return Dest;
  }
  
  if ( RouteCache[0] == None )
    return Dest;
  
  if ( RouteCache[1] == None )
    return RouteCache[0];
  
  Best = RouteCache[0];
  Dir = Normal(RouteCache[1].Location - RouteCache[0].Location);
  
  // return furthest visible path in a relatively straight line from first
  for (i = 0; i < 5; i++) {
    if ( RouteCache[i] == None || VSize(Pawn.Location - RouteCache[i].Location) > 2000 )
      break;
    if ( Normal(RouteCache[i].Location - RouteCache[0].Location) Dot Dir > 0.7 && LineOfSightTo(RouteCache[i]) )
      Best = RouteCache[i];
	}
  
  return Best;
}

//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  RemoteRole=ROLE_None
}