/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Removes the Double Damage pickup.
The class name does not include "UTPlus" so the mutator can be filtered
together with the stand-alone NoDoubleDamage mutator without requiring an
additional server filter.
******************************************************************************/


class MutNoDoubleDamage extends Mutator;


//== PostBeginPlay ============================================================
/**
Find the UTP replication info.
*/
//=============================================================================

function PostBeginPlay()
{
  local UTP_ReplicationInfo UTPlusRI;
  
  Super.PostBeginPlay();
  UTPlusRI = class'UTP_ReplicationInfo'.static.FindRI(Self);
  if ( UTPlusRI != None )
    UTPlusRI.bNoDoubleDamage = True;
}


//== CheckReplacement =========================================================
/**
Tell the UTP replication info to hide the UDamage base clientsidely.
*/
//=============================================================================

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
  local int i, j;
  
  if ( UTP_ReplicationInfo(Other) != None )
    UTP_ReplicationInfo(Other).bNoDoubleDamage = True;
  else if ( WildcardBase(Other) != None ) {
    for (i = 0; i < ArrayCount(WildcardBase(Other).PickupClasses); ++i) {
      if ( class<UDamagePack>(WildcardBase(Other).PickupClasses[i]) != None ) {
        for (j = i + 1; j < ArrayCount(WildcardBase(Other).PickupClasses); ++j) {
          WildcardBase(Other).PickupClasses[j - 1] = WildcardBase(Other).PickupClasses[j];
        }
        WildcardBase(Other).PickupClasses[j - 1] = None;
      }
    }
  }
  else if ( xPickupBase(Other) != None && class<UDamagePack>(xPickupBase(Other).Powerup) != None )
    xPickupBase(Other).bHidden = True;
  else if ( UDamagePack(Other) != None )
    return false;
  
  return Super.CheckReplacement(Other, bSuperRelevant);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FriendlyName = "UTPlus v1 No Double Damage"
  GroupName = "DoubleDamage"
  Description = "Removes the Double Damage pickup from the map."
}
