/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


UTPlus main mutator.
******************************************************************************/


class MutUTPlus extends Mutator
    dependson(UTP_ServerUtil);


//=============================================================================
// Configuration
//=============================================================================

var config bool bEnableProjectileFix;
var config bool bEnableColoredDeathMessages;
var config bool bEnableBrightPlayerSkins;
var config bool bEnableHitSounds;
var config bool bEnablePingCompensation;
var config bool bEnableAdvancedMovement;
var config bool bEnableSpawnGrenades;
var config bool bDisableDoubleDamage;
var config bool bLessLockdown;
var config bool bFixShieldBug;
var config bool bAlwaysEnableClientGUI;
var config bool bShowIPsInWhois;
var config bool bAutoServerDemoRecording;
var config string AutoServerDemoName;
var config int EmptyServerRestartDelay;
var config string EmptyServerRestartURL;


//=============================================================================
// Localization
//=============================================================================

var localized string lblEnableProjectileFix;
var localized string lblEnableAdvancedMovement;
var localized string lblEnableBrightPlayerSkins;
var localized string lblEnableColoredDeathMessages;
var localized string lblEnableHitSounds;
var localized string lblEnablePingCompensation;
var localized string lblEnableSpawnGrenades;
var localized string lblDisableDoubleDamage;
var localized string lblLessLockdown;
var localized string lblFixShieldBug;
var localized string lblAlwaysEnableClientGUI;
var localized string lblShowIPsInWhois;
var localized string lblAutoServerDemoRecording;
var localized string lblAutoServerDemoName;
var localized string lblEmptyServerRestartDelay;
var localized string lblEmptyServerRestartURL;

var localized string descEnableProjectileFix;
var localized string descEnableAdvancedMovement;
var localized string descEnableBrightPlayerSkins;
var localized string descEnableColoredDeathMessages;
var localized string descEnableHitSounds;
var localized string descEnablePingCompensation;
var localized string descEnableSpawnGrenades;
var localized string descDisableDoubleDamage;
var localized string descLessLockdown;
var localized string descFixShieldBug;
var localized string descAlwaysEnableClientGUI;
var localized string descShowIPsInWhois;
var localized string descAutoServerDemoRecording;
var localized string descAutoServerDemoName;
var localized string descEmptyServerRestartDelay;
var localized string descEmptyServerRestartURL;


//=============================================================================
// Variables
//=============================================================================

var(UTPlus) noexport const editconst string Build;
var(UTPlus) noexport const editconst string Copyright;

var color WhoAmIColor, ShowIDsColor, WhoIsColor;

var private transient editconst UTP_PlayerIDs  PlayerIDs;
var private transient editconst UTP_PlayerInfo NextRI;
var private transient editconst bool           bPlayerLogin;

var editconst editconstarray array<UTP_PlayerInfo> UTP_PIs;
var editconst editconstarray array<UTP_Extension> Extensions;
var editconst editconstarray array< class<UTP_Extension> > RegisteredExtensionClasses;

var UTP_ReplicationInfo UTPlusRI;
var UTP_GameRules       MyGameRules;

//var string DatabaseName;
//var bool bAlwaysExportDB;
var UTP_AccessControl AccessControl;
var float ServerEmptyTime;
var string DemoName;


//== PreBeginPlay =============================================================
/**
Check, if this mutator is actually required.
Replaces the DMMutator (if available) or add as first mutator in the list.
*/
//=============================================================================

function PreBeginPlay()
{
  if (EmptyServerRestartURL == "") {
    EmptyServerRestartURL = "?restart";
    SaveConfig();
  }
  if ( GetUrlOption("bLessLockdown") != "" )
    bLessLockdown = bool(GetUrlOption("bLessLockdown"));
  if ( GetUrlOption("bFixShieldBug") != "" )
    bFixShieldBug = bool(GetUrlOption("bFixShieldBug"));
  if ( GetUrlOption("bEnableProjectileFix") != "" )
    bEnableProjectileFix = bool(GetUrlOption("bEnableProjectileFix"));
   if ( GetUrlOption("bEnableColoredDeathMessages") != "" )
    bEnableColoredDeathMessages = bool(GetUrlOption("bEnableColoredDeathMessages"));
  if ( GetUrlOption("bAutoServerDemoRecording") != "" )
    bAutoServerDemoRecording = bool(GetUrlOption("bAutoServerDemoRecording"));
  
  foreach DynamicActors(class'UTP_AccessControl', AccessControl) {
    if ( AccessControl.MyMutator == None ) {
      AccessControl.MyMutator = Self;
      break;
    }
    else {
      Destroy();
      return;
    }
  }
  default.RegisteredExtensionClasses.Length = 0;
  Level.Game.BaseMutator.AddMutator(Self);
  
  MyGameRules = class'UTP_GameRules'.static.InitUTPGR(Self);
  MyGameRules.MyMutator = Self;
  
  if ( bAlwaysEnableClientGUI )
    UTPlusRI = class'UTP_ReplicationInfo'.static.InitRI(Self);
  else
    UTPlusRI = class'UTP_ReplicationInfo'.static.FindRI(Self);
  if ( UTPlusRI != None ) {
    UTPlusRI.bRemoteAdmin = True;
    UTPlusRI.bLessLockdown = bLessLockdown;
    UTPlusRI.bFixShieldBug = bFixShieldBug;
    UTPlusRI.bEnableProjectileFix = bEnableProjectileFix;
  }
  if ( Level.Game.DeathMessageClass == class'xDeathMessage' && bEnableColoredDeathMessages ) {
    Level.Game.DeathMessageClass = class'UTP_ColorDeathMessage';
    AddToPackageMap(class'UTP_ServerUtil'.static.GetPackageName(class'UTP_ColorDeathMessage'));
  }
  if ( bEnableAdvancedMovement )
    Level.Game.AddMutator(string(class'MutMultiDodging'));
  if ( bEnablePingCompensation )
    Level.Game.AddMutator(string(class'MutPingCompensation'));
  if ( bEnableBrightPlayerSkins )
    Level.Game.AddMutator(string(class'MutUTPlusBrightSkins'));
  if ( bEnableHitSounds )
    Level.Game.AddMutator(string(class'MutUTPlusHitSounds'));
  if ( bDisableDoubleDamage )
    Level.Game.AddMutator(string(class'MutNoDoubleDamage'));
  if ( bEnableSpawnGrenades )
    Level.Game.AddMutator(string(class'MutUTPlusSpawnGrenades'));
}


//== AddMutator ===============================================================
/**
Prevent self-adding.
*/
//=============================================================================

function AddMutator(Mutator M)
{
  if ( M != Self ) {
    if ( M.Class != Class )
      Super.AddMutator(M);
    else
      M.LifeSpan = 0.1;
  }
}


//== PostBeginPlay ============================================================
/**
Initialize the Player ID database.
*/
//=============================================================================

function PostBeginPlay()
{
  local UTP_PlayerIDsExport ExportedPlayerIDs;
  local string UTPIDDBName;
  
  // URL extensions
  local string Options;
  local array<string> ExtensionClassNames;
  local class<UTP_Extension> ExtensionClass;
  
  // INI extensions
  local int i, j;
  local array<class<UTP_Extension> > ExtensionClasses;
  
  if ( Level.NetMode != NM_Standalone ) {
    PlayerIDs = new(None) class'UTP_PlayerIDs';
    if ( PlayerIDs != None ) {
      UTPIDDBName = "UTPlusIDDatabase_" $ class'UTP_DataStorageID'.static.GetServerID();
      ExportedPlayerIDs = Level.Game.LoadDataObject(class'UTP_PlayerIDsExport', "PlayerIDs", UTPIDDBName);
      if ( ExportedPlayerIDs == None )
        ExportedPlayerIDs = Level.Game.CreateDataObject(class'UTP_PlayerIDsExport', "PlayerIDs", UTPIDDBName);
      if ( ExportedPlayerIDs != None )
        PlayerIDs.ImportDatabase(ExportedPlayerIDs);
      else
        log("Couldn't initialize player database '" $ UTPIDDBName $ "'!", 'UTPlusError');
    }
    else
      log("Failed to create player ID log object!", 'UTPlusError');
  }
  
  Options = Level.GetLocalURL();
  if ( Left(Options, 1) != "?" )
    Options = Mid(Options, Instr(Options, "?"));
  
  if ( class'GameInfo'.static.HasOption(Options, "UTPExtension") && Split(class'GameInfo'.static.ParseOption(Options, "UTPExtension"), ",", ExtensionClassNames) > 0 ) {
    while (ExtensionClassNames.Length > 0) {
      ExtensionClass = class<UTP_Extension>(DynamicLoadObject(ExtensionClassNames[0], class'Class'));
      if ( ExtensionClass != None ) {
        if ( !ExtensionClass.default.bOnlyForDedicatedServer || Level.NetMode == NM_DedicatedServer ) {
          if (!ExtensionClass.default.bConfigShowInServerDetails)
            ExtensionClass.static.ResetConfig("bShowInServerDetails");
          if (ExtensionClass.default.bRequiresAccessContol && AccessControl == None)
            log("UTPlus Extension (URL) requires UTP_AccessControl:" @ ExtensionClass, 'UTPlusWarning');
          else {
            log("UTPlus Extension (URL):" @ ExtensionClass, 'UTPlus');
            Spawn(ExtensionClass);
          }
        }
        else
          log("UTPlus Extension (URL) requires a dedicated server:" @ ExtensionClass, 'UTPlusWarning');
      }
      ExtensionClassNames.Remove(0, 1);
    }
  }
  
  class'UTP_Extension'.static.GetExtensionClasses(ExtensionClasses, ExtensionClassNames, True);
  for (i = 0; i < ExtensionClasses.Length; ++i) {
    if ( ExtensionClasses[i] == None )
      log("Unable to find UTPlus Extension specified in INT file:" @ ExtensionClassNames[i], 'UTPlusWarning');
    else if ( ExtensionClasses[i].default.bLoadExtension ) {
      if ( !ExtensionClasses[i].default.bOnlyForDedicatedServer || Level.NetMode == NM_DedicatedServer ) {
        for (j = 0; j < default.RegisteredExtensionClasses.Length; j++)
          if (default.RegisteredExtensionClasses[j] == ExtensionClasses[i])
            break;
        if (j == default.RegisteredExtensionClasses.Length) {
          if (!ExtensionClasses[i].default.bConfigShowInServerDetails)
            ExtensionClasses[i].static.ResetConfig("bShowInServerDetails");
          
          if (ExtensionClasses[i].default.bRequiresAccessContol && AccessControl == None)
            log("UTPlus Extension (INI) requires UTP_AccessControl:" @ ExtensionClasses[i], 'UTPlusWarning');
          else {
            log("UTPlus Extension (INI):" @ ExtensionClassNames[i], 'UTPlus');
            Spawn(ExtensionClasses[i]);
          }
        }
      }
      else
        log("UTPlus Extension (INI) requires a dedicated server:" @ ExtensionClassNames[i], 'UTPlusWarning');
    }
  }
}


//== CheckReplacement =========================================================
/**
Spawn the security actor for the next connecting player.
*/
//=============================================================================

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
  local int i;
  local bool bReplaced;
  
  if ( bPlayerLogin && PlayerController(Other) != None && MessagingSpectator(Other) == None ) {
    if ( NextRI != None ) {
      NextRI.MyOwner = PlayerController(Other);
      NextRI.MyMutator = Self;
      NextRI.SetOwner(Other);
    }
    bPlayerLogin = False;
  }
  else if ( UTP_Extension(Other) != None ) {
    Extensions[Extensions.Length] = UTP_Extension(Other);
    default.RegisteredExtensionClasses[default.RegisteredExtensionClasses.Length] = UTP_Extension(Other).Class;
    UTP_Extension(Other).PostRegister(Self);
  }
  else if ( UTP_PlayerReplicationInfo(Other) != None ) {
    UTP_PlayerReplicationInfo(Other).OnServerSetConfig      = ConfigSetByClient;
    UTP_PlayerReplicationInfo(Other).OnServerRequestConfig  = ConfigRequestByClient;
  }
  else if ( UTP_Pawn(Other) != None ) {
    UTP_Pawn(Other).bLessLockdown = bLessLockdown;
    UTP_Pawn(Other).bFixShieldBug = bFixShieldBug;
  }
  else if ( UTP_ReplicationInfo(Other) != None ) {
    UTPlusRI = UTP_ReplicationInfo(Other);
    UTPlusRI.bRemoteAdmin = True;
    if ( Level.NetMode == NM_DedicatedServer ) {
      UTPlusRI.bWantDemoRecording = True;
      class'UTP_ClientSettings'.default.bAutoDemoRecording = bAutoServerDemoRecording;
      class'UTP_ClientSettings'.default.AutoDemoName = AutoServerDemoName;
    }
  }
  else if ( Other.IsA('LockRolloffServerActor') ) {
    log(Other.Class @ "is incompatible to UTPlus and has been disabled.", 'UTPlusWarning');
    return True;  // the LockRolloff server actor is incompatible
  }
  else if ( bEnableProjectileFix && Weapon(Other) != None ) {
    for (i = 0; i < ArrayCount(Weapon(Other).FireModeClass); i++) {
      switch (Weapon(Other).FireModeClass[i]) {
        case class'AssaultGrenade':
          Weapon(Other).FireModeClass[i] = class'UTP_AssaultGrenade';
          break;
        case class'FlakAltFire':
          Weapon(Other).FireModeClass[i] = class'UTP_FlakAltFire';
          break;
        case class'RocketFire':
          Weapon(Other).FireModeClass[i] = class'UTP_RocketFire';
          break;
        case class'RocketMultiFire':
          Weapon(Other).FireModeClass[i] = class'UTP_RocketMultiFire';
          break;
      }
    }
  }
  
  for (i = 0; i < Extensions.Length; i++) {
    if ( Extensions[i] == None )
      Extensions.Remove(i--, 1);
    else if ( Extensions[i].bNotifyActorSpawn && Extensions[i].ActorSpawned(Other, byte(bReplaced)) && !bReplaced )
      bReplaced = True;
  }
  
  return !bReplaced;
}


//== GetServerDetails =========================================================
/**
Adds UTPlus extensions to the server details.
*/
//=============================================================================

function GetServerDetails(out GameInfo.ServerResponseLine ServerState)
{
  local int i;
  
  Super.GetServerDetails(ServerState);
  
  for (i = 0; i < Extensions.Length; i++) {
    if (Extensions[i] == None)
      Extensions.Remove(i--, 1);
    else if (Extensions[i].bShowInServerDetails)
        Extensions[i].GetServerDetails(ServerState);
  }
}


//== ConfigSetByClient ========================================================
/**
Sets the server's configuration.
*/
//=============================================================================

function ConfigSetByClient(UTP_PlayerReplicationInfo UTP_PRI, int ConfigData1, int ConfigData2, int ConfigData3, string AutoDemoName)
{
  local int MaxDodgesPerJump;
  
  if ( Level.NetMode != NM_Standalone && !Level.Game.AccessControl.CanPerform(UTP_PRI.MyOwner, "Ms") )
    return;
  
  // main options
  Class.default.bEnableColoredDeathMessages = bool(ConfigData1 & 0x00000001);
  Class.default.bEnableBrightPlayerSkins    = bool(ConfigData1 & 0x00000002);
  Class.default.bEnableHitSounds            = bool(ConfigData1 & 0x00000004);
  Class.default.bEnablePingCompensation     = bool(ConfigData1 & 0x00000008);
  Class.default.bEnableAdvancedMovement     = bool(ConfigData1 & 0x00000010);
  Class.default.bAlwaysEnableClientGUI      = bool(ConfigData1 & 0x00000020);
  Class.default.bShowIPsInWhois             = bool(ConfigData1 & 0x00000040);
  Class.default.bAutoServerDemoRecording    = bool(ConfigData1 & 0x00000080);
  Class.default.AutoServerDemoName          = AutoDemoName;
  Class.default.bEnableSpawnGrenades        = bool(ConfigData1 & 0x00000100);
  Class.default.bDisableDoubleDamage        = bool(ConfigData1 & 0x00000200);
  Class.default.bLessLockdown               = bool(ConfigData1 & 0x00000400);
  Class.default.bFixShieldBug               = bool(ConfigData1 & 0x00000800);
  
  Class.default.bEnableProjectileFix        = bool(ConfigData2 & 0x00001000);
  
  Class.static.StaticSaveConfig();
  
  // spawn grenades
  class'MutUTPlusSpawnGrenades'.default.InitialGrenades = (ConfigData1 >>> 12) & 0xF;
  class'MutUTPlusSpawnGrenades'.static.StaticSaveConfig();
  
  // hit sound options
  class'MutUTPlusHitSounds'.default.bAllowVariableHitSounds = bool(ConfigData1 & 0x00010000);
  class'MutUTPlusHitSounds'.default.bLineOfSightHitSounds   = bool(ConfigData1 & 0x00020000);
  class'MutUTPlusHitSounds'.static.StaticSaveConfig();
  
  // bright skin options
  class'MutUTPlusBrightSkins'.default.bAllowDeRes                 = bool(ConfigData1 & 0x0040000);
  class'MutUTPlusBrightSkins'.default.bAllowBrighterSkins         = bool(ConfigData1 & 0x0080000);
  class'MutUTPlusBrightSkins'.default.SkinGlow                    = byte(ConfigData1 >>> 24);
  class'MutUTPlusBrightSkins'.default.TeamBeaconMaxDist           = ConfigData3 & 0xFFFF;
  class'MutUTPlusBrightSkins'.default.TeamBeaconPlayerInfoMaxDist = ConfigData3 >>> 16;
  class'MutUTPlusBrightSkins'.static.StaticSaveConfig();
  
  // advanced movement options
  MaxDodgesPerJump = ConfigData2 & 0x3FF;
  if ( MaxDodgesPerJump > 999 )
    MaxDodgesPerJump = -1;
  class'MutMultiDodging'.default.MaxDodgesPerJump             = MaxDodgesPerJump;
  class'MutMultiDodging'.default.bAlwaysAllowDodgeDoubleJump  = bool(ConfigData1 & 0x00100000);
  class'MutMultiDodging'.default.bEnableBoostDodge            = bool(ConfigData1 & 0x00200000);
  class'MutMultiDodging'.default.bFastDodging                 = bool(ConfigData1 & 0x00400000);
  class'MutMultiDodging'.default.bPawnsIgnoreTerminalVelocity = bool(ConfigData1 & 0x00800000);
  class'MutMultiDodging'.static.StaticSaveConfig();
  
  // UTAN extension
  class'UTP_UTANPlugin'.default.bLoadExtension = bool(ConfigData2 & 0x00000400);
}


//== ConfigRequestByClient ====================================================
/**
Returns, whether the client may receive the server configuration and sets the parameters accordingly.
*/
//=============================================================================

function bool ConfigRequestByClient(UTP_PlayerReplicationInfo UTP_PRI, out int ConfigData1, out int ConfigData2, out int ConfigData3, out string AutoDemoName)
{
  if ( Level.NetMode == NM_Standalone || Level.Game.AccessControl.CanPerform(UTP_PRI.MyOwner, "Ms") ) {
    
    // int #1
    ConfigData1 = 0;
    if (Class.default.bEnableColoredDeathMessages)                      ConfigData1 += 0x00000001;
    if (Class.default.bEnableBrightPlayerSkins)                         ConfigData1 += 0x00000002;
    if (Class.default.bEnableHitSounds)                                 ConfigData1 += 0x00000004;
    if (Class.default.bEnablePingCompensation)                          ConfigData1 += 0x00000008;
    if (Class.default.bEnableAdvancedMovement)                          ConfigData1 += 0x00000010;
    if (Class.default.bAlwaysEnableClientGUI)                           ConfigData1 += 0x00000020;
    if (Class.default.bShowIPsInWhois)                                  ConfigData1 += 0x00000040;
    if (Class.default.bAutoServerDemoRecording)                         ConfigData1 += 0x00000080;
    if (Class.default.bEnableSpawnGrenades)                             ConfigData1 += 0x00000100;
    if (Class.default.bDisableDoubleDamage)                             ConfigData1 += 0x00000200;
    if (Class.default.bLessLockdown)                                    ConfigData1 += 0x00000400;
    if (Class.default.bFixShieldBug)                                    ConfigData1 += 0x00000800;
    ConfigData1 += ((class'MutUTPlusSpawnGrenades'.default.InitialGrenades << 12)    & 0x0000F000);
    if (class'MutUTPlusHitSounds'.default.bAllowVariableHitSounds)      ConfigData1 += 0x00010000;
    if (class'MutUTPlusHitSounds'.default.bLineOfSightHitSounds)        ConfigData1 += 0x00020000;
    if (class'MutUTPlusBrightSkins'.default.bAllowDeRes)                ConfigData1 += 0x00040000;
    if (class'MutUTPlusBrightSkins'.default.bAllowBrighterSkins)        ConfigData1 += 0x00080000;
    if (class'MutMultiDodging'.default.bAlwaysAllowDodgeDoubleJump)     ConfigData1 += 0x00100000;
    if (class'MutMultiDodging'.default.bEnableBoostDodge)               ConfigData1 += 0x00200000;
    if (class'MutMultiDodging'.default.bFastDodging)                    ConfigData1 += 0x00400000;
    if (class'MutMultiDodging'.default.bPawnsIgnoreTerminalVelocity)    ConfigData1 += 0x00800000;
    ConfigData1 += ((class'MutUTPlusBrightSkins'.default.SkinGlow << 24)             & 0xFF000000);
    
    // int #2
    ConfigData2 = class'MutMultiDodging'.default.MaxDodgesPerJump & 0x3FF;
    if (class'UTP_UTANPlugin'.default.bLoadExtension)   ConfigData2 += 0x0400;
    if (Class.default.bEnableProjectileFix)             ConfigData2 += 0x1000;

    // int #3
    ConfigData3  = class'MutUTPlusBrightSkins'.default.TeamBeaconMaxDist + (class'MutUTPlusBrightSkins'.default.TeamBeaconPlayerInfoMaxDist << 16);
    
    // the string
    AutoDemoName = Left(Class.default.AutoServerDemoName, 50);
    
    return true;
  }
  return false;
}


//== UnregisterExtension ======================================================
/**
Unregisters an extension.
*/
//=============================================================================

function UnregisterExtension(UTP_Extension Extension)
{
  local int i;
  
  if ( Extension == None )
    return;
  
  for (i = 0; i < Extensions.Length; ++i)
    if ( Extensions[i] == Extension )
      Extensions.Remove(i--, 1);
  for (i = 0; i < default.RegisteredExtensionClasses.Length; ++i)
    if ( default.RegisteredExtensionClasses[i] == Extension.Class )
      default.RegisteredExtensionClasses.Remove(i--, 1);
}


//== Timer ====================================================================
/**
Check, whether the server is empty and should be restarted.
*/
//=============================================================================

function Timer()
{
  if (ServerEmptyTime != 0 && ServerEmptyTime < Level.TimeSeconds - Level.TimeDilation * EmptyServerRestartDelay) {
    // restart the empty server
    Level.ServerTravel(EmptyServerRestartURL, false);
    ServerEmptyTime = 0;
  }
}


//== OverrideDownload =========================================================
/**
Wait a bit longer before restarting an empty server if someone tries to download content.
*/
//=============================================================================

event bool OverrideDownload( string PlayerIP, string PlayerID, string PlayerURL, out string RedirectURL )
{
  if (ServerEmptyTime != 0)
    ServerEmptyTime = Level.TimeSeconds;
  
  return Super.OverrideDownload(PlayerIP, PlayerID, PlayerURL, RedirectURL);
}


//== ModifyLogin ==============================================================
/**
Spawn the security actor for the next connecting player.
*/
//=============================================================================

function ModifyLogin(out string Portal, out string Options)
{
  local int i;
  
  if (ServerEmptyTime != 0)
    ServerEmptyTime = Level.TimeSeconds;
  
  for (i = 0; i < Extensions.Length; i++) {
    if (Extensions[i] == None)
      Extensions.Remove(i--, 1);
    else if (Extensions[i].bNotifyLogin)
      Extensions[i].ModifyLogin(Portal, Options);
  }
  
  if ( NextRI != None )
    NextRI.Destroy();
  bPlayerLogin = True;
  NextRI = Spawn(class'UTP_PlayerInfo', Self);
  
  Super.ModifyLogin(Portal, Options);
}


//== PostLogin ================================================================
/**
Register the new player's security actor.
*/
//=============================================================================

function PostLogin(UTP_PlayerInfo LoginRI)
{
  local int i;
  
  if ( LoginRI == None )
    return;
  
  ServerEmptyTime = 0;
  
  if ( NextRI == LoginRI )
    NextRI = None;
  
  if ( bAlwaysEnableClientGUI )
    class'UTP_PlayerReplicationInfo'.static.FindFor(LoginRI);
  
  UTP_PIs[UTP_PIs.Length] = LoginRI;
  for (i = 0; i < Extensions.Length; ++i) {
    if ( Extensions[i] == None )
      Extensions.Remove(i--, 1);
    else
      Extensions[i].PostLogin(LoginRI);
  }
}


//== NotifyLogout =============================================================
/**
Remove the exiting player's UTPRI.
*/
//=============================================================================

function NotifyLogout(Controller Exiting)
{
  local int i;
  
  for (i = 0; i < UTP_PIs.Length; ++i) {
    if ( UTP_PIs[i] == None )
      UTP_PIs.Remove(i--, 1);
    else if ( UTP_PIs[i].MyOwner == Exiting ) {
      UTP_PIs[i].NotifyLogout();
      UTP_PIs.Remove(i--, 1);
    }
  }
  if ( UTP_PIs.Length == 0 && Level.NetMode == NM_DedicatedServer && EmptyServerRestartDelay > 0 && UTPlusRI != None && UTPlusRI.bShouldDemoRecord ) {
    // bShouldDemoRecord tells, whether there's a game in progress (so clients can record it)
    Level.Pauser = None;
    ServerEmptyTime = Level.TimeSeconds;
  }
  Super.NotifyLogout(Exiting);
}


//== NickChange ===============================================================
/**
Log a new player or a player name change.
*/
//=============================================================================

function NickChange(UTP_PlayerInfo RelatedPI)
{
  local int i;
  
  if ( RelatedPI != None && RelatedPI.MyOwner != None && PlayerIDs != None ) {
    if ( RelatedPI.PlayerUID > 0 ) {
      PlayerIDs.AddPlayerName(RelatedPI.PlayerUID, RelatedPI.MyOwner.PlayerReplicationInfo.PlayerName);
    }
    else {
      RelatedPI.PlayerUID = PlayerIDs.LogPlayer(class'UTP_ServerUtil'.static.IDToGUID(RelatedPI.GetPlayerID()), class'UTP_ServerUtil'.static.IPToInt(RelatedPI.GetPlayerIP()), RelatedPI.MyOwner.PlayerReplicationInfo.PlayerName);
      if ( RelatedPI.PlayerUID > 0 ) {
        log("PlayerUID for" @ RelatedPI.MyOwner.PlayerReplicationInfo.PlayerName $ ":" @ RelatedPI.PlayerUID, 'UTPlus');
        if ( RelatedPI.UTPlusPRI != None )
          RelatedPI.UTPlusPRI.MyServerID = RelatedPI.PlayerUID;
      }
      else
        log("!!! No valid PlayerUID for" @ RelatedPI.MyOwner.PlayerReplicationInfo.PlayerName, 'UTPlusWarning');
    }
  }
  
  for (i = 0; i < Extensions.Length; i++) {
    if ( Extensions[i] == None )
      Extensions.Remove(i--, 1);
    else
      Extensions[i].NickChange(RelatedPI);
  }
}


//== GetUTPlusPIFor ===========================================================
/**
Returns the UTP_PlayerInfo for the specified player.
*/
//=============================================================================

function UTP_PlayerInfo GetUTPlusPIFor(PlayerController Other)
{
  local int i;
  
  if ( Other != None ) {
    for (i = 0; i < UTP_PIs.Length; ++i)
      if ( UTP_PIs[i] != None && UTP_PIs[i].MyOwner == Other )
        return UTP_PIs[i];
  }
  return None;
}


//== ServerTraveling ==========================================================
/**
Handle map changes.
*/
//=============================================================================

function ServerTraveling(string URL, bool bItems)
{
  local UTP_PlayerIDsExport ExportedPlayerIDs;
  local string UTPIDDBName;
  local int i;
  
  ServerEmptyTime = 0;
  
  if ( UTPlusRI != None && UTPlusRI.bWantDemoRecording ) {
    UTPlusRI.Timer();
  }
  
  if ( PlayerIDs != None && PlayerIDs.bIsDirty ) {
    UTPIDDBName = "UTPlusIDDatabase_" $ class'UTP_DataStorageID'.static.GetServerID();
    log("Saving player ID database '" $ UTPIDDBName $ "'...", 'UTPlus');
    ExportedPlayerIDs = Level.Game.LoadDataObject(class'UTP_PlayerIDsExport', "PlayerIDs", UTPIDDBName);
    if ( ExportedPlayerIDs == None )
      ExportedPlayerIDs = Level.Game.CreateDataObject(class'UTP_PlayerIDsExport', "PlayerIDs", UTPIDDBName);
    if ( ExportedPlayerIDs != None ) {
      PlayerIDs.ExportDatabase(ExportedPlayerIDs);
      if ( !Level.Game.SavePackage(UTPIDDBName) )
        log("Error saving player ID database!", 'UTPlusError');
    }
    else
      log("Error initializing player ID database!", 'UTPlusError');
    
    /*if ( bAlwaysExportDB ) {
      log("Exporting player database to" @ DatabaseName $ ".ini...", 'UTPlus');
      ExportedPlayerIDs = Level.Game.CreateDataObject(class'UTP_ExportedPlayerIDs', "ExportedDatabase", DatabaseName);
      if ( ExportedPlayerIDs != None ) {
        ExportedPlayerIDs.ClearConfig();
        PlayerIDs.ExportPlayerDB(ExportedPlayerIDs.Player);
        ExportedPlayerIDs.Version = PlayerIDs.Version;
        ExportedPlayerIDs.SaveConfig();
        ExportedPlayerIDs = None;
        Level.Game.DeleteDataObject(class'UTP_ExportedPlayerIDs', "ExportedDatabase", DatabaseName);
        log("Done.", 'UTPlus');
      }
      else
        log("Error exporting database!", 'UTPlus');
    }*/
  }
  
  for (i = 0; i < Extensions.Length; i++) {
    if ( Extensions[i] == None )
      Extensions.Remove(i--, 1);
    else
      Extensions[i].ServerTraveling(URL, bItems);
  }
  
  Super.ServerTraveling(URL, bItems);
}


//== Mutate ===================================================================
/**
Console commands for getting player information.
*/
//=============================================================================

function Mutate(string MutateString, PlayerController Sender)
{
  local UTP_PlayerInfo SenderRI;
  local int i;
  
  Super.Mutate(MutateString, Sender);
  
  for (i = 0; i < Extensions.Length; i++) {
    if ( Extensions[i] == None )
      Extensions.Remove(i--, 1);
    else if ( Extensions[i].bReceiveMutate )
      Extensions[i].Mutate(MutateString, Sender);
  }
  
  if ( Sender == None )
    return;
  
  if ( Left(MutateString, 4) ~= "UTP " )
    MutateString = Mid(MutateString, 4);
  else
    return;
  
  SenderRI = GetUTPlusPIFor(Sender);
  
  if ( Left(MutateString, 7) ~= "ShowIDs" )
    ShowIDs(Mid(MutateString, 8), SenderRI);
  else if ( Left(MutateString, 6) ~= "WhoAmI" )
    WhoAmI(Mid(MutateString, 7), SenderRI);
  else if ( Left(MutateString, 5) ~= "WhoIs" )
    WhoIs(Mid(MutateString, 6), SenderRI);
  else if ( MutateString ~= "Restart" && (Level.NetMode == NM_Standalone || Level.Game.AccessControl != None && Level.Game.AccessControl.CanPerform(Sender, "Mr")) )
    RestartMap();
  else if ( MutateString ~= "HUDShot" && UTPlusRI != None && (Level.NetMode == NM_Standalone || Level.Game.AccessControl != None && Level.Game.AccessControl.CanPerform(Sender, "Xc")) )
    UTPlusRI.TakeClientScreenshots();
}


//== GetPlayerInfo ============================================================
/**
Get information about a single player.
*/
//=============================================================================

function UTP_ServerUtil.TPlayerInfo GetPlayerInfo(UTP_PlayerInfo Player)
{
  local UTP_ServerUtil.TPlayerInfo PlayerInfo;
  
  if ( Player != None ) {
    if ( PlayerIDs == None || Player.PlayerUID == -1 ) {
      PlayerInfo.IP = class'UTP_ServerUtil'.static.IPToInt(Player.GetPlayerIP());
      PlayerInfo.ID = class'UTP_ServerUtil'.static.IDToGUID(Player.GetPlayerID());
      PlayerInfo.Names.Length = 1;
      if ( Player.LastPlayerName != "" )
        PlayerInfo.Names[0] = Player.LastPlayerName;
      else if ( Player.MyOwner != None && Player.MyOwner.PlayerReplicationInfo != None )
        PlayerInfo.Names[0] = Player.MyOwner.PlayerReplicationInfo.PlayerName;
      else
        PlayerInfo.Names[0] = "(n/a)";
    }
    else {
      PlayerInfo = PlayerIDs.GetPlayerInfo(Player.PlayerUID);
    }
  }
  else
    log("MutUTPlus.GetPlayerInfo - No player specified", 'UTPlusError');
  
  return PlayerInfo;
}


//== WhoAmI ===================================================================
/**
Tells the sender about himself.
*/
//=============================================================================

function WhoAmI(string Args, UTP_PlayerInfo Sender)
{
  local UTP_ServerUtil.TPlayerInfo PlayerInfo;
  local int i;
  local string Names;
  
  PlayerInfo = GetPlayerInfo(Sender);
  Sender.ClientMessage("--- WhoAmI ---",, WhoAmIColor);
  Sender.ClientMessage("  Your GUID is" @ class'UTP_ServerUtil'.static.GUIDToID(PlayerInfo.ID, True, True) $ ", your IP address is" @ class'UTP_ServerUtil'.static.IntToIP(PlayerInfo.IP),, WhoAmIColor);
  Sender.ClientMessage("  This combination corresponds to ID #" $ Sender.PlayerUID @ "on this server. Names used by your ID:",, WhoAmIColor);
  Names = " ->";
  for (i = 0; i < PlayerInfo.Names.Length; ++i) {
    if ( Len(Names @ " " @ PlayerInfo.Names[i]) > 80 ) {
      Sender.ClientMessage(Names,, WhoAmIColor);
      Names = " ->";
    }
    Names = Names @ " " @ PlayerInfo.Names[i];
  }
  Sender.ClientMessage(Names,, WhoAmIColor);
  Sender.ClientMessage("  Use 'Mutate UTP WhoIs #" $ Sender.PlayerUID @ "-allips' to display details for all IPs of your GUID.",, WhoAmIColor, True);
  Sender.ClientMessage("--- End of WhoAmI ---",, WhoAmIColor);
  Sender.ClientMessage("See console for WhoAmI details.", True);
}


//== ShowIDs ==================================================================
/**
Displays IDs of all other players.
*/
//=============================================================================

function ShowIDs(string Args, UTP_PlayerInfo Sender)
{
  local int i, j;
  local UTP_ServerUtil.TPlayerInfo PlayerInfo;
  local bool bAllNames, bAdmin;
  local string Names;
  
  bAllNames = class'UTP_ServerUtil'.static.HasParameter(Args, "-names");
  if ( !bAllNames && Args != "" ) {
    Sender.ClientMessage("Usage: Mutate UTP ShowIDs [-names]", True);
    return;
  }
  
  bAdmin = Level.Game.AccessControl.CanPerform(Sender.MyOwner, "Xp");
  Sender.ClientMessage("--- ShowIDs ---",, ShowIDsColor);
  for (i = 0; i < UTP_PIs.Length; ++i) {
    PlayerInfo = GetPlayerInfo(UTP_PIs[i]);
    Sender.ClientMessage("#" $ UTP_PIs[i].PlayerUID $ ":" @ class'UTP_ServerUtil'.static.GUIDToID(PlayerInfo.ID, True, True, !bAdmin) @ "-" @ UTP_PIs[i].MyOwner.PlayerReplicationInfo.PlayerName,, ShowIDsColor);
    if ( bAllNames ) {
      Names = "   ->";
      for (j = 0; j < PlayerInfo.Names.Length; ++j) {
        if ( Len(Names @ " " @ PlayerInfo.Names[j]) > 80 ) {
          Sender.ClientMessage(Names,, ShowIDsColor);
          Names = "   ->";
        }
        Names = Names @ " " @ PlayerInfo.Names[j];
      }
      Sender.ClientMessage(Names,, ShowIDsColor);
    }
  }
  if ( bAllNames )
    Sender.ClientMessage("  The entire output is also available in UTP_ClientConsole.log in your UT2004\\UserLogs directory.",, ShowIDsColor, True);
  else
    Sender.ClientMessage("  Use 'Mutate UTP ShowIDs -names' to display all names for these UIDs.",, ShowIDsColor, True);
  Sender.ClientMessage("  Use 'Mutate UTP WhoIs #id' to display details for individual UIDs.",, ShowIDsColor, True);
  Sender.ClientMessage("--- End of ShowIDs ---",, ShowIDsColor);
  Sender.ClientMessage("See console for ShowIDs details.", True);
}


//== WhoIs ====================================================================
/**
Tells the sender about the specified player.
*/
//=============================================================================

function WhoIs(string Args, UTP_PlayerInfo Sender)
{
  local string Search, Names;
  local int SearchID;
  local bool bAllIPs, bAdmin;
  local UTP_PlayerInfo Target;
  local int i, j;
  local UTP_ServerUtil.TPlayerInfo PlayerInfo;
  local array<int> IDs;
  
  if ( InStr(Args, " ") == -1 || !Divide(Args, " ", Search, Args) ) {
    Search = Args;
    Args = "";
  }
  
  if ( Search == "" || Left(Search, 1) == " " ) {
    Sender.ClientMessage("Usage: Mutate UTP WhoIs [#id|nick] [-allips]", True);
    return;
  }
  
  bAdmin = Level.Game.AccessControl.CanPerform(Sender.MyOwner, "Xp");
  bAllIPs = class'UTP_ServerUtil'.static.HasParameter(Args, "-allips");
  if ( Left(Search, 1) == "#" ) { // search for an ID
    SearchID = int(Mid(Search, 1));
    if ( SearchID > 0 ) {
      for (i = 0; i < UTP_PIs.Length; ++i) {
        if ( UTP_PIs[i].PlayerUID == SearchID ) {
          Target = UTP_PIs[i];
          break;
        }
      }
    }
  }
  if (Target == None) { // search for a nick
    Target = FindPlayerByName(Search);
    if ( Target != None )
      SearchID = Target.PlayerUID;
  }
  if ( Target == None ) {
    if ( PlayerIDs != None && Left(Search, 1) == "#" ) {
      PlayerInfo = PlayerIDs.GetPlayerInfo(SearchID);
    }
    else {
      Sender.ClientMessage("No matching player for WhoIs" @ Search, True);
      return;
    }
  }
  else
    PlayerInfo = GetPlayerInfo(Target);
  
  Sender.ClientMessage("--- WhoIs" @ Search @ "---",, WhoIsColor);
  if ( Target != None )
    Sender.ClientMessage("  Details for ID #" @ Target.PlayerUID $ ":" @ Target.MyOwner.PlayerReplicationInfo.PlayerName,, WhoIsColor);
  else
    Sender.ClientMessage("  Details for inactive ID #" @ SearchID,, WhoIsColor);
  
  if ( bShowIPsInWhois || bAdmin )
    Sender.ClientMessage("  GUID:" @ class'UTP_ServerUtil'.static.GUIDToID(PlayerInfo.ID, True, True, !bAdmin) $ ", IP address:" @ class'UTP_ServerUtil'.static.IntToIP(PlayerInfo.IP, !bAdmin),, WhoIsColor);
  else
    Sender.ClientMessage("  GUID:" @ class'UTP_ServerUtil'.static.GUIDToID(PlayerInfo.ID, True, True, !bAdmin),, WhoIsColor);
  
  Names = "   ->";
  for (i = 0; i < PlayerInfo.Names.Length; ++i) {
    if ( Len(Names @ " " @ PlayerInfo.Names[i]) > 80 ) {
      Sender.ClientMessage(Names,, WhoIsColor);
      Names = "   ->";
    }
    Names = Names @ " " @ PlayerInfo.Names[i];
  }
  Sender.ClientMessage(Names,, WhoIsColor);
  //log("PlayerIDs:"@PlayerIDs);
  if ( PlayerIDs != None ) {
    IDs = PlayerIDs.GetAllPlayers(SearchID);
    if ( IDs.Length > 1 ) {
      if ( bAllIPs ) {
        Sender.ClientMessage("  Other IDs for this GUID:",, WhoIsColor);
        for (j = 0; j < IDs.Length; ++j) {
          if ( IDs[j] != SearchID ) {
            PlayerInfo = PlayerIDs.GetPlayerInfo(IDs[j]);
            if ( bShowIPsInWhois || bAdmin )
              Sender.ClientMessage("  Names for ID #" @ IDs[j] @ "(IP address:" @ class'UTP_ServerUtil'.static.IntToIP(PlayerInfo.IP, !bAdmin) $ ")",, WhoIsColor);
            else
              Sender.ClientMessage("  Names for ID #" @ IDs[j],, WhoIsColor);
            Names = "   ->";
            for (i = 0; i < PlayerInfo.Names.Length; ++i) {
              if ( Len(Names @ " " @ PlayerInfo.Names[i]) > 80 ) {
                Sender.ClientMessage(Names,, WhoIsColor);
                Names = "   ->";
              }
              Names = Names @ " " @ PlayerInfo.Names[i];
            }
            Sender.ClientMessage(Names,, WhoIsColor);
          }
        }
      }
      else {
        Sender.ClientMessage("  There's more data for this player's GUID. (show with 'Mutate UTP WhoIs #" $ SearchID @ "-allips')",, WhoIsColor);
      }
    }
    else if ( bAllIPs ) {
      Sender.ClientMessage("  None.",, WhoIsColor);
    }
  }
  Sender.ClientMessage("--- End of WhoIs ---",, WhoIsColor);
  Sender.ClientMessage("See console for WhoIs details.", True);
}


//== FindPlayerByName =========================================================
/**
Returns the UTP_PlayerInfo of the player with the specified name.
*/
//=============================================================================

function UTP_PlayerInfo FindPlayerByName(string PlayerName)
{
  local int i;
  
  // search for exact name
  for (i = 0; i < UTP_PIs.Length; i++) {
    if ( UTP_PIs[i] != None && UTP_PIs[i].MyOwner != None && UTP_PIs[i].MyOwner.PlayerReplicationInfo != None && class'UTP_ServerUtil'.static.StripCodes(UTP_PIs[i].MyOwner.PlayerReplicationInfo.PlayerName) ~= PlayerName )
      return UTP_PIs[i];
  }
  
  // search for the first name containing the string
  for (i = 0; i < UTP_PIs.Length; i++) {
    if ( UTP_PIs[i] != None && UTP_PIs[i].MyOwner != None && UTP_PIs[i].MyOwner.PlayerReplicationInfo != None && InStr(Caps(class'UTP_ServerUtil'.static.StripCodes(UTP_PIs[i].MyOwner.PlayerReplicationInfo.PlayerName)), Caps(PlayerName)) != -1 )
      return UTP_PIs[i];
  }
  
  return None; // no player found
}


//== RestartMap ===============================================================
/**
Restarts the map without having to reload it.
*/
//=============================================================================

function RestartMap()
{
  local Actor A;
  local Controller C, NextC;
  local bool bTeamScoreRounds;
  local int i;
  
  if ( UnrealMPGameInfo(Level.Game) != None && UnrealMPGameInfo(Level.Game).bTeamScoreRounds ) {
    bTeamScoreRounds = True;
    UnrealMPGameInfo(Level.Game).bTeamScoreRounds = False;
  }
  
  for (C = Level.ControllerList; C != None; C = NextC) {
    NextC = C.NextController;
    if ( PlayerController(C) != None )
      PlayerController(C).ClientReset();
    C.Reset();
  }
  Level.Game.GameReplicationInfo.Reset();
  Level.Game.Reset();
  if ( Level.Game.IsA('ONSOnslaughtGame') || Level.Game.IsA('ASGameInfo') ) {
    // these gametypes don't reset properly
    Level.Game.bGameEnded = false;
    Level.Game.bOverTime = false;
    Level.Game.bWaitingToStartMatch = true;
    Level.Game.InitGameReplicationInfo();
    Level.Game.InitVoiceReplicationInfo();
    DeathMatch(Level.Game).ElapsedTime = DeathMatch(Level.Game).NetWait - 3;
    DeathMatch(Level.Game).bWaitForNetPlayers = Level.NetMode != NM_StandAlone;
    DeathMatch(Level.Game).bStartedCountDown = false;
    DeathMatch(Level.Game).bFinalStartup = false;
    DeathMatch(Level.Game).CountDown = DeathMatch(Level.Game).default.Countdown;
    DeathMatch(Level.Game).RemainingTime = 60 * DeathMatch(Level.Game).TimeLimit;
    if ( ASGameInfo(Level.Game) != None ) {
      if ( Level.NetMode == NM_Standalone )
        ASGameInfo(Level.Game).CurrentRound = 1;
      else
        ASGameInfo(Level.Game).CurrentRound = 0;
      ASGameInfo(Level.Game).LastDisabledObjective	= None;
      ASGameInfo(Level.Game).CurrentObjective		= None;
      
      ASGameReplicationInfo(Level.Game.GameReplicationInfo).bTeamZeroIsAttacking  = ASGameInfo(Level.Game).CurrentAttackingTeam == 0;
      ASGameReplicationInfo(Level.Game.GameReplicationInfo).ObjectiveProgress     = 0;
      ASGameReplicationInfo(Level.Game.GameReplicationInfo).CurrentRound          = ASGameInfo(Level.Game).CurrentRound;
      ASGameReplicationInfo(Level.Game.GameReplicationInfo).RoundWinner           = ERW_None;
      ASGameReplicationInfo(Level.Game.GameReplicationInfo).RoundStartTime        = ASGameInfo(Level.Game).RemainingTime;	// replicated round start stamp
      ASGameInfo(Level.Game).RoundStartTime           = ASGameInfo(Level.Game).RemainingTime;	// local round start stamp
      
      // Set Round Time Limit
      ASGameReplicationInfo(Level.Game.GameReplicationInfo).RoundTimeLimit = ASGameInfo(Level.Game).RoundTimeLimit * 60;		// default round time limit
      
      if ( !ASGameInfo(Level.Game).bDisableReinforcements ) {
        ASGameReplicationInfo(Level.Game.GameReplicationInfo).ReinforcementCountDown = 0;
        ASGameInfo(Level.Game).ReinforcementsCount = ASGameInfo(Level.Game).ReinforcementsFreq;	// Start reinforcements right away
      }
    }
    
    Level.Game.GotoState('PendingMatch');
  }
  if ( DeathMatch(Level.Game) != None ) {
    Level.Game.GameReplicationInfo.RemainingTime    = DeathMatch(Level.Game).RemainingTime;	// for new players joining
    Level.Game.GameReplicationInfo.RemainingMinute  = DeathMatch(Level.Game).RemainingTime;	// re-synch current players
    Level.Game.GameReplicationInfo.NetUpdateTime    = Level.TimeSeconds - 1;
    Level.Game.GameReplicationInfo.bStopCountDown   = true;
    Level.Game.GameReplicationInfo.ElapsedTime      = 0;
  }
  if ( Level.NetMode == NM_Standalone && DeathMatch(Level.Game) != None )
    DeathMatch(Level.Game).InitialBots = 0;
  
  foreach AllActors(class'Actor', A) {
    if ( Controller(A) == None && A != Level && A != Level.Game && A != Level.Game.GameReplicationInfo )
      A.Reset();
    // because assault overrides this
    if ( PlayerReplicationInfo(A) != None ) {
      // default reset
      PlayerReplicationInfo(A).Score = 0;
      PlayerReplicationInfo(A).Deaths = 0;
      PlayerReplicationInfo(A).HasFlag = None;
      PlayerReplicationInfo(A).bReadyToPlay = False;
      PlayerReplicationInfo(A).NumLives = 0;
      PlayerReplicationInfo(A).bOutOfLives = False;
      
      // additional reset
      PlayerReplicationInfo(A).GoalsScored = 0;
      PlayerReplicationInfo(A).bOutOfLives = False;
      PlayerReplicationInfo(A).Kills = 0;
      PlayerReplicationInfo(A).StartTime = Level.Game.GameReplicationInfo.ElapsedTime;
      if ( TeamPlayerReplicationInfo(A) != None ) {
        TeamPlayerReplicationInfo(A).bFirstBlood = False;
        TeamPlayerReplicationInfo(A).WeaponStatsArray.Length = 0;
        TeamPlayerReplicationInfo(A).VehicleStatsArray.Length = 0;
        TeamPlayerReplicationInfo(A).FlagTouches = 0;
        TeamPlayerReplicationInfo(A).FlagReturns = 0;
        TeamPlayerReplicationInfo(A).Suicides = 0;
        TeamPlayerReplicationInfo(A).flakcount = 0;
        TeamPlayerReplicationInfo(A).combocount = 0;
        TeamPlayerReplicationInfo(A).headcount = 0;
        TeamPlayerReplicationInfo(A).ranovercount = 0;
        TeamPlayerReplicationInfo(A).DaredevilPoints = 0;
        for (i = 0; i < ArrayCount(TeamPlayerReplicationInfo(A).Spree); ++i) {
          TeamPlayerReplicationInfo(A).Spree[i] = 0;
        }
        for (i = 0; i < ArrayCount(TeamPlayerReplicationInfo(A).MultiKills); ++i) {
          TeamPlayerReplicationInfo(A).MultiKills[i] = 0;
        }
        for (i = 0; i < ArrayCount(TeamPlayerReplicationInfo(A).Combos); ++i) {
          TeamPlayerReplicationInfo(A).Combos[i] = 0;
        }
      }
    }
    else if ( UTP_PlayerReplicationInfo(A) != None )
      UTP_PlayerReplicationInfo(A).ClientReset(DeathMatch(Level.Game).ElapsedTime, DeathMatch(Level.Game).RemainingTime);
    else if ( UTP_ReplicationInfo(A) != None )
      UTP_ReplicationInfo(A).MapRestart();
    else if ( Pickup(A) != None && Pickup(A).IsSuperItem() )
      A.GotoState('WaitingForMatch');
    
  }
  Level.Reset();
  if (Level.GetLocalPlayerController() != None)
    Level.GetLocalPlayerController().PlayAnnouncement(Sound'Reset', 0, True);
  
  if ( DeathMatch(Level.Game) != None )
    DeathMatch(Level.Game).EndMessageCounter = 0;
  if ( bTeamScoreRounds )
    UnrealMPGameInfo(Level.Game).bTeamScoreRounds = bTeamScoreRounds;
  UnrealMPGameInfo(Level.Game).FindNewObjectives(None);
}


//== static FillPlayInfo ======================================================
/**
Add the configuration options to the PlayInfo.
*/
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  local int i;
  local array<string> ExtensionClassNames;
  local array<class<UTP_Extension> > ExtensionClasses;
  
  if ( class'UTP_ServerUtil'.static.ClassIsInPlayInfo(PlayInfo, default.Class) )
    return;
  
  Super.FillPlayInfo(PlayInfo);
  
  PlayInfo.AddSetting(default.FriendlyName, "bEnableAdvancedMovement",      default.lblEnableAdvancedMovement,     1, 1, "Check",, "Mu");
  PlayInfo.AddSetting(default.FriendlyName, "bEnableProjectileFix",         default.lblEnableProjectileFix,        1, 2, "Check");
  PlayInfo.AddSetting(default.FriendlyName, "bEnableColoredDeathMessages",  default.lblEnableColoredDeathMessages, 1, 3, "Check");
  PlayInfo.AddSetting(default.FriendlyName, "bEnableBrightPlayerSkins",     default.lblEnableBrightPlayerSkins,    1, 4, "Check",, "Mu");
  PlayInfo.AddSetting(default.FriendlyName, "bEnableHitSounds",             default.lblEnableHitSounds,            1, 5, "Check",, "Mu");
  PlayInfo.AddSetting(default.FriendlyName, "bEnablePingCompensation",      default.lblEnablePingCompensation,     1, 6, "Check",, "Mu");
  PlayInfo.AddSetting(default.FriendlyName, "bEnableSpawnGrenades",         default.lblEnableSpawnGrenades,        1, 7, "Check",, "Mu");
  PlayInfo.AddSetting(default.FriendlyName, "bDisableDoubleDamage",         default.lblDisableDoubleDamage,        1, 8, "Check",, "Mu");
  PlayInfo.AddSetting(default.FriendlyName, "bLessLockdown",                default.lblLessLockdown,               1, 9, "Check");
  PlayInfo.AddSetting(default.FriendlyName, "bFixShieldBug",                default.lblFixShieldBug,              1, 10, "Check");
  PlayInfo.AddSetting(default.FriendlyName, "bAlwaysEnableClientGUI",       default.lblAlwaysEnableClientGUI,     1, 11, "Check",, "Mu");
  PlayInfo.AddSetting(default.FriendlyName, "bShowIPsInWhois",              default.lblShowIPsInWhois,          250, 12, "Check",,, True);
  PlayInfo.AddSetting(default.FriendlyName, "bAutoServerDemoRecording",     default.lblAutoServerDemoRecording, 255, 14, "Check",, "Xc", True);
  PlayInfo.AddSetting(default.FriendlyName, "AutoServerDemoName",           default.lblAutoServerDemoName,      255, 15, "Text", "50", "Xc", True);
  PlayInfo.AddSetting(default.FriendlyName, "EmptyServerRestartDelay",      default.lblEmptyServerRestartDelay, 250, 16, "Text", "3;0:300",, True);
  
  if ( default.bEnableAdvancedMovement )
    class'MutMultiDodging'.static.FillPlayInfo(PlayInfo);
  if ( default.bEnableBrightPlayerSkins )
    class'MutUTPlusBrightSkins'.static.FillPlayInfo(PlayInfo);
  if ( default.bEnableHitSounds )
    class'MutUTPlusHitSounds'.static.FillPlayInfo(PlayInfo);
  if ( default.bEnablePingCompensation )
    class'MutPingCompensation'.static.FillPlayInfo(PlayInfo);
  if ( default.bEnableSpawnGrenades )
    class'MutUTPlusSpawnGrenades'.static.FillPlayInfo(PlayInfo);
  
  PlayInfo.PopClass();
  
  class'UTP_Extension'.static.GetExtensionClasses(ExtensionClasses, ExtensionClassNames, True);
  for (i = 0; i < ExtensionClasses.Length; ++i)
    if ( ExtensionClasses[i] != None )
      ExtensionClasses[i].static.FillPlayInfo(PlayInfo);
  
  for (i = 0; i < default.RegisteredExtensionClasses.Length; i++)
    default.RegisteredExtensionClasses[i].static.FillPlayInfo(PlayInfo);
}


//== static GetDescriptionText ================================================
/**
Returns a description for the specified configuration option.
*/
//=============================================================================

static event string GetDescriptionText(string PropName)
{
  switch (PropName) {
    case "bEnableAdvancedMovement":     return default.descEnableAdvancedMovement;
    case "bEnableProjectileFix":        return default.descEnableProjectileFix;
    case "bEnableColoredDeathMessages": return default.descEnableColoredDeathMessages;
    case "bEnableBrightPlayerSkins":    return default.descEnableBrightPlayerSkins;
    case "bEnableHitSounds":            return default.descEnableHitSounds;
    case "bEnablePingCompensation":     return default.descEnablePingCompensation;
    case "bEnableSpawnGrenades":        return default.descEnableSpawnGrenades;
    case "bDisableDoubleDamage":        return default.descDisableDoubleDamage;
    case "bLessLockdown":               return default.descLessLockdown;
    case "bFixShieldBug":               return default.descFixShieldBug;
    case "bAlwaysEnableClientGUI":      return default.descAlwaysEnableClientGUI;
    case "bShowIPsInWhois":             return default.descShowIPsInWhois;
    case "bAutoServerDemoRecording":    return default.descAutoServerDemoRecording;
    case "AutoServerDemoName":          return default.descAutoServerDemoName;
    case "EmptyServerRestartDelay":     return default.descEmptyServerRestartDelay;
  }
  
  return Super.GetDescriptionText(PropName);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Build         = "%%%%-%%-%% %%:%%"
  Copyright     = "Copyright (c) 2005 Wormbo"
  FriendlyName  = "UTPlus v1.0"
  Description   = "UTPlus main mutator. If you want the full range of server-side features, enable the UTPlus AccessControl class instead."
  GroupName     = "UTPlus"
  
  WhoAmIColor  = (R=255,G=255,B=000,A=255)
  ShowIDsColor = (R=128,G=255,B=128,A=255)
  WhoIsColor   = (R=000,G=255,B=000,A=255)
  
  bEnableAdvancedMovement     = False
  bEnableProjectileFix        = True
  bEnableColoredDeathMessages = True
  bEnableBrightPlayerSkins    = False
  bEnableHitSounds            = False
  bEnablePingCompensation     = False
  bEnableSpawnGrenades        = False
  bDisableDoubleDamage        = False
  bLessLockdown               = False
  bFixShieldBug               = False
  bAlwaysEnableClientGUI      = True
  //bEnableUTANv1Plugin         = False
  bShowIPsInWhois             = False
  bAutoServerDemoRecording    = False
  AutoServerDemoName          = "%d_%t_%g_%m"
  EmptyServerRestartDelay     = 10.0
  EmptyServerRestartURL       = "?restart"
  
  lblEnableProjectileFix        = "Enable projectile fix"
  lblEnableAdvancedMovement     = "Enable advanced movement"
  lblEnableBrightPlayerSkins    = "Enable bright player skins"
  lblEnableColoredDeathMessages = "Enable colored death messages"
  lblEnableHitSounds            = "Enable hit sounds"
  lblEnablePingCompensation     = "Enable ping compensation"
  lblEnableSpawnGrenades        = "Enable Spawn Grenades"
  lblDisableDoubleDamage        = "Disable Double Damage"
  lblLessLockdown               = "Less lockdown"
  lblFixShieldBug               = "Fix shield bug"
  lblAlwaysEnableClientGUI      = "Always enable client-side GUI"
  lblShowIPsInWhois             = "Show IPs in WhoIs"
  lblAutoServerDemoRecording    = "Auto demo recording"
  lblAutoServerDemoName         = "Demo file name"
  lblEmptyServerRestartDelay    = "Empty server restart delay"
  lblEmptyServerRestartURL      = "Empty server restart URL"
  
  descEnableProjectileFix         = "Enable this option to activate the netcode fix for assault grenades, flak shells and rockets."
  descEnableAdvancedMovement      = "Enables the 'UTPlus Advanced Movement' mutator. (This makes your server non-standard.)"
  descEnableBrightPlayerSkins     = "Enables the 'UTPlus Bright Skins' mutator. (This makes your server non-standard.)"
  descEnableColoredDeathMessages  = "Enables team-colored death messages."
  descEnableHitSounds             = "Enables the 'UTPlus Hit Sounds' mutator. (This makes your server non-standard.)"
  descEnablePingCompensation      = "Enables the 'UTPlus Ping Compensation' mutator. (This makes your server non-standard.)"
  descEnableSpawnGrenades         = "Enables the 'UTPlus Spawn Grenades' mutator. (This makes your server non-standard.)"
  descDisableDoubleDamage         = "Enables the 'UTPlus No Double Damage' mutator. (This makes your server non-standard.)"
  descLessLockdown                = "Changes the way damage is handled to reduce lockdown."
  descFixShieldBug                = "Changes the way damage is absorbed to prevent small shield energy from being ignored while there's big shield energy left."
  descAlwaysEnableClientGUI       = "Always enable the client-side UTPlus HUD and menu extensions."
  descShowIPsInWhois              = "This will show other players' IPs in the WhoIs and WhoWas reply. The IP's last part will only be visible for admins."
  descAutoServerDemoRecording     = "Automatically record server-side demos. (only on dedicated servers)"
  descAutoServerDemoName          = "Enter the file name for automatically recorded server-side demos. The file name will be prefixed with 'UTPAutoDemo_'."
  descEmptyServerRestartDelay     = "Automatically restart the map after this number of seconds when the last player leaves the game."
  descEmptyServerRestartURL       = "Enter ?restart to simply restart the server after the last player leaves or enter a map name, possibly with URL parameters for gametype, mutators, etc."
}