/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Enhances player visibility by adding more color to the player skins.
******************************************************************************/


class MutUTPlusBrightSkins extends Mutator
    config;


//=============================================================================
// Configuration
//=============================================================================

var config bool bAllowDeRes;
//var config bool bAllowPlayerLights;
var config bool bAllowBrighterSkins;
var config byte SkinGlow;
var config int TeamBeaconMaxDist;
var config int TeamBeaconPlayerInfoMaxDist;


//=============================================================================
// Localization
//=============================================================================

var localized string lblAllowDeRes;
//var localized string lblAllowPlayerLights;
var localized string lblAllowBrighterSkins;
var localized string lblSkinGlow;
var localized string lblTeamBeaconMaxDist;
var localized string lblTeamBeaconPlayerInfoMaxDist;

var localized string descAllowDeRes;
//var localized string descAllowPlayerLights;
var localized string descAllowBrighterSkins;
var localized string descSkinGlow;
var localized string descTeamBeaconMaxDist;
var localized string descTeamBeaconPlayerInfoMaxDist;


//== PostBeginPlay ============================================================
/**
Find the UTP replication info.
*/
//=============================================================================

function PostBeginPlay()
{
  local UTP_ReplicationInfo UTPlusRI;
  
  Super.PostBeginPlay();
  
  UTPlusRI = class'UTP_ReplicationInfo'.static.InitRI(Self);
  UTPlusRI.bAllowColors                 = True;
  UTPlusRI.bAllowDeRes                  = bAllowDeRes;
  UTPlusRI.bAllowBrighterSkins          = bAllowBrighterSkins;
  UTPlusRI.SkinGlow                     = SkinGlow;
  UTPlusRI.TeamBeaconMaxDist            = Clamp(TeamBeaconMaxDist, 2000, 20000);
  UTPlusRI.TeamBeaconPlayerInfoMaxDist  = Clamp(TeamBeaconPlayerInfoMaxDist, 1000, Min(10000, TeamBeaconMaxDist));
  
  if ( Level.Game.DefaultPlayerClassName ~= string(class'xPawn') || Level.Game.DefaultPlayerClassName ~= string(class'xMutantPawn') )
    Level.Game.DefaultPlayerClassName = string(class'UTP_Pawn');
  
  if ( UTPlusRI.bAllowDeRes ) {
    AddToPackageMap(class'UTP_ServerUtil'.static.GetPackageName(class'UTP_Pawn'.default.DeResBaseBody));
    AddToPackageMap(class'UTP_ServerUtil'.static.GetPackageName(class'UTP_Pawn'.default.DeResBaseHead));
  }
}


//== CheckReplacement =========================================================
/**
Replace the pawn class for all controllers.
*/
//=============================================================================

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
  if ( Controller(Other) != None && Controller(Other).bIsPlayer && (Controller(Other).PawnClass == None || Controller(Other).PawnClass == class'xPawn' || Controller(Other).PawnClass == class'xMutantPawn') ) {
    //log("CheckReplacement"@Other);
    Controller(Other).PawnClass = class'UTP_Pawn';
    Controller(Other).PreviousPawnClass = class'UTP_Pawn';  // force PlayerChangedClass() in Mutant and for bots
  }
  return Super.CheckReplacement(Other, bSuperRelevant);
}


//== PlayerChangedClass =======================================================
/**
Make sure the players keep the UTP_Pawn as PawnClass.
*/
//=============================================================================

function PlayerChangedClass(Controller thePlayer)
{
  Super.PlayerChangedClass(thePlayer);
  
  //log("PlayerChangedClass"@thePlayer);
  if ( thePlayer.PreviousPawnClass == class'UTP_Pawn' ) {
    if ( thePlayer.PawnClass == class'xPawn' || thePlayer.PawnClass == class'xMutantPawn' )
      thePlayer.PawnClass = class'UTP_Pawn';
    else if ( thePlayer.PawnClass != class'UTP_Pawn' )
      log("Incompatible PawnClass" @ thePlayer.PawnClass @ "for" @ thePlayer, 'UTPlus');
  }
}


//== static FillPlayInfo ======================================================
/**
Add the configuration options to the PlayInfo.
*/
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  if ( class'UTP_ServerUtil'.static.ClassIsInPlayInfo(PlayInfo, default.Class) )
    return;
  
  Super.FillPlayInfo(PlayInfo);
  
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "bAllowDeRes", default.lblAllowDeRes, 0, 40, "Check");
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "bAllowBrighterSkins", default.lblAllowBrighterSkins, 0, 41, "Check");
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "SkinGlow", default.lblSkinGlow, 0, 42, "Text", "3;40:128");
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "TeamBeaconMaxDist", default.lblTeamBeaconMaxDist, 0, 43, "Text", "5;2000:20000");
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "TeamBeaconPlayerInfoMaxDist", default.lblTeamBeaconPlayerInfoMaxDist, 0, 44, "Text", "5;1000:10000");
  
  PlayInfo.PopClass();
}


//== static GetDescriptionText ================================================
/**
Returns a description for the specified configuration option.
*/
//=============================================================================

static event string GetDescriptionText(string PropName)
{
  switch (PropName) {
    case "bAllowDeRes":                 return default.descAllowDeRes;
    case "bAllowBrighterSkins":         return default.descAllowBrighterSkins;
    case "SkinGlow":                    return default.descSkinGlow;
    case "TeamBeaconMaxDist":           return default.descTeamBeaconMaxDist;
    case "TeamBeaconPlayerInfoMaxDist": return default.descTeamBeaconPlayerInfoMaxDist;
  }
  
  return Super.GetDescriptionText(PropName);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FriendlyName  = "UTPlus v1 Player Visibility"
  Description   = "Enhances player visibility by adding more color to the player skins and modifiying the de-res effects."
  GroupName     = "BrightSkins"
  
  bAllowDeRes         = True
  bAllowBrighterSkins = True
  SkinGlow            = 96
  TeamBeaconMaxDist   = 10000.0
  TeamBeaconPlayerInfoMaxDist = 1800.0
  
  lblAllowDeRes                   = "Allow custom DeRes"
  lblAllowBrighterSkins           = "Allow custom-colored skins"
  lblSkinGlow                     = "Skin Glow (UT2004: 40)"
  lblTeamBeaconMaxDist            = "Maximum Team Beacon Distance"
  lblTeamBeaconPlayerInfoMaxDist  = "Max. Team Beacon Player Info Dist."
  
  descAllowDeRes                  = "Allow clients to enable instant and/or team-colored DeRes and disable rising DeRes."
  descAllowBrighterSkins          = "Allow clients to enable a custom-colored player skin overlay."
  descSkinGlow                    = "Overall player brightness in darker map areas. The default value without the mutator is 40."
  descTeamBeaconMaxDist           = "Maximum distance the small triangle about teammates' heads are visible. (UT2004: 6000)"
  descTeamBeaconPlayerInfoMaxDist = "Maximum distance the player's name and health is displayed next to the team beacon. (UT2004: 1800)"
}
