/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


UTPlus HitSounds mutator.
******************************************************************************/


class MutUTPlusHitSounds extends Mutator
    config;


//=============================================================================
// Configuration
//=============================================================================

var config bool bAllowVariableHitSounds;
var config bool bLineOfSightHitSounds;


//=============================================================================
// Localization
//=============================================================================

var localized string lblAllowVariableHitSounds;
var localized string lblLineOfSightHitSounds;
var localized string descAllowVariableHitSounds;
var localized string descLineOfSightHitSounds;


//=============================================================================
// Variables
//=============================================================================

var UTP_GameRules HitSoundRules;


//== PostBeginPlay ============================================================
/**
Enable HitSounds in the UTP_GRI.
*/
//=============================================================================

function PostBeginPlay()
{
  local UTP_ReplicationInfo UTPlusRI;
  
  UTPlusRI = class'UTP_ReplicationInfo'.static.InitRI(Self);
  UTPlusRI.bHitSoundsAvailable      = True;
  UTPlusRI.bAllowVariableHitSounds  = bAllowVariableHitSounds;  // only for information
  UTPlusRI.bLineOfSightHitSounds    = bLineOfSightHitSounds;    // only for information
  AddToPackageMap(string('UTPlusSounds'));
  class'UTP_HitSounds'.default.bAllowVariableHitSounds = bAllowVariableHitSounds;
  
  HitSoundRules = class'UTP_GameRules'.static.InitUTPGR(Self);
  HitSoundRules.bAllowHitSounds = True;
  HitSoundRules.bLineOfSightHitSounds = bLineOfSightHitSounds;
}


//== ModifyPlayer =============================================================
/**
Make sure the hits to spawn-protected UTP_Pawns are registered as well.
*/
//=============================================================================

function ModifyPlayer(Pawn Other)
{
  Super.ModifyPlayer(Other);
  
  if ( UTP_Pawn(Other) != None && HitSoundRules != None ) {
    class'UTP_PlayerReplicationInfo'.static.FindFor(Other);
    UTP_Pawn(Other).OnCalcHitSoundsFor = HitSoundRules.CalcHitSoundsFor;
  }
}


//== static FillPlayInfo ======================================================
/**
Add the configuration options to the PlayInfo.
*/
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  if ( class'UTP_ServerUtil'.static.ClassIsInPlayInfo(PlayInfo, default.Class) )
    return;
  
  Super.FillPlayInfo(PlayInfo);
  
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "bAllowVariableHitSounds", default.lblAllowVariableHitSounds,  0, 50, "Check");
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "bLineOfSightHitSounds",   default.lblLineOfSightHitSounds,    0, 51, "Check");
  
  PlayInfo.PopClass();
}


//== static GetDescriptionText ================================================
/**
Returns a description for the specified configuration option.
*/
//=============================================================================

static event string GetDescriptionText(string PropName)
{
  switch (PropName) {
    case "bAllowVariableHitSounds": return default.descAllowVariableHitSounds;
    case "bLineOfSightHitSounds":   return default.descLineOfSightHitSounds;
  }
  
  return Super.GetDescriptionText(PropName);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FriendlyName  = "UTPlus v1 HitSounds"
  GroupName     = "HitSounds"
  Description   = "UTPlus HitSounds mutator."
  
  bAllowVariableHitSounds = True
  bLineOfSightHitSounds   = True
  
  lblAllowVariableHitSounds = "Allow variable hit sounds"
  lblLineOfSightHitSounds   = "Use line-of-sight hit sounds"
  
  descAllowVariableHitSounds  = "Allow variable hit sounds pitch based on damage."
  descLineOfSightHitSounds    = "Hit sounds will only play if the attacker has a line of sight to the victim."
}