/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Enables Q-Mode.
******************************************************************************/


class MutUTPlusQMode extends Mutator;


//== PostBeginPlay ============================================================
/**
Find the UTP replication info.
*/
//=============================================================================

function PostBeginPlay()
{
  Level.Game.AddGameModifier(Spawn(class'UTP_QModeGameRules'));
  
  Super.PostBeginPlay();
  
  if ( Level.Game.DefaultPlayerClassName ~= string(class'xPawn') || Level.Game.DefaultPlayerClassName ~= string(class'xMutantPawn') )
    Level.Game.DefaultPlayerClassName = string(class'UTP_Pawn');
}


//== CheckReplacement =========================================================
/**
Replace the pawn class for all controllers.
*/
//=============================================================================

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
  if ( Controller(Other) != None && Controller(Other).bIsPlayer && (Controller(Other).PawnClass == None || Controller(Other).PawnClass == class'xPawn' || Controller(Other).PawnClass == class'xMutantPawn') ) {
    //log("CheckReplacement"@Other);
    Controller(Other).PawnClass = class'UTP_Pawn';
    Controller(Other).PreviousPawnClass = class'UTP_Pawn';  // force PlayerChangedClass() in Mutant and for bots
  }
  else if (UTP_Pawn(Other) != None) {
    UTP_Pawn(Other).Health = 125;
    UTP_Pawn(Other).SuperHealthMax = 200;
    UTP_Pawn(Other).ShieldStrengthMax = 200;
    UTP_Pawn(Other).bDrainingHealth = True;
    UTP_Pawn(Other).bContinousShield = True;
  }
  else if (UTP_ReplicationInfo(Other) != None) {
    UTP_ReplicationInfo(Other).AmpDamageMultiplier = 3.0;
  }
  else if (WeaponPickup(Other) != None) {
    if (!Level.Game.bWeaponStay || !WeaponPickup(Other).bWeaponStay)
      WeaponPickup(Other).RespawnTime /= 1.090909;   // should be 25 seconds at default game speed
    else
      WeaponPickup(Other).RespawnTime /= 5.454545;   // should be 5 seconds at default game speed
    WeaponPickup(Other).bWeaponStay = False;
  }
  else if (Ammo(Other) != None) {
    Ammo(Other).RespawnTime *= 1.1; // should be 30 seconds at default game speed
  }
  else if (AdrenalinePickup(Other) != None) {
    ReplaceWith(Other, string(class'UTP_MiniShieldPack'));
    return false;
  }
  else if (ShieldPickup(Other) != None) {
    ShieldPickup(Other).RespawnTime = 27.5; // 25 seconds at default game speed for all shield types
  }
  else if (TournamentHealth(Other) != None) {
    TournamentHealth(Other).RespawnTime = 38.5; // 35 seconds at default game speed for all shield types
  }
  else if (xPickupBase(Other) != None && (class<ShieldPickup>(xPickupBase(Other).PowerUp) != None || class<TournamentHealth>(xPickupBase(Other).PowerUp) != None)) {
    xPickupBase(Other).bDelayedSpawn = False;
  }
  return Super.CheckReplacement(Other, bSuperRelevant);
}


//== PlayerChangedClass =======================================================
/**
Make sure the players keep the UTP_Pawn as PawnClass.
*/
//=============================================================================

function PlayerChangedClass(Controller thePlayer)
{
  Super.PlayerChangedClass(thePlayer);
  
  if ( thePlayer.PreviousPawnClass == class'UTP_Pawn' ) {
    if ( thePlayer.PawnClass == class'xPawn' || thePlayer.PawnClass == class'xMutantPawn' )
      thePlayer.PawnClass = class'UTP_Pawn';
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FriendlyName  = "UTPlus v1 Q-Mode"
  Description   = "Enables Q-Mode."
  GroupName     = "QuakeStyle"
}