/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Modifies the initial number of Assault Grenades.
******************************************************************************/


class MutUTPlusSpawnGrenades extends Mutator;


//=============================================================================
// Configuration
//=============================================================================

var config byte InitialGrenades;


//=============================================================================
// Localization
//=============================================================================

var localized string lblInitialGrenades;
var localized string descInitialGrenades;


//== PostBeginPlay ============================================================
/**
Modifies the initial amount of grenades.
*/
//=============================================================================

event PostBeginPlay()
{
  Super.PostBeginPlay();
  if ( GetUrlOption("InitialGrenades") != "" )
    InitialGrenades = Clamp(byte(GetUrlOption("InitialGrenades")), 0, 8);
  
  class'GrenadeAmmo'.default.InitialAmount = InitialGrenades;
}


//== ServerTraveling ==========================================================
/**
Resets the initial amount of grenades.
*/
//=============================================================================

function ServerTraveling(string URL, bool bItems)
{
  Super.ServerTraveling(URL, bItems);
  class'GrenadeAmmo'.default.InitialAmount = 4; // restore default
}


//== static FillPlayInfo ======================================================
/**
Adds the Multi-Dodging configuration to the webadmin interface.
*/
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  if ( class'UTP_ServerUtil'.static.ClassIsInPlayInfo(PlayInfo, default.Class) )
    return;
  
  Super.FillPlayInfo(PlayInfo);
  
  PlayInfo.AddSetting(class'MutUTPlus'.default.FriendlyName, "InitialGrenades", default.lblInitialGrenades, 0, 70, "Text", "1;0:8");
  
  PlayInfo.PopClass();
}


//== static GetDescriptionText ================================================
/**
Returns a description text for the specified property.
*/
//=============================================================================

static event string GetDescriptionText(string PropName)
{
  Switch (PropName) {
    Case "InitialGrenades": return default.descInitialGrenades;
  }
  return Super.GetDescriptionText(PropName);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FriendlyName = "UTPlus v1 Spawn Grenades"
  GroupName = "SpawnGrenades"
  Description = "Modifies the initial number of Assault Grenades."
  
  InitialGrenades = 1
  
  lblInitialGrenades = "Initial grenades"
  descInitialGrenades = "The initial amount of grenades for the Assault Rifle."
}
