/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


The UTPlus access control class.
******************************************************************************/


class UTP_AccessControl extends AccessControlIni;


//=============================================================================
// Localization
//=============================================================================

var localized string InvalidNickError, InvalidNickFailCode;


//=============================================================================
// Configuration
//=============================================================================

var globalconfig bool bAddToServerPackages;

var config bool bDisabled;


//=============================================================================
// Variables
//=============================================================================

var MutUTPlus MyMutator;


//== PreBeginPlay =============================================================
/**
Check whether UTPlus can actually be used.
*/
//=============================================================================

function PreBeginPlay()
{
  Super.PreBeginPlay();
  
  log("", 'UTPlus');
  log(class'UTP_Util'.static.RepeatChar("=", 79),             'UTPlus');
  log("" @ class'UTP_ServerUtil'.static.GetVersionString(),   'UTPlus');
  log("" @ class'UTP_ServerUtil'.static.GetCopyrightString(), 'UTPlus');
  log(class'UTP_Util'.static.RepeatChar("=", 79),             'UTPlus');
  
  if ( int(Level.EngineVersion) < 3339 ) {
    bDisabled = True;
    log("ERROR: Engine version 3339 or above is required.", 'UTPlusError');
  }
  if ( Level.NetMode != NM_DedicatedServer ) {
    bDisabled = True;
    log("ERROR: The UTPlus Access Control can only be used on a dedicated server.", 'UTPlusError');
  }
  if ( Level.Game.AccessControl != None && Level.Game.AccessControl != Self ) {
    bDisabled = True;
    log("ERROR:" @ Level.Game.AccessControl.Class @ "is already running as Access Control.", 'UTPlusError');
    Destroy();
  }
}


//== PostBeginPlay ============================================================
/**
Initialize the UTPlus access control.
*/
//=============================================================================

function PostBeginPlay()
{
  Super.PostBeginPlay();
  
  if ( bDisabled || bDeleteMe )
    return;
  
  Level.Game.AddMutator(string(class'MutUTPlus'), True);
  if ( MyMutator != None && MyMutator.UTPlusRI != None )
    MyMutator.UTPlusRI.bAdvancedAdmin = True;
  
  AddToServerPackages();
  log("", 'UTPlus');
}


//== AddToServerPackages ======================================================
/**
Adds the client package to the temporary ServerPackages list.
*/
//=============================================================================

function AddToServerPackages()
{
  local int i;
  
  AddToPackageMap(string(class'UTP_Security'.Outer));
  for (i = 0; i < class'GameEngine'.default.ServerPackages.Length; ++i)
    if ( class'GameEngine'.default.ServerPackages[i] ~= string(class'UTP_Security'.Outer) )
      break;
  
  if ( bAddToServerPackages && i == class'GameEngine'.default.ServerPackages.Length )
    class'GameEngine'.default.ServerPackages[i] = string(class'UTP_Security'.Outer);
  else if ( !bAddToServerPackages && i < class'GameEngine'.default.ServerPackages.Length )
    class'GameEngine'.default.ServerPackages.Remove(i, 1);
}


//== PreLogin =================================================================
/**
Allow extensions to reject players logging in.
*/
//=============================================================================

function PreLogin(string Options, string Address, string PlayerID, out string Error, out string FailCode, bool bSpectator)
{
  local int i;
  local string PlayerName;
  
  if ( !bDisabled && MyMutator != None && Error == "" && FailCode == "" ) {
    for (i = 0; i < MyMutator.Extensions.Length; ++i) {
      if ( MyMutator.Extensions[i] != None && MyMutator.Extensions[i].bNotifyLogin ) {
        MyMutator.Extensions[i].PreLogin(Options, Address, PlayerID, Error, FailCode, bSpectator);
        if ( Error != "" || FailCode != "" )
          break;
      }
    }
  }
  if ( Error == "" && FailCode == "" )
    Super.PreLogin(Options, Address, PlayerID, Error, FailCode, bSpectator);
  if ( Error == "" && FailCode == "" ) {
    PlayerName = Left(class'GameInfo'.static.ParseOption(Options, "Name"), 20);
    if ( PlayerName ~= "WebAdmin" || PlayerName ~= "UTV2004" && !bSpectator ) {
      Error = Repl(InvalidNickError, "%%nick%%", PlayerName, True);
      FailCode = Repl(InvalidNickFailCode, " ", "");
    }
  }
}


//== FillPlayInfo =============================================================
/**
Add the configuration options to the PlayInfo.
*/
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  Super.FillPlayInfo(PlayInfo);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bAddToServerPackages=False
  
  AdminClass=class'UTP_Admin'
  
  InvalidNickFailCode="Invalid player name"
  InvalidNickError="You are not allowed to use the name '%%nick%%'."
}