/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


UTPlus HitSounds GameRules.
******************************************************************************/


class UTP_GameRules extends GameRules;


//=============================================================================
// Variables
//=============================================================================

var array<UTP_HitSounds> HitSoundsList;
var bool bAllowHitSounds, bLineOfSightHitSounds;
var MutUTPlus MyMutator;


//== static FindFor ===========================================================
/**
Returns a UTP_PlayerReplicationInfo actor for a player. If none is found, a new one is spawned and returned.
*/
//=============================================================================

static function UTP_GameRules InitUTPGR(Actor Referencer)
{
  local UTP_GameRules UTP_GR;
  
  if ( Referencer == None )
    return None;
  
  foreach Referencer.DynamicActors(class'UTP_GameRules', UTP_GR) {
    return UTP_GR;
  }
  
  if ( Referencer.Level.Game != None ) {
    UTP_GR = Referencer.Spawn(default.Class);
    //log("Spawned"@UTP_GR);
    Referencer.Level.Game.AddGameModifier(UTP_GR);
  }
  return UTP_GR;
}


//== PostBeginPlay ============================================================
/**
Enable HitSounds in the UTP_GRI.
*/
//=============================================================================

function PostBeginPlay()
{
  //class'UTP_RealTime'.static.StartTimer(Self, 0.05, True);
  if ( TeamGame(Level.Game) != None && TeamGame(Level.Game).FriendlyFireScale == 0.0 )
    TeamGame(Level.Game).FriendlyFireScale = 0.0001;  // still no actual damage, but NetDamage() gets called now
}


//== Timer ====================================================================
/**
Lets each HitSounds handler check if HitSounds should be played.
*/
//=============================================================================

function Timer()
{
  local int i;
  
  for (i = 0; i < HitSoundsList.Length; ++i) {
    if ( HitSoundsList[i] != None ) {
      //HitSoundsList[i].DispatchHitSounds();
    }
    else {
      HitSoundsList.Remove(i--, 1);
    }
  }
}


//== OverridePickupQuery ======================================================
/**
Notify extensions of item pickup events.
*/
//=============================================================================

function bool OverridePickupQuery(Pawn Other, Pickup Item, out byte bAllowPickup)
{
  local bool bOverride;
  local int i;
  
  bOverride = Super.OverridePickupQuery(Other, item, bAllowPickup);
  if ( !bOverride || bool(bAllowPickup) && MyMutator != None ) {
    for (i = 0; i < MyMutator.Extensions.Length; ++i) {
      MyMutator.Extensions[i].PickupQuery(Other, Item);
    }
  }
  return bOverride;
}


//== NetDamage ================================================================
/**
Check for HitSound damage.
*/
//=============================================================================

function int NetDamage(int OriginalDamage, int Damage, Pawn Injured, Pawn InstigatedBy, vector HitLocation, out vector Momentum, class<DamageType> DamageType)
{
  if ( bAllowHitSounds && InstigatedBy != None && Injured != None && UTP_Pawn(Injured) == None && InstigatedBy != Injured && InstigatedBy.Health > 0 )
    CalcHitSoundsFor(class'UTP_PlayerReplicationInfo'.static.FindFor(InstigatedBy), Damage, Injured);
  
  return Super.NetDamage(OriginalDamage, Damage, Injured, InstigatedBy, HitLocation, Momentum, DamageType);
}


//== CalcHitSoundsFor =========================================================
/**
Plays a hit sound on the player.
*/
//=============================================================================

function CalcHitSoundsFor(UTP_PlayerReplicationInfo Player, int Damage, Pawn Victim)
{
  local UTP_HitSounds FoundHitSounds;
  local int i;
  
  if ( !bAllowHitSounds || Player == None || Controller(Player.Owner) == None || Damage < 0 || Victim == None || bLineOfSightHitSounds && !Controller(Player.Owner).LineOfSightTo(Victim) )
    return;
  
  for (i = 0; i < HitSoundsList.Length; i++) {
    if ( HitSoundsList[i] == None ) {
      HitSoundsList.Remove(i, 1);
      i--;
    }
    else if ( HitSoundsList[i].Owner == Player ) {
      FoundHitSounds = HitSoundsList[i];
      break;
    }
    else if ( HitSoundsList[i].Owner == None )
      HitSoundsList[i].Destroy();
  }
  
  if ( i == HitSoundsList.Length )
    HitSoundsList[i] = Spawn(class'UTP_HitSounds', Player);
  if ( HitSoundsList[i] != None )
    HitSoundsList[i].AddDamage(Damage, Victim);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
}