/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Server-side information holder class for individual players.
******************************************************************************/


class UTP_PlayerInfo extends Info;


//=============================================================================
// Variables
//=============================================================================

var editconst PlayerController          MyOwner;
var editconst MutUTPlus                 MyMutator;
var editconst UTP_PlayerReplicationInfo UTPlusPRI;
var editconst string                    PlayerID;
var editconst string                    PlayerIP;
var editconst int                       PlayerUID;
var editconst string                    LastPlayerName;


//== ClientMessage ============================================================
/**
Called when the player leaves the server.
*/
//=============================================================================

function ClientMessage(string Message, optional bool bDisplayInChatArea, optional color MsgColor, optional bool bNoLog)
{
  if ( MsgColor.A != 0 )
    Message = class'UTP_Util'.static.ColorCode(MsgColor) $ Message;
  if ( !bDisplayInChatArea ) {
    if ( UTPlusPRI != None )
      UTPlusPRI.QueueClientMessage(Message, bNoLog);
    else if ( MyOwner.PlayerSecurity.IsA('AntiTCCSecurity') || MyOwner.PlayerSecurity.IsA('UTP_Security') )
      MyOwner.PlayerSecurity.ClientMessage(Message);
  }
  else
    MyOwner.TeamMessage(None, Message, 'UTPlus');
  //log(Message, 'UTPlusClient');
}


//== NotifyLogout =============================================================
/**
Called when the player leaves the server.
*/
//=============================================================================

function NotifyLogout();


//== Tick =====================================================================
/**
Check for nick changes and make sure the nick is unique and doesn't contain any illegal characters.
*/
//=============================================================================

function Tick(float DeltaTime)
{
  local PlayerReplicationInfo OtherPRI;
  local string StrippedNick, ChangedNick;
  local bool bCollision;
  local int CollisionCount;
  
  if ( MyOwner != None && MyOwner.PlayerReplicationInfo != None ) {
    if ( LastPlayerName != MyOwner.PlayerReplicationInfo.PlayerName ) {
      if ( Len(MyOwner.PlayerReplicationInfo.PlayerName) > 1 )
        StrippedNick = class'UTP_ServerUtil'.static.StripNick(MyOwner.PlayerReplicationInfo.PlayerName);
      if ( StrippedNick ~= "WebAdmin" || !MyOwner.PlayerReplicationInfo.bOnlySpectator && StrippedNick ~= "UTV2004" )
        StrippedNick = "";
      if ( Len(StrippedNick) <= 1 ) {
        // stripped nick too short (must be at least 2 characters)
        if ( LastPlayerName != "" && !(LastPlayerName ~= "WebAdmin" || LastPlayerName ~= "UTV2004") )
          StrippedNick = LastPlayerName;
        else    // no valid previous nick available, generate a unique name for the player
          StrippedNick = "Player" $ Left(GetPlayerID(), 8);
      }
      
      if ( MyOwner.PlayerReplicationInfo.PlayerName != StrippedNick ) {
        // stripped nick differs from current nick
        ChangedNick = StrippedNick;
        do {
          // check for nick collisions
          bCollision = False;
          foreach DynamicActors(class'PlayerReplicationInfo', OtherPRI) {
            if ( OtherPRI != MyOwner.PlayerReplicationInfo && OtherPRI.PlayerName ~= ChangedNick ) {
              // nick collision!
              bCollision = True;
              break;
            }
          }
          if ( bCollision ) {
            // find new unique name by adding predefined affixes to it
            // (we can select from a total of 330 combinations, so we should find a valid one sooner or later)
            ChangedNick = Left(class'DeathMatch'.default.NamePrefixes[class'DeathMatch'.default.NameNumber % 10] $ StrippedNick $ class'DeathMatch'.default.NameSuffixes[class'DeathMatch'.default.NameNumber++ % 10], 20);
            if ( ++CollisionCount % 10 == 0 ) {
              // affixes didn't help, try again with a different backup nick
              if ( MyOwner.PlayerReplicationInfo.bIsFemale )
                StrippedNick = class'DeathMatch'.default.FemaleBackupNames[class'DeathMatch'.default.FemaleBackupNameOffset++ % 32];
              else
                StrippedNick = class'DeathMatch'.default.MaleBackupNames[class'DeathMatch'.default.MaleBackupNameOffset++ % 32];
              
              // first try without any affixes
              ChangedNick = StrippedNick;
            }
          }
        } until (!bCollision);
        MyOwner.PlayerReplicationInfo.PlayerName = ChangedNick;
      }
      if ( LastPlayerName != MyOwner.PlayerReplicationInfo.PlayerName ) {
        if ( MyMutator != None )
          MyMutator.NickChange(Self);
        LastPlayerName = MyOwner.PlayerReplicationInfo.PlayerName;
      }
    }
  }
}


//== state WaitForPlayer ======================================================
/**
Initial state. Wait for a PlayerController.
*/
//=============================================================================

auto state WaitForPlayer
{
  function Tick(float DeltaTime);

Begin:
  Sleep(0.0);
  if ( MyOwner == None || MyMutator == None )
    Destroy();
  else {
    MyMutator.PostLogin(Self);
    GotoState('Initializing');
  }
}


//== state Initializing =======================================================
/**
Initialize this security actor.
*/
//=============================================================================

state Initializing
{
Begin:
  Tick(0.0);
  UTPlusPRI = class'UTP_PlayerReplicationInfo'.static.FindFor(MyOwner);
  if ( UTPlusPRI != None && PlayerUID != -1 )
    UTPlusPRI.MyServerID = PlayerUID;
}


//== GetHumanReadableName =====================================================
/**
Returns the player ID hash.
*/
//=============================================================================

simulated function string GetHumanReadableName()
{
  if ( MyOwner != None )
    return class'UTP_ServerUtil'.static.StripNick(MyOwner.GetHumanReadableName());
  if ( LastPlayerName != "" )
    return LastPlayerName;
  if (PlayerUID != -1)
    return "Player" $ PlayerUID;
  return "Player";
}


//== GetPlayerID ==============================================================
/**
Returns the player ID hash.
*/
//=============================================================================

simulated function string GetPlayerID()
{
  if ( PlayerID == "" )
    PlayerID = MyOwner.GetPlayerIDHash();
  return PlayerID;
}


//== GetPlayerIP ==============================================================
/**
Returns the player's IP.
*/
//=============================================================================

function string GetPlayerIP()
{
  if ( PlayerIP == "" || PlayerIP == "(n/a)" ) {
    PlayerIP = MyOwner.GetPlayerNetworkAddress();
    if ( PlayerIP == "" )
      PlayerIP = "(n/a)";
    else
      PlayerIP = Left(PlayerIP, InStr(PlayerIP, ":"));
  }
  return PlayerIP;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  PlayerUID = -1
}