/******************************************************************************
UTPlus
Copyright (c) 2005-2006 by Wormbo

This program is free software; you can redistribute and/or modify 
it under the terms of the Open Unreal Mod License version 1.1.
http://wiki.beyondunreal.com/wiki/OpenUnrealModLicense


Implements ReservedSlots from El Muerte's ServerExt as a UTPlus extension.
******************************************************************************/


class UTP_ReservedSlotsPlugin extends UTP_Extension;


//=============================================================================
// Configuration
//=============================================================================

/** allows you to add more than one slot manager */
var config array<string> SlotManagerClasses;


//=============================================================================
// Variables
//=============================================================================

/** configured slot managers */
var array<SlotManagerBase> SlotManagers;
/** used to call FillPlayInfo */
var protected array< class<SlotManagerBase> > SMClasses;


//== PreBeginPlay =============================================================
/**
Initialize the slot manager(s).
*/
//=============================================================================

event PreBeginPlay()
{
  local int i;
  local class<SlotManagerBase> SM;
  
  Super.PreBeginPlay();
  Log("Loading ReservedSlots ...", name);
  Level.Game.AddMutator("ServerExt.ResSltMut");
  default.SMClasses.Length = 0;
  for (i = 0; i < SlotManagerClasses.Length; i++) {
    SM = class<SlotManagerBase>(DynamicLoadObject(SlotManagerClasses[i], class'Class'));
    if (SM != none) {
      default.SMClasses[default.SMClasses.Length] = SM;
      SlotManagers[SlotManagers.Length] = Spawn(SM);
    }
  }
}


//== PreLogin =================================================================
/**
Let the slot manager(s) check for a valid reserved slot.
*/
//=============================================================================

event PreLogin(string Options, string Address, string PlayerID, out string Error, out string FailCode, bool bSpectator)
{
  local int i;
  
  for (i = 0; i < SlotManagers.Length; i++)
    if (SlotManagers[i].PreLogin(Options, Address, PlayerID, Error, FailCode, bSpectator))
      return;
}


//== ModifyLogin ==============================================================
/**
Allow the slot manager(s) to modify login info.
*/
//=============================================================================

function ModifyLogin(out string Portal, out string Options)
{
  local int i;
  
  for (i = 0; i < SlotManagers.Length; i++)
    SlotManagers[i].ModifyLogin(Portal, Options);
}


//== FillExtensionInfo ========================================================
/**
Add the slot managers' configuration options to the PlayInfo.
*/
//=============================================================================

static function FillExtensionInfo(PlayInfo PlayInfo)
{
  local int i;
  
  Super.FillExtensionInfo(PlayInfo);
  
  for (i = 0; i < default.SMClasses.Length; i++) {
    default.SMClasses[i].static.FillPlayInfo(PlayInfo);
    PlayInfo.PopClass();
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FriendlyName  = "Plugin for El Muerte's Reserved Slots"
  Description   = "This plugin loads the Reserved Slots feature from El Muerte's ServerExt v105 and above. (requires UTP_AccessControl)"
  
  bOnlyForDedicatedServer   = True
  bNotifyLogin              = True
  bRequiresAccessContol     = True
  
  SlotManagerClasses[0] = "ServerExt.SlotManager"
}
